@@ -13,7 +13,6 @@ and show the job(s) logs.
 """
 
 import argparse
-import os
 import re
 import sys
 import time
@@ -26,7 +25,6 @@ from typing import Callable, Dict, TYPE_CHECKING, Iterable, Literal, Optional, T
 
 import gitlab
 import gitlab.v4.objects
-from colorama import Fore, Style
 from gitlab_common import (
     GITLAB_URL,
     TOKEN_DIR,
@@ -34,11 +32,11 @@ from gitlab_common import (
     get_gitlab_project,
     get_token_from_default_dir,
     pretty_duration,
-    print_once,
     read_token,
     wait_for_pipeline,
 )
 from gitlab_gql import GitlabGQL, create_job_needs_dag, filter_dag, print_dag, print_formatted_list
+from rich.console import Console
 
 if TYPE_CHECKING:
     from gitlab_gql import Dag
@@ -47,16 +45,13 @@ REFRESH_WAIT_LOG = 10
 REFRESH_WAIT_JOBS = 6
 MAX_ENABLE_JOB_ATTEMPTS = 3
 
-URL_START = "\033]8;;"
-URL_END = "\033]8;;\a"
-
 STATUS_COLORS = {
     "created": "",
-    "running": Fore.BLUE,
-    "success": Fore.GREEN,
-    "failed": Fore.RED,
-    "canceled": Fore.MAGENTA,
-    "canceling": Fore.MAGENTA,
+    "running": "[blue]",
+    "success": "[green]",
+    "failed": "[red]",
+    "canceled": "[magenta]",
+    "canceling": "[magenta]",
     "manual": "",
     "pending": "",
     "skipped": "",
@@ -65,6 +60,9 @@ STATUS_COLORS = {
 COMPLETED_STATUSES = frozenset({"success", "failed"})
 RUNNING_STATUSES = frozenset({"created", "pending", "running"})
 
+console = Console(highlight=False)
+print = console.print
+
 
 def print_job_status(
     job: gitlab.v4.objects.ProjectPipelineJob,
@@ -86,13 +84,12 @@ def print_job_status(
 
     duration = job_duration(job)
 
-    print_once(
-        STATUS_COLORS[job.status]
-        + f"{jtype:{type_field_pad}} "  # U+1F78B Round target
-        + link2print(job.web_url, job.name, name_field_pad)
-        + (f" has new status: {job.status}" if new_status else f" {job.status}")
-        + (f" ({pretty_duration(duration)})" if job.started_at else "")
-        + Style.RESET_ALL
+    print(
+        f"{STATUS_COLORS[job.status]}"
+        f"{jtype:{type_field_pad}} "  # U+1F78B Round target
+        f"{link2print(job.web_url, job.name, name_field_pad)} " +
+        (f" has new status: {job.status}" if new_status else f" {job.status}") +
+        (f" ({pretty_duration(duration)})" if job.started_at else "")
     )
 
 
@@ -145,9 +142,7 @@ def run_target_job(
 def monitor_pipeline(
     project: gitlab.v4.objects.Project,
     pipeline: gitlab.v4.objects.ProjectPipeline,
-    target_jobs_regex: re.Pattern,
-    include_stage_regex: re.Pattern,
-    exclude_stage_regex: re.Pattern,
+    job_filter: callable,
     dependencies: set[str],
     stress: int,
 ) -> tuple[Optional[int], Optional[int], Dict[str, Dict[int, Tuple[float, str, str]]]]:
@@ -167,10 +162,11 @@ def monitor_pipeline(
     if stress:
         # When stress test, it is necessary to collect this information before start.
         for job in pipeline.jobs.list(all=True, include_retried=True):
-            if target_jobs_regex.fullmatch(job.name) and \
-               include_stage_regex.fullmatch(job.stage) and \
-               not exclude_stage_regex.fullmatch(job.stage) and \
-               job.status in COMPLETED_STATUSES:
+            if job_filter(
+                job_name=job.name,
+                job_stage=job.stage,
+                job_tags=job.tag_list,
+            ) and job.status in COMPLETED_STATUSES:
                 execution_times[job.name][job.id] = (job_duration(job), job.status, job.web_url)
 
     # jobs_waiting is a list of job names that are waiting for status update.
@@ -191,9 +187,11 @@ def monitor_pipeline(
         jobs_waiting.clear()
         for job in sorted(pipeline.jobs.list(all=True), key=lambda j: j.name):
             job = cast(gitlab.v4.objects.ProjectPipelineJob, job)
-            if target_jobs_regex.fullmatch(job.name) and \
-               include_stage_regex.fullmatch(job.stage) and \
-               not exclude_stage_regex.fullmatch(job.stage):
+            if job_filter(
+                job_name=job.name,
+                job_stage=job.stage,
+                job_tags=job.tag_list,
+            ):
                 run_target_job(
                     job,
                     enable_job_fn,
@@ -235,7 +233,6 @@ def monitor_pipeline(
                     f"* {job_name:{name_field_pad}} succ: {n_succeed}; "
                     f"fail: {n_failed}; "
                     f"total: {n_total_seen} of {stress}",
-                    flush=False,
                 )
                 if stress < 0 or n_total_completed < stress:
                     enough = False
@@ -245,9 +242,8 @@ def monitor_pipeline(
                 continue
 
         if jobs_waiting:
-            print(f"{Fore.YELLOW}Waiting for jobs to update status:")
-            print_formatted_list(jobs_waiting, indentation=8)
-            print(Style.RESET_ALL, end='')
+            print(f"[yellow]Waiting for jobs to update status:")
+            print_formatted_list(jobs_waiting, indentation=8, color="[yellow]")
             pretty_wait(REFRESH_WAIT_JOBS)
             continue
 
@@ -269,10 +265,7 @@ def monitor_pipeline(
             and not RUNNING_STATUSES.intersection(target_statuses.values())
         ):
             print(
-                Fore.RED,
-                "Target in skipped state, aborting. Failed dependencies:",
-                deps_failed,
-                Fore.RESET,
+                f"[red]Target in skipped state, aborting. Failed dependencies:{deps_failed}"
             )
             return None, 1, execution_times
 
@@ -348,9 +341,7 @@ def enable_job(
     type_field_pad = len(jtype) if len(jtype) > type_field_pad else type_field_pad
     name_field_pad = len(job_name) if len(job_name) > name_field_pad else name_field_pad
     print(
-        Fore.MAGENTA +
-        f"{jtype:{type_field_pad}} {job.name:{name_field_pad}} manually enabled" +
-        Style.RESET_ALL
+        f"[magenta]{jtype:{type_field_pad}} {job.name:{name_field_pad}} manually enabled"
     )
 
     return True
@@ -416,7 +407,7 @@ def print_log(
         printed_lines = len(lines)
 
         if job.status in COMPLETED_STATUSES:
-            print(Fore.GREEN + f"Job finished: {job.web_url}" + Style.RESET_ALL)
+            print(f"[green]Job finished: {job.web_url}")
             return
         pretty_wait(REFRESH_WAIT_LOG)
 
@@ -465,12 +456,24 @@ def parse_args() -> argparse.Namespace:
         default=["performance", ".*-postmerge", ".*-nightly"],
         nargs=argparse.ONE_OR_MORE,
     )
+    parser.add_argument(
+        "--job-tags",
+        metavar="job-tags",
+        help="Job tags to require when searching for target jobs. If multiple "
+             "values are passed, eg. `--job-tags 'foo.*' 'bar'`, the job will "
+             "need to have a tag matching `foo.*` *and* a tag matching `bar` "
+             "to qualify. Passing `--job-tags '.*'` makes sure the job has "
+             "a tag defined, while not passing `--job-tags` also allows "
+             "untagged jobs.",
+        default=[],
+        nargs=argparse.ONE_OR_MORE,
+    )
     parser.add_argument(
         "--token",
         metavar="token",
         type=str,
         default=get_token_from_default_dir(),
-        help="Use the provided GitLab token or token file, "
+        help="Use the provided GitLab token (with `api` scope) or token file, "
              f"otherwise it's read from {TOKEN_DIR / 'gitlab-token'}",
     )
     parser.add_argument(
@@ -539,23 +542,19 @@ def print_detected_jobs(
 ) -> None:
     def print_job_set(color: str, kind: str, job_set: Iterable[str]):
         job_list = list(job_set)
-        print(color + f"Running {len(job_list)} {kind} jobs:")
-        print_formatted_list(job_list, indentation=8)
-        print(Style.RESET_ALL)
+        print(f"{color}Running {len(job_list)} {kind} jobs:")
+        print_formatted_list(job_list, indentation=8, color=color)
 
-    print(Fore.YELLOW + "Detected target job and its dependencies:")
-    print_dag(target_dep_dag, indentation=8)
-    print(Style.RESET_ALL)
-    print_job_set(Fore.MAGENTA, "dependency", dependency_jobs)
-    print_job_set(Fore.BLUE, "target", target_jobs)
+    print("[yellow]Detected target job and its dependencies:")
+    print_dag(target_dep_dag, indentation=8, color="[yellow]")
+    print_job_set("[magenta]", "dependency", dependency_jobs)
+    print_job_set("[blue]", "target", target_jobs)
 
 
 def find_dependencies(
     server: str,
     token: str | None,
-    target_jobs_regex: re.Pattern,
-    include_stage_regex: re.Pattern,
-    exclude_stage_regex: re.Pattern,
+    job_filter: callable,
     project_path: str,
     iid: int
 ) -> set[str]:
@@ -588,9 +587,9 @@ def find_dependencies(
         gql_instance, {"projectPath": project_path.path_with_namespace, "iid": iid}
     )
 
-    target_dep_dag = filter_dag(dag, target_jobs_regex, include_stage_regex, exclude_stage_regex)
+    target_dep_dag = filter_dag(dag, job_filter)
     if not target_dep_dag:
-        print(Fore.RED + "The job(s) were not found in the pipeline." + Fore.RESET)
+        print("[red]The job(s) were not found in the pipeline.")
         sys.exit(1)
 
     dependency_jobs = set(chain.from_iterable(d["needs"] for d in target_dep_dag.values()))
@@ -627,15 +626,16 @@ def __job_duration_record(dict_item: tuple) -> str:
     """
     job_id = f"{dict_item[0]}"  # dictionary key
     job_duration, job_status, job_url = dict_item[1]  # dictionary value, the tuple
-    return (f"{STATUS_COLORS[job_status]}"
-            f"{link2print(job_url, job_id)}: {pretty_duration(job_duration):>8}"
-            f"{Style.RESET_ALL}")
+    return (
+        f"{STATUS_COLORS[job_status]}"
+        f"{link2print(job_url, job_id)}: {pretty_duration(job_duration):>8}"
+    )
 
 
 def link2print(url: str, text: str, text_pad: int = 0) -> str:
     text = str(text)
     text_pad = len(text) if text_pad < 1 else text_pad
-    return f"{URL_START}{url}\a{text:{text_pad}}{URL_END}"
+    return f"[link={url}]{text:{text_pad}}[/link]"
 
 
 def main() -> None:
@@ -700,7 +700,7 @@ def main() -> None:
         target = '|'.join(args.target)
         target = target.strip()
 
-        print("🞋 target job: " + Fore.BLUE + target + Style.RESET_ALL)  # U+1F78B Round target
+        print(f"🞋 target job: [blue]{target}")  # U+1F78B Round target
 
         # Implicitly include `parallel:` jobs
         target = f'({target})' + r'( \d+/\d+)?'
@@ -710,23 +710,46 @@ def main() -> None:
         include_stage = '|'.join(args.include_stage)
         include_stage = include_stage.strip()
 
-        print("🞋 target from stages: " + Fore.BLUE + include_stage + Style.RESET_ALL)  # U+1F78B Round target
+        print(f"🞋 target from stages: [blue]{include_stage}")  # U+1F78B Round target
 
         include_stage_regex = re.compile(include_stage)
 
         exclude_stage = '|'.join(args.exclude_stage)
         exclude_stage = exclude_stage.strip()
 
-        print("🞋 target excluding stages: " + Fore.BLUE + exclude_stage + Style.RESET_ALL)  # U+1F78B Round target
+        print(f"🞋 target excluding stages: [blue]{exclude_stage}")  # U+1F78B Round target
 
         exclude_stage_regex = re.compile(exclude_stage)
 
+        print(f"🞋 target jobs with tags: [blue]{str(args.job_tags)}")  # U+1F78B Round target
+        job_tags_regexes = [re.compile(job_tag) for job_tag in args.job_tags]
+
+        def job_filter(
+            job_name: str,
+            job_stage: str,
+            job_tags: set[str],
+        ) -> bool:
+            """
+            Apply user-specified filters to a job, and return whether the
+            filters allow that job (True) or not (False).
+            """
+            if not target_jobs_regex.fullmatch(job_name):
+                return False
+            if not include_stage_regex.fullmatch(job_stage):
+                return False
+            if exclude_stage_regex.fullmatch(job_stage):
+                return False
+            if not all(
+                any(job_tags_regex.fullmatch(tag) for tag in job_tags)
+                for job_tags_regex in job_tags_regexes
+            ):
+                return False
+            return True
+
         deps = find_dependencies(
             server=args.server,
             token=token,
-            target_jobs_regex=target_jobs_regex,
-            include_stage_regex=include_stage_regex,
-            exclude_stage_regex=exclude_stage_regex,
+            job_filter=job_filter,
             iid=pipe.iid,
             project_path=cur_project
         )
@@ -737,9 +760,7 @@ def main() -> None:
         target_job_id, ret, exec_t = monitor_pipeline(
             cur_project,
             pipe,
-            target_jobs_regex,
-            include_stage_regex,
-            exclude_stage_regex,
+            job_filter,
             deps,
             args.stress
         )
