@@ -686,6 +686,10 @@ gfx12_init_graphics_preamble_state(const struct ac_preamble_state *state,
    ac_pm4_set_reg(pm4, R_00B4D0_SPI_SHADER_USER_ACCUM_LSHS_2, 0);
    ac_pm4_set_reg(pm4, R_00B4D4_SPI_SHADER_USER_ACCUM_LSHS_3, 0);
 
+   /* Shader registers - PS */
+   ac_pm4_set_reg(pm4, R_00B024_SPI_SHADER_PGM_HI_PS,
+                  S_00B024_MEM_BASE(info->address32_hi >> 8));
+
    /* Context registers */
    ac_pm4_set_reg(pm4, R_028040_DB_GL1_INTERFACE_CONTROL, 0);
    ac_pm4_set_reg(pm4, R_028048_DB_MEM_TEMPORAL,
@@ -853,3 +857,215 @@ ac_init_graphics_preamble_state(const struct ac_preamble_state *state,
       gfx6_init_graphics_preamble_state(state, pm4);
    }
 }
+
+void
+ac_emit_cond_exec(struct ac_cmdbuf *cs, enum amd_gfx_level gfx_level,
+                  uint64_t va, uint32_t count)
+{
+   ac_cmdbuf_begin(cs);
+   if (gfx_level >= GFX7) {
+      ac_cmdbuf_emit(PKT3(PKT3_COND_EXEC, 3, 0));
+      ac_cmdbuf_emit(va);
+      ac_cmdbuf_emit(va >> 32);
+      ac_cmdbuf_emit(0);
+      ac_cmdbuf_emit(count);
+   } else {
+      ac_cmdbuf_emit(PKT3(PKT3_COND_EXEC, 2, 0));
+      ac_cmdbuf_emit(va);
+      ac_cmdbuf_emit(va >> 32);
+      ac_cmdbuf_emit(count);
+   }
+   ac_cmdbuf_end();
+}
+
+void
+ac_emit_write_data_imm(struct ac_cmdbuf *cs, unsigned engine_sel, uint64_t va, uint32_t value)
+{
+   ac_cmdbuf_begin(cs);
+   ac_cmdbuf_emit(PKT3(PKT3_WRITE_DATA, 3, 0));
+   ac_cmdbuf_emit(S_370_DST_SEL(V_370_MEM) | S_370_WR_CONFIRM(1) | S_370_ENGINE_SEL(engine_sel));
+   ac_cmdbuf_emit(va);
+   ac_cmdbuf_emit(va >> 32);
+   ac_cmdbuf_emit(value);
+   ac_cmdbuf_end();
+}
+
+void
+ac_emit_cp_wait_mem(struct ac_cmdbuf *cs, uint64_t va, uint32_t ref,
+                    uint32_t mask, unsigned flags)
+{
+   ac_cmdbuf_begin(cs);
+   ac_cmdbuf_emit(PKT3(PKT3_WAIT_REG_MEM, 5, 0));
+   ac_cmdbuf_emit(WAIT_REG_MEM_MEM_SPACE(1) | flags);
+   ac_cmdbuf_emit(va);
+   ac_cmdbuf_emit(va >> 32);
+   ac_cmdbuf_emit(ref);  /* reference value */
+   ac_cmdbuf_emit(mask); /* mask */
+   ac_cmdbuf_emit(4);    /* poll interval */
+   ac_cmdbuf_end();
+}
+
+static bool
+is_ts_event(unsigned event_type)
+{
+   return event_type == V_028A90_CACHE_FLUSH_TS ||
+          event_type == V_028A90_CACHE_FLUSH_AND_INV_TS_EVENT ||
+          event_type == V_028A90_BOTTOM_OF_PIPE_TS ||
+          event_type == V_028A90_FLUSH_AND_INV_DB_DATA_TS ||
+          event_type == V_028A90_FLUSH_AND_INV_CB_DATA_TS;
+}
+
+/* This will wait or insert into the pipeline a wait for a previous
+ * RELEASE_MEM PWS event.
+ *
+ * "event_type" must be the same as the RELEASE_MEM PWS event.
+ *
+ * "stage_sel" determines when the waiting happens. It can be CP_PFP, CP_ME,
+ * PRE_SHADER, PRE_DEPTH, or PRE_PIX_SHADER, allowing to wait later in the
+ * pipeline instead of completely idling the hw at the frontend.
+ *
+ * "gcr_cntl" must be 0 if not waiting in PFP or ME. When waiting later in the
+ * pipeline, any cache flushes must be part of RELEASE_MEM, not ACQUIRE_MEM.
+ *
+ * "distance" determines how many RELEASE_MEM PWS events ago it should wait
+ * for, minus one (starting from 0). There are 3 event types: PS_DONE,
+ * CS_DONE, and TS events. The distance counter increments separately for each
+ * type, so 0 with PS_DONE means wait for the last PS_DONE event, while 0 with
+ * *_TS means wait for the last TS event (even if it's a different TS event
+ * because all TS events share the same counter).
+ *
+ * PRE_SHADER waits before the first shader that has IMAGE_OP=1, while
+ * PRE_PIX_SHADER waits before PS if it has IMAGE_OP=1 (IMAGE_OP should really
+ * be called SYNC_ENABLE) PRE_DEPTH waits before depth/stencil tests.
+ *
+ * PRE_COLOR also exists but shouldn't be used because it can hang. It's
+ * recommended to use PRE_PIX_SHADER instead, which means all PS that have
+ * color exports with enabled color buffers, non-zero colormask, and non-zero
+ * sample mask must have IMAGE_OP=1 to enable the sync before PS.
+ *
+ * Waiting for a PWS fence that was generated by a previous IB is valid, but
+ * if there is an IB from another process in between and that IB also inserted
+ * a PWS fence, the hw will wait for the newer fence instead because the PWS
+ * counter was incremented.
+ */
+void
+ac_emit_cp_acquire_mem_pws(struct ac_cmdbuf *cs, ASSERTED enum amd_gfx_level gfx_level,
+                           ASSERTED enum amd_ip_type ip_type, uint32_t event_type,
+                           uint32_t stage_sel, uint32_t count,
+                           uint32_t gcr_cntl)
+{
+   assert(gfx_level >= GFX11 && ip_type == AMD_IP_GFX);
+
+   const bool ts = is_ts_event(event_type);
+   const bool ps_done = event_type == V_028A90_PS_DONE;
+   const bool cs_done = event_type == V_028A90_CS_DONE;
+   const uint32_t counter_sel = ts ? V_580_TS_SELECT : ps_done ? V_580_PS_SELECT : V_580_CS_SELECT;
+
+   assert((int)ts + (int)cs_done + (int)ps_done == 1);
+   assert(!gcr_cntl || stage_sel == V_580_CP_PFP || stage_sel == V_580_CP_ME);
+   assert(stage_sel != V_580_PRE_COLOR);
+
+   ac_cmdbuf_begin(cs);
+   ac_cmdbuf_emit(PKT3(PKT3_ACQUIRE_MEM, 6, 0));
+   ac_cmdbuf_emit(S_580_PWS_STAGE_SEL(stage_sel) |
+                  S_580_PWS_COUNTER_SEL(counter_sel) |
+                  S_580_PWS_ENA2(1) |
+                  S_580_PWS_COUNT(count));
+   ac_cmdbuf_emit(0xffffffff); /* GCR_SIZE */
+   ac_cmdbuf_emit(0x01ffffff); /* GCR_SIZE_HI */
+   ac_cmdbuf_emit(0);          /* GCR_BASE_LO */
+   ac_cmdbuf_emit(0);          /* GCR_BASE_HI */
+   ac_cmdbuf_emit(S_585_PWS_ENA(1));
+   ac_cmdbuf_emit(gcr_cntl); /* GCR_CNTL (this has no effect if PWS_STAGE_SEL isn't PFP or ME) */
+   ac_cmdbuf_end();
+}
+
+/* Insert CS_DONE, PS_DONE, or a *_TS event into the pipeline, which will
+ * signal after the work indicated by the event is complete, which optionally
+ * includes flushing caches using "gcr_cntl" after the completion of the work.
+ * *_TS events are always signaled at the end of the pipeline, while CS_DONE
+ * and PS_DONE are signaled when those shaders finish. This call only inserts
+ * the event into the pipeline. It doesn't wait for anything and it doesn't
+ * execute anything immediately. The only way to wait for the event completion
+ * is to call si_cp_acquire_mem_pws with the same "event_type".
+ */
+void
+ac_emit_cp_release_mem_pws(struct ac_cmdbuf *cs, ASSERTED enum amd_gfx_level gfx_level,
+                           ASSERTED enum amd_ip_type ip_type, uint32_t event_type,
+                           uint32_t gcr_cntl)
+{
+   assert(gfx_level >= GFX11 && ip_type == AMD_IP_GFX);
+
+   /* Extract GCR_CNTL fields because the encoding is different in RELEASE_MEM. */
+   assert(G_586_GLI_INV(gcr_cntl) == 0);
+   assert(G_586_GL1_RANGE(gcr_cntl) == 0);
+   const uint32_t glm_wb = G_586_GLM_WB(gcr_cntl);
+   const uint32_t glm_inv = G_586_GLM_INV(gcr_cntl);
+   const uint32_t glk_wb = G_586_GLK_WB(gcr_cntl);
+   const uint32_t glk_inv = G_586_GLK_INV(gcr_cntl);
+   const uint32_t glv_inv = G_586_GLV_INV(gcr_cntl);
+   const uint32_t gl1_inv = G_586_GL1_INV(gcr_cntl);
+   assert(G_586_GL2_US(gcr_cntl) == 0);
+   assert(G_586_GL2_RANGE(gcr_cntl) == 0);
+   assert(G_586_GL2_DISCARD(gcr_cntl) == 0);
+   const uint32_t gl2_inv = G_586_GL2_INV(gcr_cntl);
+   const uint32_t gl2_wb = G_586_GL2_WB(gcr_cntl);
+   const uint32_t gcr_seq = G_586_SEQ(gcr_cntl);
+   const bool ts = is_ts_event(event_type);
+
+   ac_cmdbuf_begin(cs);
+   ac_cmdbuf_emit(PKT3(PKT3_RELEASE_MEM, 6, 0));
+   ac_cmdbuf_emit(S_490_EVENT_TYPE(event_type) |
+                   S_490_EVENT_INDEX(ts ? 5 : 6) |
+                   S_490_GLM_WB(glm_wb) |
+                   S_490_GLM_INV(glm_inv) |
+                   S_490_GLV_INV(glv_inv) |
+                   S_490_GL1_INV(gl1_inv) |
+                   S_490_GL2_INV(gl2_inv) |
+                   S_490_GL2_WB(gl2_wb) |
+                   S_490_SEQ(gcr_seq) |
+                   S_490_GLK_WB(glk_wb) |
+                   S_490_GLK_INV(glk_inv) |
+                   S_490_PWS_ENABLE(1));
+   ac_cmdbuf_emit(0); /* DST_SEL, INT_SEL, DATA_SEL */
+   ac_cmdbuf_emit(0); /* ADDRESS_LO */
+   ac_cmdbuf_emit(0); /* ADDRESS_HI */
+   ac_cmdbuf_emit(0); /* DATA_LO */
+   ac_cmdbuf_emit(0); /* DATA_HI */
+   ac_cmdbuf_emit(0); /* INT_CTXID */
+   ac_cmdbuf_end();
+}
+
+void
+ac_emit_cp_copy_data(struct ac_cmdbuf *cs, uint32_t src_sel, uint32_t dst_sel,
+                     uint64_t src_va, uint64_t dst_va,
+                     enum ac_cp_copy_data_flags flags)
+{
+   uint32_t dword0 = COPY_DATA_SRC_SEL(src_sel) |
+                     COPY_DATA_DST_SEL(dst_sel);
+
+   if (flags & AC_CP_COPY_DATA_WR_CONFIRM)
+      dword0 |= COPY_DATA_WR_CONFIRM;
+   if (flags & AC_CP_COPY_DATA_COUNT_SEL)
+      dword0 |= COPY_DATA_COUNT_SEL;
+   if (flags & AC_CP_COPY_DATA_ENGINE_PFP)
+      dword0 |= COPY_DATA_ENGINE_PFP;
+
+   ac_cmdbuf_begin(cs);
+   ac_cmdbuf_emit(PKT3(PKT3_COPY_DATA, 4, 0));
+   ac_cmdbuf_emit(dword0);
+   ac_cmdbuf_emit(src_va);
+   ac_cmdbuf_emit(src_va >> 32);
+   ac_cmdbuf_emit(dst_va);
+   ac_cmdbuf_emit(dst_va >> 32);
+   ac_cmdbuf_end();
+}
+
+void
+ac_emit_cp_pfp_sync_me(struct ac_cmdbuf *cs)
+{
+   ac_cmdbuf_begin(cs);
+   ac_cmdbuf_emit(PKT3(PKT3_PFP_SYNC_ME, 0, 0));
+   ac_cmdbuf_emit(0);
+   ac_cmdbuf_end();
+}
