@@ -1182,7 +1182,7 @@ static void print_vcn_preencode_input_picture(FILE *f, struct ac_ib_parser *ib,
 
 static void parse_vcn_enc_ib(FILE *f, struct ac_ib_parser *ib)
 {
-   rvcn_enc_cmd_t cmd = {};
+   rvcn_enc_cmd_t cmd = { 0 };
    ac_vcn_enc_init_cmds(&cmd, ib->vcn_version);
 
    while (ib->cur_dw < ib->num_dw) {
@@ -1558,6 +1558,8 @@ static void parse_vcn_enc_ib(FILE *f, struct ac_ib_parser *ib)
                     instruction == RENCODE_HEVC_HEADER_INSTRUCTION_SLICE_QP_DELTA ? "SLICE QP DELTA" :
                     instruction == RENCODE_HEVC_HEADER_INSTRUCTION_SAO_ENABLE ? "SAO ENABLE" :
                     instruction == RENCODE_HEVC_HEADER_INSTRUCTION_LOOP_FILTER_ACROSS_SLICES_ENABLE ? "LOOP FILTER ACROSS SLICES ENABLE" :
+                    instruction == RENCODE_HEVC_HEADER_INSTRUCTION_SLICE_SEGMENT_ADDRESS ? "SLICE SEGMENT ADDRESS" :
+                    instruction == RENCODE_HEVC_HEADER_INSTRUCTION_DEPENDENT_SLICE_SEGMENT_FLAG ? "DEPENDENT SLICE SEGMENT FLAG" :
                     instruction == RENCODE_H264_HEADER_INSTRUCTION_FIRST_MB ? "FIRST MB" :
                     instruction == RENCODE_H264_HEADER_INSTRUCTION_SLICE_QP_DELTA ? "SLICE QP DELTA" :
                     "???");
@@ -2152,7 +2154,7 @@ void ac_parse_ib_chunk(struct ac_ib_parser *ib)
    else if (ib->ip_type == AMD_IP_VCN_DEC || ib->ip_type == AMD_IP_VCN_ENC)
       parse_vcn_ib(memf, &tmp_ib);
    else
-      unreachable("unsupported IP type");
+      UNREACHABLE("unsupported IP type");
 
    u_memstream_close(&mem);
 
