@@ -33,6 +33,7 @@
 #include "opp.h"
 #include "vector.h"
 #include "hw_shared.h"
+#include "color_bg.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -63,12 +64,6 @@ struct resource {
     struct vpe_priv *vpe_priv;
     struct vpec      vpec;
 
-    bool (*check_input_color_space)(struct vpe_priv *vpe_priv, enum vpe_surface_pixel_format format,
-        const struct vpe_color_space *vcs);
-
-    bool (*check_output_color_space)(struct vpe_priv *vpe_priv,
-        enum vpe_surface_pixel_format format, const struct vpe_color_space *vcs);
-
     bool (*check_h_mirror_support)(bool *input_mirror, bool *output_miror);
 
     enum vpe_status (*calculate_segments)(
@@ -76,6 +71,10 @@ struct resource {
 
     enum vpe_status(*check_bg_color_support)(struct vpe_priv* vpe_priv, struct vpe_color* bg_color);
 
+    void (*bg_color_convert)(enum color_space output_cs, struct transfer_func *output_tf,
+        enum vpe_surface_pixel_format pixel_format, struct vpe_color *mpc_bg_color,
+        struct vpe_color *opp_bg_color, bool enable_3dlut);
+
     enum vpe_status (*set_num_segments)(struct vpe_priv *vpe_priv, struct stream_ctx *stream_ctx,
         struct scaler_data *scl_data, struct vpe_rect *src_rect, struct vpe_rect *dst_rect,
         uint32_t *max_seg_width, uint32_t recout_width_alignment);
@@ -164,7 +163,7 @@ void calculate_scaling_ratios(struct scaler_data *scl_data, struct vpe_rect *src
 uint16_t vpe_get_num_segments(struct vpe_priv *vpe_priv, const struct vpe_rect *src,
     const struct vpe_rect *dst, const uint32_t max_seg_width);
 
-bool should_generate_cmd_info(enum vpe_stream_type stream_type);
+bool should_generate_cmd_info(struct stream_ctx *stream_ctx);
 
 enum vpe_status vpe_resource_build_scaling_params(struct segment_ctx *segment);
 
@@ -182,6 +181,10 @@ void vpe_backend_config_callback(
 
 bool vpe_rec_is_equal(struct vpe_rect rec1, struct vpe_rect rec2);
 
+const struct vpe_caps *vpe_get_capability(enum vpe_ip_level ip_level);
+
+void vpe_setup_check_funcs(struct vpe_check_support_funcs *funcs, enum vpe_ip_level ip_level);
+
 #ifdef __cplusplus
 }
 #endif
