@@ -34,13 +34,13 @@ load_ring(nir_builder *b, unsigned ring, lower_abi_state *s)
 
    nir_def *ring_offsets = ac_nir_load_arg(b, &s->args->ac, arg);
    ring_offsets = nir_pack_64_2x32_split(b, nir_channel(b, ring_offsets, 0), nir_channel(b, ring_offsets, 1));
-   return nir_load_smem_amd(b, 4, ring_offsets, nir_imm_int(b, ring * 16u), .align_mul = 4u);
+   return ac_nir_load_smem(b, 4, ring_offsets, nir_imm_int(b, ring * 16u), 4, ACCESS_CAN_SPECULATE);
 }
 
 static nir_def *
 nggc_bool_setting(nir_builder *b, unsigned mask, lower_abi_state *s)
 {
-   nir_def *settings = ac_nir_load_arg(b, &s->args->ac, s->args->ngg_culling_settings);
+   nir_def *settings = ac_nir_load_arg(b, &s->args->ac, s->args->nggc_settings);
    return nir_test_mask(b, settings, mask);
 }
 
@@ -62,7 +62,7 @@ static bool
 lower_abi_instr(nir_builder *b, nir_intrinsic_instr *intrin, void *state)
 {
    lower_abi_state *s = (lower_abi_state *)state;
-   gl_shader_stage stage = b->shader->info.stage;
+   mesa_shader_stage stage = b->shader->info.stage;
 
    b->cursor = nir_before_instr(&intrin->instr);
 
@@ -134,7 +134,7 @@ lower_abi_instr(nir_builder *b, nir_intrinsic_instr *intrin, void *state)
             replacement = nir_iadd_imm_nuw(b, n, 1);
          }
       } else
-         unreachable("invalid tessellation shader stage");
+         UNREACHABLE("invalid tessellation shader stage");
       break;
    case nir_intrinsic_load_pipeline_stat_query_enabled_amd:
       replacement = shader_query_bool_setting(b, radv_shader_query_pipeline_stat, s);
@@ -173,7 +173,7 @@ lower_abi_instr(nir_builder *b, nir_intrinsic_instr *intrin, void *state)
       nir_def *mask =
          nir_bcsel(b, small_workgroup, nir_imm_int(b, radv_nggc_none),
                    nir_imm_int(b, radv_nggc_front_face | radv_nggc_back_face | radv_nggc_small_primitives));
-      nir_def *settings = ac_nir_load_arg(b, &s->args->ac, s->args->ngg_culling_settings);
+      nir_def *settings = ac_nir_load_arg(b, &s->args->ac, s->args->nggc_settings);
       replacement = nir_ine_imm(b, nir_iand(b, settings, mask), 0);
       break;
    }
@@ -195,7 +195,7 @@ lower_abi_instr(nir_builder *b, nir_intrinsic_instr *intrin, void *state)
        * exponent = nggc_settings >> 24
        * precision = 1.0 * 2 ^ exponent
        */
-      nir_def *settings = ac_nir_load_arg(b, &s->args->ac, s->args->ngg_culling_settings);
+      nir_def *settings = ac_nir_load_arg(b, &s->args->ac, s->args->nggc_settings);
       nir_def *exponent = nir_ishr_imm(b, settings, 24u);
       replacement = nir_ldexp(b, nir_imm_float(b, 1.0f), exponent);
       break;
@@ -203,10 +203,10 @@ lower_abi_instr(nir_builder *b, nir_intrinsic_instr *intrin, void *state)
 
    case nir_intrinsic_load_cull_triangle_viewport_xy_scale_and_offset_amd: {
       nir_def *comps[] = {
-         ac_nir_load_arg(b, &s->args->ac, s->args->ngg_viewport_scale[0]),
-         ac_nir_load_arg(b, &s->args->ac, s->args->ngg_viewport_scale[1]),
-         ac_nir_load_arg(b, &s->args->ac, s->args->ngg_viewport_translate[0]),
-         ac_nir_load_arg(b, &s->args->ac, s->args->ngg_viewport_translate[1]),
+         ac_nir_load_arg(b, &s->args->ac, s->args->nggc_viewport_scale[0]),
+         ac_nir_load_arg(b, &s->args->ac, s->args->nggc_viewport_scale[1]),
+         ac_nir_load_arg(b, &s->args->ac, s->args->nggc_viewport_translate[0]),
+         ac_nir_load_arg(b, &s->args->ac, s->args->nggc_viewport_translate[1]),
       };
       replacement = nir_vec(b, comps, 4);
       break;
@@ -243,7 +243,7 @@ lower_abi_instr(nir_builder *b, nir_intrinsic_instr *intrin, void *state)
          replacement = ac_nir_load_arg(b, &s->args->ac, s->args->vgt_esgs_ring_itemsize);
       } else {
          const unsigned stride =
-            s->info->is_ngg ? s->info->ngg_info.vgt_esgs_ring_itemsize : s->info->legacy_gs_info.esgs_itemsize;
+            s->info->is_ngg ? s->info->ngg_info.vgt_esgs_ring_itemsize : s->info->gs_ring_info.esgs_itemsize;
          replacement = nir_imm_int(b, stride);
       }
       break;
@@ -292,8 +292,10 @@ lower_abi_instr(nir_builder *b, nir_intrinsic_instr *intrin, void *state)
          offset = nir_iadd(b, offset, nir_ishl_imm(b, intrin->src[1].ssa, 3));
       }
 
-      replacement =
-         nir_load_global_amd(b, 2, 32, addr, offset, .base = sample_pos_offset, .access = ACCESS_NON_WRITEABLE);
+      offset = nir_iadd_imm_nuw(b, offset, sample_pos_offset);
+      addr = nir_iadd(b, addr, nir_u2u64(b, offset));
+      replacement = nir_build_load_global(b, 2, 32, addr,
+                                          .access = ACCESS_NON_WRITEABLE | ACCESS_CAN_SPECULATE | ACCESS_CAN_REORDER);
       break;
    }
    case nir_intrinsic_load_rasterization_samples_amd:
@@ -320,7 +322,7 @@ lower_abi_instr(nir_builder *b, nir_intrinsic_instr *intrin, void *state)
                /* TES won't use this intrinsic, because it can get primitive id directly
                 * instead of using this intrinsic to pass primitive id by LDS.
                 */
-               unreachable("load_provoking_vtx_in_prim_amd is only supported in VS and GS");
+               UNREACHABLE("load_provoking_vtx_in_prim_amd is only supported in VS and GS");
             }
          }
 
@@ -366,7 +368,8 @@ lower_abi_instr(nir_builder *b, nir_intrinsic_instr *intrin, void *state)
    case nir_intrinsic_load_streamout_buffer_amd: {
       nir_def *ptr = nir_pack_64_2x32_split(b, ac_nir_load_arg(b, &s->args->ac, s->args->streamout_buffers),
                                             nir_imm_int(b, s->address32_hi));
-      replacement = nir_load_smem_amd(b, 4, ptr, nir_imm_int(b, nir_intrinsic_base(intrin) * 16));
+      replacement =
+         ac_nir_load_smem(b, 4, ptr, nir_imm_int(b, nir_intrinsic_base(intrin) * 16), 4, ACCESS_CAN_SPECULATE);
       break;
    }
    case nir_intrinsic_load_xfb_state_address_gfx12_amd:
@@ -408,7 +411,7 @@ lower_abi_instr(nir_builder *b, nir_intrinsic_instr *intrin, void *state)
             num_vertices = 3;
             break;
          default:
-            unreachable("invalid GS output primitive");
+            UNREACHABLE("invalid GS output primitive");
             break;
          }
          replacement = nir_imm_int(b, num_vertices);
