@@ -31,7 +31,7 @@ struct radv_shader_context {
    const struct radv_shader_info *shader_info;
    const struct radv_shader_args *args;
 
-   gl_shader_stage stage;
+   mesa_shader_stage stage;
 
    unsigned max_workgroup_size;
    LLVMContextRef context;
@@ -57,13 +57,13 @@ create_llvm_function(struct ac_llvm_context *ctx, LLVMModuleRef module, LLVMBuil
    }
 
    ac_llvm_set_workgroup_size(main_function.value, max_workgroup_size);
-   ac_llvm_set_target_features(main_function.value, ctx, true);
+   ac_llvm_set_target_features(main_function.value, ctx, options->wgp_mode);
 
    return main_function;
 }
 
 static enum ac_llvm_calling_convention
-get_llvm_calling_convention(LLVMValueRef func, gl_shader_stage stage)
+get_llvm_calling_convention(LLVMValueRef func, mesa_shader_stage stage)
 {
    switch (stage) {
    case MESA_SHADER_VERTEX:
@@ -83,19 +83,19 @@ get_llvm_calling_convention(LLVMValueRef func, gl_shader_stage stage)
       return AC_LLVM_AMDGPU_CS;
       break;
    default:
-      unreachable("Unhandle shader type");
+      UNREACHABLE("Unhandle shader type");
    }
 }
 
 /* Returns whether the stage is a stage that can be directly before the GS */
 static bool
-is_pre_gs_stage(gl_shader_stage stage)
+is_pre_gs_stage(mesa_shader_stage stage)
 {
    return stage == MESA_SHADER_VERTEX || stage == MESA_SHADER_TESS_EVAL;
 }
 
 static void
-create_function(struct radv_shader_context *ctx, gl_shader_stage stage, bool has_previous_stage)
+create_function(struct radv_shader_context *ctx, mesa_shader_stage stage, bool has_previous_stage)
 {
    if (ctx->ac.gfx_level >= GFX10) {
       if (is_pre_gs_stage(stage) && ctx->shader_info->is_ngg) {
@@ -108,13 +108,6 @@ create_function(struct radv_shader_context *ctx, gl_shader_stage stage, bool has
    ctx->main_function = create_llvm_function(&ctx->ac, ctx->ac.module, ctx->ac.builder, &ctx->args->ac,
                                              get_llvm_calling_convention(ctx->main_function.value, stage),
                                              ctx->max_workgroup_size, ctx->options);
-
-   if (stage == MESA_SHADER_TESS_CTRL || (stage == MESA_SHADER_VERTEX && ctx->shader_info->vs.as_ls) ||
-       ctx->shader_info->is_ngg ||
-       /* GFX9 has the ESGS ring buffer in LDS. */
-       (stage == MESA_SHADER_GEOMETRY && has_previous_stage)) {
-      ac_declare_lds_as_pointer(&ctx->ac);
-   }
 }
 
 static LLVMValueRef
@@ -130,7 +123,7 @@ radv_load_rsrc(struct radv_shader_context *ctx, LLVMValueRef ptr, LLVMTypeRef ty
    if (ptr && LLVMTypeOf(ptr) == ctx->ac.i32) {
       LLVMValueRef result;
 
-      LLVMTypeRef ptr_type = LLVMPointerType(type, AC_ADDR_SPACE_CONST_32BIT);
+      LLVMTypeRef ptr_type = LLVMPointerTypeInContext(ctx->ac.context, AC_ADDR_SPACE_CONST_32BIT);
       ptr = LLVMBuildIntToPtr(ctx->ac.builder, ptr, ptr_type, "");
       LLVMSetMetadata(ptr, ctx->ac.uniform_md_kind, ctx->ac.empty_md);
 
@@ -183,15 +176,6 @@ radv_get_sampler_desc(struct ac_shader_abi *abi, LLVMValueRef index, enum ac_des
    return radv_load_rsrc(ctx, index, v4 ? ctx->ac.v4i32 : ctx->ac.v8i32);
 }
 
-static LLVMValueRef
-radv_load_output(struct radv_shader_context *ctx, unsigned index, unsigned chan)
-{
-   int idx = ac_llvm_reg_index_soa(index, chan);
-   LLVMValueRef output = ctx->abi.outputs[idx];
-   LLVMTypeRef type = ctx->abi.is_16bit[idx] ? ctx->ac.f16 : ctx->ac.f32;
-   return LLVMBuildLoad2(ctx->ac.builder, type, output, "");
-}
-
 static void
 ac_llvm_finalize_module(struct radv_shader_context *ctx, struct ac_midend_optimizer *meo)
 {
@@ -199,22 +183,6 @@ ac_llvm_finalize_module(struct radv_shader_context *ctx, struct ac_midend_optimi
    ac_llvm_context_dispose(&ctx->ac);
 }
 
-/* Ensure that the esgs ring is declared.
- *
- * We declare it with 64KB alignment as a hint that the
- * pointer value will always be 0.
- */
-static void
-declare_esgs_ring(struct radv_shader_context *ctx)
-{
-   assert(!LLVMGetNamedGlobal(ctx->ac.module, "esgs_ring"));
-
-   LLVMValueRef esgs_ring =
-      LLVMAddGlobalInAddressSpace(ctx->ac.module, LLVMArrayType(ctx->ac.i32, 0), "esgs_ring", AC_ADDR_SPACE_LDS);
-   LLVMSetLinkage(esgs_ring, LLVMExternalLinkage);
-   LLVMSetAlignment(esgs_ring, 64 * 1024);
-}
-
 static LLVMModuleRef
 ac_translate_nir_to_llvm(struct ac_llvm_compiler *ac_llvm, const struct radv_nir_compiler_options *options,
                          const struct radv_shader_info *info, struct nir_shader *const *shaders, int shader_count,
@@ -238,8 +206,7 @@ ac_translate_nir_to_llvm(struct ac_llvm_compiler *ac_llvm, const struct radv_nir
       exports_color_null = !exports_mrtz || (shaders[0]->info.outputs_written & (0xffu << FRAG_RESULT_DATA0));
    }
 
-   ac_llvm_context_init(&ctx.ac, ac_llvm, options->info, float_mode, info->wave_size, info->ballot_bit_size,
-                        exports_color_null, exports_mrtz);
+   ac_llvm_context_init(&ctx.ac, ac_llvm, options->info, float_mode, info->wave_size, exports_color_null, exports_mrtz);
 
    uint32_t length = 1;
    for (uint32_t i = 0; i < shader_count; i++)
@@ -280,26 +247,6 @@ ac_translate_nir_to_llvm(struct ac_llvm_compiler *ac_llvm, const struct radv_nir
       ac_init_exec_full_mask(&ctx.ac);
 
    if (is_ngg) {
-      if (!info->is_ngg_passthrough)
-         declare_esgs_ring(&ctx);
-
-      if (ctx.stage == MESA_SHADER_GEOMETRY) {
-         /* Scratch space used by NGG GS for repacking vertices at the end. */
-         LLVMTypeRef ai32 = LLVMArrayType(ctx.ac.i32, 8);
-         LLVMValueRef gs_ngg_scratch =
-            LLVMAddGlobalInAddressSpace(ctx.ac.module, ai32, "ngg_scratch", AC_ADDR_SPACE_LDS);
-         LLVMSetInitializer(gs_ngg_scratch, LLVMGetUndef(ai32));
-         LLVMSetLinkage(gs_ngg_scratch, LLVMExternalLinkage);
-         LLVMSetAlignment(gs_ngg_scratch, 4);
-
-         /* Vertex emit space used by NGG GS for storing all vertex attributes. */
-         LLVMValueRef gs_ngg_emit =
-            LLVMAddGlobalInAddressSpace(ctx.ac.module, LLVMArrayType(ctx.ac.i32, 0), "ngg_emit", AC_ADDR_SPACE_LDS);
-         LLVMSetInitializer(gs_ngg_emit, LLVMGetUndef(ai32));
-         LLVMSetLinkage(gs_ngg_emit, LLVMExternalLinkage);
-         LLVMSetAlignment(gs_ngg_emit, 4);
-      }
-
       /* GFX10 hang workaround - there needs to be an s_barrier before gs_alloc_req always */
       if (ctx.ac.gfx_level == GFX10 && shader_count == 1)
          ac_build_s_barrier(&ctx.ac, shaders[0]->info.stage);
@@ -443,6 +390,7 @@ ac_compile_llvm_module(struct ac_llvm_compiler *ac_llvm, LLVMModuleRef llvm_modu
 
    rbin->base.type = RADV_BINARY_TYPE_RTLD;
    rbin->base.total_size = alloc_size;
+   rbin->base.config.wgp_mode = options->wgp_mode;
    rbin->elf_size = elf_size;
    rbin->llvm_ir_size = llvm_ir_size;
    *rbinary = &rbin->base;
