@@ -32,7 +32,7 @@ radv_tex_wrap(VkSamplerAddressMode address_mode)
    case VK_SAMPLER_ADDRESS_MODE_MIRROR_CLAMP_TO_EDGE:
       return V_008F30_SQ_TEX_MIRROR_ONCE_LAST_TEXEL;
    default:
-      unreachable("illegal tex wrap mode");
+      UNREACHABLE("illegal tex wrap mode");
       break;
    }
    return 0;
@@ -59,7 +59,7 @@ radv_tex_compare(VkCompareOp op)
    case VK_COMPARE_OP_ALWAYS:
       return V_008F30_SQ_TEX_DEPTH_COMPARE_ALWAYS;
    default:
-      unreachable("illegal compare mode");
+      UNREACHABLE("illegal compare mode");
       break;
    }
    return 0;
@@ -208,7 +208,7 @@ radv_sampler_init(struct radv_device *device, struct radv_sampler *sampler, cons
    unsigned depth_compare_func = V_008F30_SQ_TEX_DEPTH_COMPARE_NEVER;
    bool trunc_coord = ((pCreateInfo->minFilter == VK_FILTER_NEAREST && pCreateInfo->magFilter == VK_FILTER_NEAREST) ||
                        pdev->info.conformant_trunc_coord) &&
-                      !instance->drirc.disable_trunc_coord;
+                      !instance->drirc.debug.disable_trunc_coord;
    bool uses_border_color = pCreateInfo->addressModeU == VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER ||
                             pCreateInfo->addressModeV == VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER ||
                             pCreateInfo->addressModeW == VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER;
@@ -259,7 +259,7 @@ radv_sampler_init(struct radv_device *device, struct radv_sampler *sampler, cons
       .min_lod = pCreateInfo->minLod,
       .max_lod = pCreateInfo->maxLod,
       .lod_bias = pCreateInfo->mipLodBias,
-      .aniso_single_level = !instance->drirc.disable_aniso_single_level,
+      .aniso_single_level = !instance->drirc.debug.disable_aniso_single_level,
       .border_color_type = radv_tex_bordercolor(border_color),
       .border_color_ptr = border_color_ptr,
    };
