@@ -23,8 +23,8 @@ struct radv_sampler {
 
 VK_DEFINE_NONDISP_HANDLE_CASTS(radv_sampler, vk.base, VkSampler, VK_OBJECT_TYPE_SAMPLER)
 
-void radv_sampler_init(struct radv_device *device, struct radv_sampler *sampler,
-                       const VkSamplerCreateInfo *pCreateInfo);
+VkResult radv_sampler_init(struct radv_device *device, struct radv_sampler *sampler,
+                           const VkSamplerCreateInfo *pCreateInfo);
 void radv_sampler_finish(struct radv_device *device, struct radv_sampler *sampler);
 
 #endif /* RADV_SAMPLER_H */
