@@ -30,23 +30,20 @@ struct radv_device;
 struct radv_pipeline;
 struct radv_ray_tracing_pipeline;
 struct radv_shader_args;
-struct radv_vertex_input_state;
 struct radv_shader_args;
 struct radv_serialized_shader_arena_block;
 struct vk_pipeline_robustness_state;
 
-enum {
-   RADV_GRAPHICS_STAGE_BITS =
-      (VK_SHADER_STAGE_ALL_GRAPHICS | VK_SHADER_STAGE_MESH_BIT_EXT | VK_SHADER_STAGE_TASK_BIT_EXT),
-   RADV_RT_STAGE_BITS =
-      (VK_SHADER_STAGE_RAYGEN_BIT_KHR | VK_SHADER_STAGE_ANY_HIT_BIT_KHR | VK_SHADER_STAGE_CLOSEST_HIT_BIT_KHR |
-       VK_SHADER_STAGE_MISS_BIT_KHR | VK_SHADER_STAGE_INTERSECTION_BIT_KHR | VK_SHADER_STAGE_CALLABLE_BIT_KHR)
-};
+#define RADV_GRAPHICS_STAGE_BITS                                                                                       \
+   (VK_SHADER_STAGE_ALL_GRAPHICS | VK_SHADER_STAGE_MESH_BIT_EXT | VK_SHADER_STAGE_TASK_BIT_EXT)
+#define RADV_RT_STAGE_BITS                                                                                             \
+   (VK_SHADER_STAGE_RAYGEN_BIT_KHR | VK_SHADER_STAGE_ANY_HIT_BIT_KHR | VK_SHADER_STAGE_CLOSEST_HIT_BIT_KHR |           \
+    VK_SHADER_STAGE_MISS_BIT_KHR | VK_SHADER_STAGE_INTERSECTION_BIT_KHR | VK_SHADER_STAGE_CALLABLE_BIT_KHR)
 
 #define RADV_STAGE_MASK ((1 << MESA_VULKAN_SHADER_STAGES) - 1)
 
 #define radv_foreach_stage(stage, stage_bits)                                                                          \
-   for (gl_shader_stage stage, __tmp = (gl_shader_stage)((stage_bits) & RADV_STAGE_MASK);                              \
+   for (mesa_shader_stage stage, __tmp = (mesa_shader_stage)((stage_bits) & RADV_STAGE_MASK);                          \
         stage = ffs(__tmp) - 1, __tmp; __tmp &= ~(1 << (stage)))
 
 enum radv_nggc_settings {
@@ -73,6 +70,7 @@ enum radv_required_subgroup_size {
 struct radv_shader_stage_key {
    uint8_t subgroup_required_size : 2; /* radv_required_subgroup_size */
    uint8_t subgroup_require_full : 1;  /* whether full subgroups are required */
+   uint8_t subgroup_allow_varying : 1; /* whether subgroup size can differ from the api constant */
 
    uint8_t storage_robustness2 : 1;
    uint8_t uniform_robustness2 : 1;
@@ -257,8 +255,8 @@ struct radv_shader_layout {
 };
 
 struct radv_shader_stage {
-   gl_shader_stage stage;
-   gl_shader_stage next_stage;
+   mesa_shader_stage stage;
+   mesa_shader_stage next_stage;
 
    struct {
       const struct vk_object_base *object;
@@ -292,33 +290,6 @@ radv_is_last_vgt_stage(const struct radv_shader_stage *stage)
           (stage->info.next_stage == MESA_SHADER_FRAGMENT || stage->info.next_stage == MESA_SHADER_NONE);
 }
 
-struct radv_vertex_input_state {
-   uint32_t attribute_mask;
-
-   uint32_t instance_rate_inputs;
-   uint32_t nontrivial_divisors;
-   uint32_t zero_divisors;
-   uint32_t post_shuffle;
-   /* Having two separate fields instead of a single uint64_t makes it easier to remove attributes
-    * using bitwise arithmetic.
-    */
-   uint32_t alpha_adjust_lo;
-   uint32_t alpha_adjust_hi;
-   uint32_t nontrivial_formats;
-
-   uint8_t bindings[MAX_VERTEX_ATTRIBS];
-   uint32_t divisors[MAX_VERTEX_ATTRIBS];
-   uint32_t offsets[MAX_VERTEX_ATTRIBS];
-   uint8_t formats[MAX_VERTEX_ATTRIBS];
-   uint8_t format_align_req_minus_1[MAX_VERTEX_ATTRIBS];
-   uint8_t component_align_req_minus_1[MAX_VERTEX_ATTRIBS];
-   uint8_t format_sizes[MAX_VERTEX_ATTRIBS];
-   uint32_t attrib_index_offset[MAX_VERTEX_ATTRIBS]; /* Only used with static strides. */
-   uint32_t non_trivial_format[MAX_VERTEX_ATTRIBS];
-
-   bool bindings_match_attrib;
-};
-
 struct radv_vs_prolog_key {
    /* All the fields are pre-masked with BITFIELD_MASK(num_attributes).
     * Some of the fields are pre-masked by other conditions. See lookup_vs_prolog.
@@ -339,7 +310,7 @@ struct radv_vs_prolog_key {
    bool as_ls;
    bool is_ngg;
    bool wave32;
-   gl_shader_stage next_stage;
+   mesa_shader_stage next_stage;
 };
 
 enum radv_shader_binary_type { RADV_BINARY_TYPE_LEGACY, RADV_BINARY_TYPE_RTLD };
@@ -504,7 +475,7 @@ struct radv_shader_part_cache {
 struct radv_shader_dma_submission {
    struct list_head list;
 
-   struct radeon_cmdbuf *cs;
+   struct radv_cmd_stream *cs;
    struct radeon_winsys_bo *bo;
    uint64_t bo_size;
    char *ptr;
@@ -513,11 +484,13 @@ struct radv_shader_dma_submission {
    uint64_t seq;
 };
 
-struct radv_pipeline_layout;
 struct radv_shader_stage;
 
 void radv_optimize_nir(struct nir_shader *shader, bool optimize_conservatively);
-void radv_optimize_nir_algebraic(nir_shader *shader, bool opt_offsets, bool opt_mqsad);
+void radv_optimize_nir_algebraic_early(nir_shader *shader);
+void radv_optimize_nir_algebraic_late(nir_shader *shader);
+void radv_optimize_nir_algebraic(nir_shader *shader, bool opt_offsets, bool opt_mqsad,
+                                 enum amd_gfx_level gfx_level);
 
 void radv_nir_lower_rt_io(nir_shader *shader, bool monolithic, uint32_t payload_offset);
 
@@ -526,7 +499,7 @@ struct radv_ray_tracing_stage_info;
 void radv_nir_lower_rt_abi(nir_shader *shader, const VkRayTracingPipelineCreateInfoKHR *pCreateInfo,
                            const struct radv_shader_args *args, const struct radv_shader_info *info,
                            uint32_t *stack_size, bool resume_shader, struct radv_device *device,
-                           struct radv_ray_tracing_pipeline *pipeline, bool monolithic,
+                           struct radv_ray_tracing_pipeline *pipeline, bool monolithic, bool has_position_fetch,
                            const struct radv_ray_tracing_stage_info *traversal_info);
 
 void radv_gather_unused_args(struct radv_ray_tracing_stage_info *info, nir_shader *nir);
@@ -595,7 +568,7 @@ struct radv_shader_part *radv_create_ps_epilog(struct radv_device *device, const
 
 void radv_shader_part_destroy(struct radv_device *device, struct radv_shader_part *shader_part);
 
-bool radv_shader_part_cache_init(struct radv_shader_part_cache *cache, struct radv_shader_part_cache_ops *ops);
+void radv_shader_part_cache_init(struct radv_shader_part_cache *cache, struct radv_shader_part_cache_ops *ops);
 void radv_shader_part_cache_finish(struct radv_device *device, struct radv_shader_part_cache *cache);
 struct radv_shader_part *radv_shader_part_cache_get(struct radv_device *device, struct radv_shader_part_cache *cache,
                                                     struct set *local_entries, const void *key);
@@ -608,7 +581,7 @@ unsigned radv_get_max_waves(const struct radv_device *device, const struct ac_sh
 
 unsigned radv_get_max_scratch_waves(const struct radv_device *device, struct radv_shader *shader);
 
-const char *radv_get_shader_name(const struct radv_shader_info *info, gl_shader_stage stage);
+const char *radv_get_shader_name(const struct radv_shader_info *info, mesa_shader_stage stage);
 
 unsigned radv_compute_spi_ps_input(const struct radv_physical_device *pdev,
                                    const struct radv_graphics_state_key *gfx_state,
@@ -619,7 +592,7 @@ bool radv_can_dump_shader(struct radv_device *device, nir_shader *nir);
 bool radv_can_dump_shader_stats(struct radv_device *device, nir_shader *nir);
 
 VkResult radv_dump_shader_stats(struct radv_device *device, struct radv_pipeline *pipeline, struct radv_shader *shader,
-                                gl_shader_stage stage, FILE *output);
+                                mesa_shader_stage stage, FILE *output);
 
 /* Returns true on success and false on failure */
 bool radv_shader_reupload(struct radv_device *device, struct radv_shader *shader);
@@ -675,7 +648,7 @@ get_tcs_input_vertex_stride(unsigned tcs_num_inputs)
 
 void radv_get_tess_wg_info(const struct radv_physical_device *pdev, const ac_nir_tess_io_info *io_info,
                            unsigned tcs_vertices_out, unsigned tcs_num_input_vertices, unsigned tcs_num_lds_inputs,
-                           unsigned *num_patches_per_wg, unsigned *hw_lds_size);
+                           unsigned *num_patches_per_wg, unsigned *lds_size);
 
 void radv_lower_ngg(struct radv_device *device, struct radv_shader_stage *ngg_stage,
                     const struct radv_graphics_state_key *gfx_state);
@@ -700,7 +673,7 @@ enum radv_rt_priority {
 };
 
 static inline enum radv_rt_priority
-radv_get_rt_priority(gl_shader_stage stage)
+radv_get_rt_priority(mesa_shader_stage stage)
 {
    switch (stage) {
    case MESA_SHADER_RAYGEN:
@@ -714,7 +687,7 @@ radv_get_rt_priority(gl_shader_stage stage)
    case MESA_SHADER_CALLABLE:
       return radv_rt_priority_callable;
    default:
-      unreachable("Unimplemented RT shader stage.");
+      UNREACHABLE("Unimplemented RT shader stage.");
    }
 }
 
@@ -736,12 +709,24 @@ uint32_t radv_get_user_sgpr_loc(const struct radv_shader *shader, int idx);
 
 uint32_t radv_get_user_sgpr(const struct radv_shader *shader, int idx);
 
-void radv_precompute_registers_hw_gs(struct radv_device *device, struct radv_shader_info *es_info, struct radv_shader_info *gs_info);
+static inline bool
+radv_shader_need_indirect_descriptors(const struct radv_shader *shader)
+{
+   const struct radv_userdata_info *loc = radv_get_user_sgpr_info(shader, AC_UD_INDIRECT_DESCRIPTORS);
+   return loc->sgpr_idx != -1;
+}
+
+static inline bool
+radv_shader_need_push_constants_upload(const struct radv_shader *shader)
+{
+   const struct radv_userdata_info *loc = radv_get_user_sgpr_info(shader, AC_UD_PUSH_CONSTANTS);
+   return loc->sgpr_idx != -1;
+}
 
 void radv_precompute_registers_hw_ngg(struct radv_device *device, const struct ac_shader_config *config,
                                       struct radv_shader_info *info);
 
-void radv_set_stage_key_robustness(const struct vk_pipeline_robustness_state *rs, gl_shader_stage stage,
+void radv_set_stage_key_robustness(const struct vk_pipeline_robustness_state *rs, mesa_shader_stage stage,
                                    struct radv_shader_stage_key *key);
 
 #endif /* RADV_SHADER_H */
