@@ -78,7 +78,7 @@ struct agx_rodata {
 };
 
 struct agx_shader_info {
-   enum pipe_shader_type stage;
+   mesa_shader_stage stage;
    uint32_t binary_size;
 
    union agx_varyings varyings;
@@ -140,8 +140,11 @@ struct agx_shader_info {
    /* Uses txf and hence needs a txf sampler mapped */
    bool uses_txf;
 
-   /* Number of texture state registers pushed by the preamble. */
-   uint8_t texture_state_count;
+   /* Potentially uses the sampler heap (conservative) */
+   bool uses_sampler_heap;
+
+   /* Number of texture/sampler state registers pushed by the preamble. */
+   uint8_t texture_state_count, sampler_state_count;
 
    /* Number of 16-bit registers used by the main shader and preamble
     * respectively.
@@ -287,9 +290,10 @@ struct agx_shader_key {
     */
    bool promote_constants;
 
-   /* Similarly whether the driver supports promoting bindless textures.
-    * Currently this works only if non-bindless textures are not used, but
-    * none of our drivers mix bindless / non-bindless usage.
+   /* Similarly whether the driver supports promoting bindless
+    * textures/samplers.  Currently this works only if non-bindless
+    * textures/samplers are not used, but none of our drivers mix bindless /
+    * non-bindless usage.
     */
    bool promote_textures;
 
@@ -310,7 +314,6 @@ struct agx_shader_key {
    };
 };
 
-struct agx_interp_info agx_gather_interp_info(nir_shader *nir);
 uint64_t agx_gather_texcoords(nir_shader *nir);
 
 void agx_preprocess_nir(nir_shader *nir);
@@ -337,6 +340,12 @@ struct agx_occupancy {
 struct agx_occupancy agx_occupancy_for_register_count(unsigned halfregs);
 unsigned agx_max_registers_for_occupancy(unsigned occupancy);
 
+static inline unsigned
+agx_round_registers(unsigned halfregs)
+{
+   return agx_occupancy_for_register_count(halfregs).max_registers;
+}
+
 static const nir_shader_compiler_options agx_nir_options = {
    .lower_fdiv = true,
    .fuse_ffma16 = true,
@@ -391,7 +400,7 @@ static const nir_shader_compiler_options agx_nir_options = {
    .support_indirect_inputs = BITFIELD_BIT(MESA_SHADER_TESS_CTRL) |
                               BITFIELD_BIT(MESA_SHADER_TESS_EVAL) |
                               BITFIELD_BIT(MESA_SHADER_FRAGMENT),
-   .support_indirect_outputs = (uint8_t)BITFIELD_MASK(PIPE_SHADER_TYPES),
+   .support_indirect_outputs = (uint8_t)BITFIELD_MASK(MESA_SHADER_STAGES),
    .lower_fquantize2f16 = true,
    .compact_arrays = true,
    .discard_is_demote = true,
