@@ -16,13 +16,11 @@
 #include "util/list.h"
 #include "util/vma.h"
 
-/* Stride of the image heap, equal to the size of a texture/PBE descriptor */
-#define HK_IMAGE_STRIDE (24)
-
 struct hk_descriptor_set_layout;
 
 struct hk_sampled_image_descriptor {
-   uint32_t image_offset;
+   struct agx_texture_packed tex;
+   struct agx_sampler_packed sampler;
    uint16_t sampler_index;
 
    /* Negative if there is no border colour, else the clamp=0 sampler index used
@@ -42,24 +40,18 @@ struct hk_sampled_image_descriptor {
    uint16_t pad;
    /* TODO: This should probably be a heap! */
    uint32_t border[4];
+   uint8_t pad2[4];
 };
-static_assert(sizeof(struct hk_sampled_image_descriptor) == 32,
+static_assert(sizeof(struct hk_sampled_image_descriptor) == 64,
               "hk_sampled_image_descriptor has no holes");
 
 struct hk_storage_image_descriptor {
-   uint32_t tex_offset;
-   uint32_t pbe_offset;
+   struct agx_texture_packed tex;
+   struct agx_pbe_packed pbe;
 };
-static_assert(sizeof(struct hk_storage_image_descriptor) == 8,
+static_assert(sizeof(struct hk_storage_image_descriptor) == 48,
               "hk_storage_image_descriptor has no holes");
 
-struct hk_buffer_view_descriptor {
-   uint32_t tex_offset;
-   uint32_t pbe_offset;
-};
-static_assert(sizeof(struct hk_buffer_view_descriptor) == 8,
-              "hk_buffer_view_descriptor has no holes");
-
 /* This has to match nir_address_format_64bit_bounded_global */
 struct hk_buffer_address {
    uint64_t base_addr;
