@@ -77,6 +77,19 @@ typedef enum {
    nir_divergence_uniform_load_tears = (1 << 7),
    /* If used, this allows phis for divergent merges with undef and a uniform source to be considered uniform */
    nir_divergence_ignore_undef_if_phi_srcs = (1 << 8),
+
+   /* Whether to compute vertex divergence (meaning between vertices
+    * of the same primitive) instead of subgroup invocation divergence
+    * (between invocations of the same subgroup). For example, patch input
+    * loads are always convergent, while subgroup intrinsics are divergent.
+    */
+   nir_divergence_vertex = (1 << 11),
+
+   /* Whether to compute divergence of subgroup operations as if multiple
+    * subgroups ran in lock-step (for example subgroup operations normally
+    * convergent are divergent).
+    */
+   nir_divergence_across_subgroups = (1 << 12),
 } nir_divergence_options;
 
 /** An instruction filtering callback
@@ -364,7 +377,14 @@ typedef struct nir_shader_compiler_options {
    /**
     * If enabled, gl_HelperInvocation will be lowered as:
     *
-    *   !((1 << sample_id) & sample_mask_in))
+    * - non-sample-shading: sample_mask_in == 0.
+    * - sample shading:     !((1 << sample_id) & sample_mask_in))
+    *
+    * For this to be correct, it requires that fs.uses_sample_shading is set to
+    * true when sample shading is enabled.  This means that you need shader
+    * variants to set the flag when Vulkan's
+    * VkPipelineMultisampleStateCreateInfo->sampleShadingEnable or GL's
+    * glMinSampleshading() are enabled.
     *
     * This depends on some possibly hw implementation details, which may
     * not be true for all hw.  In particular that the FS is only executed
@@ -466,6 +486,19 @@ typedef struct nir_shader_compiler_options {
     */
    bool lower_bfloat16_conversions;
 
+   /**
+    * Set if f2u_sat (or f2i_sat) is supported for converting from 16-, 32-,
+    * or 64-bit float types to 8-, 16-, or 32-bit integer types (with small
+    * exceptions).
+    *
+    * Due to the prevalence of drivers using \c nir_split_conversion for
+    * conversions from 64-bit float to 8-bit integer, these flags will not
+    * enable generation of f2u_sat from 64-bit float types to 8-bit integer
+    * types.
+    */
+   bool has_f2u_sat;
+   bool has_f2i_sat;
+
    bool vectorize_tess_levels;
    bool lower_to_scalar;
    nir_instr_filter_cb lower_to_scalar_filter;
@@ -633,6 +666,15 @@ typedef struct nir_shader_compiler_options {
    /** Backend supports load_global_bounded intrinsics. */
    bool has_load_global_bounded;
 
+   /** Backend supports f2i32_rtne opcode. */
+   bool has_f2i32_rtne;
+
+   /** Backend supports atomic isub. */
+   bool has_atomic_isub;
+
+   /** Backend supports atomic load/store. */
+   bool has_atomic_load_store;
+
    /**
     * Is this the Intel vec4 backend?
     *
@@ -649,6 +691,11 @@ typedef struct nir_shader_compiler_options {
     */
    bool avoid_ternary_with_two_constants;
 
+   /* Avoid using fabs on sources to ternary operations
+    * r600 can't use source modifiers for these
+    */
+   bool avoid_ternary_with_fabs;
+
    /** Whether 8-bit ALU is supported. */
    bool support_8bit_alu;
 
@@ -699,6 +746,11 @@ typedef struct nir_shader_compiler_options {
    uint8_t support_indirect_inputs;
    uint8_t support_indirect_outputs;
 
+   /**
+    * Lower fmulz to `min(abs(a), abs(b)) == 0.0 ? 0.0 : a * b`.
+    */
+   bool lower_fmulz_with_abs_min;
+
    /** store the variable offset into the instrinsic range_base instead
     *  of adding it to the image index.
     */
@@ -732,13 +784,6 @@ typedef struct nir_shader_compiler_options {
     */
    bool discard_is_demote;
 
-   /**
-    * Whether the new-style derivative intrinsics are supported. If false,
-    * legacy ALU derivative ops will be emitted. This transitional option will
-    * be removed once all drivers are converted to derivative intrinsics.
-    */
-   bool has_ddx_intrinsics;
-
    /** Whether derivative intrinsics must be scalarized. */
    bool scalarize_ddx;
 
@@ -819,6 +864,23 @@ typedef struct nir_shader_compiler_options {
     * the next shader according to default_varying_estimate_instr_cost.
     */
    unsigned max_varying_expression_cost;
+
+   /**
+    * Used by nir_lower_explicit_io to determine the maximum offset_shift to
+    * use when lowering the deref address of the given intrinsic.
+    */
+   unsigned (*max_offset_shift)(nir_intrinsic_instr *, const void *);
+
+   /**
+    * Passed to the callbacks that accept a data pointer.
+    */
+   const void *cb_data;
+
+   /** Maximum amount of invocations per workgroup. */
+   unsigned max_workgroup_invocations;
+
+   /** Maximum compute shader / kernel dispatchable work size. */
+   unsigned max_workgroup_count[3];
 } nir_shader_compiler_options;
 
 #ifdef __cplusplus
