@@ -473,7 +473,7 @@ add_gpus([
             RB_DBG_ECO_CNTL = 0x04100000,
             RB_DBG_ECO_CNTL_blit = 0x04100000,
             HLSQ_DBG_ECO_CNTL = 0,
-            RB_UNKNOWN_8E01 = 0x00000001,
+            RB_RBP_CNTL = 0x00000001,
             VPC_DBG_ECO_CNTL = 0x0,
             UCHE_UNKNOWN_0E12 = 0x10000000,
         ),
@@ -486,7 +486,7 @@ add_gpus([
         GPUId(619),
     ], A6xxGPUInfo(
         CHIP.A6XX,
-        [a6xx_base, a6xx_gen1],
+        [a6xx_base, a6xx_gen1, A6XXProps(blit_wfi_quirk = True)],
         num_ccu = 1,
         tile_align_w = 32,
         tile_align_h = 32,
@@ -507,7 +507,7 @@ add_gpus([
             RB_DBG_ECO_CNTL = 0x04100000,
             RB_DBG_ECO_CNTL_blit = 0x04100000,
             HLSQ_DBG_ECO_CNTL = 0x00080000,
-            RB_UNKNOWN_8E01 = 0x00000001,
+            RB_RBP_CNTL = 0x00000001,
             VPC_DBG_ECO_CNTL = 0x0,
             UCHE_UNKNOWN_0E12 = 0x00000001
         )
@@ -536,7 +536,7 @@ add_gpus([
             RB_DBG_ECO_CNTL = 0x04100000,
             RB_DBG_ECO_CNTL_blit = 0x04100000,
             HLSQ_DBG_ECO_CNTL = 0x0,
-            RB_UNKNOWN_8E01 = 0x0,
+            RB_RBP_CNTL = 0x0,
             VPC_DBG_ECO_CNTL = 0x02000000,
             UCHE_UNKNOWN_0E12 = 0x00000001
         )
@@ -567,7 +567,7 @@ add_gpus([
             RB_DBG_ECO_CNTL = 0x04100000,
             RB_DBG_ECO_CNTL_blit = 0x04100000,
             HLSQ_DBG_ECO_CNTL = 0x0,
-            RB_UNKNOWN_8E01 = 0x0,
+            RB_RBP_CNTL = 0x0,
             VPC_DBG_ECO_CNTL = 0x02000000,
             UCHE_UNKNOWN_0E12 = 0x00000001
         )
@@ -598,7 +598,7 @@ add_gpus([
             RB_DBG_ECO_CNTL = 0x04100000,
             RB_DBG_ECO_CNTL_blit = 0x05100000,
             HLSQ_DBG_ECO_CNTL = 0x00080000,
-            RB_UNKNOWN_8E01 = 0x00000001,
+            RB_RBP_CNTL = 0x00000001,
             VPC_DBG_ECO_CNTL = 0x0,
             UCHE_UNKNOWN_0E12 = 0x10000001
         )
@@ -629,7 +629,7 @@ add_gpus([
             RB_DBG_ECO_CNTL = 0x04100000,
             RB_DBG_ECO_CNTL_blit = 0x04100000,
             HLSQ_DBG_ECO_CNTL = 0x0,
-            RB_UNKNOWN_8E01 = 0x00000001,
+            RB_RBP_CNTL = 0x00000001,
             VPC_DBG_ECO_CNTL = 0x02000000,
             UCHE_UNKNOWN_0E12 = 0x00000001
         )
@@ -660,7 +660,7 @@ add_gpus([
             RB_DBG_ECO_CNTL = 0x04100000,
             RB_DBG_ECO_CNTL_blit = 0x04100000,
             HLSQ_DBG_ECO_CNTL = 0x0,
-            RB_UNKNOWN_8E01 = 0x00000001,
+            RB_RBP_CNTL = 0x00000001,
             VPC_DBG_ECO_CNTL = 0x02000000,
             UCHE_UNKNOWN_0E12 = 0x00000001
         )
@@ -691,7 +691,7 @@ add_gpus([
             RB_DBG_ECO_CNTL = 0x04100000,
             RB_DBG_ECO_CNTL_blit = 0x04100000,
             HLSQ_DBG_ECO_CNTL = 0x0,
-            RB_UNKNOWN_8E01 = 0x0,
+            RB_RBP_CNTL = 0x0,
             VPC_DBG_ECO_CNTL = 0x02000000,
             UCHE_UNKNOWN_0E12 = 0x00000001
         )
@@ -727,7 +727,7 @@ add_gpus([
             RB_DBG_ECO_CNTL = 0x04100000,
             RB_DBG_ECO_CNTL_blit = 0x04100000,
             HLSQ_DBG_ECO_CNTL = 0x0,
-            RB_UNKNOWN_8E01 = 0x0,
+            RB_RBP_CNTL = 0x0,
             VPC_DBG_ECO_CNTL = 0x02000000,
             UCHE_UNKNOWN_0E12 = 0x00000001
         )
@@ -757,7 +757,7 @@ add_gpus([
             RB_DBG_ECO_CNTL = 0x04100000,
             RB_DBG_ECO_CNTL_blit = 0x04100000,
             HLSQ_DBG_ECO_CNTL = 0x0,
-            RB_UNKNOWN_8E01 = 0x0,
+            RB_RBP_CNTL = 0x0,
             VPC_DBG_ECO_CNTL = 0x02000000,
             UCHE_UNKNOWN_0E12 = 0x00000001
         )
@@ -787,7 +787,7 @@ add_gpus([
             RB_DBG_ECO_CNTL = 0x04100000,
             RB_DBG_ECO_CNTL_blit = 0x04100000,
             HLSQ_DBG_ECO_CNTL = 0x0,
-            RB_UNKNOWN_8E01 = 0x0,
+            RB_RBP_CNTL = 0x0,
             VPC_DBG_ECO_CNTL = 0x02000000,
             UCHE_UNKNOWN_0E12 = 0x00000001
         )
@@ -818,7 +818,7 @@ add_gpus([
             RB_DBG_ECO_CNTL = 0x100000,
             RB_DBG_ECO_CNTL_blit = 0x00100000,  # ???
             HLSQ_DBG_ECO_CNTL = 0x0,
-            RB_UNKNOWN_8E01 = 0x0,
+            RB_RBP_CNTL = 0x0,
             VPC_DBG_ECO_CNTL = 0x2000400,
             UCHE_UNKNOWN_0E12 = 0x00000001
         ),
@@ -866,7 +866,7 @@ add_gpus([
             RB_DBG_ECO_CNTL = 0x100000,
             RB_DBG_ECO_CNTL_blit = 0x100000,
             HLSQ_DBG_ECO_CNTL = 0x02000000,
-            RB_UNKNOWN_8E01 = 0x1,
+            RB_RBP_CNTL = 0x1,
             VPC_DBG_ECO_CNTL = 0x0,
             UCHE_UNKNOWN_0E12 = 0x1,
         ),
@@ -911,6 +911,7 @@ a7xx_base = A6XXProps(
         line_width_min = 1.0,
         line_width_max = 127.5,
         has_scalar_alu = True,
+        has_scalar_predicates = True,
         has_coherent_ubwc_flag_caches = True,
         has_isam_v = True,
         has_ssbo_imm_offsets = True,
@@ -943,6 +944,7 @@ a7xx_gen2 = A7XXProps(
         has_primitive_shading_rate = True,
         reading_shading_rate_requires_smask_quirk = True,
         has_ray_intersection = True,
+        has_hw_bin_scaling = True,
     )
 
 a7xx_gen3 = A7XXProps(
@@ -970,6 +972,7 @@ a7xx_gen3 = A7XXProps(
         has_alias_rt = True,
         has_abs_bin_mask = True,
         new_control_regs = True,
+        has_hw_bin_scaling = True,
     )
 
 a730_magic_regs = dict(
@@ -981,7 +984,7 @@ a730_magic_regs = dict(
         SP_DBG_ECO_CNTL = 0x10000000,
         RB_DBG_ECO_CNTL = 0x00000000,
         RB_DBG_ECO_CNTL_blit = 0x00000000,  # is it even needed?
-        RB_UNKNOWN_8E01 = 0x0,
+        RB_RBP_CNTL = 0x0,
         VPC_DBG_ECO_CNTL = 0x02000000,
         UCHE_UNKNOWN_0E12 = 0x3200000,
 
@@ -992,7 +995,7 @@ a730_raw_magic_regs = [
         [A6XXRegs.REG_A6XX_UCHE_CACHE_WAYS, 0x00840004],
         [A6XXRegs.REG_A6XX_TPL1_DBG_ECO_CNTL1, 0x00040724],
 
-        [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_1, 0x00002400],
+        [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_1, 0x00402400],
         [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_2, 0x00000000],
         [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_3, 0x00000000],
         [A6XXRegs.REG_A7XX_UCHE_UNKNOWN_0E10, 0x00000000],
@@ -1001,18 +1004,15 @@ a730_raw_magic_regs = [
         [A6XXRegs.REG_A6XX_PC_DBG_ECO_CNTL, 0x20080000],
         [A6XXRegs.REG_A7XX_PC_UNKNOWN_9E24, 0x21fc7f00],
         [A6XXRegs.REG_A7XX_VFD_DBG_ECO_CNTL, 0x00000000],
-        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE06, 0x00000000],
+        [A6XXRegs.REG_A7XX_SP_ISDB_CNTL, 0x00000000],
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE6A, 0x00000000],
-        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE6B, 0x00000080],
-        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE73, 0x00000000],
+        [A6XXRegs.REG_A7XX_SP_HLSQ_TIMEOUT_THRESHOLD_DP, 0x00000080],
+        [A6XXRegs.REG_A7XX_SP_HLSQ_DBG_ECO_CNTL_1, 0x00000000],
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB02, 0x00000000],
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB01, 0x00000000],
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB22, 0x00000000],
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_B310, 0x00000000],
 
-        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8120, 0x09510840],
-        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8121, 0x00000a62],
-
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE2,   0x00000000],
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE2+1, 0x00000000],
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE4,   0x00000000],
@@ -1020,11 +1020,10 @@ a730_raw_magic_regs = [
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE6,   0x00000000],
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE6+1, 0x00000000],
 
-        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_80A7, 0x00000000],
+        [A6XXRegs.REG_A7XX_GRAS_ROTATION_CNTL, 0x00000000],
 
         [A6XXRegs.REG_A7XX_RB_UNKNOWN_8E79,   0x00000000],
         [A6XXRegs.REG_A7XX_RB_UNKNOWN_8899,   0x00000000],
-        [A6XXRegs.REG_A7XX_RB_UNKNOWN_88F5,   0x00000000],
     ]
 
 a740_magic_regs = dict(
@@ -1040,7 +1039,7 @@ a740_magic_regs = dict(
         RB_DBG_ECO_CNTL = 0x00000000,
         RB_DBG_ECO_CNTL_blit = 0x00000000,  # is it even needed?
         # HLSQ_DBG_ECO_CNTL = 0x0,
-        RB_UNKNOWN_8E01 = 0x0,
+        RB_RBP_CNTL = 0x0,
         VPC_DBG_ECO_CNTL = 0x02000000,
         UCHE_UNKNOWN_0E12 = 0x00000000,
 
@@ -1051,7 +1050,7 @@ a740_raw_magic_regs = [
         [A6XXRegs.REG_A6XX_UCHE_CACHE_WAYS, 0x00040004],
         [A6XXRegs.REG_A6XX_TPL1_DBG_ECO_CNTL1, 0x00040724],
 
-        [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_1, 0x00000400],
+        [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_1, 0x00400400],
         [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_2, 0x00430800],
         [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_3, 0x00000000],
         [A6XXRegs.REG_A7XX_UCHE_UNKNOWN_0E10, 0x00000000],
@@ -1060,23 +1059,15 @@ a740_raw_magic_regs = [
         [A6XXRegs.REG_A6XX_PC_DBG_ECO_CNTL, 0x00100000],
         [A6XXRegs.REG_A7XX_PC_UNKNOWN_9E24, 0x21585600],
         [A6XXRegs.REG_A7XX_VFD_DBG_ECO_CNTL, 0x00008000],
-        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE06, 0x00000000],
+        [A6XXRegs.REG_A7XX_SP_ISDB_CNTL, 0x00000000],
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE6A, 0x00000000],
-        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE6B, 0x00000080],
-        [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE73, 0x00000000],
+        [A6XXRegs.REG_A7XX_SP_HLSQ_TIMEOUT_THRESHOLD_DP, 0x00000080],
+        [A6XXRegs.REG_A7XX_SP_HLSQ_DBG_ECO_CNTL_1, 0x00000000],
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB02, 0x00000000],
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB01, 0x00000000],
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB22, 0x00000000],
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_B310, 0x00000000],
 
-        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8120, 0x09510840],
-        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8121, 0x00000a62],
-
-        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8009, 0x00000000],
-        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_800A, 0x00000000],
-        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_800B, 0x00000000],
-        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_800C, 0x00000000],
-
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE2,   0x00000000],
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE2+1, 0x00000000],
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE4,   0x00000000],
@@ -1084,14 +1075,11 @@ a740_raw_magic_regs = [
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE6,   0x00000000],
         [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE6+1, 0x00000000],
 
-        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_80A7, 0x00000000],
+        [A6XXRegs.REG_A7XX_GRAS_ROTATION_CNTL, 0x00000000],
 
         [A6XXRegs.REG_A7XX_RB_UNKNOWN_8E79,   0x00000000],
         [A6XXRegs.REG_A7XX_RB_UNKNOWN_8899,   0x00000000],
-        [A6XXRegs.REG_A7XX_RB_UNKNOWN_88F5,   0x00000000],
         [A6XXRegs.REG_A7XX_RB_UNKNOWN_8C34,   0x00000000],
-
-        [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8008, 0x00000000],
     ]
 
 add_gpus([
@@ -1153,7 +1141,7 @@ add_gpus([
             SP_DBG_ECO_CNTL = 0x10000000,
             RB_DBG_ECO_CNTL = 0x00000001,
             RB_DBG_ECO_CNTL_blit = 0x00000001,  # is it even needed?
-            RB_UNKNOWN_8E01 = 0x0,
+            RB_RBP_CNTL = 0x0,
             VPC_DBG_ECO_CNTL = 0x02000000,
             UCHE_UNKNOWN_0E12 = 0x00000000,
 
@@ -1163,7 +1151,7 @@ add_gpus([
             [A6XXRegs.REG_A6XX_UCHE_CACHE_WAYS, 0x00000000],
             [A6XXRegs.REG_A6XX_TPL1_DBG_ECO_CNTL1, 0x00040724],
 
-            [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_1, 0x00000400],
+            [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_1, 0x00400400],
             [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_2, 0x00430800],
             [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_3, 0x00000000],
             [A6XXRegs.REG_A7XX_UCHE_UNKNOWN_0E10, 0x00000000],
@@ -1172,23 +1160,15 @@ add_gpus([
             [A6XXRegs.REG_A6XX_PC_DBG_ECO_CNTL, 0x00100000],
             [A6XXRegs.REG_A7XX_PC_UNKNOWN_9E24, 0x01585600],
             [A6XXRegs.REG_A7XX_VFD_DBG_ECO_CNTL, 0x00008000],
-            [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE06, 0x00000000],
+            [A6XXRegs.REG_A7XX_SP_ISDB_CNTL, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE6A, 0x00000000],
-            [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE6B, 0x00000080],
-            [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE73, 0x00000000],
+            [A6XXRegs.REG_A7XX_SP_HLSQ_TIMEOUT_THRESHOLD_DP, 0x00000080],
+            [A6XXRegs.REG_A7XX_SP_HLSQ_DBG_ECO_CNTL_1, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB02, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB01, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB22, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_B310, 0x00000000],
 
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8120, 0x09510840],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8121, 0x00000a62],
-
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8009, 0x00000000],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_800A, 0x00000000],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_800B, 0x00000000],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_800C, 0x00000000],
-
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE2,   0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE2+1, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE4,   0x00000000],
@@ -1196,14 +1176,11 @@ add_gpus([
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE6,   0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE6+1, 0x00000000],
 
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_80A7, 0x00000000],
+            [A6XXRegs.REG_A7XX_GRAS_ROTATION_CNTL, 0x00000000],
 
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_8E79,   0x00000000],
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_8899,   0x00000000],
-            [A6XXRegs.REG_A7XX_RB_UNKNOWN_88F5,   0x00000000],
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_8C34,   0x00000000],
-
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8008, 0x00000000],
         ],
     ))
 
@@ -1246,7 +1223,7 @@ add_gpus([
             [A6XXRegs.REG_A6XX_UCHE_CACHE_WAYS, 0x00040004],
             [A6XXRegs.REG_A6XX_TPL1_DBG_ECO_CNTL1, 0x00000700],
 
-            [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_1, 0x00000400],
+            [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_1, 0x00400400],
             [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_2, 0x00430820],
             [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_3, 0x00000000],
             [A6XXRegs.REG_A7XX_UCHE_UNKNOWN_0E10, 0x00000000],
@@ -1255,23 +1232,15 @@ add_gpus([
             [A6XXRegs.REG_A6XX_PC_DBG_ECO_CNTL, 0x00100000],
             [A6XXRegs.REG_A7XX_PC_UNKNOWN_9E24, 0x21585600],
             [A6XXRegs.REG_A7XX_VFD_DBG_ECO_CNTL, 0x00008000],
-            [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE06, 0x00000000],
+            [A6XXRegs.REG_A7XX_SP_ISDB_CNTL, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE6A, 0x00000000],
-            [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE6B, 0x00000080],
-            [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE73, 0x00000000],
+            [A6XXRegs.REG_A7XX_SP_HLSQ_TIMEOUT_THRESHOLD_DP, 0x00000080],
+            [A6XXRegs.REG_A7XX_SP_HLSQ_DBG_ECO_CNTL_1, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB02, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB01, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB22, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_B310, 0x00000000],
 
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8120, 0x09510840],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8121, 0x00000a62],
-
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8009, 0x00000000],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_800A, 0x00000000],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_800B, 0x00000000],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_800C, 0x00000000],
-
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE2,   0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE2+1, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE4,   0x00000000],
@@ -1279,11 +1248,10 @@ add_gpus([
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE6,   0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE6+1, 0x00000000],
 
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_80A7, 0x00000000],
+            [A6XXRegs.REG_A7XX_GRAS_ROTATION_CNTL, 0x00000000],
 
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_8E79,   0x00000000],
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_8899,   0x00000000],
-            [A6XXRegs.REG_A7XX_RB_UNKNOWN_88F5,   0x00000000],
         ],
     ))
 
@@ -1313,7 +1281,7 @@ add_gpus([
             RB_DBG_ECO_CNTL = 0x00000001,
             RB_DBG_ECO_CNTL_blit = 0x00000000,  # is it even needed?
             # HLSQ_DBG_ECO_CNTL = 0x0,
-            RB_UNKNOWN_8E01 = 0x0,
+            RB_RBP_CNTL = 0x0,
             VPC_DBG_ECO_CNTL = 0x02000000,
             UCHE_UNKNOWN_0E12 = 0x00000000,
 
@@ -1344,7 +1312,7 @@ add_gpus([
             SP_DBG_ECO_CNTL = 0x10000000,
             RB_DBG_ECO_CNTL = 0x00000001,
             RB_DBG_ECO_CNTL_blit = 0x00000001,
-            RB_UNKNOWN_8E01 = 0x0,
+            RB_RBP_CNTL = 0x0,
             VPC_DBG_ECO_CNTL = 0x02000000,
             UCHE_UNKNOWN_0E12 = 0x40000000,
 
@@ -1354,27 +1322,21 @@ add_gpus([
             [A6XXRegs.REG_A6XX_UCHE_CACHE_WAYS, 0x00000000],
             [A6XXRegs.REG_A7XX_UCHE_UNKNOWN_0E10, 0x00000000],
             [A6XXRegs.REG_A7XX_UCHE_UNKNOWN_0E11, 0x00000080],
-            [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_1, 0x00000000],
+            [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_1, 0x00400000],
             [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_2, 0x00431800],
             [A6XXRegs.REG_A7XX_SP_CHICKEN_BITS_3, 0x00800000],
             [A6XXRegs.REG_A7XX_SP_HLSQ_DBG_ECO_CNTL, 0x00000000],
             [A6XXRegs.REG_A6XX_PC_DBG_ECO_CNTL, 0x00100000],
             [A6XXRegs.REG_A7XX_PC_UNKNOWN_9E24, 0x01585600],
             [A6XXRegs.REG_A7XX_VFD_DBG_ECO_CNTL, 0x00008000],
-            [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE06, 0x00000000],
+            [A6XXRegs.REG_A7XX_SP_ISDB_CNTL, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE6A, 0x00000000],
-            [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE6B, 0x00000080],
-            [A6XXRegs.REG_A7XX_SP_UNKNOWN_AE73, 0x00000000],
+            [A6XXRegs.REG_A7XX_SP_HLSQ_TIMEOUT_THRESHOLD_DP, 0x00000080],
+            [A6XXRegs.REG_A7XX_SP_HLSQ_DBG_ECO_CNTL_1, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB02, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB01, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_AB22, 0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_B310, 0x00000000],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8120, 0x09510840],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8121, 0x00000a62],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8009, 0x00000000],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_800A, 0x00000000],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_800B, 0x00000000],
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_800C, 0x00000000],
 
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE2,   0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE2+1, 0x00000000],
@@ -1383,16 +1345,14 @@ add_gpus([
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE6,   0x00000000],
             [A6XXRegs.REG_A7XX_SP_UNKNOWN_0CE6+1, 0x00000000],
 
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_80A7, 0x00000000],
+            [A6XXRegs.REG_A7XX_GRAS_ROTATION_CNTL, 0x00000000],
 
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_8899,   0x00000000],
-            [A6XXRegs.REG_A7XX_RB_UNKNOWN_88F5,   0x00000000],
             [A6XXRegs.REG_A7XX_RB_UNKNOWN_8C34,   0x00000000],
 
-            [A6XXRegs.REG_A7XX_GRAS_UNKNOWN_8008, 0x00000000],
+            [A6XXRegs.REG_A7XX_VPC_UNKNOWN_930A, 0],
+            [A6XXRegs.REG_A7XX_VPC_FLATSHADE_MODE_CNTL, 1],
 
-            [0x930a, 0],
-            [0x960a, 1],
             [A6XXRegs.REG_A7XX_SP_PS_OUTPUT_CONST_CNTL, 0],
             [A6XXRegs.REG_A7XX_SP_PS_OUTPUT_CONST_MASK, 0],
         ],
