@@ -474,12 +474,61 @@ reg_disasm_gpuaddr64(const char *name, uint64_t qword, int level)
    disasm_gpuaddr(name, qword, level);
 }
 
+/* Get the value of the corresponding SP_xS_TSIZE reg: */
+static unsigned
+get_tsize(const char *name)
+{
+   char tsize_reg[12];
+   sprintf(tsize_reg, "%.5s_TSIZE", name);
+   return reg_val(regbase(tsize_reg));
+}
+
+static unsigned
+get_usize(const char *name)
+{
+   char usize_reg[12];
+   sprintf(usize_reg, "%.5s_USIZE", name);
+   return reg_val(regbase(usize_reg));
+}
+
+static void
+reg_dump_texmemobj64(const char *name, uint64_t gpuaddr, int level)
+{
+   unsigned num_unit = get_tsize(name);
+   void *buf = hostptr(gpuaddr);
+   if (!buf)
+      return;
+   dump_tex_const(buf, num_unit, level + 1);
+}
+
+static void
+reg_dump_sampler64(const char *name, uint64_t gpuaddr, int level)
+{
+   unsigned num_unit = get_tsize(name);
+   void *buf = hostptr(gpuaddr);
+   if (!buf)
+      return;
+   dump_tex_samp(buf, STATE_SRC_DIRECT, num_unit, level + 1);
+}
+
+static void
+reg_dump_uav64(const char *name, uint64_t gpuaddr, int level)
+{
+   unsigned num_unit = get_usize(name);
+   void *buf = hostptr(gpuaddr);
+   if (!buf)
+      return;
+   dump_tex_const(buf, num_unit, level + 1);
+}
+
 /* Find the value of the TEX_COUNT register that corresponds to the named
  * TEX_SAMP/TEX_CONST reg.
  *
  * Note, this kinda assumes an equal # of samplers and textures, but not
  * really sure if there is a much better option.  I suppose on a6xx we
  * could instead decode the bitfields in SP_xS_CONFIG
+ *
+ * For a6xx+ use get_tsize()
  */
 static int
 get_tex_count(const char *name)
@@ -711,6 +760,31 @@ static struct {
       REG64(SP_PS_BASE, reg_disasm_gpuaddr64),
       REG64(SP_CS_BASE, reg_disasm_gpuaddr64),
 
+      {NULL},
+}, reg_a8xx[] = {
+      REG64(SP_VS_BASE, reg_disasm_gpuaddr64),
+      REG64(SP_HS_BASE, reg_disasm_gpuaddr64),
+      REG64(SP_DS_BASE, reg_disasm_gpuaddr64),
+      REG64(SP_GS_BASE, reg_disasm_gpuaddr64),
+      REG64(SP_PS_BASE, reg_disasm_gpuaddr64),
+      REG64(SP_CS_BASE, reg_disasm_gpuaddr64),
+
+      REG64(SP_VS_TEXMEMOBJ_BASE, reg_dump_texmemobj64),
+      REG64(SP_VS_SAMPLER_BASE, reg_dump_sampler64),
+      REG64(SP_HS_TEXMEMOBJ_BASE, reg_dump_texmemobj64),
+      REG64(SP_HS_SAMPLER_BASE, reg_dump_sampler64),
+      REG64(SP_DS_TEXMEMOBJ_BASE, reg_dump_texmemobj64),
+      REG64(SP_DS_SAMPLER_BASE, reg_dump_sampler64),
+      REG64(SP_GS_TEXMEMOBJ_BASE, reg_dump_texmemobj64),
+      REG64(SP_GS_SAMPLER_BASE, reg_dump_sampler64),
+      REG64(SP_PS_TEXMEMOBJ_BASE, reg_dump_texmemobj64),
+      REG64(SP_PS_SAMPLER_BASE, reg_dump_sampler64),
+      REG64(SP_CS_TEXMEMOBJ_BASE, reg_dump_texmemobj64),
+      REG64(SP_CS_SAMPLER_BASE, reg_dump_sampler64),
+
+      REG64(SP_GFX_UAV_BASE, reg_dump_uav64),
+      REG64(SP_CS_UAV_BASE, reg_dump_uav64),
+
       {NULL},
 }, *type0_reg;
 
@@ -794,6 +868,10 @@ cffdec_init(const struct cffdec_options *_options)
       type0_reg = reg_a7xx;
       init_rnn("a7xx");
       break;
+   case 8:
+      type0_reg = reg_a8xx;
+      init_rnn("a8xx");
+      break;
    default:
       errx(-1, "unsupported generation: %u", options->info->chip);
    }
@@ -817,6 +895,12 @@ regbase(const char *name)
    return rnn_regbase(rnn, name);
 }
 
+int
+enumval(const char *enumname, const char *enumval)
+{
+   return rnn_enumval(rnn, enumname, enumval);
+}
+
 static int
 endswith(uint32_t regbase, const char *suffix)
 {
@@ -1203,7 +1287,7 @@ cp_im_loadi(uint32_t *dwords, uint32_t sizedwords, int level)
    uint32_t start = dwords[1] >> 16;
    uint32_t size = dwords[1] & 0xffff;
    const char *type = NULL, *ext = NULL;
-   gl_shader_stage disasm_type;
+   mesa_shader_stage disasm_type;
 
    switch (dwords[0]) {
    case 0:
@@ -1280,13 +1364,13 @@ enum adreno_state_block {
  */
 
 static void
-a3xx_get_state_type(uint32_t *dwords, gl_shader_stage *stage,
+a3xx_get_state_type(uint32_t *dwords, mesa_shader_stage *stage,
                     enum state_t *state, enum state_src_t *src)
 {
    unsigned state_block_id = (dwords[0] >> 19) & 0x7;
    unsigned state_type = dwords[1] & 0x3;
    static const struct {
-      gl_shader_stage stage;
+      mesa_shader_stage stage;
       enum state_t state;
    } lookup[0xf][0x3] = {
       [SB_VERT_TEX][0] = {MESA_SHADER_VERTEX, TEX_SAMP},
@@ -1325,10 +1409,10 @@ _get_state_src(unsigned dword0)
 
 static void
 _get_state_type(unsigned state_block_id, unsigned state_type,
-                gl_shader_stage *stage, enum state_t *state)
+                mesa_shader_stage *stage, enum state_t *state)
 {
    static const struct {
-      gl_shader_stage stage;
+      mesa_shader_stage stage;
       enum state_t state;
    } lookup[0x10][0x4] = {
       // SB4_VS_TEX:
@@ -1401,7 +1485,7 @@ _get_state_type(unsigned state_block_id, unsigned state_type,
 }
 
 static void
-a4xx_get_state_type(uint32_t *dwords, gl_shader_stage *stage,
+a4xx_get_state_type(uint32_t *dwords, mesa_shader_stage *stage,
                     enum state_t *state, enum state_src_t *src)
 {
    unsigned state_block_id = (dwords[0] >> 18) & 0xf;
@@ -1411,7 +1495,7 @@ a4xx_get_state_type(uint32_t *dwords, gl_shader_stage *stage,
 }
 
 static void
-a6xx_get_state_type(uint32_t *dwords, gl_shader_stage *stage,
+a6xx_get_state_type(uint32_t *dwords, mesa_shader_stage *stage,
                     enum state_t *state, enum state_src_t *src)
 {
    unsigned state_block_id = (dwords[0] >> 18) & 0xf;
@@ -1446,6 +1530,10 @@ dump_tex_samp(uint32_t *texsamp, enum state_src_t src, int num_unit, int level)
          dump_domain(texsamp, 4, level + 2, "A6XX_TEX_SAMP");
          dump_hex(texsamp, 4, level + 1);
          texsamp += src == STATE_SRC_BINDLESS ? 16 : 4;
+      } else if ((8 <= options->info->chip) && (options->info->chip < 9)) {
+         dump_domain(texsamp, 4, level + 2, "A8XX_TEX_SAMP");
+         dump_hex(texsamp, 4, level + 1);
+         texsamp += src == STATE_SRC_BINDLESS ? 16 : 4;
       }
    }
 }
@@ -1491,6 +1579,15 @@ dump_tex_const(uint32_t *texconst, int num_unit, int level)
          }
          dump_hex(texconst, 16, level + 1);
          texconst += 16;
+      } else if ((8 <= options->info->chip) && (options->info->chip < 9)) {
+         dump_domain(texconst, 16, level + 2, "A8XX_TEX_MEMOBJ");
+         if (options->dump_textures) {
+            uint64_t addr =
+               (((uint64_t)texconst[5] & 0x1ffff) << 32) | texconst[4];
+            dump_gpuaddr_size(addr, level - 2, hostlen(addr) / 4, 3);
+         }
+         dump_hex(texconst, 16, level + 1);
+         texconst += 16;
       }
    }
 }
@@ -1562,7 +1659,7 @@ dump_bindless_descriptors(bool is_compute, int level)
 static void
 cp_load_state(uint32_t *dwords, uint32_t sizedwords, int level)
 {
-   gl_shader_stage stage;
+   mesa_shader_stage stage;
    enum state_t state;
    enum state_src_t src;
    uint32_t num_unit = (dwords[0] >> 22) & 0x1ff;
@@ -2894,28 +2991,8 @@ cp_context_reg_bunch(uint32_t *dwords, uint32_t sizedwords, int level)
    summary = saved_summary;
 }
 
-/* Looks similar to CP_CONTEXT_REG_BUNCH, but not quite the same...
- * discarding first two dwords??
- *
- *   CP_CONTEXT_REG_BUNCH:
- *        0221: 9c1ff606  (rep)(xmov3)mov $usraddr, $data
- *        ; mov $data, $data
- *        ; mov $usraddr, $data
- *        ; mov $data, $data
- *        0222: d8000000  waitin
- *        0223: 981f0806  mov $01, $data
- *
- *   CP_UNK5D:
- *        0224: 981f0006  mov $00, $data
- *        0225: 981f0006  mov $00, $data
- *        0226: 9c1ff206  (rep)(xmov1)mov $usraddr, $data
- *        ; mov $data, $data
- *        0227: d8000000  waitin
- *        0228: 981f0806  mov $01, $data
- *
- */
 static void
-cp_context_reg_bunch2(uint32_t *dwords, uint32_t sizedwords, int level)
+cp_non_context_reg_bunch(uint32_t *dwords, uint32_t sizedwords, int level)
 {
    dwords += 2;
    sizedwords -= 2;
@@ -2934,7 +3011,7 @@ cp_reg_write(uint32_t *dwords, uint32_t sizedwords, int level)
 }
 
 static void
-cp_set_ctxswitch_ib(uint32_t *dwords, uint32_t sizedwords, int level)
+cp_set_amble(uint32_t *dwords, uint32_t sizedwords, int level)
 {
    uint64_t addr;
    uint32_t size = dwords[2] & 0xffff;
@@ -3019,7 +3096,7 @@ static const struct type3_op {
    CP(REG_WRITE, cp_reg_write),
    CP(DRAW_AUTO, cp_draw_auto, {.load_all_groups = true}),
 
-   CP(SET_CTXSWITCH_IB, cp_set_ctxswitch_ib),
+   CP(SET_AMBLE, cp_set_amble),
 
    CP(START_BIN, cp_start_bin),
 
@@ -3027,7 +3104,7 @@ static const struct type3_op {
 
    /* for a7xx */
    CP(THREAD_CONTROL, cp_set_thread_control),
-   CP(CONTEXT_REG_BUNCH2, cp_context_reg_bunch2),
+   CP(NON_CONTEXT_REG_BUNCH, cp_non_context_reg_bunch),
    CP(EVENT_WRITE7, cp_event_write),
 };
 
