@@ -222,6 +222,8 @@ struct fd_device {
 
    bool closefd; /* call close(fd) upon destruction */
 
+   bool disable_explicit_sync_heuristic;
+
    /* just for valgrind: */
    int bo_size;
 
@@ -321,6 +323,7 @@ struct fd_pipe {
    struct fd_device *dev;
    enum fd_pipe_id id;
    struct fd_dev_id dev_id;
+   uint8_t gen;
 
    /**
     * Note refcnt is *not* atomic, but protected by fence_lock, since the
