@@ -17,6 +17,7 @@
 
 #include "disasm.h"
 #include "instr-a3xx.h"
+#include "ir3.h"
 
 static enum debug_t debug;
 
@@ -39,6 +40,8 @@ static const char *levels[] = {
    "x",
 };
 
+typedef BITSET_DECLARE(gpr_bitset, GPR_REG_SIZE);
+
 struct disasm_ctx {
    FILE *out;
    struct isa_decode_options *options;
@@ -66,6 +69,9 @@ struct disasm_ctx {
       bool ss;
       uint8_t nop;
       uint8_t repeat;
+      bool alias;
+      ir3_alias_scope alias_scope;
+      bool alias_full;
    } last;
 
    /**
@@ -77,10 +83,17 @@ struct disasm_ctx {
       enum {
          FILE_GPR = 1,
          FILE_CONST = 2,
+         FILE_RT = 3,
       } file;
       unsigned num;
    } reg;
 
+   /* Track which registers are currently aliases because they shouldn't be
+    * included in the GPR footprint.
+    */
+   gpr_bitset full_aliases;
+   gpr_bitset half_aliases;
+
    struct shader_stats *stats;
 };
 
@@ -172,6 +185,7 @@ static const struct opc_info {
    OPC(1, OPC_SWZ,          swz),
    OPC(1, OPC_SCT,          sct),
    OPC(1, OPC_GAT,          gat),
+   OPC(1, OPC_MOVS,         movs),
    OPC(1, OPC_BALLOT_MACRO, ballot.macro),
    OPC(1, OPC_ANY_MACRO,    any.macro),
    OPC(1, OPC_ALL_MACRO,    all.macro),
@@ -437,6 +451,8 @@ disasm_field_cb(void *d, const char *field_name, struct isa_decode_value *val)
          ctx->options->stop = true;
       } else if (!strcmp("bary.f", val->str)) {
          ctx->stats->last_baryf = ctx->cur_n;
+      } else if (!strcmp("alias", val->str)) {
+         ctx->last.alias = true;
       }
    } else if (!strcmp(field_name, "REPEAT")) {
       ctx->extra_cycles += val->num;
@@ -461,6 +477,8 @@ disasm_field_cb(void *d, const char *field_name, struct isa_decode_value *val)
          ctx->reg.num = val->num;
          ctx->reg.file = FILE_GPR;
       }
+   } else if (!strcmp(field_name, "RT")) {
+      ctx->reg.file = FILE_RT;
    } else if (!strcmp(field_name, "SRC_R") || !strcmp(field_name, "SRC1_R") ||
               !strcmp(field_name, "SRC2_R") || !strcmp(field_name, "SRC3_R")) {
       ctx->reg.r = val->num;
@@ -472,8 +490,20 @@ disasm_field_cb(void *d, const char *field_name, struct isa_decode_value *val)
        * that case either.
        */
       ctx->reg.r = true;
+
+      if (ctx->last.alias && ctx->last.alias_scope == ALIAS_TEX) {
+         if (ctx->last.alias_full) {
+            BITSET_SET(ctx->full_aliases, val->num);
+         } else {
+            BITSET_SET(ctx->half_aliases, val->num);
+         }
+      }
    } else if (strstr(field_name, "HALF")) {
       ctx->reg.half = val->num;
+   } else if (strstr(field_name, "TYPE_SIZE")) {
+      if (ctx->last.alias && ctx->last.alias_scope == ALIAS_TEX) {
+         ctx->last.alias_full = val->num == 1;
+      }
    } else if (!strcmp(field_name, "SWIZ")) {
       unsigned num = (ctx->reg.num << 2) | val->num;
       if (ctx->reg.r)
@@ -482,14 +512,16 @@ disasm_field_cb(void *d, const char *field_name, struct isa_decode_value *val)
       if (ctx->reg.file == FILE_CONST) {
          ctx->stats->constlen = MAX2(ctx->stats->constlen, num);
       } else if (ctx->reg.file == FILE_GPR) {
-         if (ctx->reg.half) {
+         if (ctx->reg.half && !BITSET_TEST(ctx->half_aliases, num)) {
             ctx->stats->halfreg = MAX2(ctx->stats->halfreg, num);
-         } else {
+         } else if (!BITSET_TEST(ctx->full_aliases, num)){
             ctx->stats->fullreg = MAX2(ctx->stats->fullreg, num);
          }
       }
 
       memset(&ctx->reg, 0, sizeof(ctx->reg));
+   } else if (!strcmp(field_name, "SCOPE")) {
+      ctx->last.alias_scope = val->num;
    }
 }
 
@@ -512,6 +544,12 @@ disasm_handle_last(struct disasm_ctx *ctx)
       ctx->sfu_delay -= n;
    }
 
+   if (ctx->cur_opc_cat == 5) {
+      /* tex instruction clear the alias table. */
+      memset(&ctx->full_aliases, 0, sizeof(ctx->full_aliases));
+      memset(&ctx->half_aliases, 0, sizeof(ctx->half_aliases));
+   }
+
    memset(&ctx->last, 0, sizeof(ctx->last));
 }
 
@@ -585,6 +623,8 @@ disasm_a3xx_stat(uint32_t *dwords, int sizedwords, int level, FILE *out,
       .cur_n = -1,
    };
 
+   memset(&ctx.full_aliases, 0, sizeof(ctx.full_aliases));
+   memset(&ctx.half_aliases, 0, sizeof(ctx.half_aliases));
    memset(stats, 0, sizeof(*stats));
 
    decode_options.cbdata = &ctx;
