@@ -21,8 +21,6 @@
 
 #include <fcntl.h>
 
-#include "vulkan/vulkan_android.h"
-
 #include "util/mesa-sha1.h"
 #include "vk_descriptors.h"
 #include "vk_util.h"
@@ -974,6 +972,7 @@ tu_FreeDescriptorSets(VkDevice _device,
    return VK_SUCCESS;
 }
 
+template <chip CHIP>
 static void
 write_texel_buffer_descriptor_addr(uint32_t *dst,
                                    const VkDescriptorAddressInfoEXT *buffer_info)
@@ -983,9 +982,9 @@ write_texel_buffer_descriptor_addr(uint32_t *dst,
    } else {
       uint8_t swiz[4] = { PIPE_SWIZZLE_X, PIPE_SWIZZLE_Y, PIPE_SWIZZLE_Z,
                           PIPE_SWIZZLE_W };
-      fdl6_buffer_view_init(dst,
-                            vk_format_to_pipe_format(buffer_info->format),
-                            swiz, buffer_info->address, buffer_info->range);
+      fdl6_buffer_view_init<CHIP>(dst,
+                                  vk_format_to_pipe_format(buffer_info->format),
+                                  swiz, buffer_info->address, buffer_info->range);
    }
 }
 
@@ -1160,15 +1159,16 @@ write_sampler_descriptor(uint32_t *dst, VkSampler _sampler)
 }
 
 static void
-write_accel_struct(uint32_t *dst, uint64_t va, uint64_t size)
+write_accel_struct(uint32_t *dst, uint64_t va)
 {
    dst[0] = A6XX_TEX_CONST_0_TILE_MODE(TILE6_LINEAR) | A6XX_TEX_CONST_0_FMT(FMT6_32_UINT);
 
-   /* The overall range of the entire AS may be more than the max range, but
-    * the SSBO is only used to access the instance descriptors and header.
-    * Make sure that we don't specify a too-large range.
+   /* We don't actually use the bounds checking in the shader, since the
+    * instance array is accessed entirely with a driver-controlled offset.
+    * Therefore just always specify the maximum possible size to avoid having
+    * to keep track of the size.
     */
-   dst[1] = MAX2(DIV_ROUND_UP(size, AS_RECORD_SIZE), MAX_TEXEL_ELEMENTS);
+   dst[1] = MAX_TEXEL_ELEMENTS;
    dst[2] =
       A6XX_TEX_CONST_2_STRUCTSIZETEXELS(AS_RECORD_SIZE / 4) |
       A6XX_TEX_CONST_2_STARTOFFSETTEXELS(0) |
@@ -1205,10 +1205,10 @@ tu_GetDescriptorEXT(
       write_buffer_descriptor_addr(device, dest, pDescriptorInfo->data.pStorageBuffer);
       break;
    case VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER:
-      write_texel_buffer_descriptor_addr(dest, pDescriptorInfo->data.pUniformTexelBuffer);
+      TU_CALLX(device, write_texel_buffer_descriptor_addr)(dest, pDescriptorInfo->data.pUniformTexelBuffer);
       break;
    case VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER:
-      write_texel_buffer_descriptor_addr(dest, pDescriptorInfo->data.pStorageTexelBuffer);
+      TU_CALLX(device, write_texel_buffer_descriptor_addr)(dest, pDescriptorInfo->data.pStorageTexelBuffer);
       break;
    case VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE:
       write_image_descriptor(dest, VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE,
@@ -1229,13 +1229,9 @@ tu_GetDescriptorEXT(
       break;
    case VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR: {
       if (pDescriptorInfo->data.accelerationStructure == 0) {
-         write_accel_struct(dest, device->null_accel_struct_bo->iova,
-                            device->null_accel_struct_bo->size);
+         write_accel_struct(dest, device->null_accel_struct_bo->iova);
       } else {
-         VkDeviceSize size = *(VkDeviceSize *)
-            util_sparse_array_get(&device->accel_struct_ranges,
-                                  pDescriptorInfo->data.accelerationStructure);
-         write_accel_struct(dest, pDescriptorInfo->data.accelerationStructure, size);
+         write_accel_struct(dest, pDescriptorInfo->data.accelerationStructure);
       }
       break;
    }
@@ -1244,7 +1240,7 @@ tu_GetDescriptorEXT(
                              pDescriptorInfo->data.pInputAttachmentImage);
       break;
    default:
-      unreachable("unimplemented descriptor type");
+      UNREACHABLE("unimplemented descriptor type");
       break;
    }
 }
@@ -1361,16 +1357,14 @@ tu_update_descriptor_sets(const struct tu_device *device,
             VK_FROM_HANDLE(vk_acceleration_structure, accel_struct, accel_structs->pAccelerationStructures[j]);
             if (accel_struct) {
                write_accel_struct(ptr,
-                                  vk_acceleration_structure_get_va(accel_struct),
-                                  accel_struct->size);
+                                  vk_acceleration_structure_get_va(accel_struct));
             } else {
-               write_accel_struct(ptr, device->null_accel_struct_bo->iova,
-                                  device->null_accel_struct_bo->size);
+               write_accel_struct(ptr, device->null_accel_struct_bo->iova);
             }
             break;
          }
          default:
-            unreachable("unimplemented descriptor type");
+            UNREACHABLE("unimplemented descriptor type");
             break;
          }
          ptr += binding_layout->size / 4;
@@ -1703,16 +1697,14 @@ tu_update_descriptor_set_with_template(
             VK_FROM_HANDLE(vk_acceleration_structure, accel_struct, *(const VkAccelerationStructureKHR *)src);
             if (accel_struct) {
                write_accel_struct(ptr,
-                                  vk_acceleration_structure_get_va(accel_struct),
-                                  accel_struct->size);
+                                  vk_acceleration_structure_get_va(accel_struct));
             } else {
-               write_accel_struct(ptr, device->null_accel_struct_bo->iova,
-                                  device->null_accel_struct_bo->size);
+               write_accel_struct(ptr, device->null_accel_struct_bo->iova);
             }
             break;
          }
          default:
-            unreachable("unimplemented descriptor type");
+            UNREACHABLE("unimplemented descriptor type");
             break;
          }
          src = (char *) src + templ->entry[i].src_stride;
