@@ -15,12 +15,9 @@
 #include "vk_acceleration_structure.h"
 #include "drm-uapi/drm_fourcc.h"
 
-#include "tu_android.h"
 #include "tu_device.h"
 #include "tu_image.h"
 
-#include <vulkan/vulkan_android.h>
-
 static bool
 tu6_format_vtx_supported(enum pipe_format format)
 {
@@ -223,10 +220,11 @@ tu_physical_device_get_format_properties(
 
       /* TODO: The blob also exposes these for R16G16_UINT/R16G16_SINT/
        * R32G32_SFLOAT/R32G32B32A32_SFLOAT, but we don't have any tests for those.
-       * R32_SFLOAT is also included here by the blob, but that requires
-       * implementing VK_EXT_shader_atomic_float.
+       * The WoA blob on X1 also supports VK_EXT_shader_image_atomic_int64 (for
+       * R64_UINT and R64_SINT).
        */
-      if (vk_format == VK_FORMAT_R32_UINT || vk_format == VK_FORMAT_R32_SINT) {
+      if (vk_format == VK_FORMAT_R32_UINT || vk_format == VK_FORMAT_R32_SINT ||
+          vk_format == VK_FORMAT_R32_SFLOAT) {
          optimal |= VK_FORMAT_FEATURE_2_STORAGE_IMAGE_ATOMIC_BIT;
          buffer |= VK_FORMAT_FEATURE_2_STORAGE_TEXEL_BUFFER_ATOMIC_BIT;
       }
@@ -346,7 +344,7 @@ tu_GetPhysicalDeviceFormatProperties2(
       /* note: ubwc_possible() argument values to be ignored except for format */
       if (pFormatProperties->formatProperties.optimalTilingFeatures &&
           tiling_possible(format) &&
-          ubwc_possible(NULL, format, VK_IMAGE_TYPE_2D, 0, 0,
+          ubwc_possible(NULL, format, VK_IMAGE_TYPE_2D, 0, 0, 0,
                         physical_device->info, VK_SAMPLE_COUNT_1_BIT, 1,
                         false)) {
          vk_outarray_append_typed(VkDrmFormatModifierPropertiesEXT, &out, mod_props) {
@@ -405,6 +403,10 @@ tu_get_image_format_properties(
       if (info->flags & VK_IMAGE_CREATE_SUBSAMPLED_BIT_EXT)
          return VK_ERROR_FORMAT_NOT_SUPPORTED;
 
+      /* Don't allow modifiers with sparse */
+      if (info->flags & VK_IMAGE_CREATE_SPARSE_BINDING_BIT)
+         return VK_ERROR_FORMAT_NOT_SUPPORTED;
+
       switch (drm_info->drmFormatModifier) {
       case DRM_FORMAT_MOD_QCOM_COMPRESSED:
          /* falling back to linear/non-UBWC isn't possible with explicit modifier */
@@ -423,9 +425,9 @@ tu_get_image_format_properties(
                return VK_ERROR_FORMAT_NOT_SUPPORTED;
          }
 
-         if (!ubwc_possible(NULL, info->format, info->type, info->usage,
-                            info->usage, physical_device->info, sampleCounts,
-                            1, false)) {
+         if (!ubwc_possible(NULL, info->format, info->type, info->flags,
+                            info->usage, info->usage, physical_device->info,
+                            sampleCounts, 1, false)) {
             return VK_ERROR_FORMAT_NOT_SUPPORTED;
          }
 
@@ -442,19 +444,44 @@ tu_get_image_format_properties(
       format_feature_flags = format_props.optimalTilingFeatures;
       break;
    default:
-      unreachable("bad VkPhysicalDeviceImageFormatInfo2");
+      UNREACHABLE("bad VkPhysicalDeviceImageFormatInfo2");
    }
 
    if (format_feature_flags == 0)
       return tu_image_unsupported_format(pImageFormatProperties);
 
+   if (info->flags & VK_IMAGE_CREATE_SPARSE_BINDING_BIT) {
+      if (!physical_device->has_sparse)
+         return tu_image_unsupported_format(pImageFormatProperties);
+   }
+
+   if (info->flags & VK_IMAGE_CREATE_SPARSE_RESIDENCY_BIT) {
+      /* Don't support multi-planar formats with sparse yet */
+      if (vk_format_get_plane_count(info->format) > 1)
+         return tu_image_unsupported_format(pImageFormatProperties);
+
+      /* Sparse isn't compatible with HIC */
+      if (info->usage & VK_IMAGE_USAGE_HOST_TRANSFER_BIT_EXT)
+         return tu_image_unsupported_format(pImageFormatProperties);
+
+      /* We can't support sparse when we force linear tiling, so disable
+       * sparse with formats or usages which could cause us to fall back to
+       * linear. We also currently don't support sparse for 3D images.
+       */
+      if (info->type != VK_IMAGE_TYPE_2D ||
+          info->tiling != VK_IMAGE_TILING_OPTIMAL ||
+          !tiling_possible(info->format) ||
+          (info->usage & VK_IMAGE_USAGE_FRAGMENT_DENSITY_MAP_BIT_EXT))
+         return tu_image_unsupported_format(pImageFormatProperties);
+   }
+
    if (info->type != VK_IMAGE_TYPE_2D &&
        vk_format_is_depth_or_stencil(info->format))
       return tu_image_unsupported_format(pImageFormatProperties);
 
    switch (info->type) {
    default:
-      unreachable("bad vkimage type\n");
+      UNREACHABLE("bad vkimage type\n");
    case VK_IMAGE_TYPE_1D:
       maxExtent.width = 16384;
       maxExtent.height = 1;
@@ -536,7 +563,8 @@ tu_get_image_format_properties(
       }
    }
 
-   if (image_usage & VK_IMAGE_USAGE_INPUT_ATTACHMENT_BIT) {
+   if (image_usage & (VK_IMAGE_USAGE_INPUT_ATTACHMENT_BIT |
+                      VK_IMAGE_USAGE_TRANSIENT_ATTACHMENT_BIT)) {
       if (!(format_feature_flags &
             (VK_FORMAT_FEATURE_2_COLOR_ATTACHMENT_BIT |
              VK_FORMAT_FEATURE_2_DEPTH_STENCIL_ATTACHMENT_BIT))) {
@@ -589,6 +617,8 @@ tu_get_external_image_format_properties(
     *    VK_ERROR_FORMAT_NOT_SUPPORTED.
     */
 
+   assert(handleType !=
+          VK_EXTERNAL_MEMORY_HANDLE_TYPE_ANDROID_HARDWARE_BUFFER_BIT_ANDROID);
    switch (handleType) {
    case VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT:
    case VK_EXTERNAL_MEMORY_HANDLE_TYPE_DMA_BUF_BIT_EXT:
@@ -607,14 +637,6 @@ tu_get_external_image_format_properties(
                           handleType, pImageFormatInfo->type);
       }
       break;
-#if DETECT_OS_ANDROID
-   case VK_EXTERNAL_MEMORY_HANDLE_TYPE_ANDROID_HARDWARE_BUFFER_BIT_ANDROID:
-      flags = VK_EXTERNAL_MEMORY_FEATURE_DEDICATED_ONLY_BIT |
-              VK_EXTERNAL_MEMORY_FEATURE_EXPORTABLE_BIT |
-              VK_EXTERNAL_MEMORY_FEATURE_IMPORTABLE_BIT;
-      compat_flags = export_flags = VK_EXTERNAL_MEMORY_HANDLE_TYPE_ANDROID_HARDWARE_BUFFER_BIT_ANDROID;
-      break;
-#endif
    case VK_EXTERNAL_MEMORY_HANDLE_TYPE_HOST_ALLOCATION_BIT_EXT:
       flags = VK_EXTERNAL_MEMORY_FEATURE_IMPORTABLE_BIT;
       compat_flags = VK_EXTERNAL_MEMORY_HANDLE_TYPE_HOST_ALLOCATION_BIT_EXT;
@@ -647,7 +669,6 @@ tu_GetPhysicalDeviceImageFormatProperties2(
    const VkPhysicalDeviceExternalImageFormatInfo *external_info = NULL;
    const VkPhysicalDeviceImageViewImageFormatInfoEXT *image_view_info = NULL;
    VkExternalImageFormatProperties *external_props = NULL;
-   VkAndroidHardwareBufferUsageANDROID *android_usage = NULL;
    VkFilterCubicImageViewImageFormatPropertiesEXT *cubic_props = NULL;
    VkFormatFeatureFlags format_feature_flags;
    VkSamplerYcbcrConversionImageFormatProperties *ycbcr_props = NULL;
@@ -681,9 +702,6 @@ tu_GetPhysicalDeviceImageFormatProperties2(
       case VK_STRUCTURE_TYPE_EXTERNAL_IMAGE_FORMAT_PROPERTIES:
          external_props = (VkExternalImageFormatProperties *) s;
          break;
-      case VK_STRUCTURE_TYPE_ANDROID_HARDWARE_BUFFER_USAGE_ANDROID:
-         android_usage = (VkAndroidHardwareBufferUsageANDROID *) s;
-         break;
       case VK_STRUCTURE_TYPE_FILTER_CUBIC_IMAGE_VIEW_IMAGE_FORMAT_PROPERTIES_EXT:
          cubic_props = (VkFilterCubicImageViewImageFormatPropertiesEXT *) s;
          break;
@@ -705,11 +723,31 @@ tu_GetPhysicalDeviceImageFormatProperties2(
     *    present and VkExternalImageFormatProperties will be ignored.
     */
    if (external_info && external_info->handleType != 0) {
-      result = tu_get_external_image_format_properties(
-         physical_device, base_info, external_info->handleType,
-         external_props);
-      if (result != VK_SUCCESS)
-         goto fail;
+      if (external_info->handleType ==
+          VK_EXTERNAL_MEMORY_HANDLE_TYPE_ANDROID_HARDWARE_BUFFER_BIT_ANDROID) {
+         result = vk_android_get_ahb_image_properties(physicalDevice,
+                                                      base_info, base_props);
+         if (result != VK_SUCCESS)
+            goto fail;
+
+         VkImageFormatProperties *props = &base_props->imageFormatProperties;
+         if (!(props->sampleCounts & VK_SAMPLE_COUNT_1_BIT)) {
+            result = vk_errorf(physical_device, VK_ERROR_FORMAT_NOT_SUPPORTED,
+                               "sampleCounts (%x) unsupported for AHB",
+                               props->sampleCounts);
+            goto fail;
+         }
+
+         /* AHBs with mipmap usage will ignore this property */
+         props->maxMipLevels = 1;
+         props->sampleCounts = VK_SAMPLE_COUNT_1_BIT;
+      } else {
+         result = tu_get_external_image_format_properties(
+            physical_device, base_info, external_info->handleType,
+            external_props);
+         if (result != VK_SUCCESS)
+            goto fail;
+      }
    }
 
    if (cubic_props) {
@@ -727,43 +765,6 @@ tu_GetPhysicalDeviceImageFormatProperties2(
       }
    }
 
-   if (android_usage) {
-      /* Don't expect gralloc to be able to allocate anything other than 3D: */
-      if (base_info->type != VK_IMAGE_TYPE_2D) {
-         result = vk_errorf(physical_device, VK_ERROR_FORMAT_NOT_SUPPORTED,
-                            "type (%u) unsupported for AHB", base_info->type);
-         goto fail;
-      }
-      VkImageFormatProperties *props = &base_props->imageFormatProperties;
-      if (!(props->sampleCounts & VK_SAMPLE_COUNT_1_BIT)) {
-         result = vk_errorf(physical_device, VK_ERROR_FORMAT_NOT_SUPPORTED,
-                          "sampleCounts (%x) unsupported for AHB", props->sampleCounts);
-         goto fail;
-      }
-      android_usage->androidHardwareBufferUsage =
-         vk_image_usage_to_ahb_usage(base_info->flags, base_info->usage);
-      uint32_t format = vk_image_format_to_ahb_format(base_info->format);
-      if (!format) {
-         result = vk_errorf(physical_device, VK_ERROR_FORMAT_NOT_SUPPORTED,
-                            "format (%u) unsupported for AHB", base_info->format);
-         goto fail;
-      }
-      /* We can't advertise support for anything that gralloc cannot allocate
-       * so we are stuck without any better option than attempting a test
-       * allocation:
-       */
-      if (!vk_ahb_probe_format(base_info->format, base_info->flags, base_info->usage)) {
-         result = vk_errorf(physical_device, VK_ERROR_FORMAT_NOT_SUPPORTED,
-                            "format (%x) with flags (%x) and usage (%x) unsupported for AHB",
-                            base_info->format, base_info->flags, base_info->usage);
-         goto fail;
-      }
-
-      /* AHBs with mipmap usage will ignore this property */
-      props->maxMipLevels = 1;
-      props->sampleCounts = VK_SAMPLE_COUNT_1_BIT;
-   }
-
    if (ycbcr_props)
       ycbcr_props->combinedImageSamplerDescriptorCount = 1;
 
@@ -794,6 +795,7 @@ tu_GetPhysicalDeviceImageFormatProperties2(
          (fd6_color_swap(vk_format_to_pipe_format(base_info->format),
                                                   TILE6_LINEAR, false) == WZYX &&
          !ubwc_possible(NULL, base_info->format, base_info->type,
+                        base_info->flags,
                         (base_info->usage & ~VK_IMAGE_USAGE_HOST_TRANSFER_BIT_EXT),
                         (base_info->usage & ~VK_IMAGE_USAGE_HOST_TRANSFER_BIT_EXT),
                         physical_device->info, VK_SAMPLE_COUNT_1_BIT, 1,
@@ -816,14 +818,3 @@ fail:
 
    return result;
 }
-
-VKAPI_ATTR void VKAPI_CALL
-tu_GetPhysicalDeviceSparseImageFormatProperties2(
-   VkPhysicalDevice physicalDevice,
-   const VkPhysicalDeviceSparseImageFormatInfo2 *pFormatInfo,
-   uint32_t *pPropertyCount,
-   VkSparseImageFormatProperties2 *pProperties)
-{
-   /* Sparse images are not yet supported. */
-   *pPropertyCount = 0;
-}
