@@ -36,6 +36,8 @@
 
 #include "gallivm/lp_bld_sample.h"
 #include "gallivm/lp_bld_limits.h"
+#include "gallivm/lp_bld_jit_types.h"
+#include "gallivm/lp_bld_jit_sample.h"
 
 #include "pipe/p_context.h"
 #include "util/list.h"
@@ -47,105 +49,6 @@ struct llvm_geometry_shader;
 struct llvm_tess_ctrl_shader;
 struct llvm_tess_eval_shader;
 
-struct draw_jit_texture
-{
-   uint32_t width;
-   uint32_t height;
-   uint32_t depth;
-   const void *base;
-   uint32_t row_stride[PIPE_MAX_TEXTURE_LEVELS];
-   uint32_t img_stride[PIPE_MAX_TEXTURE_LEVELS];
-   uint32_t first_level;
-   uint32_t last_level;
-   uint32_t mip_offsets[PIPE_MAX_TEXTURE_LEVELS];
-   uint32_t num_samples;
-   uint32_t sample_stride;
-};
-
-
-struct draw_sampler_static_state
-{
-   /*
-    * These attributes are effectively interleaved for more sane key handling.
-    * However, there might be lots of null space if the amount of samplers and
-    * textures isn't the same.
-    */
-   struct lp_static_sampler_state sampler_state;
-   struct lp_static_texture_state texture_state;
-};
-
-struct draw_image_static_state
-{
-   struct lp_static_texture_state image_state;
-};
-
-
-struct draw_jit_sampler
-{
-   float min_lod;
-   float max_lod;
-   float lod_bias;
-   float border_color[4];
-   float max_aniso;
-};
-
-
-struct draw_jit_image
-{
-   uint32_t width;
-   uint32_t height;
-   uint32_t depth;
-   const void *base;
-   uint32_t row_stride;
-   uint32_t img_stride;
-   uint32_t num_samples;
-   uint32_t sample_stride;
-};
-
-enum {
-   DRAW_JIT_TEXTURE_WIDTH = 0,
-   DRAW_JIT_TEXTURE_HEIGHT,
-   DRAW_JIT_TEXTURE_DEPTH,
-   DRAW_JIT_TEXTURE_BASE,
-   DRAW_JIT_TEXTURE_ROW_STRIDE,
-   DRAW_JIT_TEXTURE_IMG_STRIDE,
-   DRAW_JIT_TEXTURE_FIRST_LEVEL,
-   DRAW_JIT_TEXTURE_LAST_LEVEL,
-   DRAW_JIT_TEXTURE_MIP_OFFSETS,
-   DRAW_JIT_TEXTURE_NUM_SAMPLES,
-   DRAW_JIT_TEXTURE_SAMPLE_STRIDE,
-   DRAW_JIT_TEXTURE_NUM_FIELDS  /* number of fields above */
-};
-
-
-enum {
-   DRAW_JIT_SAMPLER_MIN_LOD,
-   DRAW_JIT_SAMPLER_MAX_LOD,
-   DRAW_JIT_SAMPLER_LOD_BIAS,
-   DRAW_JIT_SAMPLER_BORDER_COLOR,
-   DRAW_JIT_SAMPLER_MAX_ANISO,
-   DRAW_JIT_SAMPLER_NUM_FIELDS  /* number of fields above */
-};
-
-
-enum {
-   DRAW_JIT_VERTEX_VERTEX_ID = 0,
-   DRAW_JIT_VERTEX_CLIP_POS,
-   DRAW_JIT_VERTEX_DATA
-};
-
-enum {
-   DRAW_JIT_IMAGE_WIDTH = 0,
-   DRAW_JIT_IMAGE_HEIGHT,
-   DRAW_JIT_IMAGE_DEPTH,
-   DRAW_JIT_IMAGE_BASE,
-   DRAW_JIT_IMAGE_ROW_STRIDE,
-   DRAW_JIT_IMAGE_IMG_STRIDE,
-   DRAW_JIT_IMAGE_NUM_SAMPLES,
-   DRAW_JIT_IMAGE_SAMPLE_STRIDE,
-   DRAW_JIT_IMAGE_NUM_FIELDS  /* number of fields above */
-};
-
 /**
  * This structure is passed directly to the generated vertex shader.
  *
@@ -157,83 +60,27 @@ enum {
  * Only use types with a clear size and padding here, in particular prefer the
  * stdint.h types to the basic integer types.
  */
-struct draw_jit_context
+struct draw_vs_jit_context
 {
-   const float *vs_constants[LP_MAX_TGSI_CONST_BUFFERS];
-   int num_vs_constants[LP_MAX_TGSI_CONST_BUFFERS];
    float (*planes) [DRAW_TOTAL_CLIP_PLANES][4];
    struct pipe_viewport_state *viewports;
-
-   struct draw_jit_texture textures[PIPE_MAX_SHADER_SAMPLER_VIEWS];
-   struct draw_jit_sampler samplers[PIPE_MAX_SAMPLERS];
-   struct draw_jit_image images[PIPE_MAX_SHADER_IMAGES];
-
-   const uint32_t *vs_ssbos[LP_MAX_TGSI_SHADER_BUFFERS];
-   int num_vs_ssbos[LP_MAX_TGSI_SHADER_BUFFERS];
-
-   const float *aniso_filter_table;
 };
 
 enum {
-   DRAW_JIT_CTX_CONSTANTS            = 0,
-   DRAW_JIT_CTX_NUM_CONSTANTS        = 1,
-   DRAW_JIT_CTX_PLANES               = 2,
-   DRAW_JIT_CTX_VIEWPORT             = 3,
-   DRAW_JIT_CTX_TEXTURES             = 4,
-   DRAW_JIT_CTX_SAMPLERS             = 5,
-   DRAW_JIT_CTX_IMAGES               = 6,
-   DRAW_JIT_CTX_SSBOS                = 7,
-   DRAW_JIT_CTX_NUM_SSBOS            = 8,
-   DRAW_JIT_CTX_ANISO_FILTER_TABLE   = 9,
-   DRAW_JIT_CTX_NUM_FIELDS
+   DRAW_VS_JIT_CTX_PLANES               = 0,
+   DRAW_VS_JIT_CTX_VIEWPORT             = 1,
+   DRAW_VS_JIT_CTX_NUM_FIELDS
 };
 
-#define draw_jit_context_vs_constants(_variant, _ptr) \
-   lp_build_struct_get_ptr2(_variant->gallivm, _variant->context_type, _ptr, DRAW_JIT_CTX_CONSTANTS, "vs_constants")
+#define draw_vs_jit_context_planes(_gallivm, _type, _ptr) \
+   lp_build_struct_get2(_gallivm, _type, _ptr, DRAW_VS_JIT_CTX_PLANES, "planes")
 
-#define draw_jit_context_num_vs_constants(_variant, _ptr) \
-   lp_build_struct_get_ptr2(_variant->gallivm, _variant->context_type, _ptr, DRAW_JIT_CTX_NUM_CONSTANTS, "num_vs_constants")
+#define draw_vs_jit_context_viewports(_variant, _ptr) \
+   lp_build_struct_get2(_variant->gallivm, _variant->context_type, _ptr, DRAW_VS_JIT_CTX_VIEWPORT, "viewports")
 
-#define draw_jit_context_planes(_gallivm, _type, _ptr) \
-   lp_build_struct_get2(_gallivm, _type, _ptr, DRAW_JIT_CTX_PLANES, "planes")
-
-#define draw_jit_context_viewports(_variant, _ptr) \
-   lp_build_struct_get2(_variant->gallivm, _variant->context_type, _ptr, DRAW_JIT_CTX_VIEWPORT, "viewports")
-
-#define draw_jit_context_textures(_gallivm, _ptr) \
-   lp_build_struct_get_ptr(_gallivm, _ptr, DRAW_JIT_CTX_TEXTURES, "textures")
-
-#define draw_jit_context_samplers(_gallivm, _ptr) \
-   lp_build_struct_get_ptr(_gallivm, _ptr, DRAW_JIT_CTX_SAMPLERS, "samplers")
-
-#define draw_jit_context_images(_gallivm, _ptr) \
-   lp_build_struct_get_ptr(_gallivm, _ptr, DRAW_JIT_CTX_IMAGES, "images")
-
-#define draw_jit_context_vs_ssbos(_variant, _ptr) \
-   lp_build_struct_get_ptr2(_variant->gallivm, _variant->context_type, _ptr, DRAW_JIT_CTX_SSBOS, "vs_ssbos")
-
-#define draw_jit_context_num_vs_ssbos(_variant, _ptr) \
-   lp_build_struct_get_ptr2(_variant->gallivm, _variant->context_type, _ptr, DRAW_JIT_CTX_NUM_SSBOS, "num_vs_ssbos")
-
-#define draw_jit_context_aniso_filter_table(_variant, _ptr) \
-   lp_build_struct_get2(_variant->gallivm, _variant->context_type, _ptr, DRAW_JIT_CTX_ANISO_FILTER_TABLE, "aniso_filter_table")
-
-
-#define draw_jit_header_id(_gallivm, _type, _ptr)              \
-   lp_build_struct_get_ptr2(_gallivm, _type, _ptr, DRAW_JIT_VERTEX_VERTEX_ID, "id")
-
-#define draw_jit_header_clip_pos(_gallivm, _type, _ptr) \
-   lp_build_struct_get_ptr2(_gallivm, _type, _ptr, DRAW_JIT_VERTEX_CLIP_POS, "clip_pos")
-
-#define draw_jit_header_data(_gallivm, _type, _ptr)            \
-   lp_build_struct_get_ptr2(_gallivm, _type, _ptr, DRAW_JIT_VERTEX_DATA, "data")
-
-
-#define draw_jit_vbuffer_stride(_gallivm, _type, _ptr)         \
-   lp_build_struct_get2(_gallivm, _type, _ptr, 0, "stride")
 
 #define draw_jit_vbuffer_offset(_gallivm, _type, _ptr)         \
-   lp_build_struct_get2(_gallivm, _type, _ptr, 2, "buffer_offset")
+   lp_build_struct_get2(_gallivm, _type, _ptr, 1, "buffer_offset")
 
 enum {
    DRAW_JIT_DVBUFFER_MAP = 0,
@@ -261,68 +108,29 @@ enum {
  */
 struct draw_gs_jit_context
 {
-   const float *constants[LP_MAX_TGSI_CONST_BUFFERS];
-   int num_constants[LP_MAX_TGSI_CONST_BUFFERS];
    float (*planes) [DRAW_TOTAL_CLIP_PLANES][4];
    struct pipe_viewport_state *viewports;
 
-   /* There two need to be exactly at DRAW_JIT_CTX_TEXTURES and
-    * DRAW_JIT_CTX_SAMPLERS positions in the struct */
-   struct draw_jit_texture textures[PIPE_MAX_SHADER_SAMPLER_VIEWS];
-   struct draw_jit_sampler samplers[PIPE_MAX_SAMPLERS];
-   struct draw_jit_image images[PIPE_MAX_SHADER_IMAGES];
-
    int **prim_lengths;
    int *emitted_vertices;
    int *emitted_prims;
-   const uint32_t *ssbos[LP_MAX_TGSI_SHADER_BUFFERS];
-   int num_ssbos[LP_MAX_TGSI_SHADER_BUFFERS];
-
-   const float *aniso_filter_table;
 };
 
 enum {
-   DRAW_GS_JIT_CTX_CONSTANTS = 0,
-   DRAW_GS_JIT_CTX_NUM_CONSTANTS = 1,
-   DRAW_GS_JIT_CTX_PLANES = 2,
-   DRAW_GS_JIT_CTX_VIEWPORT = 3,
-   /* Textures and samples are reserved for DRAW_JIT_CTX_TEXTURES
-    * and DRAW_JIT_CTX_SAMPLERS, because they both need
-    * to be at exactly the same locations as they are in the
-    * VS ctx structure for sampling to work. */
-   DRAW_GS_JIT_CTX_TEXTURES = DRAW_JIT_CTX_TEXTURES,
-   DRAW_GS_JIT_CTX_SAMPLERS = DRAW_JIT_CTX_SAMPLERS,
-   DRAW_GS_JIT_CTX_IMAGES = DRAW_JIT_CTX_IMAGES,
-   DRAW_GS_JIT_CTX_PRIM_LENGTHS = 7,
-   DRAW_GS_JIT_CTX_EMITTED_VERTICES = 8,
-   DRAW_GS_JIT_CTX_EMITTED_PRIMS = 9,
-   DRAW_GS_JIT_CTX_SSBOS = 10,
-   DRAW_GS_JIT_CTX_NUM_SSBOS = 11,
-   DRAW_GS_JIT_CTX_ANISO_FILTER_TABLE = 12,
-   DRAW_GS_JIT_CTX_NUM_FIELDS = 13
+   DRAW_GS_JIT_CTX_PLANES = 0,
+   DRAW_GS_JIT_CTX_VIEWPORT = 1,
+   DRAW_GS_JIT_CTX_PRIM_LENGTHS = 2,
+   DRAW_GS_JIT_CTX_EMITTED_VERTICES = 3,
+   DRAW_GS_JIT_CTX_EMITTED_PRIMS = 4,
+   DRAW_GS_JIT_CTX_NUM_FIELDS = 5
 };
 
-#define draw_gs_jit_context_constants(_variant, _ptr) \
-   lp_build_struct_get_ptr2(_variant->gallivm, _variant->context_type, _ptr, DRAW_GS_JIT_CTX_CONSTANTS, "constants")
-
-#define draw_gs_jit_context_num_constants(_variant, _ptr) \
-   lp_build_struct_get_ptr2(_variant->gallivm, _variant->context_type, _ptr, DRAW_GS_JIT_CTX_NUM_CONSTANTS, "num_constants")
-
 #define draw_gs_jit_context_planes(_gallivm, _ptr) \
    lp_build_struct_get(_gallivm, _ptr, DRAW_GS_JIT_CTX_PLANES, "planes")
 
 #define draw_gs_jit_context_viewports(_gallivm, _ptr) \
    lp_build_struct_get(_gallivm, _ptr, DRAW_GS_JIT_CTX_VIEWPORT, "viewports")
 
-#define draw_gs_jit_context_textures(_gallivm, _ptr) \
-   lp_build_struct_get_ptr(_gallivm, _ptr, DRAW_GS_JIT_CTX_TEXTURES, "textures")
-
-#define draw_gs_jit_context_samplers(_gallivm, _ptr) \
-   lp_build_struct_get_ptr(_gallivm, _ptr, DRAW_GS_JIT_CTX_SAMPLERS, "samplers")
-
-#define draw_gs_jit_context_images(_gallivm, _ptr)                      \
-   lp_build_struct_get_ptr(_gallivm, _ptr, DRAW_GS_JIT_CTX_IMAGES, "images")
-
 #define draw_gs_jit_prim_lengths(_variant, _ptr) \
    lp_build_struct_get2(_variant->gallivm, _variant->context_type, _ptr, DRAW_GS_JIT_CTX_PRIM_LENGTHS, "prim_lengths")
 
@@ -332,129 +140,14 @@ enum {
 #define draw_gs_jit_emitted_prims(_variant, _ptr) \
    lp_build_struct_get2(_variant->gallivm, _variant->context_type, _ptr, DRAW_GS_JIT_CTX_EMITTED_PRIMS, "emitted_prims")
 
-#define draw_gs_jit_context_ssbos(_variant, _ptr) \
-   lp_build_struct_get_ptr2(_variant->gallivm, _variant->context_type, _ptr, DRAW_GS_JIT_CTX_SSBOS, "ssbos")
 
-#define draw_gs_jit_context_num_ssbos(_variant, _ptr) \
-   lp_build_struct_get_ptr2(_variant->gallivm, _variant->context_type, _ptr, DRAW_GS_JIT_CTX_NUM_SSBOS, "num_ssbos")
-
-#define draw_gs_jit_context_aniso_filter_table(_variant, _ptr) \
-   lp_build_struct_get2(_variant->gallivm, _variant->context_type, _ptr, DRAW_GS_JIT_CTX_ANISO_FILTER_TABLE, "aniso_filter_table")
-
-struct draw_tcs_jit_context {
-   const float *constants[LP_MAX_TGSI_CONST_BUFFERS];
-   int num_constants[LP_MAX_TGSI_CONST_BUFFERS];
-
-   int dummy1;
-   int dummy2;
-   /* There two need to be exactly at DRAW_JIT_CTX_TEXTURES and
-    * DRAW_JIT_CTX_SAMPLERS positions in the struct */
-   struct draw_jit_texture textures[PIPE_MAX_SHADER_SAMPLER_VIEWS];
-   struct draw_jit_sampler samplers[PIPE_MAX_SAMPLERS];
-   struct draw_jit_image images[PIPE_MAX_SHADER_IMAGES];
-
-   const uint32_t *ssbos[LP_MAX_TGSI_SHADER_BUFFERS];
-   int num_ssbos[LP_MAX_TGSI_SHADER_BUFFERS];
-
-   const float *aniso_filter_table;
-};
-
-enum {
-   DRAW_TCS_JIT_CTX_CONSTANTS = 0,
-   DRAW_TCS_JIT_CTX_NUM_CONSTANTS = 1,
-   DRAW_TCS_JIT_CTX_TEXTURES = DRAW_JIT_CTX_TEXTURES,
-   DRAW_TCS_JIT_CTX_SAMPLERS = DRAW_JIT_CTX_SAMPLERS,
-   DRAW_TCS_JIT_CTX_IMAGES = DRAW_JIT_CTX_IMAGES,
-   DRAW_TCS_JIT_CTX_SSBOS = 7,
-   DRAW_TCS_JIT_CTX_NUM_SSBOS = 8,
-   DRAW_TCS_JIT_CTX_ANISO_FILTER_TABLE = 9,
-   DRAW_TCS_JIT_CTX_NUM_FIELDS = 10,
-};
-
-#define draw_tcs_jit_context_constants(_variant, _ptr) \
-   lp_build_struct_get_ptr2(_variant->gallivm, _variant->context_type, _ptr, DRAW_TCS_JIT_CTX_CONSTANTS, "constants")
-
-#define draw_tcs_jit_context_num_constants(_variant, _ptr) \
-   lp_build_struct_get_ptr2(_variant->gallivm, _variant->context_type, _ptr, DRAW_TCS_JIT_CTX_NUM_CONSTANTS, "num_constants")
-
-#define draw_tcs_jit_context_textures(_gallivm, _ptr) \
-   lp_build_struct_get_ptr(_gallivm, _ptr, DRAW_TCS_JIT_CTX_TEXTURES, "textures")
-
-#define draw_tcs_jit_context_samplers(_gallivm, _ptr) \
-   lp_build_struct_get_ptr(_gallivm, _ptr, DRAW_TCS_JIT_CTX_SAMPLERS, "samplers")
-
-#define draw_tcs_jit_context_images(_gallivm, _ptr)                      \
-   lp_build_struct_get_ptr(_gallivm, _ptr, DRAW_TCS_JIT_CTX_IMAGES, "images")
-
-#define draw_tcs_jit_context_ssbos(_variant, _ptr) \
-   lp_build_struct_get_ptr2(_variant->gallivm, _variant->context_type, _ptr, DRAW_TCS_JIT_CTX_SSBOS, "ssbos")
-
-#define draw_tcs_jit_context_num_ssbos(_variant, _ptr) \
-   lp_build_struct_get_ptr2(_variant->gallivm, _variant->context_type, _ptr, DRAW_TCS_JIT_CTX_NUM_SSBOS, "num_ssbos")
-
-#define draw_tcs_jit_context_aniso_filter_table(_variant, _ptr) \
-   lp_build_struct_get2(_variant->gallivm, _variant->context_type, _ptr, DRAW_TCS_JIT_CTX_ANISO_FILTER_TABLE, "aniso_filter_table")
-
-struct draw_tes_jit_context {
-   const float *constants[LP_MAX_TGSI_CONST_BUFFERS];
-   int num_constants[LP_MAX_TGSI_CONST_BUFFERS];
-
-   int dummy1;
-   int dummy2;
-   /* There two need to be exactly at DRAW_JIT_CTX_TEXTURES and
-    * DRAW_JIT_CTX_SAMPLERS positions in the struct */
-   struct draw_jit_texture textures[PIPE_MAX_SHADER_SAMPLER_VIEWS];
-   struct draw_jit_sampler samplers[PIPE_MAX_SAMPLERS];
-   struct draw_jit_image images[PIPE_MAX_SHADER_IMAGES];
-
-   const uint32_t *ssbos[LP_MAX_TGSI_SHADER_BUFFERS];
-   int num_ssbos[LP_MAX_TGSI_SHADER_BUFFERS];
-
-   const float *aniso_filter_table;
-};
-
-enum {
-   DRAW_TES_JIT_CTX_CONSTANTS = 0,
-   DRAW_TES_JIT_CTX_NUM_CONSTANTS = 1,
-   DRAW_TES_JIT_CTX_TEXTURES = DRAW_JIT_CTX_TEXTURES,
-   DRAW_TES_JIT_CTX_SAMPLERS = DRAW_JIT_CTX_SAMPLERS,
-   DRAW_TES_JIT_CTX_IMAGES = DRAW_JIT_CTX_IMAGES,
-   DRAW_TES_JIT_CTX_SSBOS = 7,
-   DRAW_TES_JIT_CTX_NUM_SSBOS = 8,
-   DRAW_TES_JIT_CTX_ANISO_FILTER_TABLE = 9,
-   DRAW_TES_JIT_CTX_NUM_FIELDS = 10,
-};
-
-#define draw_tes_jit_context_constants(_variant, _ptr) \
-   lp_build_struct_get_ptr2(_variant->gallivm, _variant->context_type, _ptr, DRAW_TES_JIT_CTX_CONSTANTS, "constants")
-
-#define draw_tes_jit_context_num_constants(_variant, _ptr) \
-   lp_build_struct_get_ptr2(_variant->gallivm, _variant->context_type, _ptr, DRAW_TES_JIT_CTX_NUM_CONSTANTS, "num_constants")
-
-#define draw_tes_jit_context_textures(_gallivm, _ptr) \
-   lp_build_struct_get_ptr(_gallivm, _ptr, DRAW_TES_JIT_CTX_TEXTURES, "textures")
-
-#define draw_tes_jit_context_samplers(_gallivm, _ptr) \
-   lp_build_struct_get_ptr(_gallivm, _ptr, DRAW_TES_JIT_CTX_SAMPLERS, "samplers")
-
-#define draw_tes_jit_context_images(_gallivm, _ptr)                      \
-   lp_build_struct_get_ptr(_gallivm, _ptr, DRAW_TES_JIT_CTX_IMAGES, "images")
-
-#define draw_tes_jit_context_ssbos(_variant, _ptr) \
-   lp_build_struct_get_ptr2(_variant->gallivm, _variant->context_type, _ptr, DRAW_TES_JIT_CTX_SSBOS, "ssbos")
-
-#define draw_tes_jit_context_num_ssbos(_variant, _ptr) \
-   lp_build_struct_get_ptr2(_variant->gallivm, _variant->context_type, _ptr, DRAW_TES_JIT_CTX_NUM_SSBOS, "num_ssbos")
-
-#define draw_tes_jit_context_aniso_filter_table(_variant, _ptr) \
-   lp_build_struct_get2(_variant->gallivm, _variant->context_type, _ptr, DRAW_TES_JIT_CTX_ANISO_FILTER_TABLE, "aniso_filter_table")
-
-typedef boolean
-(*draw_jit_vert_func)(struct draw_jit_context *context,
+typedef bool
+(*draw_jit_vert_func)(struct draw_vs_jit_context *context,
+                      const struct lp_jit_resources *resources,
                       struct vertex_header *io,
                       const struct draw_vertex_buffer vbuffers[PIPE_MAX_ATTRIBS],
                       unsigned count,
-                      unsigned start_or_maxelt,
+                      unsigned start,
                       unsigned stride,
                       struct pipe_vertex_buffer *vertex_buffers,
                       unsigned instance_id,
@@ -466,6 +159,7 @@ typedef boolean
 
 typedef int
 (*draw_gs_jit_func)(struct draw_gs_jit_context *context,
+                    const struct lp_jit_resources *resources,
                     float inputs[6][PIPE_MAX_SHADER_INPUTS][TGSI_NUM_CHANNELS][TGSI_NUM_CHANNELS],
                     struct vertex_header **output,
                     unsigned num_prims,
@@ -475,14 +169,14 @@ typedef int
                     unsigned view_id);
 
 typedef int
-(*draw_tcs_jit_func)(struct draw_tcs_jit_context *context,
+(*draw_tcs_jit_func)(const struct lp_jit_resources *resources,
                      float inputs[32][NUM_TCS_INPUTS][TGSI_NUM_CHANNELS],
                      float outputs[32][PIPE_MAX_SHADER_INPUTS][TGSI_NUM_CHANNELS],
                      uint32_t prim_id, uint32_t patch_vertices_in,
                      unsigned view_id);
 
 typedef int
-(*draw_tes_jit_func)(struct draw_tes_jit_context *context,
+(*draw_tes_jit_func)(const struct lp_jit_resources *resources,
                      float inputs[32][PIPE_MAX_SHADER_INPUTS][TGSI_NUM_CHANNELS],
                      struct vertex_header *io,
                      uint32_t prim_id, uint32_t num_tess_coord,
@@ -528,7 +222,7 @@ struct draw_gs_llvm_variant_key
    unsigned num_outputs:8;
    /* note padding here - must use memset */
    unsigned clamp_vertex_color:1;
-   struct draw_sampler_static_state samplers[1];
+   struct lp_sampler_static_state samplers[1];
    /* Followed by variable number of images.*/
 };
 
@@ -537,7 +231,7 @@ struct draw_tcs_llvm_variant_key
    unsigned nr_samplers:8;
    unsigned nr_sampler_views:8;
    unsigned nr_images:8;
-   struct draw_sampler_static_state samplers[1];
+   struct lp_sampler_static_state samplers[1];
    /* Followed by variable number of images.*/
 };
 
@@ -549,30 +243,30 @@ struct draw_tes_llvm_variant_key
    unsigned primid_output:7;
    unsigned primid_needed:1;
    unsigned clamp_vertex_color:1;
-   struct draw_sampler_static_state samplers[1];
+   struct lp_sampler_static_state samplers[1];
    /* Followed by variable number of images.*/
 };
 
 #define DRAW_LLVM_MAX_VARIANT_KEY_SIZE \
    (sizeof(struct draw_llvm_variant_key) +	\
-    PIPE_MAX_SHADER_SAMPLER_VIEWS * sizeof(struct draw_sampler_static_state) +	\
-    PIPE_MAX_SHADER_IMAGES * sizeof(struct draw_image_static_state) + \
+    PIPE_MAX_SHADER_SAMPLER_VIEWS * sizeof(struct lp_sampler_static_state) +	\
+    PIPE_MAX_SHADER_IMAGES * sizeof(struct lp_image_static_state) + \
     (PIPE_MAX_ATTRIBS-1) * sizeof(struct pipe_vertex_element))
 
 #define DRAW_GS_LLVM_MAX_VARIANT_KEY_SIZE \
    (sizeof(struct draw_gs_llvm_variant_key) +	\
-    PIPE_MAX_SHADER_IMAGES * sizeof(struct draw_image_static_state) + \
-    PIPE_MAX_SHADER_SAMPLER_VIEWS * sizeof(struct draw_sampler_static_state))
+    PIPE_MAX_SHADER_IMAGES * sizeof(struct lp_image_static_state) + \
+    PIPE_MAX_SHADER_SAMPLER_VIEWS * sizeof(struct lp_sampler_static_state))
 
 #define DRAW_TCS_LLVM_MAX_VARIANT_KEY_SIZE \
    (sizeof(struct draw_tcs_llvm_variant_key) +	\
-    PIPE_MAX_SHADER_IMAGES * sizeof(struct draw_image_static_state) + \
-    PIPE_MAX_SHADER_SAMPLER_VIEWS * sizeof(struct draw_sampler_static_state))
+    PIPE_MAX_SHADER_IMAGES * sizeof(struct lp_image_static_state) + \
+    PIPE_MAX_SHADER_SAMPLER_VIEWS * sizeof(struct lp_sampler_static_state))
 
 #define DRAW_TES_LLVM_MAX_VARIANT_KEY_SIZE \
    (sizeof(struct draw_tes_llvm_variant_key) +	\
-    PIPE_MAX_SHADER_IMAGES * sizeof(struct draw_image_static_state) + \
-    PIPE_MAX_SHADER_SAMPLER_VIEWS * sizeof(struct draw_sampler_static_state))
+    PIPE_MAX_SHADER_IMAGES * sizeof(struct lp_image_static_state) + \
+    PIPE_MAX_SHADER_SAMPLER_VIEWS * sizeof(struct lp_sampler_static_state))
 
 
 static inline size_t
@@ -584,8 +278,8 @@ draw_llvm_variant_key_size(unsigned nr_vertex_elements,
    return (sizeof(struct draw_llvm_variant_key) +
            (nr_vertex_elements - 1) * sizeof(struct pipe_vertex_element) +
            MAX2(nr_samplers, nr_sampler_views) *
-               sizeof(struct draw_sampler_static_state) +
-           nr_images * sizeof(struct draw_image_static_state));
+               sizeof(struct lp_sampler_static_state) +
+           nr_images * sizeof(struct lp_image_static_state));
 }
 
 
@@ -596,8 +290,8 @@ draw_gs_llvm_variant_key_size(unsigned nr_samplers,
 {
    return (sizeof(struct draw_gs_llvm_variant_key) +
            (MAX2(nr_samplers, nr_sampler_views) - 1) *
-               sizeof(struct draw_sampler_static_state) +
-           nr_images * sizeof(struct draw_sampler_static_state));
+               sizeof(struct lp_sampler_static_state) +
+           nr_images * sizeof(struct lp_sampler_static_state));
 }
 
 static inline size_t
@@ -607,8 +301,8 @@ draw_tcs_llvm_variant_key_size(unsigned nr_samplers,
 {
    return (sizeof(struct draw_tcs_llvm_variant_key) +
            (MAX2(nr_samplers, nr_sampler_views) - 1) *
-               sizeof(struct draw_sampler_static_state) +
-           nr_images * sizeof(struct draw_sampler_static_state));
+               sizeof(struct lp_sampler_static_state) +
+           nr_images * sizeof(struct lp_sampler_static_state));
 }
 
 static inline size_t
@@ -618,44 +312,44 @@ draw_tes_llvm_variant_key_size(unsigned nr_samplers,
 {
    return (sizeof(struct draw_tes_llvm_variant_key) +
            (MAX2(nr_samplers, nr_sampler_views) - 1) *
-               sizeof(struct draw_sampler_static_state) +
-           nr_images * sizeof(struct draw_sampler_static_state));
+               sizeof(struct lp_sampler_static_state) +
+           nr_images * sizeof(struct lp_sampler_static_state));
 }
 
-static inline struct draw_sampler_static_state *
+static inline struct lp_sampler_static_state *
 draw_llvm_variant_key_samplers(struct draw_llvm_variant_key *key)
 {
-   return (struct draw_sampler_static_state *)
+   return (struct lp_sampler_static_state *)
       &key->vertex_element[key->nr_vertex_elements];
 }
 
-static inline struct draw_image_static_state *
+static inline struct lp_image_static_state *
 draw_llvm_variant_key_images(struct draw_llvm_variant_key *key)
 {
-   struct draw_sampler_static_state *samplers = (struct draw_sampler_static_state *)
+   struct lp_sampler_static_state *samplers = (struct lp_sampler_static_state *)
       (&key->vertex_element[key->nr_vertex_elements]);
-   return (struct draw_image_static_state *)
+   return (struct lp_image_static_state *)
       &samplers[MAX2(key->nr_samplers, key->nr_sampler_views)];
 }
 
-static inline struct draw_image_static_state *
+static inline struct lp_image_static_state *
 draw_gs_llvm_variant_key_images(struct draw_gs_llvm_variant_key *key)
 {
-   return (struct draw_image_static_state *)
+   return (struct lp_image_static_state *)
       &key->samplers[MAX2(key->nr_samplers, key->nr_sampler_views)];
 }
 
-static inline struct draw_image_static_state *
+static inline struct lp_image_static_state *
 draw_tcs_llvm_variant_key_images(struct draw_tcs_llvm_variant_key *key)
 {
-   return (struct draw_image_static_state *)
+   return (struct lp_image_static_state *)
       &key->samplers[MAX2(key->nr_samplers, key->nr_sampler_views)];
 }
 
-static inline struct draw_image_static_state *
+static inline struct lp_image_static_state *
 draw_tes_llvm_variant_key_images(struct draw_tes_llvm_variant_key *key)
 {
-   return (struct draw_image_static_state *)
+   return (struct lp_image_static_state *)
       &key->samplers[MAX2(key->nr_samplers, key->nr_sampler_views)];
 }
 
@@ -691,6 +385,9 @@ struct draw_llvm_variant
    LLVMTypeRef context_type;
    LLVMTypeRef context_ptr_type;
 
+   LLVMTypeRef resources_type;
+   LLVMTypeRef resources_ptr_type;
+   
    LLVMTypeRef buffer_type;
    LLVMTypeRef buffer_ptr_type;
 
@@ -701,6 +398,7 @@ struct draw_llvm_variant
    LLVMTypeRef vertex_header_ptr_type;
 
    LLVMValueRef function;
+   char *function_name;
    draw_jit_vert_func jit_func;
 
    struct llvm_vertex_shader *shader;
@@ -722,6 +420,9 @@ struct draw_gs_llvm_variant
    LLVMTypeRef context_type;
    LLVMTypeRef context_ptr_type;
 
+   LLVMTypeRef resources_type;
+   LLVMTypeRef resources_ptr_type;
+
    LLVMTypeRef vertex_header_type;
    LLVMTypeRef vertex_header_ptr_type;
 
@@ -731,6 +432,7 @@ struct draw_gs_llvm_variant
    LLVMValueRef io_ptr;
    LLVMValueRef num_prims;
    LLVMValueRef function;
+   char *function_name;
    draw_gs_jit_func jit_func;
 
    struct llvm_geometry_shader *shader;
@@ -748,8 +450,8 @@ struct draw_tcs_llvm_variant
    struct gallivm_state *gallivm;
 
    /* LLVM JIT builder types */
-   LLVMTypeRef context_type;
-   LLVMTypeRef context_ptr_type;
+   LLVMTypeRef resources_type;
+   LLVMTypeRef resources_ptr_type;
    LLVMTypeRef input_array_type;
    LLVMTypeRef output_array_type;
 
@@ -757,6 +459,7 @@ struct draw_tcs_llvm_variant
    /* LLVMValueRef io_ptr; */
    LLVMValueRef num_prims;
    LLVMValueRef function;
+   char *function_name;
    draw_tcs_jit_func jit_func;
 
    struct llvm_tess_ctrl_shader *shader;
@@ -774,8 +477,8 @@ struct draw_tes_llvm_variant
    struct gallivm_state *gallivm;
 
    /* LLVM JIT builder types */
-   LLVMTypeRef context_type;
-   LLVMTypeRef context_ptr_type;
+   LLVMTypeRef resources_type;
+   LLVMTypeRef resources_ptr_type;
    LLVMTypeRef vertex_header_ptr_type;
    LLVMTypeRef input_array_type;
    LLVMTypeRef patch_input_array_type;
@@ -787,6 +490,7 @@ struct draw_tes_llvm_variant
    LLVMValueRef io_ptr;
    LLVMValueRef num_prims;
    LLVMValueRef function;
+   char *function_name;
    draw_tes_jit_func jit_func;
 
    struct llvm_tess_eval_shader *shader;
@@ -838,13 +542,12 @@ struct llvm_tess_eval_shader {
 struct draw_llvm {
    struct draw_context *draw;
 
-   LLVMContextRef context;
-   boolean context_owned;
+   lp_context_ref context;
 
-   struct draw_jit_context jit_context;
+   struct draw_vs_jit_context vs_jit_context;
    struct draw_gs_jit_context gs_jit_context;
-   struct draw_tcs_jit_context tcs_jit_context;
-   struct draw_tes_jit_context tes_jit_context;
+
+   struct lp_jit_resources jit_resources[DRAW_MAX_SHADER_STAGE];
 
    struct draw_llvm_variant_list_item vs_variants_list;
    int nr_variants;
@@ -885,7 +588,7 @@ llvm_tess_eval_shader(struct draw_tess_eval_shader *tes)
 }
 
 struct draw_llvm *
-draw_llvm_create(struct draw_context *draw, LLVMContextRef llvm_context);
+draw_llvm_create(struct draw_context *draw, lp_context_ref *llvm_context);
 
 void
 draw_llvm_destroy(struct draw_llvm *llvm);
@@ -947,21 +650,13 @@ draw_tes_llvm_make_variant_key(struct draw_llvm *llvm, char *store);
 void
 draw_tes_llvm_dump_variant_key(struct draw_tes_llvm_variant_key *key);
 
-struct lp_build_sampler_soa *
-draw_llvm_sampler_soa_create(const struct draw_sampler_static_state *static_state,
-                             unsigned nr_samplers);
-
-struct lp_build_image_soa *
-draw_llvm_image_soa_create(const struct draw_image_static_state *static_state,
-                           unsigned nr_images);
-
 void
 draw_llvm_set_sampler_state(struct draw_context *draw,
-                            enum pipe_shader_type shader_stage);
+                            mesa_shader_stage shader_stage);
 
 void
 draw_llvm_set_mapped_texture(struct draw_context *draw,
-                             enum pipe_shader_type shader_stage,
+                             mesa_shader_stage shader_stage,
                              unsigned sview_idx,
                              uint32_t width, uint32_t height, uint32_t depth,
                              uint32_t first_level, uint32_t last_level,
@@ -974,7 +669,7 @@ draw_llvm_set_mapped_texture(struct draw_context *draw,
 
 void
 draw_llvm_set_mapped_image(struct draw_context *draw,
-                           enum pipe_shader_type shader_stage,
+                           mesa_shader_stage shader_stage,
                            unsigned idx,
                            uint32_t width, uint32_t height, uint32_t depth,
                            const void *base_ptr,
@@ -982,4 +677,17 @@ draw_llvm_set_mapped_image(struct draw_context *draw,
                            uint32_t img_stride,
                            uint32_t num_samples,
                            uint32_t sample_stride);
+
+void
+draw_store_aos_array(struct gallivm_state *gallivm,
+                     struct lp_type soa_type,
+                     LLVMTypeRef io_type,
+                     LLVMValueRef io_ptr,
+                     LLVMValueRef *indices,
+                     LLVMValueRef* aos,
+                     int attrib,
+                     LLVMValueRef clipmask,
+                     bool need_edgeflag,
+                     bool per_prim);
+
 #endif
