@@ -105,29 +105,6 @@ trace_screen_get_device_vendor(struct pipe_screen *_screen)
 }
 
 
-static const struct nir_shader_compiler_options *
-trace_screen_get_compiler_options(struct pipe_screen *_screen,
-                                  enum pipe_shader_type shader)
-{
-   struct trace_screen *tr_scr = trace_screen(_screen);
-   struct pipe_screen *screen = tr_scr->screen;
-   const struct nir_shader_compiler_options *result;
-
-   trace_dump_call_begin("pipe_screen", "get_compiler_options");
-
-   trace_dump_arg(ptr, screen);
-   trace_dump_arg_enum(pipe_shader_type, shader);
-
-   result = screen->get_compiler_options(screen, shader);
-
-   trace_dump_ret(ptr, result);
-
-   trace_dump_call_end();
-
-   return result;
-}
-
-
 static struct disk_cache *
 trace_screen_get_disk_shader_cache(struct pipe_screen *_screen)
 {
@@ -832,27 +809,6 @@ trace_screen_resource_get_param(struct pipe_screen *_screen,
    return result;
 }
 
-static void
-trace_screen_resource_get_info(struct pipe_screen *_screen,
-                               struct pipe_resource *resource,
-                               unsigned *stride,
-                               unsigned *offset)
-{
-   struct trace_screen *tr_screen = trace_screen(_screen);
-   struct pipe_screen *screen = tr_screen->screen;
-
-   trace_dump_call_begin("pipe_screen", "resource_get_info");
-   trace_dump_arg(ptr, screen);
-   trace_dump_arg(ptr, resource);
-
-   screen->resource_get_info(screen, resource, stride, offset);
-
-   trace_dump_arg(uint, *stride);
-   trace_dump_arg(uint, *offset);
-
-   trace_dump_call_end();
-}
-
 static uint64_t
 trace_screen_resource_get_address(struct pipe_screen *_screen,
                                   struct pipe_resource *resource)
@@ -959,6 +915,25 @@ trace_screen_fence_reference(struct pipe_screen *_screen,
 }
 
 
+static struct pipe_fence_handle *
+trace_screen_semaphore_create(struct pipe_screen *_screen)
+{
+   struct trace_screen *tr_scr = trace_screen(_screen);
+   struct pipe_screen *screen = tr_scr->screen;
+   struct pipe_fence_handle *res;
+
+   trace_dump_call_begin("pipe_screen", "fence_reference");
+   trace_dump_arg(ptr, screen);
+
+   res = screen->semaphore_create(screen);
+
+   trace_dump_ret(ptr, res);
+   trace_dump_call_end();
+
+   return res;
+}
+
+
 static int
 trace_screen_fence_get_fd(struct pipe_screen *_screen,
                           struct pipe_fence_handle *fence)
@@ -1094,12 +1069,12 @@ trace_screen_get_timestamp(struct pipe_screen *_screen)
    return result;
 }
 
-static char *
+static void
 trace_screen_finalize_nir(struct pipe_screen *_screen, struct nir_shader *nir)
 {
    struct pipe_screen *screen = trace_screen(_screen)->screen;
 
-   return screen->finalize_nir(screen, nir);
+   screen->finalize_nir(screen, nir);
 }
 
 static void
@@ -1499,7 +1474,6 @@ trace_screen_create(struct pipe_screen *screen)
    tr_scr->base.get_name = trace_screen_get_name;
    tr_scr->base.get_vendor = trace_screen_get_vendor;
    tr_scr->base.get_device_vendor = trace_screen_get_device_vendor;
-   SCR_INIT(get_compiler_options);
    SCR_INIT(get_disk_shader_cache);
    SCR_INIT(get_video_param);
    tr_scr->base.is_format_supported = trace_screen_is_format_supported;
@@ -1526,12 +1500,12 @@ trace_screen_create(struct pipe_screen *screen)
    SCR_INIT(check_resource_capability);
    tr_scr->base.resource_get_handle = trace_screen_resource_get_handle;
    SCR_INIT(resource_get_param);
-   SCR_INIT(resource_get_info);
    SCR_INIT(resource_get_address);
    SCR_INIT(resource_from_memobj);
    SCR_INIT(resource_changed);
    tr_scr->base.resource_destroy = trace_screen_resource_destroy;
    tr_scr->base.fence_reference = trace_screen_fence_reference;
+   SCR_INIT(semaphore_create);
    SCR_INIT(fence_get_fd);
    SCR_INIT(create_fence_win32);
    tr_scr->base.fence_finish = trace_screen_fence_finish;
@@ -1571,6 +1545,7 @@ trace_screen_create(struct pipe_screen *screen)
    *(struct pipe_caps *)&tr_scr->base.caps = screen->caps;
    *(struct pipe_compute_caps *)&tr_scr->base.compute_caps = screen->compute_caps;
    memcpy((void *)tr_scr->base.shader_caps, screen->shader_caps, sizeof(screen->shader_caps));
+   memcpy((void *)tr_scr->base.nir_options, screen->nir_options, sizeof(screen->nir_options));
 
    return &tr_scr->base;
 
