@@ -77,6 +77,7 @@ nir_lower_pstipple_block(nir_block *block,
    tex->dest_type = nir_type_float32;
    tex->texture_index = state->stip_tex->data.binding;
    tex->sampler_index = state->stip_tex->data.binding;
+   tex->can_speculate = true;
    tex->src[0] = nir_tex_src_for_ssa(nir_tex_src_coord, texcoord);
    nir_def_init(&tex->instr, &tex->def, 4, 32);
 
@@ -93,7 +94,7 @@ nir_lower_pstipple_block(nir_block *block,
                              nir_imm_floatN_t(b, 0.0, tex->def.bit_size));
       break;
    default:
-      unreachable("Invalid Boolean type.");
+      UNREACHABLE("Invalid Boolean type.");
    }
 
    nir_discard_if(b, condition);
@@ -110,7 +111,7 @@ nir_lower_pstipple_impl(nir_function_impl *impl,
    nir_lower_pstipple_block(start, state);
 }
 
-void
+bool
 nir_lower_pstipple_fs(struct nir_shader *shader,
                       unsigned *samplerUnitOut,
                       unsigned fixedUnit,
@@ -126,8 +127,10 @@ nir_lower_pstipple_fs(struct nir_shader *shader,
    assert(bool_type == nir_type_bool1 ||
           bool_type == nir_type_bool32);
 
-   if (shader->info.stage != MESA_SHADER_FRAGMENT)
-      return;
+   if (shader->info.stage != MESA_SHADER_FRAGMENT) {
+      nir_shader_preserve_all_metadata(shader);
+      return false;
+   }
 
    int binding = 0;
    nir_foreach_uniform_variable(var, shader) {
@@ -148,10 +151,13 @@ nir_lower_pstipple_fs(struct nir_shader *shader,
    BITSET_SET(shader->info.samplers_used, binding);
    state.stip_tex = tex_var;
 
+   bool progress = false;
    nir_foreach_function_impl(impl, shader) {
       nir_lower_pstipple_impl(impl, &state);
+      progress |= nir_progress(true, impl, nir_metadata_none);
    }
    *samplerUnitOut = binding;
+   return progress;
 }
 
 typedef struct {
@@ -336,7 +342,7 @@ nir_lower_aapoint_impl(nir_function_impl *impl, lower_aapoint *state,
       comp = nir_slt(b, chan_val_one, dist);
       break;
    default:
-      unreachable("Invalid Boolean type.");
+      UNREACHABLE("Invalid Boolean type.");
    }
 
    nir_discard_if(b, comp);
@@ -392,7 +398,7 @@ nir_lower_aapoint_impl(nir_function_impl *impl, lower_aapoint *state,
       break;
    }
    default:
-      unreachable("Invalid Boolean type.");
+      UNREACHABLE("Invalid Boolean type.");
    }
 
    nir_foreach_block(block, impl) {
