@@ -31,9 +31,9 @@
 
 #include "pipe_loader_priv.h"
 
+#include "util/detect_os.h"
 #include "util/os_file.h"
 #include "util/u_memory.h"
-#include "util/u_dl.h"
 #include "sw/dri/dri_sw_winsys.h"
 #include "sw/kms-dri/kms_dri_sw_winsys.h"
 #include "sw/null/null_sw_winsys.h"
@@ -48,9 +48,6 @@
 struct pipe_loader_sw_device {
    struct pipe_loader_device base;
    const struct sw_driver_descriptor *dd;
-#ifndef GALLIUM_STATIC_TARGETS
-   struct util_dl_library *lib;
-#endif
    struct sw_winsys *ws;
    int fd;
 };
@@ -62,59 +59,57 @@ static const struct pipe_loader_ops pipe_loader_sw_ops;
 static const struct pipe_loader_ops pipe_loader_vk_ops;
 #endif
 
-#ifdef GALLIUM_STATIC_TARGETS
 static const struct sw_driver_descriptor driver_descriptors = {
    .create_screen = sw_screen_create_vk,
    .winsys = {
 #ifdef HAVE_DRI
       {
          .name = "dri",
-         .create_winsys = dri_create_sw_winsys,
+         .create_winsys_dri = dri_create_sw_winsys,
       },
 #endif
 #ifdef HAVE_DRISW_KMS
       {
          .name = "kms_dri",
-         .create_winsys = kms_dri_create_winsys,
+         .create_winsys_kms_dri = kms_dri_create_winsys,
       },
 #endif
-#ifndef __ANDROID__
+#if !DETECT_OS_ANDROID
       {
          .name = "null",
          .create_winsys = null_sw_create,
       },
       {
          .name = "wrapped",
-         .create_winsys = wrapper_sw_winsys_wrap_pipe_screen,
+         .create_winsys_wrapped = wrapper_sw_winsys_wrap_pipe_screen,
       },
 #endif
       { 0 },
    }
 };
-#endif
 
-#if defined(GALLIUM_STATIC_TARGETS) && defined(HAVE_ZINK) && defined(HAVE_DRI)
+#if defined(HAVE_ZINK) && defined(HAVE_DRI)
 static const struct sw_driver_descriptor kopper_driver_descriptors = {
    .create_screen = sw_screen_create_zink,
    .winsys = {
       {
          .name = "dri",
-         .create_winsys = dri_create_sw_winsys,
+         .create_winsys_dri = dri_create_sw_winsys,
       },
 #ifdef HAVE_DRISW_KMS
       {
          .name = "kms_dri",
-         .create_winsys = kms_dri_create_winsys,
+         .create_winsys_kms_dri = kms_dri_create_winsys,
       },
 #endif
-#ifndef __ANDROID__
+#if !DETECT_OS_ANDROID
       {
          .name = "null",
          .create_winsys = null_sw_create,
       },
       {
          .name = "wrapped",
-         .create_winsys = wrapper_sw_winsys_wrap_pipe_screen,
+         .create_winsys_wrapped = wrapper_sw_winsys_wrap_pipe_screen,
       },
 #endif
       { 0 },
@@ -130,29 +125,9 @@ pipe_loader_sw_probe_init_common(struct pipe_loader_sw_device *sdev)
    sdev->base.ops = &pipe_loader_sw_ops;
    sdev->fd = -1;
 
-#ifdef GALLIUM_STATIC_TARGETS
    sdev->dd = &driver_descriptors;
    if (!sdev->dd)
       return false;
-#else
-   const char *search_dir = getenv("GALLIUM_PIPE_SEARCH_DIR");
-   if (search_dir == NULL)
-      search_dir = PIPE_SEARCH_DIR;
-
-   sdev->lib = pipe_loader_find_module("swrast", search_dir);
-   if (!sdev->lib)
-      return false;
-
-   sdev->dd = (const struct sw_driver_descriptor *)
-      util_dl_get_proc_address(sdev->lib, "swrast_driver_descriptor");
-
-   if (!sdev->dd){
-      util_dl_close(sdev->lib);
-      sdev->lib = NULL;
-      return false;
-   }
-#endif
-
    return true;
 }
 
@@ -165,42 +140,13 @@ pipe_loader_vk_probe_init_common(struct pipe_loader_sw_device *sdev)
    sdev->base.ops = &pipe_loader_vk_ops;
    sdev->fd = -1;
 
-#ifdef GALLIUM_STATIC_TARGETS
    sdev->dd = &kopper_driver_descriptors;
    if (!sdev->dd)
       return false;
-#else
-   const char *search_dir = getenv("GALLIUM_PIPE_SEARCH_DIR");
-   if (search_dir == NULL)
-      search_dir = PIPE_SEARCH_DIR;
-
-   sdev->lib = pipe_loader_find_module("swrast", search_dir);
-   if (!sdev->lib)
-      return false;
-
-   sdev->dd = (const struct sw_driver_descriptor *)
-      util_dl_get_proc_address(sdev->lib, "swrast_driver_descriptor");
-
-   if (!sdev->dd){
-      util_dl_close(sdev->lib);
-      sdev->lib = NULL;
-      return false;
-   }
-#endif
-
    return true;
 }
 #endif
 
-static void
-pipe_loader_sw_probe_teardown_common(struct pipe_loader_sw_device *sdev)
-{
-#ifndef GALLIUM_STATIC_TARGETS
-   if (sdev->lib)
-      util_dl_close(sdev->lib);
-#endif
-}
-
 #ifdef HAVE_DRI
 bool
 pipe_loader_sw_probe_dri(struct pipe_loader_device **devs, const struct drisw_loader_funcs *drisw_lf)
@@ -216,7 +162,7 @@ pipe_loader_sw_probe_dri(struct pipe_loader_device **devs, const struct drisw_lo
 
    for (i = 0; sdev->dd->winsys[i].name; i++) {
       if (strcmp(sdev->dd->winsys[i].name, "dri") == 0) {
-         sdev->ws = sdev->dd->winsys[i].create_winsys(drisw_lf);
+         sdev->ws = sdev->dd->winsys[i].create_winsys_dri(drisw_lf);
          break;
       }
    }
@@ -227,13 +173,12 @@ pipe_loader_sw_probe_dri(struct pipe_loader_device **devs, const struct drisw_lo
    return true;
 
 fail:
-   pipe_loader_sw_probe_teardown_common(sdev);
    FREE(sdev);
    return false;
 }
 #ifdef HAVE_ZINK
 bool
-pipe_loader_vk_probe_dri(struct pipe_loader_device **devs, const struct drisw_loader_funcs *drisw_lf)
+pipe_loader_vk_probe_dri(struct pipe_loader_device **devs)
 {
    struct pipe_loader_sw_device *sdev = CALLOC_STRUCT(pipe_loader_sw_device);
    int i;
@@ -246,7 +191,7 @@ pipe_loader_vk_probe_dri(struct pipe_loader_device **devs, const struct drisw_lo
 
    for (i = 0; sdev->dd->winsys[i].name; i++) {
       if (strcmp(sdev->dd->winsys[i].name, "dri") == 0) {
-         sdev->ws = sdev->dd->winsys[i].create_winsys(drisw_lf);
+         sdev->ws = sdev->dd->winsys[i].create_winsys_dri(NULL);
          break;
       }
    }
@@ -257,7 +202,6 @@ pipe_loader_vk_probe_dri(struct pipe_loader_device **devs, const struct drisw_lo
    return true;
 
 fail:
-   pipe_loader_sw_probe_teardown_common(sdev);
    FREE(sdev);
    return false;
 }
@@ -282,7 +226,7 @@ pipe_loader_sw_probe_kms(struct pipe_loader_device **devs, int fd)
 
    for (i = 0; sdev->dd->winsys[i].name; i++) {
       if (strcmp(sdev->dd->winsys[i].name, "kms_dri") == 0) {
-         sdev->ws = sdev->dd->winsys[i].create_winsys(sdev->fd);
+         sdev->ws = sdev->dd->winsys[i].create_winsys_kms_dri(sdev->fd);
          break;
       }
    }
@@ -293,7 +237,6 @@ pipe_loader_sw_probe_kms(struct pipe_loader_device **devs, int fd)
    return true;
 
 fail:
-   pipe_loader_sw_probe_teardown_common(sdev);
    if (sdev->fd != -1)
       close(sdev->fd);
    FREE(sdev);
@@ -326,7 +269,6 @@ pipe_loader_sw_probe_null(struct pipe_loader_device **devs)
    return true;
 
 fail:
-   pipe_loader_sw_probe_teardown_common(sdev);
    FREE(sdev);
    return false;
 }
@@ -345,7 +287,7 @@ pipe_loader_sw_probe(struct pipe_loader_device **devs, int ndev)
    return i;
 }
 
-boolean
+bool
 pipe_loader_sw_probe_wrapped(struct pipe_loader_device **dev,
                              struct pipe_screen *screen)
 {
@@ -360,7 +302,7 @@ pipe_loader_sw_probe_wrapped(struct pipe_loader_device **dev,
 
    for (i = 0; sdev->dd->winsys[i].name; i++) {
       if (strcmp(sdev->dd->winsys[i].name, "wrapped") == 0) {
-         sdev->ws = sdev->dd->winsys[i].create_winsys(screen);
+         sdev->ws = sdev->dd->winsys[i].create_winsys_wrapped(screen);
          break;
       }
    }
@@ -371,7 +313,6 @@ pipe_loader_sw_probe_wrapped(struct pipe_loader_device **dev,
    return true;
 
 fail:
-   pipe_loader_sw_probe_teardown_common(sdev);
    FREE(sdev);
    return false;
 }
@@ -382,10 +323,7 @@ pipe_loader_sw_release(struct pipe_loader_device **dev)
    UNUSED struct pipe_loader_sw_device *sdev =
       pipe_loader_sw_device(*dev);
 
-#ifndef GALLIUM_STATIC_TARGETS
-   if (sdev->lib)
-      util_dl_close(sdev->lib);
-#endif
+   sdev->ws->destroy(sdev->ws);
 
 #ifdef HAVE_DRISW_KMS
    if (sdev->fd != -1)
@@ -404,7 +342,7 @@ pipe_loader_sw_get_driconf(struct pipe_loader_device *dev, unsigned *count)
 
 #if defined(HAVE_DRI) && defined(HAVE_ZINK)
 static const driOptionDescription zink_driconf[] = {
-      #include "zink/driinfo_zink.h"
+      #include "gallium/drivers/zink/driinfo_zink.h"
 };
 
 static const struct driOptionDescription *
@@ -423,8 +361,6 @@ pipe_loader_sw_create_screen(struct pipe_loader_device *dev,
    struct pipe_screen *screen;
 
    screen = sdev->dd->create_screen(sdev->ws, config, sw_vk);
-   if (!screen)
-      sdev->ws->destroy(sdev->ws);
 
    return screen ? debug_screen_wrap(screen) : NULL;
 }
