@@ -63,9 +63,9 @@ util_pstipple_update_stipple_texture(struct pipe_context *pipe,
                                      struct pipe_resource *tex,
                                      const uint32_t pattern[32])
 {
-   static const uint bit31 = 1u << 31;
+   static const unsigned bit31 = 1u << 31;
    struct pipe_transfer *transfer;
-   ubyte *data;
+   uint8_t *data;
    int i, j;
 
    /* map texture memory */
@@ -157,7 +157,6 @@ util_pstipple_create_sampler(struct pipe_context *pipe)
    templat.min_mip_filter = PIPE_TEX_MIPFILTER_NONE;
    templat.min_img_filter = PIPE_TEX_FILTER_NEAREST;
    templat.mag_img_filter = PIPE_TEX_FILTER_NEAREST;
-   templat.normalized_coords = 1;
    templat.min_lod = 0.0f;
    templat.max_lod = 0.0f;
 
@@ -175,14 +174,14 @@ util_pstipple_create_sampler(struct pipe_context *pipe)
 struct pstip_transform_context {
    struct tgsi_transform_context base;
    struct tgsi_shader_info info;
-   uint tempsUsed;  /**< bitmask */
+   unsigned tempsUsed;  /**< bitmask */
    int wincoordInput;
    unsigned wincoordFile;
    int maxInput;
-   uint samplersUsed;  /**< bitfield of samplers used */
+   unsigned samplersUsed;  /**< bitfield of samplers used */
    int freeSampler;  /** an available sampler for the pstipple */
    int numImmed;
-   uint coordOrigin;
+   unsigned coordOrigin;
    unsigned fixedUnit;
    bool hasFixedUnit;
 };
@@ -202,7 +201,7 @@ pstip_transform_decl(struct tgsi_transform_context *ctx,
    /* XXX we can use tgsi_shader_info instead of some of this */
 
    if (decl->Declaration.File == TGSI_FILE_SAMPLER) {
-      uint i;
+      unsigned i;
       for (i = decl->Range.First; i <= decl->Range.Last; i++) {
          pctx->samplersUsed |= 1u << i;
       }
@@ -213,7 +212,7 @@ pstip_transform_decl(struct tgsi_transform_context *ctx,
          pctx->wincoordInput = (int) decl->Range.First;
    }
    else if (decl->Declaration.File == TGSI_FILE_TEMPORARY) {
-      uint i;
+      unsigned i;
       for (i = decl->Range.First; i <= decl->Range.Last; i++) {
          pctx->tempsUsed |= (1 << i);
       }
@@ -238,7 +237,7 @@ pstip_transform_immed(struct tgsi_transform_context *ctx,
  * Find the lowest zero bit in the given word, or -1 if bitfield is all ones.
  */
 static int
-free_bit(uint bitfield)
+free_bit(unsigned bitfield)
 {
    return ffs(~bitfield) - 1;
 }
@@ -355,7 +354,7 @@ pstip_transform_prolog(struct tgsi_transform_context *ctx)
    /* KILL_IF -texTemp;   # if -texTemp < 0, kill fragment */
    tgsi_transform_kill_inst(ctx,
                             TGSI_FILE_TEMPORARY, texTemp,
-                            TGSI_SWIZZLE_W, TRUE);
+                            TGSI_SWIZZLE_W, true);
 }
 
 
@@ -378,14 +377,9 @@ util_pstipple_create_fragment_shader(const struct tgsi_token *tokens,
                                      unsigned wincoordFile)
 {
    struct pstip_transform_context transform;
-   const uint newLen = tgsi_num_tokens(tokens) + NUM_NEW_TOKENS;
+   const unsigned newLen = tgsi_num_tokens(tokens) + NUM_NEW_TOKENS;
    struct tgsi_token *new_tokens;
 
-   new_tokens = tgsi_alloc_tokens(newLen);
-   if (!new_tokens) {
-      return NULL;
-   }
-
    /* Setup shader transformation info/context.
     */
    memset(&transform, 0, sizeof(transform));
@@ -404,9 +398,11 @@ util_pstipple_create_fragment_shader(const struct tgsi_token *tokens,
    transform.coordOrigin =
       transform.info.properties[TGSI_PROPERTY_FS_COORD_ORIGIN];
 
-   tgsi_transform_shader(tokens, new_tokens, newLen, &transform.base);
+   new_tokens = tgsi_transform_shader(tokens, newLen, &transform.base);
+   if (!new_tokens)
+      return NULL;
 
-#if 0 /* DEBUG */
+#if 0 /* debug */
    tgsi_dump(fs->tokens, 0);
    tgsi_dump(new_fs->tokens, 0);
 #endif
