@@ -30,7 +30,7 @@
 #define U_SIMPLE_SHADERS_H
 
 
-#include "pipe/p_compiler.h"
+#include "util/compiler.h"
 #include "pipe/p_shader_tokens.h"
 
 
@@ -46,16 +46,16 @@ extern "C" {
 
 extern void *
 util_make_vertex_passthrough_shader(struct pipe_context *pipe,
-                                    uint num_attribs,
+                                    unsigned num_attribs,
                                     const enum tgsi_semantic *semantic_names,
-                                    const uint *semantic_indexes,
+                                    const unsigned *semantic_indexes,
                                     bool window_space);
 
 extern void *
 util_make_vertex_passthrough_shader_with_so(struct pipe_context *pipe,
-                                    uint num_attribs,
+                                    unsigned num_attribs,
                                     const enum tgsi_semantic *semantic_names,
-                                    const uint *semantic_indexes,
+                                    const unsigned *semantic_indexes,
                                     bool window_space, bool layered,
                                     const struct pipe_stream_output_info *so);
 
@@ -68,24 +68,9 @@ util_make_layered_clear_helper_vertex_shader(struct pipe_context *pipe);
 extern void *
 util_make_layered_clear_geometry_shader(struct pipe_context *pipe);
 
-void *
-util_make_fragment_tex_shader_xrbias(struct pipe_context *pipe,
-                                     enum tgsi_texture_type tex_target);
-
-extern void *
-util_make_fragment_tex_shader_writemask(struct pipe_context *pipe,
-                                        enum tgsi_texture_type tex_target,
-                                        enum tgsi_interpolate_mode interp_mode,
-                                        unsigned writemask,
-                                        enum tgsi_return_type stype,
-                                        enum tgsi_return_type dtype,
-                                        bool load_level_zero,
-                                        bool use_txf);
-
 extern void *
 util_make_fragment_tex_shader(struct pipe_context *pipe,
                               enum tgsi_texture_type tex_target,
-                              enum tgsi_interpolate_mode interp_mode,
                               enum tgsi_return_type stype,
                               enum tgsi_return_type dtype,
                               bool load_level_zero,
@@ -100,7 +85,7 @@ extern void *
 util_make_fragment_passthrough_shader(struct pipe_context *pipe,
                                       int input_semantic,
                                       int input_interpolate,
-                                      boolean write_all_cbufs);
+                                      bool write_all_cbufs);
 
 
 extern void *
@@ -118,44 +103,43 @@ util_make_fs_blit_msaa_color(struct pipe_context *pipe,
                              enum tgsi_texture_type tgsi_tex,
                              enum tgsi_return_type stype,
                              enum tgsi_return_type dtype,
-                             bool sample_shading);
+                             bool sample_shading, bool has_txq);
 
 
 extern void *
 util_make_fs_blit_msaa_depth(struct pipe_context *pipe,
                              enum tgsi_texture_type tgsi_tex,
-                             bool sample_shading);
+                             bool sample_shading, bool has_txq);
 
 
 extern void *
 util_make_fs_blit_msaa_depthstencil(struct pipe_context *pipe,
                                     enum tgsi_texture_type tgsi_tex,
-                                    bool sample_shading);
+                                    bool sample_shading, bool has_txq);
 
 
 void *
 util_make_fs_blit_msaa_stencil(struct pipe_context *pipe,
                                enum tgsi_texture_type tgsi_tex,
-                               bool sample_shading);
+                               bool sample_shading, bool has_txq);
 
 
 void *
 util_make_fs_msaa_resolve(struct pipe_context *pipe,
                           enum tgsi_texture_type tgsi_tex, unsigned nr_samples,
-                          enum tgsi_return_type stype);
+                          bool has_txq);
 
 
 void *
 util_make_fs_msaa_resolve_bilinear(struct pipe_context *pipe,
                                    enum tgsi_texture_type tgsi_tex,
-                                   unsigned nr_samples,
-                                   enum tgsi_return_type stype);
+                                   unsigned nr_samples, bool has_txq);
 
 extern void *
 util_make_geometry_passthrough_shader(struct pipe_context *pipe,
-                                      uint num_attribs,
-                                      const ubyte *semantic_names,
-                                      const ubyte *semantic_indexes);
+                                      unsigned num_attribs,
+                                      const uint8_t *semantic_names,
+                                      const uint8_t *semantic_indexes);
 
 void *
 util_make_fs_pack_color_zs(struct pipe_context *pipe,
@@ -165,16 +149,19 @@ util_make_fs_pack_color_zs(struct pipe_context *pipe,
 
 extern void *
 util_make_tess_ctrl_passthrough_shader(struct pipe_context *pipe,
-                                       uint num_vs_outputs,
-                                       uint num_tes_inputs,
-                                       const ubyte *vs_semantic_names,
-                                       const ubyte *vs_semantic_indexes,
-                                       const ubyte *tes_semantic_names,
-                                       const ubyte *tes_semantic_indexes,
+                                       unsigned num_vs_outputs,
+                                       unsigned num_tes_inputs,
+                                       const uint8_t *vs_semantic_names,
+                                       const uint8_t *vs_semantic_indexes,
+                                       const uint8_t *tes_semantic_names,
+                                       const uint8_t *tes_semantic_indexes,
                                        const unsigned vertices_per_patch);
 
 void *
-util_make_fs_stencil_blit(struct pipe_context *pipe, bool msaa_src);
+util_make_fs_stencil_blit(struct pipe_context *pipe, bool msaa_src, bool has_txq);
+
+void *
+util_make_fs_clear_color(struct pipe_context *pipe, bool write_all_cbufs);
 
 #ifdef __cplusplus
 }
