@@ -55,7 +55,7 @@ struct cs_shader {
    unsigned num_samplers;
    nir_variable *samplers[3];
    nir_variable *image;
-   nir_def *params[11];
+   nir_def *params[17];
    nir_def *fone;
    nir_def *fzero;
 };
@@ -66,6 +66,12 @@ enum coords_flags {
    COORDS_CHROMA_OFFSET = 0x2,
 };
 
+enum color_conversion {
+   YUV2RGB,
+   RGB2YUV,
+   PRIMARIES,
+};
+
 static nir_def *cs_create_shader(struct vl_compositor *c, struct cs_shader *s)
 {
    /*
@@ -77,10 +83,10 @@ static nir_def *cs_create_shader(struct vl_compositor *c, struct cs_shader *s)
 
       layout (std140, binding = 0) uniform ubo
       {
-         vec4 csc_mat[3];      // params[0-2]
-         float luma_min;       // params[3].x
-         float luma_max;       // params[3].y
+         vec4 yuv2rgb[3];      // params[0-2]
          vec2 chroma_offset;   // params[3].zw
+         int trc_in;           // params[4].x
+         int trc_out;          // params[4].y
          ivec2 translate;      // params[4].zw
          vec2 sampler0_wh;     // params[5].xy
          vec2 subsample_ratio; // params[5].zw
@@ -88,6 +94,8 @@ static nir_def *cs_create_shader(struct vl_compositor *c, struct cs_shader *s)
          vec2 chroma_clamp;    // params[6].zw
          vec4 proj[3];         // params[7-8]
          vec4 chroma_proj[3];  // params[9-10]
+         vec4 rgb2yuv[3];      // params[11-13]
+         vec4 primaries[3];    // params[14-16]
       };
 
       void main()
@@ -100,8 +108,7 @@ static nir_def *cs_create_shader(struct vl_compositor *c, struct cs_shader *s)
       glsl_sampler_type(sampler_dim, /*is_shadow*/ false, s->array, GLSL_TYPE_FLOAT);
    const struct glsl_type *image_type =
       glsl_image_type(GLSL_SAMPLER_DIM_2D, /*is_array*/ false, GLSL_TYPE_FLOAT);
-   const nir_shader_compiler_options *options =
-      c->pipe->screen->get_compiler_options(c->pipe->screen, PIPE_SHADER_COMPUTE);
+   const nir_shader_compiler_options *options = c->pipe->screen->nir_options[MESA_SHADER_COMPUTE];
 
    s->b = nir_builder_init_simple_shader(MESA_SHADER_COMPUTE, options, "vl:%s", s->name);
    nir_builder *b = &s->b;
@@ -189,19 +196,6 @@ static inline nir_def *cs_proj(struct cs_shader *s, nir_def *src, unsigned flags
    return nir_vec3(b, x, y, s->fzero);
 }
 
-static inline nir_def *cs_luma_key(struct cs_shader *s, nir_def *src)
-{
-   /*
-      bool luma_min = params[3].x >= src;
-      bool luma_max = params[3].y < src;
-      return float(luma_min || luma_max);
-   */
-   nir_builder *b = &s->b;
-   nir_def *luma_min = nir_fge(b, nir_channel(b, s->params[3], 0), src);
-   nir_def *luma_max = nir_flt(b, nir_channel(b, s->params[3], 1), src);
-   return nir_b2f32(b, nir_ior(b, luma_min, luma_max));
-}
-
 static inline nir_def *cs_chroma_offset(struct cs_shader *s, nir_def *src, unsigned flags)
 {
    /*
@@ -247,13 +241,168 @@ static inline nir_def *cs_normalize(struct cs_shader *s, nir_def *src, unsigned
    return nir_fdiv(b, src, div);
 }
 
-static inline nir_def *cs_color_space_conversion(struct cs_shader *s, nir_def *src, unsigned comp)
+static nir_def *cs_color_conversion(struct cs_shader *s, nir_def *src,
+                                    unsigned comp, enum color_conversion conversion)
 {
    /*
-      return dot(src, params[comp]);
+      return dot(src, params[idx + comp]);
    */
    nir_builder *b = &s->b;
-   return nir_fdot4(b, src, s->params[comp]);
+   unsigned idx = 0;
+   if (conversion == RGB2YUV)
+      idx = 11;
+   else if (conversion == PRIMARIES)
+      idx = 14;
+   return nir_fdot4(b, src, s->params[idx + comp]);
+}
+
+#define SDR_WHITE (203.0)
+#define PQ_C1     (3424.0 / 4096.0)
+#define PQ_C2     (2413.0 / 4096.0 * 32.0)
+#define PQ_C3     (2392.0 / 4096.0 * 32.0)
+#define PQ_M1     (2610.0 / 4096.0 * 0.25)
+#define PQ_M2     (2523.0 / 4096.0 * 128.0)
+
+static void cs_trc_to_linear(struct cs_shader *s, nir_def *src, nir_variable **out, unsigned trc)
+{
+   nir_builder *b = &s->b;
+
+   for (unsigned i = 0; i < 3; ++i) {
+      nir_def *c = nir_fmax(b, nir_channel(b, src, i), s->fzero);
+      switch (trc) {
+      case PIPE_VIDEO_VPP_TRC_GAMMA22:
+         c = nir_fpow_imm(b, c, 2.2f);
+         break;
+      case PIPE_VIDEO_VPP_TRC_GAMMA28:
+         c = nir_fpow_imm(b, c, 2.8f);
+         break;
+      case PIPE_VIDEO_VPP_TRC_LINEAR:
+         break;
+      case PIPE_VIDEO_VPP_TRC_IEC61966_2_1:
+         /* sRGB */
+         nir_push_if(b, nir_fgt_imm(b, c, 0.04045f));
+            nir_def *p1 = nir_fadd_imm(b, c, 0.055f);
+            p1 = nir_fpow_imm(b, nir_fdiv_imm(b, p1, 1.055f), 2.4f);
+         nir_push_else(b, NULL);
+            nir_def *p2 = nir_fdiv_imm(b, c, 12.92f);
+         nir_pop_if(b, NULL);
+         c = nir_if_phi(b, p1, p2);
+         break;
+      case PIPE_VIDEO_VPP_TRC_SMPTE2084: {
+         /* PQ */
+         c = nir_fpow_imm(b, c, 1.0f / PQ_M2);
+         nir_def *d1 = nir_fmax(b, nir_fadd_imm(b, c, -PQ_C1), s->fzero);
+         nir_def *d2 = nir_fsub_imm(b, PQ_C2, nir_fmul_imm(b, c, PQ_C3));
+         c = nir_fmul_imm(b, nir_fpow_imm(b, nir_fdiv(b, d1, d2), 1.0f / PQ_M1), 10000.0f / SDR_WHITE);
+         break;
+      }
+      case PIPE_VIDEO_VPP_TRC_BT709:
+      case PIPE_VIDEO_VPP_TRC_SMPTE170M:
+      case PIPE_VIDEO_VPP_TRC_SMPTE240M:
+      case PIPE_VIDEO_VPP_TRC_IEC61966_2_4:
+      case PIPE_VIDEO_VPP_TRC_BT1361_ECG:
+      case PIPE_VIDEO_VPP_TRC_BT2020_10:
+      case PIPE_VIDEO_VPP_TRC_BT2020_12:
+      default:
+         /* BT.1886 Lb=0 Lw=1 */
+         c = nir_fpow_imm(b, c, 2.4f);
+         break;
+      }
+      nir_store_var(b, out[i], c, 0x1);
+   }
+}
+
+static void cs_trc_from_linear(struct cs_shader *s, nir_def *src, nir_variable **out, unsigned trc)
+{
+   nir_builder *b = &s->b;
+
+   for (unsigned i = 0; i < 3; ++i) {
+      nir_def *c = nir_fmax(b, nir_channel(b, src, i), s->fzero);
+      switch (trc) {
+      case PIPE_VIDEO_VPP_TRC_GAMMA22:
+         c = nir_fpow_imm(b, c, 1.0f / 2.2f);
+         break;
+      case PIPE_VIDEO_VPP_TRC_GAMMA28:
+         c = nir_fpow_imm(b, c, 1.0f / 2.8f);
+         break;
+      case PIPE_VIDEO_VPP_TRC_LINEAR:
+         break;
+      case PIPE_VIDEO_VPP_TRC_IEC61966_2_1:
+         /* sRGB */
+         nir_push_if(b, nir_fge_imm(b, c, 0.0031308f));
+            nir_def *p1 = nir_fpow_imm(b, c, 1.0f / 2.4f);
+            p1 = nir_fadd_imm(b, nir_fmul_imm(b, p1, 1.055f), -0.055f);
+         nir_push_else(b, NULL);
+            nir_def *p2 = nir_fmul_imm(b, c, 12.92f);
+         nir_pop_if(b, NULL);
+         c = nir_if_phi(b, p1, p2);
+         break;
+      case PIPE_VIDEO_VPP_TRC_SMPTE2084: {
+         /* PQ */
+         c = nir_fpow_imm(b, nir_fmul_imm(b, c, SDR_WHITE / 10000.0f), PQ_M1);
+         nir_def *d1 = nir_fadd_imm(b, nir_fmul_imm(b, c, PQ_C2), PQ_C1);
+         nir_def *d2 = nir_fadd_imm(b, nir_fmul_imm(b, c, PQ_C3), 1.0f);
+         c = nir_fpow_imm(b, nir_fdiv(b, d1, d2), PQ_M2);
+         break;
+      }
+      case PIPE_VIDEO_VPP_TRC_BT709:
+      case PIPE_VIDEO_VPP_TRC_SMPTE170M:
+      case PIPE_VIDEO_VPP_TRC_SMPTE240M:
+      case PIPE_VIDEO_VPP_TRC_IEC61966_2_4:
+      case PIPE_VIDEO_VPP_TRC_BT1361_ECG:
+      case PIPE_VIDEO_VPP_TRC_BT2020_10:
+      case PIPE_VIDEO_VPP_TRC_BT2020_12:
+      default:
+         /* BT.1886 Lb=0 Lw=1 */
+         c = nir_fpow_imm(b, c, 1.0f / 2.4f);
+         break;
+      }
+      nir_store_var(b, out[i], c, 0x1);
+   }
+}
+
+static nir_def *cs_trc_apply(struct cs_shader *s, nir_def *src, bool in,
+                             void (*trc_func)(struct cs_shader *, nir_def *, nir_variable **, unsigned))
+{
+   nir_builder *b = &s->b;
+   nir_def *trc = nir_channels(b, s->params[4], in ? 0x1 : 0x2);
+   nir_variable *col[3] = {
+      nir_local_variable_create(b->impl, glsl_float_type(), "col0"),
+      nir_local_variable_create(b->impl, glsl_float_type(), "col1"),
+      nir_local_variable_create(b->impl, glsl_float_type(), "col2"),
+   };
+
+   enum pipe_video_vpp_transfer_characteristic trcs[] = {
+      PIPE_VIDEO_VPP_TRC_GAMMA22,
+      PIPE_VIDEO_VPP_TRC_GAMMA28,
+      PIPE_VIDEO_VPP_TRC_LINEAR,
+      PIPE_VIDEO_VPP_TRC_IEC61966_2_1,
+      PIPE_VIDEO_VPP_TRC_SMPTE2084,
+   };
+
+   for (unsigned i = 0; i < ARRAY_SIZE(trcs); i++) {
+      nir_push_if(b, nir_ieq_imm(b, trc, trcs[i]));
+         trc_func(s, src, col, trcs[i]);
+      nir_push_else(b, NULL);
+   }
+   trc_func(s, src, col, PIPE_VIDEO_VPP_TRC_BT709);
+   for (unsigned i = 0; i < ARRAY_SIZE(trcs); i++)
+      nir_pop_if(b, NULL);
+
+   return nir_vec4(b, nir_load_var(b, col[0]), nir_load_var(b, col[1]),
+                      nir_load_var(b, col[2]), s->fone);
+}
+
+static nir_def *cs_prim_trc_conversion(struct cs_shader *s, nir_def *src)
+{
+   nir_def *col[3];
+   nir_def *color = cs_trc_apply(s, src, true, cs_trc_to_linear);
+
+   for (unsigned i = 0; i < 3; i++)
+      col[i] = cs_color_conversion(s, color, i, PRIMARIES);
+
+   color = nir_vec4(&s->b, col[0], col[1], col[2], s->fone);
+   return cs_trc_apply(s, color, false, cs_trc_from_linear);
 }
 
 static inline nir_def *cs_fetch_texel(struct cs_shader *s, nir_def *coords, unsigned sampler)
@@ -264,7 +413,8 @@ static inline nir_def *cs_fetch_texel(struct cs_shader *s, nir_def *coords, unsi
    nir_builder *b = &s->b;
    nir_deref_instr *tex_deref = nir_build_deref_var(b, s->samplers[sampler]);
    nir_component_mask_t mask = s->array ? 0x7 : 0x3;
-   return nir_tex_deref(b, tex_deref, tex_deref, nir_channels(b, coords, mask));
+   return nir_tex(b, nir_channels(b, coords, mask),
+                  .texture_deref = tex_deref, .sampler_deref = tex_deref);
 }
 
 static inline nir_def *cs_image_load(struct cs_shader *s, nir_def *pos)
@@ -331,13 +481,12 @@ static void *create_video_buffer_shader(struct vl_compositor *c)
    for (unsigned i = 0; i < 3; ++i)
       col[i] = cs_fetch_texel(&s, pos[MIN2(i, 1)], i);
 
-   nir_def *alpha = cs_luma_key(&s, col[2]);
-
    nir_def *color = nir_vec4(b, col[0], col[1], col[2], s.fone);
    for (unsigned i = 0; i < 3; ++i)
-      col[i] = cs_color_space_conversion(&s, color, i);
+      col[i] = cs_color_conversion(&s, color, i, YUV2RGB);
 
-   color = nir_vec4(b, col[0], col[1], col[2], alpha);
+   color = nir_vec4(b, col[0], col[1], col[2], s.fone);
+   color = cs_prim_trc_conversion(&s, color);
    cs_image_store(&s, cs_translate(&s, ipos), color);
 
    return cs_create_shader_state(c, &s);
@@ -352,19 +501,37 @@ static void *create_yuv_progressive_shader(struct vl_compositor *c, enum vl_comp
    nir_builder *b = &s.b;
 
    nir_def *ipos = cs_create_shader(c, &s);
-   nir_def *pos = cs_tex_coords(&s, ipos, plane == VL_COMPOSITOR_PLANE_Y ? COORDS_LUMA : COORDS_CHROMA);
+   nir_def *pos_luma = cs_tex_coords(&s, ipos, COORDS_LUMA);
+   nir_def *pos_chroma = cs_tex_coords(&s, ipos, COORDS_CHROMA |
+                                       (plane == VL_COMPOSITOR_PLANE_Y ? COORDS_CHROMA_OFFSET : 0));
+
+   nir_def *col[3];
+   for (unsigned i = 0; i < 3; i++)
+      col[i] = cs_fetch_texel(&s, i == 0 ? pos_luma : pos_chroma, i);
+
+   nir_def *color = nir_vec4(b, col[0], col[1], col[2], s.fone);
+
+   for (unsigned i = 0; i < 3; i++)
+      col[i] = cs_color_conversion(&s, color, i, YUV2RGB);
+
+   color = nir_vec4(b, col[0], col[1], col[2], s.fone);
+   color = cs_prim_trc_conversion(&s, color);
+
+   for (unsigned i = 0; i < 3; i++)
+      col[i] = cs_color_conversion(&s, color, i, RGB2YUV);
+
+   color = nir_vec4(b, col[0], col[1], col[2], s.fone);
 
-   nir_def *color;
    if (plane != VL_COMPOSITOR_PLANE_UV) {
       unsigned c = 0;
       if (plane == VL_COMPOSITOR_PLANE_U)
          c = 1;
       else if (plane == VL_COMPOSITOR_PLANE_V)
          c = 2;
-      color = nir_channel(b, cs_fetch_texel(&s, pos, c), c);
+      color = nir_channel(b, color, c);
    } else {
-      nir_def *col1 = cs_fetch_texel(&s, pos, 1);
-      nir_def *col2 = cs_fetch_texel(&s, pos, 2);
+      nir_def *col1 = nir_channel(b, color, 1);
+      nir_def *col2 = nir_channel(b, color, 2);
       color = nir_vec2(b, col1, col2);
    }
 
@@ -432,6 +599,7 @@ static void *create_rgb_yuv_shader(struct vl_compositor *c, enum vl_compositor_p
    }
 
    color = nir_vector_insert_imm(b, color, s.fone, 3);
+   color = cs_prim_trc_conversion(&s, color);
 
    if (plane != VL_COMPOSITOR_PLANE_UV) {
       unsigned c = 0;
@@ -439,10 +607,10 @@ static void *create_rgb_yuv_shader(struct vl_compositor *c, enum vl_compositor_p
          c = 1;
       else if (plane == VL_COMPOSITOR_PLANE_V)
          c = 2;
-      color = cs_color_space_conversion(&s, color, c);
+      color = cs_color_conversion(&s, color, c, RGB2YUV);
    } else {
-      nir_def *col1 = cs_color_space_conversion(&s, color, 1);
-      nir_def *col2 = cs_color_space_conversion(&s, color, 2);
+      nir_def *col1 = cs_color_conversion(&s, color, 1, RGB2YUV);
+      nir_def *col2 = cs_color_conversion(&s, color, 2, RGB2YUV);
       color = nir_vec2(b, col1, col2);
    }
 
@@ -558,10 +726,9 @@ static nir_def *create_weave_shader(struct vl_compositor *c, bool rgb, bool y)
    nir_def *color = nir_flrp(b, color_down, color_top, tex_layer);
 
    if (rgb) {
-      nir_def *alpha = cs_luma_key(&s, nir_channel(b, color, 2));
       for (unsigned i = 0; i < 3; ++i)
-         col[i] = cs_color_space_conversion(&s, color, i);
-      color = nir_vec4(b, col[0], col[1], col[2], alpha);
+         col[i] = cs_color_conversion(&s, color, i, YUV2RGB);
+      color = nir_vec4(b, col[0], col[1], col[2], s.fone);
    } else if (y) {
       color = nir_channel(b, color, 0);
    } else {
@@ -615,7 +782,7 @@ cs_launch(struct vl_compositor *c,
    image.shader_access = image.access = PIPE_IMAGE_ACCESS_READ_WRITE;
    image.format = c->fb_state.cbufs[0].texture->format;
 
-   ctx->set_shader_images(c->pipe, PIPE_SHADER_COMPUTE, 0, 1, 0, &image);
+   ctx->set_shader_images(c->pipe, MESA_SHADER_COMPUTE, 0, 1, 0, &image);
 
    /* Bind compute shader */
    ctx->bind_compute_state(ctx, cs);
@@ -774,17 +941,18 @@ set_viewport(struct vl_compositor_state *s,
    if (!ptr)
      return false;
 
-   memcpy(ptr, &s->csc_matrix, sizeof(vl_csc_matrix));
+   memcpy(ptr, &s->yuv2rgb, sizeof(vl_csc_matrix));
 
    float *ptr_float = (float *)ptr;
    ptr_float += sizeof(vl_csc_matrix) / sizeof(float);
-   *ptr_float++ = s->luma_min;
-   *ptr_float++ = s->luma_max;
+
+   ptr_float += 2; /* pad */
    *ptr_float++ = drawn->chroma_offset_x;
    *ptr_float++ = drawn->chroma_offset_y;
-   ptr_float += 2; /* pad */
 
    int *ptr_int = (int *)ptr_float;
+   *ptr_int++ = s->in_transfer_characteristic;
+   *ptr_int++ = s->out_transfer_characteristic;
    *ptr_int++ = drawn->translate_x;
    *ptr_int++ = drawn->translate_y;
 
@@ -816,6 +984,13 @@ set_viewport(struct vl_compositor_state *s,
    memcpy(ptr_float, drawn->proj, sizeof(drawn->proj));
    ptr_float += sizeof(drawn->proj) / sizeof(float);
    memcpy(ptr_float, drawn->chroma_proj, sizeof(drawn->chroma_proj));
+   ptr_float += sizeof(drawn->chroma_proj) / sizeof(float);
+
+   memcpy(ptr_float, &s->rgb2yuv, sizeof(vl_csc_matrix));
+   ptr_float += sizeof(vl_csc_matrix) / sizeof(float);
+
+   memcpy(ptr_float, &s->primaries, sizeof(vl_csc_matrix));
+   ptr_float += sizeof(vl_csc_matrix) / sizeof(float);
 
    pipe_buffer_unmap(s->pipe, buf_transfer);
 
@@ -854,20 +1029,20 @@ draw_layers(struct vl_compositor       *c,
          calc_proj(layer, sampler1->texture, drawn.chroma_proj);
          set_viewport(s, &drawn, samplers);
 
-         c->pipe->bind_sampler_states(c->pipe, PIPE_SHADER_COMPUTE, 0,
+         c->pipe->bind_sampler_states(c->pipe, MESA_SHADER_COMPUTE, 0,
                         num_sampler_views, layer->samplers);
-         c->pipe->set_sampler_views(c->pipe, PIPE_SHADER_COMPUTE, 0,
+         c->pipe->set_sampler_views(c->pipe, MESA_SHADER_COMPUTE, 0,
                         num_sampler_views, 0, samplers);
 
          cs_launch(c, layer->cs, &(drawn.area));
 
          /* Unbind. */
-         c->pipe->set_shader_images(c->pipe, PIPE_SHADER_COMPUTE, 0, 0, 1, NULL);
-         c->pipe->set_constant_buffer(c->pipe, PIPE_SHADER_COMPUTE, 0, false, NULL);
-         c->pipe->set_sampler_views(c->pipe, PIPE_SHADER_COMPUTE, 0, 0,
+         c->pipe->set_shader_images(c->pipe, MESA_SHADER_COMPUTE, 0, 0, 1, NULL);
+         c->pipe->set_constant_buffer(c->pipe, MESA_SHADER_COMPUTE, 0, NULL);
+         c->pipe->set_sampler_views(c->pipe, MESA_SHADER_COMPUTE, 0, 0,
                         num_sampler_views, NULL);
          c->pipe->bind_compute_state(c->pipe, NULL);
-         c->pipe->bind_sampler_states(c->pipe, PIPE_SHADER_COMPUTE, 0,
+         c->pipe->bind_sampler_states(c->pipe, MESA_SHADER_COMPUTE, 0,
                         num_sampler_views, NULL);
 
          if (dirty) {
@@ -910,7 +1085,7 @@ vl_compositor_cs_render(struct vl_compositor_state *s,
       dirty_area->x1 = dirty_area->y1 = VL_COMPOSITOR_MIN_DIRTY;
    }
 
-   pipe_set_constant_buffer(c->pipe, PIPE_SHADER_COMPUTE, 0, s->shader_params);
+   pipe_set_constant_buffer(c->pipe, MESA_SHADER_COMPUTE, 0, s->shader_params);
 
    draw_layers(c, s, dirty_area);
 }
