@@ -175,11 +175,7 @@ struct D3D12EncodeCapabilities
 
    D3D12_VIDEO_ENCODER_SUPPORT_FLAGS                          m_SupportFlags = {};
    D3D12_VIDEO_ENCODER_VALIDATION_FLAGS                       m_ValidationFlags = {};
-#if D3D12_VIDEO_USE_NEW_ENCODECMDLIST4_INTERFACE
    D3D12_FEATURE_DATA_VIDEO_ENCODER_RESOLUTION_SUPPORT_LIMITS1 m_currentResolutionSupportCaps = {};
-#else
-   D3D12_FEATURE_DATA_VIDEO_ENCODER_RESOLUTION_SUPPORT_LIMITS m_currentResolutionSupportCaps = {};
-#endif
    union
    {
       D3D12_VIDEO_ENCODER_PROFILE_H264 m_H264Profile;
@@ -208,11 +204,7 @@ struct D3D12EncodeCapabilities
    // The maximum number of slices that the output of the current frame to be encoded will contain
    uint32_t m_MaxSlicesInOutput = 0;
 
-#if D3D12_VIDEO_USE_NEW_ENCODECMDLIST4_INTERFACE
    D3D12_FEATURE_DATA_VIDEO_ENCODER_RESOURCE_REQUIREMENTS1 m_ResourceRequirementsCaps = {};
-#else
-   D3D12_FEATURE_DATA_VIDEO_ENCODER_RESOURCE_REQUIREMENTS m_ResourceRequirementsCaps = {};
-#endif
 };
 
 struct D3D12EncodeRateControlState
@@ -232,6 +224,35 @@ struct D3D12EncodeRateControlState
       D3D12_VIDEO_ENCODER_RATE_CONTROL_VBR1  m_Configuration_VBR1;
       D3D12_VIDEO_ENCODER_RATE_CONTROL_QVBR1 m_Configuration_QVBR1;  
    } m_Config;
+
+   bool CompareEqual(D3D12EncodeRateControlState& other)
+   {
+      if (m_Mode != other.m_Mode)
+         return false;
+      {
+         // Ignore differences in the ENABLE_FRAME_ANALYSIS flag when comparing rate control flags.
+         const uint32_t ignoreFlagMask =
+         static_cast<uint32_t>(D3D12_VIDEO_ENCODER_RATE_CONTROL_FLAG_ENABLE_FRAME_ANALYSIS);
+         if ((static_cast<uint32_t>(m_Flags) & ~ignoreFlagMask) !=
+          (static_cast<uint32_t>(other.m_Flags) & ~ignoreFlagMask))
+         return false;
+      }
+      if (max_frame_size != other.max_frame_size)
+         return false;
+      if (memcmp(&m_FrameRate, &other.m_FrameRate, sizeof(m_FrameRate)) != 0)
+         return false;
+      if (memcmp(&m_Config, &other.m_Config, sizeof(m_Config)) != 0)
+         return false;
+      return true;
+   }
+};
+
+enum d3d12_video_encoder_input_qp_mode
+{
+   d3d12_video_encoder_input_disable = 0,
+   d3d12_video_encoder_input8_cpu_mode = 1,
+   d3d12_video_encoder_input16_cpu_mode = 2,
+   d3d12_video_encoder_input_gpu_mode = 3,
 };
 
 struct D3D12EncodeConfiguration
@@ -296,11 +317,7 @@ struct D3D12EncodeConfiguration
    union
    {
       D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_H264 m_H264PicData;
-#if D3D12_VIDEO_USE_NEW_ENCODECMDLIST4_INTERFACE
       D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_HEVC2 m_HEVCPicData;
-#else
-      D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA_HEVC m_HEVCPicData;
-#endif // D3D12_VIDEO_USE_NEW_ENCODECMDLIST4_INTERFACE
       D3D12_VIDEO_ENCODER_AV1_PICTURE_CONTROL_CODEC_DATA m_AV1PicData;
    } m_encoderPicParamsDesc = {};
 
@@ -322,10 +339,9 @@ struct D3D12EncodeConfiguration
 
    bool m_bUsedAsReference; // Set if frame will be used as reference frame
 
-#if D3D12_VIDEO_USE_NEW_ENCODECMDLIST4_INTERFACE
    struct{
       D3D12_VIDEO_ENCODER_INPUT_MAP_SOURCE MapSource;
-      union {
+      struct {
          // D3D12_VIDEO_ENCODER_INPUT_MAP_SOURCE_CPU_BUFFER
          D3D12_VIDEO_ENCODER_DIRTY_RECT_INFO RectsInfo;
          // D3D12_VIDEO_ENCODER_INPUT_MAP_SOURCE_GPU_TEXTURE
@@ -346,32 +362,39 @@ struct D3D12EncodeConfiguration
          struct d3d12_resource* InputMap;
          D3D12_FEATURE_DATA_VIDEO_ENCODER_RESOLVE_INPUT_PARAM_LAYOUT capInputLayoutQPMap;
       } GPUInput;
+      // ROI map provided in CPU memory
       struct {
          bool AppRequested;
          // AV1 uses 16 bit integers, H26x uses 8 bit integers
          std::vector<int8_t> m_pRateControlQPMap8Bit;
          std::vector<int16_t> m_pRateControlQPMap16Bit;
       } CPUInput;
+      // When app provides a QP Map with handle and size
+      struct
+      {
+         bool AppRequested;
+         enum d3d12_video_encoder_input_qp_mode m_input_qp_mode;
+         void *m_p_qp_map_cpu;
+         uint32_t m_qp_map_count;
+      } CPUInputBuffer;
    } m_QuantizationMatrixDesc = {};
    struct{
       D3D12_VIDEO_ENCODER_INPUT_MAP_SOURCE MapSource;
-      struct { // union doesn't play well with std::vector
-         // D3D12_VIDEO_ENCODER_INPUT_MAP_SOURCE_CPU_BUFFER
-         D3D12_VIDEO_ENCODER_MOVEREGION_INFO RectsInfo;
-         // D3D12_VIDEO_ENCODER_INPUT_MAP_SOURCE_GPU_TEXTURE
-         struct
-         {
-            D3D12_VIDEO_ENCODER_FRAME_MOTION_SEARCH_MODE_CONFIG MotionSearchModeConfiguration;
-            UINT NumHintsPerPixel;
-            std::vector<ID3D12Resource*> ppMotionVectorMaps;
-            UINT*            pMotionVectorMapsSubresources;
-            std::vector<ID3D12Resource*> ppMotionVectorMapsMetadata;
-            UINT*            pMotionVectorMapsMetadataSubresources;
-            D3D12_VIDEO_ENCODER_FRAME_INPUT_MOTION_UNIT_PRECISION MotionUnitPrecision;
-            D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA1 PictureControlConfiguration;
-            D3D12_FEATURE_DATA_VIDEO_ENCODER_RESOLVE_INPUT_PARAM_LAYOUT capInputLayoutMotionVectors;
-         } MapInfo;
-      };
+      // D3D12_VIDEO_ENCODER_INPUT_MAP_SOURCE_CPU_BUFFER
+      D3D12_VIDEO_ENCODER_MOVEREGION_INFO RectsInfo;
+      // D3D12_VIDEO_ENCODER_INPUT_MAP_SOURCE_GPU_TEXTURE
+      struct
+      {
+         D3D12_VIDEO_ENCODER_FRAME_MOTION_SEARCH_MODE_CONFIG MotionSearchModeConfiguration;
+         UINT NumHintsPerPixel;
+         std::vector<ID3D12Resource*> ppMotionVectorMaps;
+         UINT*            pMotionVectorMapsSubresources;
+         std::vector<ID3D12Resource*> ppMotionVectorMapsMetadata;
+         UINT*            pMotionVectorMapsMetadataSubresources;
+         D3D12_VIDEO_ENCODER_FRAME_INPUT_MOTION_UNIT_PRECISION MotionUnitPrecision;
+         D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA1 PictureControlConfiguration;
+         D3D12_FEATURE_DATA_VIDEO_ENCODER_RESOLVE_INPUT_PARAM_LAYOUT capInputLayoutMotionVectors;
+      } MapInfo;
    } m_MoveRectsDesc = {};
    std::vector<RECT> m_DirtyRectsArray;
    std::vector<D3D12_VIDEO_ENCODER_MOVE_RECT> m_MoveRectsArray;
@@ -423,7 +446,6 @@ struct D3D12EncodeConfiguration
          D3D12_VIDEO_ENCODER_RECONSTRUCTED_PICTURE FrameAnalysisReconstructedPictureOutput;
 
    } m_TwoPassEncodeDesc = {};
-#endif
 };
 
 struct EncodedBitstreamResolvedMetadata
@@ -478,9 +500,7 @@ struct EncodedBitstreamResolvedMetadata
    * stream from EncodeFrame.
    */
    std::vector<ComPtr<ID3D12Resource>> spStagingBitstreams;
-#if D3D12_VIDEO_USE_NEW_ENCODECMDLIST4_INTERFACE
    D3D12_VIDEO_ENCODER_COMPRESSED_BITSTREAM_NOTIFICATION_MODE SubregionNotificationMode;
-#endif // D3D12_VIDEO_USE_NEW_ENCODECMDLIST4_INTERFACE
    std::vector<ComPtr<ID3D12Resource>> pspSubregionSizes;
    std::vector<ComPtr<ID3D12Resource>> pspSubregionOffsets;
    std::vector<ComPtr<ID3D12Fence>> pspSubregionFences;
@@ -515,7 +535,7 @@ struct EncodedBitstreamResolvedMetadata
    */
    std::vector<uint8_t> m_StagingBitstreamConstruction;
 
-   /* Stores encode result for get_feedback readback in the D3D12_VIDEO_ENC_METADATA_BUFFERS_COUNT slots */
+   /* Stores encode result for get_feedback readback in the m_MaxMetadataBuffersCount slots */
    enum pipe_video_feedback_encode_result_flags encode_result = PIPE_VIDEO_FEEDBACK_METADATA_ENCODE_FLAG_OK;
 
    /* Expected max frame, slice sizes */
@@ -553,6 +573,9 @@ struct d3d12_video_encoder
 
    const uint m_NodeMask  = 0u;
    const uint m_NodeIndex = 0u;
+   UINT m_MaxOutputBitstreamSize  = 0;
+   size_t m_MaxQueueAsyncDepth = 0;
+   size_t m_MaxMetadataBuffersCount = 0;
 
    ComPtr<ID3D12Fence> m_spFence;
    uint64_t            m_fenceValue = 1u;
@@ -594,7 +617,7 @@ struct d3d12_video_encoder
       uint64_t m_InputSurfaceFenceValue = 0;
       d3d12_unique_fence m_CompletionFence;
 
-      /* Stores encode result for submission error control in the D3D12_VIDEO_ENC_ASYNC_DEPTH slots */
+      /* Stores encode result for submission error control in the m_MaxQueueAsyncDepth slots */
       enum pipe_video_feedback_encode_result_flags encode_result = PIPE_VIDEO_FEEDBACK_METADATA_ENCODE_FLAG_OK;
 
       ComPtr<ID3D12Resource> m_spDirtyRectsResolvedOpaqueMap; // output of ID3D12VideoEncodeCommandList::ResolveInputParamLayout
@@ -632,12 +655,10 @@ bool
 d3d12_video_encoder_reconfigure_encoder_objects(struct d3d12_video_encoder *pD3D12Enc,
                                                 struct pipe_video_buffer *  srcTexture,
                                                 struct pipe_picture_desc *  picture);
-D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA
-d3d12_video_encoder_get_current_picture_param_settings(struct d3d12_video_encoder *pD3D12Enc);
-#if D3D12_VIDEO_USE_NEW_ENCODECMDLIST4_INTERFACE
 D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA1
-d3d12_video_encoder_get_current_picture_param_settings1(struct d3d12_video_encoder *pD3D12Enc);
-#endif // D3D12_VIDEO_USE_NEW_ENCODECMDLIST4_INTERFACE
+d3d12_video_encoder_get_current_picture_param_settings(struct d3d12_video_encoder *pD3D12Enc);
+D3D12_VIDEO_ENCODER_PICTURE_CONTROL_CODEC_DATA
+d3d12_video_encoder_get_current_picture_param_settings_legacy(struct d3d12_video_encoder *pD3D12Enc);
 D3D12_VIDEO_ENCODER_LEVEL_SETTING
 d3d12_video_encoder_get_current_level_desc(struct d3d12_video_encoder *pD3D12Enc);
 D3D12_VIDEO_ENCODER_CODEC_CONFIGURATION
@@ -689,18 +710,10 @@ d3d12_video_encoder_get_current_codec(struct d3d12_video_encoder *pD3D12Enc);
 
 bool
 d3d12_video_encoder_negotiate_requested_features_and_d3d12_driver_caps(struct d3d12_video_encoder *pD3D12Enc,
-#if D3D12_VIDEO_USE_NEW_ENCODECMDLIST4_INTERFACE
                                                                        D3D12_FEATURE_DATA_VIDEO_ENCODER_SUPPORT2 &capEncoderSupportData);
-#else
-                                                                       D3D12_FEATURE_DATA_VIDEO_ENCODER_SUPPORT1 &capEncoderSupportData);
-#endif
 bool
 d3d12_video_encoder_query_d3d12_driver_caps(struct d3d12_video_encoder *pD3D12Enc,
-#if D3D12_VIDEO_USE_NEW_ENCODECMDLIST4_INTERFACE
                                             D3D12_FEATURE_DATA_VIDEO_ENCODER_SUPPORT2 &capEncoderSupportData);
-#else
-                                            D3D12_FEATURE_DATA_VIDEO_ENCODER_SUPPORT1 &capEncoderSupportData);
-#endif
 bool
 d3d12_video_encoder_check_subregion_mode_support(struct d3d12_video_encoder *pD3D12Enc,
                                                  D3D12_VIDEO_ENCODER_FRAME_SUBREGION_LAYOUT_MODE requestedSlicesMode);
@@ -748,7 +761,7 @@ d3d12_video_encoder_prepare_input_buffers(struct d3d12_video_encoder *pD3D12Enc)
 
 void
 d3d12_video_encoder_update_qpmap_input(struct d3d12_video_encoder *pD3D12Enc,
-                                       struct pipe_resource* qpmap,
+                                       struct pipe_enc_qpmap_input_info &qp_info,
                                        struct pipe_enc_roi roi,
                                        uint32_t temporal_id);
 void
