@@ -1,24 +1,6 @@
 /*
- * Copyright (C) 2016 Rob Clark <robclark@freedesktop.org>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
+ * Copyright © 2016 Rob Clark <robclark@freedesktop.org>
+ * SPDX-License-Identifier: MIT
  *
  * Authors:
  *    Rob Clark <robclark@freedesktop.org>
@@ -41,12 +23,13 @@ struct fd_ringbuffer;
 
 /* grouped together emit-state for prog/vertex/state emit: */
 struct fd5_emit {
-   struct pipe_debug_callback *debug;
+   struct util_debug_callback *debug;
    const struct fd_vertex_state *vtx;
    const struct fd5_program_state *prog;
    const struct pipe_draw_info *info;
+        unsigned drawid_offset;
    const struct pipe_draw_indirect_info *indirect;
-   const struct pipe_draw_start_count *draw;
+	const struct pipe_draw_start_count_bias *draw;
    bool binning_pass;
    struct ir3_cache_key key;
    enum fd_dirty_3d_state dirty;
@@ -84,7 +67,7 @@ fd5_emit_get_vp(struct fd5_emit *emit)
        * is what has all the outputs that might be involved in TFB.
        */
       if (emit->binning_pass &&
-          !emit->prog->vs->shader->stream_output.num_outputs)
+          !emit->prog->vs->stream_output.num_outputs)
          emit->vs = emit->prog->bs;
       else
          emit->vs = emit->prog->vs;
@@ -171,7 +154,7 @@ fd5_emit_render_cntl(struct fd_context *ctx, bool blit, bool binning) assert_dt
     * Other bits seem to depend on query state, like if samples-passed
     * query is active.
     */
-   bool samples_passed = (fd5_context(ctx)->samples_passed_queries > 0);
+   bool samples_passed = (ctx->occlusion_queries_active > 0);
    OUT_PKT4(ring, REG_A5XX_RB_RENDER_CNTL, 1);
    OUT_RING(ring, 0x00000000 | /* RB_RENDER_CNTL */
                      COND(binning, A5XX_RB_RENDER_CNTL_BINNING_PASS) |
