@@ -1,24 +1,6 @@
 /*
  * Copyright © 2021 Google, Inc.
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE.
+ * SPDX-License-Identifier: MIT
  */
 
 #ifndef FREEDRENO_AUTOTUNE_H
@@ -126,7 +108,7 @@ struct fd_autotune_results {
     */
    struct {
       uint64_t samples_start;
-      uint64_t __pad0;
+      uint64_t samples_result;
       uint64_t samples_end;
       uint64_t __pad1;
    } result[127];
@@ -134,7 +116,7 @@ struct fd_autotune_results {
 
 #define __offset(base, ptr) ((uint8_t *)(ptr) - (uint8_t *)(base))
 #define results_ptr(at, member)                                                \
-   (at)->results_mem, __offset((at)->results, &(at)->results->member), 0, 0
+   (at)->results_mem, __offset((at)->results, &(at)->results->member)
 
 struct fd_batch_history;
 
