@@ -430,7 +430,7 @@ fd_dirty_resource(struct fd_context *ctx, struct pipe_resource *prsc,
 
 static inline void
 fd_dirty_shader_resource(struct fd_context *ctx, struct pipe_resource *prsc,
-                         enum pipe_shader_type shader,
+                         mesa_shader_stage shader,
                          BITMASK_ENUM(fd_dirty_shader_state) dirty,
                          bool write)
    assert_dt
@@ -464,7 +464,7 @@ fdl_type_from_pipe_target(enum pipe_texture_target target) {
       return FDL_VIEW_TYPE_3D;
    case PIPE_MAX_TEXTURE_TYPES:
    default:
-      unreachable("bad texture type");
+      UNREACHABLE("bad texture type");
    }
 }
 
