@@ -169,22 +169,11 @@ static const struct nir_shader_compiler_options gallivm_nir_options = {
    .lower_vector_cmp = true,
    .lower_device_index_to_zero = true,
    /* .support_16bit_alu = true, */
-   .support_indirect_inputs = (uint8_t)BITFIELD_MASK(PIPE_SHADER_TYPES),
-   .support_indirect_outputs = (uint8_t)BITFIELD_MASK(PIPE_SHADER_TYPES),
-   .has_ddx_intrinsics = true,
+   .support_indirect_inputs = (uint8_t)BITFIELD_MASK(MESA_SHADER_STAGES),
+   .support_indirect_outputs = (uint8_t)BITFIELD_MASK(MESA_SHADER_STAGES),
    .no_integers = true,
 };
 
-static const struct nir_shader_compiler_options *
-i915_get_compiler_options(struct pipe_screen *pscreen,
-                          enum pipe_shader_type shader)
-{
-   if (shader == PIPE_SHADER_FRAGMENT)
-      return &i915_compiler_options;
-   else
-      return &gallivm_nir_options;
-}
-
 static void
 i915_optimize_nir(struct nir_shader *s)
 {
@@ -232,10 +221,10 @@ i915_optimize_nir(struct nir_shader *s)
    /* Group texture loads together to try to avoid hitting the
     * texture indirection phase limit.
     */
-   NIR_PASS(_, s, nir_group_loads, nir_group_all, ~0);
+   NIR_PASS(_, s, nir_opt_group_loads, nir_group_all, ~0);
 }
 
-static char *
+static void
 i915_finalize_nir(struct pipe_screen *pscreen, struct nir_shader *s)
 {
    if (s->info.stage == MESA_SHADER_FRAGMENT)
@@ -258,14 +247,13 @@ i915_finalize_nir(struct pipe_screen *pscreen, struct nir_shader *s)
    nir_validate_shader(s, "after uniform var removal");
 
    nir_sweep(s);
-   return NULL;
 }
 
 static void
 i915_init_shader_caps(struct i915_screen *is)
 {
    struct pipe_shader_caps *caps =
-      (struct pipe_shader_caps *)&is->base.shader_caps[PIPE_SHADER_VERTEX];
+      (struct pipe_shader_caps *)&is->base.shader_caps[MESA_SHADER_VERTEX];
 
    draw_init_shader_caps(caps);
 
@@ -292,7 +280,7 @@ i915_init_shader_caps(struct i915_screen *is)
    caps->max_shader_buffers = false;
    caps->max_shader_images = false;
 
-   caps = (struct pipe_shader_caps *)&is->base.shader_caps[PIPE_SHADER_FRAGMENT];
+   caps = (struct pipe_shader_caps *)&is->base.shader_caps[MESA_SHADER_FRAGMENT];
 
    caps->supported_irs = (1 << PIPE_SHADER_IR_NIR) | (1 << PIPE_SHADER_IR_TGSI);
    /* XXX: some of these are just shader model 2.0 values, fix this! */
@@ -560,7 +548,6 @@ i915_screen_create(struct i915_winsys *iws)
    is->base.get_vendor = i915_get_vendor;
    is->base.get_device_vendor = i915_get_device_vendor;
    is->base.get_screen_fd = i915_screen_get_fd;
-   is->base.get_compiler_options = i915_get_compiler_options;
    is->base.finalize_nir = i915_finalize_nir;
    is->base.is_format_supported = i915_is_format_supported;
 
@@ -569,6 +556,9 @@ i915_screen_create(struct i915_winsys *iws)
    is->base.fence_reference = i915_fence_reference;
    is->base.fence_finish = i915_fence_finish;
 
+   is->base.nir_options[MESA_SHADER_VERTEX] = &gallivm_nir_options;
+   is->base.nir_options[MESA_SHADER_FRAGMENT] = &i915_compiler_options;
+
    i915_init_screen_resource_functions(is);
 
    i915_init_shader_caps(is);
