@@ -142,7 +142,7 @@ typedef struct {
 
    /* for node pred_list */
    struct list_head pred_link;
-   /* for ndoe succ_list */
+   /* for node succ_list */
    struct list_head succ_link;
 } gpir_dep;
 
@@ -398,15 +398,9 @@ typedef struct gpir_compiler {
    /* Find the gpir node for a given NIR SSA def. */
    gpir_node **node_for_ssa;
 
-   /* Find the gpir node for a given NIR register. */
-   gpir_node **node_for_reg;
-
    /* Find the gpir register for a given NIR SSA def. */
    gpir_reg **reg_for_ssa;
 
-   /* Find the gpir register for a given NIR register. */
-   gpir_reg **reg_for_reg;
-
    /* gpir block for NIR block. */
    gpir_block **blocks;
 
