@@ -210,7 +210,7 @@ static ppir_codegen_outmod ppir_codegen_get_outmod(ppir_outmod outmod)
       case ppir_outmod_round:
          return ppir_codegen_outmod_round;
       default:
-         unreachable("invalid ppir_outmod");
+         UNREACHABLE("invalid ppir_outmod");
    }
 }
 
@@ -888,8 +888,7 @@ bool ppir_codegen_prog(ppir_compiler *comp)
       }
    }
 
-   if (comp->prog->shader)
-      ralloc_free(comp->prog->shader);
+   ralloc_free(comp->prog->shader);
 
    comp->prog->shader = prog;
    comp->prog->state.shader_size = size * sizeof(uint32_t);
