@@ -67,14 +67,14 @@ llvmpipe_create_sampler_state(struct pipe_context *pipe,
 
 static void
 llvmpipe_bind_sampler_states(struct pipe_context *pipe,
-                             enum pipe_shader_type shader,
+                             mesa_shader_stage shader,
                              unsigned start,
                              unsigned num,
                              void **samplers)
 {
    struct llvmpipe_context *llvmpipe = llvmpipe_context(pipe);
 
-   assert(shader < PIPE_SHADER_MESH_TYPES);
+   assert(shader < MESA_SHADER_MESH_STAGES);
    assert(start + num <= ARRAY_SIZE(llvmpipe->samplers[shader]));
 
    draw_flush(llvmpipe->draw);
@@ -97,29 +97,29 @@ llvmpipe_bind_sampler_states(struct pipe_context *pipe,
    }
 
    switch (shader) {
-   case PIPE_SHADER_VERTEX:
-   case PIPE_SHADER_GEOMETRY:
-   case PIPE_SHADER_TESS_CTRL:
-   case PIPE_SHADER_TESS_EVAL:
+   case MESA_SHADER_VERTEX:
+   case MESA_SHADER_GEOMETRY:
+   case MESA_SHADER_TESS_CTRL:
+   case MESA_SHADER_TESS_EVAL:
       draw_set_samplers(llvmpipe->draw,
                         shader,
                         llvmpipe->samplers[shader],
                         llvmpipe->num_samplers[shader]);
       break;
-   case PIPE_SHADER_COMPUTE:
+   case MESA_SHADER_COMPUTE:
       llvmpipe->cs_dirty |= LP_CSNEW_SAMPLER;
       break;
-   case PIPE_SHADER_FRAGMENT:
+   case MESA_SHADER_FRAGMENT:
       llvmpipe->dirty |= LP_NEW_SAMPLER;
       break;
-   case PIPE_SHADER_TASK:
+   case MESA_SHADER_TASK:
       llvmpipe->dirty |= LP_NEW_TASK_SAMPLER;
       break;
-   case PIPE_SHADER_MESH:
+   case MESA_SHADER_MESH:
       llvmpipe->dirty |= LP_NEW_MESH_SAMPLER;
       break;
    default:
-      unreachable("Illegal shader type");
+      UNREACHABLE("Illegal shader type");
       break;
    }
 }
@@ -127,7 +127,7 @@ llvmpipe_bind_sampler_states(struct pipe_context *pipe,
 
 static void
 llvmpipe_set_sampler_views(struct pipe_context *pipe,
-                           enum pipe_shader_type shader,
+                           mesa_shader_stage shader,
                            unsigned start,
                            unsigned num,
                            unsigned unbind_num_trailing_slots,
@@ -138,7 +138,7 @@ llvmpipe_set_sampler_views(struct pipe_context *pipe,
 
    assert(num <= PIPE_MAX_SHADER_SAMPLER_VIEWS);
 
-   assert(shader < PIPE_SHADER_MESH_TYPES);
+   assert(shader < MESA_SHADER_MESH_STAGES);
    assert(start + num <= ARRAY_SIZE(llvmpipe->sampler_views[shader]));
 
    draw_flush(llvmpipe->draw);
@@ -180,32 +180,32 @@ llvmpipe_set_sampler_views(struct pipe_context *pipe,
    }
 
    switch (shader) {
-   case PIPE_SHADER_VERTEX:
-   case PIPE_SHADER_GEOMETRY:
-   case PIPE_SHADER_TESS_CTRL:
-   case PIPE_SHADER_TESS_EVAL:
+   case MESA_SHADER_VERTEX:
+   case MESA_SHADER_GEOMETRY:
+   case MESA_SHADER_TESS_CTRL:
+   case MESA_SHADER_TESS_EVAL:
       draw_set_sampler_views(llvmpipe->draw,
                              shader,
                              llvmpipe->sampler_views[shader],
                              llvmpipe->num_sampler_views[shader]);
       break;
-   case PIPE_SHADER_COMPUTE:
+   case MESA_SHADER_COMPUTE:
       llvmpipe->cs_dirty |= LP_CSNEW_SAMPLER_VIEW;
       break;
-   case PIPE_SHADER_FRAGMENT:
+   case MESA_SHADER_FRAGMENT:
       llvmpipe->dirty |= LP_NEW_SAMPLER_VIEW;
       lp_setup_set_fragment_sampler_views(llvmpipe->setup,
-                                          llvmpipe->num_sampler_views[PIPE_SHADER_FRAGMENT],
-                                          llvmpipe->sampler_views[PIPE_SHADER_FRAGMENT]);
+                                          llvmpipe->num_sampler_views[MESA_SHADER_FRAGMENT],
+                                          llvmpipe->sampler_views[MESA_SHADER_FRAGMENT]);
       break;
-   case PIPE_SHADER_TASK:
+   case MESA_SHADER_TASK:
       llvmpipe->dirty |= LP_NEW_TASK_SAMPLER_VIEW;
       break;
-   case PIPE_SHADER_MESH:
+   case MESA_SHADER_MESH:
       llvmpipe->dirty |= LP_NEW_MESH_SAMPLER_VIEW;
       break;
    default:
-      unreachable("Illegal shader type");
+      UNREACHABLE("Illegal shader type");
       break;
    }
 }
@@ -290,7 +290,7 @@ static void
 prepare_shader_sampling(struct llvmpipe_context *lp,
                         unsigned num,
                         struct pipe_sampler_view **views,
-                        enum pipe_shader_type shader_type)
+                        mesa_shader_stage shader_type)
 {
    uint32_t row_stride[PIPE_MAX_TEXTURE_LEVELS];
    uint32_t img_stride[PIPE_MAX_TEXTURE_LEVELS];
@@ -390,7 +390,7 @@ llvmpipe_prepare_vertex_sampling(struct llvmpipe_context *lp,
                                  unsigned num,
                                  struct pipe_sampler_view **views)
 {
-   prepare_shader_sampling(lp, num, views, PIPE_SHADER_VERTEX);
+   prepare_shader_sampling(lp, num, views, MESA_SHADER_VERTEX);
 }
 
 
@@ -402,7 +402,7 @@ llvmpipe_prepare_geometry_sampling(struct llvmpipe_context *lp,
                                    unsigned num,
                                    struct pipe_sampler_view **views)
 {
-   prepare_shader_sampling(lp, num, views, PIPE_SHADER_GEOMETRY);
+   prepare_shader_sampling(lp, num, views, MESA_SHADER_GEOMETRY);
 }
 
 
@@ -414,7 +414,7 @@ llvmpipe_prepare_tess_ctrl_sampling(struct llvmpipe_context *lp,
                                     unsigned num,
                                     struct pipe_sampler_view **views)
 {
-   prepare_shader_sampling(lp, num, views, PIPE_SHADER_TESS_CTRL);
+   prepare_shader_sampling(lp, num, views, MESA_SHADER_TESS_CTRL);
 }
 
 
@@ -426,13 +426,13 @@ llvmpipe_prepare_tess_eval_sampling(struct llvmpipe_context *lp,
                                     unsigned num,
                                     struct pipe_sampler_view **views)
 {
-   prepare_shader_sampling(lp, num, views, PIPE_SHADER_TESS_EVAL);
+   prepare_shader_sampling(lp, num, views, MESA_SHADER_TESS_EVAL);
 }
 
 
 void
 llvmpipe_cleanup_stage_sampling(struct llvmpipe_context *ctx,
-                                enum pipe_shader_type stage)
+                                mesa_shader_stage stage)
 {
    assert(ctx);
    assert(stage < ARRAY_SIZE(ctx->num_sampler_views));
@@ -458,7 +458,7 @@ static void
 prepare_shader_images(struct llvmpipe_context *lp,
                       unsigned num,
                       struct pipe_image_view *views,
-                      enum pipe_shader_type shader_type)
+                      mesa_shader_stage shader_type)
 {
    assert(num <= PIPE_MAX_SHADER_SAMPLER_VIEWS);
    if (!num)
@@ -549,7 +549,7 @@ llvmpipe_prepare_vertex_images(struct llvmpipe_context *lp,
                                unsigned num,
                                struct pipe_image_view *views)
 {
-   prepare_shader_images(lp, num, views, PIPE_SHADER_VERTEX);
+   prepare_shader_images(lp, num, views, MESA_SHADER_VERTEX);
 }
 
 
@@ -561,7 +561,7 @@ llvmpipe_prepare_geometry_images(struct llvmpipe_context *lp,
                                  unsigned num,
                                  struct pipe_image_view *views)
 {
-   prepare_shader_images(lp, num, views, PIPE_SHADER_GEOMETRY);
+   prepare_shader_images(lp, num, views, MESA_SHADER_GEOMETRY);
 }
 
 
@@ -573,7 +573,7 @@ llvmpipe_prepare_tess_ctrl_images(struct llvmpipe_context *lp,
                                   unsigned num,
                                   struct pipe_image_view *views)
 {
-   prepare_shader_images(lp, num, views, PIPE_SHADER_TESS_CTRL);
+   prepare_shader_images(lp, num, views, MESA_SHADER_TESS_CTRL);
 }
 
 
@@ -585,13 +585,13 @@ llvmpipe_prepare_tess_eval_images(struct llvmpipe_context *lp,
                                   unsigned num,
                                   struct pipe_image_view *views)
 {
-   prepare_shader_images(lp, num, views, PIPE_SHADER_TESS_EVAL);
+   prepare_shader_images(lp, num, views, MESA_SHADER_TESS_EVAL);
 }
 
 
 void
 llvmpipe_cleanup_stage_images(struct llvmpipe_context *ctx,
-                              enum pipe_shader_type stage)
+                              mesa_shader_stage stage)
 {
    assert(ctx);
    assert(stage < ARRAY_SIZE(ctx->num_images));
@@ -622,5 +622,6 @@ llvmpipe_init_sampler_funcs(struct llvmpipe_context *llvmpipe)
    llvmpipe->pipe.set_sampler_views = llvmpipe_set_sampler_views;
    llvmpipe->pipe.sampler_view_destroy = llvmpipe_sampler_view_destroy;
    llvmpipe->pipe.sampler_view_release = u_default_sampler_view_release;
+   llvmpipe->pipe.resource_release = u_default_resource_release;
    llvmpipe->pipe.delete_sampler_state = llvmpipe_delete_sampler_state;
 }
