@@ -7,9 +7,10 @@
 struct nv50_tsc_entry {
    int id;
    uint32_t tsc[8];
+   bool seamless_cube_map;
 };
 
-static INLINE struct nv50_tsc_entry *
+static inline struct nv50_tsc_entry *
 nv50_tsc_entry(void *hwcso)
 {
    return (struct nv50_tsc_entry *)hwcso;
@@ -19,9 +20,10 @@ struct nv50_tic_entry {
    struct pipe_sampler_view pipe;
    int id;
    uint32_t tic[8];
+   uint32_t bindless;
 };
 
-static INLINE struct nv50_tic_entry *
+static inline struct nv50_tic_entry *
 nv50_tic_entry(struct pipe_sampler_view *view)
 {
    return (struct nv50_tic_entry *)view;
