@@ -512,6 +512,34 @@ csf_attach_sync_points(struct panfrost_batch *batch, uint32_t vm_sync_handle,
                              vm_sync_handle, vm_sync_signal_point, 0);
 }
 
+static void
+update_reset_status(struct panfrost_context *ctx,
+                    enum pipe_reset_status new_status)
+{
+   switch (new_status) {
+   case PIPE_GUILTY_CONTEXT_RESET:
+      /* Guilty reset overrides everything. */
+      ctx->csf.reset_status = new_status;
+      break;
+   case PIPE_UNKNOWN_CONTEXT_RESET:
+      /* Preserve guilty reset, override otherwise. */
+      if (ctx->csf.reset_status != PIPE_GUILTY_CONTEXT_RESET)
+         ctx->csf.reset_status = new_status;
+      break;
+   case PIPE_INNOCENT_CONTEXT_RESET:
+      /* Preserve guilty/unknown resets, override otherwise. */
+      if (ctx->csf.reset_status != PIPE_GUILTY_CONTEXT_RESET &&
+          ctx->csf.reset_status != PIPE_UNKNOWN_CONTEXT_RESET)
+         ctx->csf.reset_status = new_status;
+      break;
+   case PIPE_NO_RESET:
+      break;
+   default:
+      assert(!"Invalid reset status");
+      break;
+   }
+}
+
 static void
 csf_check_ctx_state_and_reinit(struct panfrost_context *ctx)
 {
@@ -524,13 +552,17 @@ csf_check_ctx_state_and_reinit(struct panfrost_context *ctx)
    ret = pan_kmod_ioctl(panfrost_device_fd(dev),
                         DRM_IOCTL_PANTHOR_GROUP_GET_STATE, &state);
    if (ret) {
+      update_reset_status(ctx, PIPE_UNKNOWN_CONTEXT_RESET);
       mesa_loge("DRM_IOCTL_PANTHOR_GROUP_GET_STATE failed (err=%d)", errno);
       return;
    }
 
    /* Context is still usable. This was a transient error. */
-   if (state.state == 0)
+   if (!(state.state & (DRM_PANTHOR_GROUP_STATE_FATAL_FAULT |
+                        DRM_PANTHOR_GROUP_STATE_TIMEDOUT))) {
+      update_reset_status(ctx, PIPE_NO_RESET);
       return;
+   }
 
    /* If the VM is unusable, we can't do much, as this is shared between all
     * contexts, and restoring the VM state is non-trivial.
@@ -540,6 +572,15 @@ csf_check_ctx_state_and_reinit(struct panfrost_context *ctx)
       assert(!"VM became unusable, we can't reset the context");
    }
 
+   /* DRM_PANTHOR_GROUP_STATE_INNOCENT only exists since panthor 1.3, which
+    * means we consider all resets as guilty until that point, but that
+    * should be fine.
+    */
+   update_reset_status(ctx, state.state & DRM_PANTHOR_GROUP_STATE_INNOCENT
+                               ? PIPE_INNOCENT_CONTEXT_RESET
+                               : PIPE_GUILTY_CONTEXT_RESET);
+
+   mesa_loge("Group became unusable, re-initializing context");
    panfrost_context_reinit(ctx);
 }
 
@@ -859,18 +900,18 @@ GENX(csf_emit_fragment_job)(struct panfrost_batch *batch,
 }
 
 static void
-csf_emit_shader_regs(struct panfrost_batch *batch, enum pipe_shader_type stage,
+csf_emit_shader_regs(struct panfrost_batch *batch, mesa_shader_stage stage,
                      uint64_t shader)
 {
    uint64_t resources = panfrost_emit_resources(batch, stage);
 
-   assert(stage == PIPE_SHADER_VERTEX || stage == PIPE_SHADER_FRAGMENT ||
-          stage == PIPE_SHADER_COMPUTE);
+   assert(stage == MESA_SHADER_VERTEX || stage == MESA_SHADER_FRAGMENT ||
+          stage == MESA_SHADER_COMPUTE);
 
 #if PAN_ARCH >= 12
-   unsigned offset = (stage == PIPE_SHADER_FRAGMENT) ? 2 : 0;
+   unsigned offset = (stage == MESA_SHADER_FRAGMENT) ? 2 : 0;
 #else
-   unsigned offset = (stage == PIPE_SHADER_FRAGMENT) ? 4 : 0;
+   unsigned offset = (stage == MESA_SHADER_FRAGMENT) ? 4 : 0;
 #endif
 
    unsigned fau_count = DIV_ROUND_UP(batch->nr_push_uniforms[stage], 2);
@@ -887,16 +928,16 @@ GENX(csf_launch_grid)(struct panfrost_batch *batch,
                       const struct pipe_grid_info *info)
 {
    /* Empty compute programs are invalid and don't make sense */
-   if (batch->rsd[PIPE_SHADER_COMPUTE] == 0)
+   if (batch->rsd[MESA_SHADER_COMPUTE] == 0)
       return;
 
    struct panfrost_context *ctx = batch->ctx;
    struct panfrost_device *dev = pan_device(ctx->base.screen);
-   struct panfrost_compiled_shader *cs = ctx->prog[PIPE_SHADER_COMPUTE];
+   struct panfrost_compiled_shader *cs = ctx->prog[MESA_SHADER_COMPUTE];
    struct cs_builder *b = batch->csf.cs.builder;
 
-   csf_emit_shader_regs(batch, PIPE_SHADER_COMPUTE,
-                        batch->rsd[PIPE_SHADER_COMPUTE]);
+   csf_emit_shader_regs(batch, MESA_SHADER_COMPUTE,
+                        batch->rsd[MESA_SHADER_COMPUTE]);
 
    cs_move64_to(b, cs_sr_reg64(b, COMPUTE, TSD_0), batch->tls.gpu);
 
@@ -1038,8 +1079,8 @@ GENX(csf_launch_xfb)(struct panfrost_batch *batch,
    cs_move32_to(b, cs_sr_reg32(b, COMPUTE, JOB_SIZE_Y), info->instance_count);
    cs_move32_to(b, cs_sr_reg32(b, COMPUTE, JOB_SIZE_Z), 1);
 
-   csf_emit_shader_regs(batch, PIPE_SHADER_VERTEX,
-                        batch->rsd[PIPE_SHADER_VERTEX]);
+   csf_emit_shader_regs(batch, MESA_SHADER_VERTEX,
+                        batch->rsd[MESA_SHADER_VERTEX]);
    /* force a barrier to avoid read/write sync issues with buffers */
    cs_wait_slot(b, 2);
 
@@ -1073,8 +1114,8 @@ csf_emit_draw_state(struct panfrost_batch *batch,
                     const struct pipe_draw_info *info, unsigned drawid_offset)
 {
    struct panfrost_context *ctx = batch->ctx;
-   struct panfrost_compiled_shader *vs = ctx->prog[PIPE_SHADER_VERTEX];
-   struct panfrost_compiled_shader *fs = ctx->prog[PIPE_SHADER_FRAGMENT];
+   struct panfrost_compiled_shader *vs = ctx->prog[MESA_SHADER_VERTEX];
+   struct panfrost_compiled_shader *fs = ctx->prog[MESA_SHADER_FRAGMENT];
 
    bool idvs = vs->info.vs.idvs;
    bool fs_required = panfrost_fs_required(
@@ -1090,12 +1131,12 @@ csf_emit_draw_state(struct panfrost_batch *batch,
       cs_vt_start(batch->csf.cs.builder, cs_now());
    }
 
-   csf_emit_shader_regs(batch, PIPE_SHADER_VERTEX,
+   csf_emit_shader_regs(batch, MESA_SHADER_VERTEX,
       panfrost_get_position_shader(batch, info));
 
    if (fs_required) {
-      csf_emit_shader_regs(batch, PIPE_SHADER_FRAGMENT,
-                           batch->rsd[PIPE_SHADER_FRAGMENT]);
+      csf_emit_shader_regs(batch, MESA_SHADER_FRAGMENT,
+                           batch->rsd[MESA_SHADER_FRAGMENT]);
    } else {
       cs_move64_to(b, cs_sr_reg64(b, IDVS, FRAGMENT_SRT), 0);
       cs_move64_to(b, cs_sr_reg64(b, IDVS, FRAGMENT_FAU), 0);
@@ -1139,7 +1180,7 @@ csf_emit_draw_state(struct panfrost_batch *batch,
    if (ctx->occlusion_query && ctx->active_queries) {
       struct panfrost_resource *rsrc = pan_resource(ctx->occlusion_query->rsrc);
       cs_move64_to(b, cs_sr_reg64(b, IDVS, OQ), rsrc->plane.base);
-      panfrost_batch_write_rsrc(ctx->batch, rsrc, PIPE_SHADER_FRAGMENT);
+      panfrost_batch_write_rsrc(ctx->batch, rsrc, MESA_SHADER_FRAGMENT);
    }
 
    cs_move32_to(b, cs_sr_reg32(b, IDVS, VARY_SIZE),
@@ -1316,7 +1357,7 @@ csf_emit_draw_id_register(struct panfrost_batch *batch, unsigned offset)
 {
    struct cs_builder *b = batch->csf.cs.builder;
    struct panfrost_context *ctx = batch->ctx;
-   struct panfrost_uncompiled_shader *vs = ctx->uncompiled[PIPE_SHADER_VERTEX];
+   struct panfrost_uncompiled_shader *vs = ctx->uncompiled[MESA_SHADER_VERTEX];
 
    if (!BITSET_TEST(vs->nir->info.system_values_read, SYSTEM_VALUE_DRAW_ID))
       return cs_undef();
@@ -1431,6 +1472,17 @@ get_panthor_group_priority(struct panfrost_context *ctx)
    return PANTHOR_GROUP_PRIORITY_MEDIUM;
 }
 
+static enum pipe_reset_status
+get_device_reset_status(struct pipe_context *pctx)
+{
+   struct panfrost_context *ctx = pan_context(pctx);
+   enum pipe_reset_status reset_status = ctx->csf.reset_status;
+
+   /* Reset the status before returning. */
+   ctx->csf.reset_status = PIPE_NO_RESET;
+   return reset_status;
+}
+
 int
 GENX(csf_init_context)(struct panfrost_context *ctx)
 {
@@ -1576,6 +1628,8 @@ GENX(csf_init_context)(struct panfrost_context *ctx)
 
    panfrost_bo_unreference(cs_bo);
 
+   ctx->csf.reset_status = PIPE_NO_RESET;
+   ctx->base.get_device_reset_status = get_device_reset_status;
    ctx->csf.is_init = true;
    return 0;
 err_g_submit:
@@ -1641,5 +1695,5 @@ GENX(csf_emit_write_timestamp)(struct panfrost_batch *batch,
    cs_move64_to(b, address, dst->plane.base + offset);
    cs_store_state(b, address, 0, MALI_CS_STATE_TIMESTAMP, cs_now());
 
-   panfrost_batch_write_rsrc(batch, dst, PIPE_SHADER_VERTEX);
+   panfrost_batch_write_rsrc(batch, dst, MESA_SHADER_VERTEX);
 }
