@@ -14,8 +14,6 @@
 #include "util/u_upload_mgr.h"
 #include "util/os_time.h"
 #include "util/hex.h"
-#include "vl/vl_decoder.h"
-#include "vl/vl_video_buffer.h"
 #include "radeon_video.h"
 #include "git_sha1.h"
 
@@ -158,7 +156,7 @@ void r600_draw_rectangle(struct blitter_context *blitter,
 	/* Upload vertices. The hw rectangle has only 3 vertices,
 	 * The 4th one is derived from the first 3.
 	 * The vertex specification should match u_blitter's vertex element state. */
-	u_upload_alloc(rctx->b.stream_uploader, 0, sizeof(float) * 24,
+	u_upload_alloc_ref(rctx->b.stream_uploader, 0, sizeof(float) * 24,
 		       rctx->screen->info.tcc_cache_line_size,
                        &offset, &buf, (void**)&vb);
 	if (!buf)
@@ -200,7 +198,7 @@ void r600_draw_rectangle(struct blitter_context *blitter,
 	vbuffer.buffer.resource = buf;
 	vbuffer.buffer_offset = offset;
 
-	util_set_vertex_buffers(&rctx->b, 1, false, &vbuffer);
+	rctx->b.set_vertex_buffers(&rctx->b, 1, &vbuffer);
 	util_draw_arrays_instanced(&rctx->b, R600_PRIM_RECTANGLE_LIST, 0, 3,
 				   0, num_instances);
 	pipe_resource_reference(&buf, NULL);
@@ -543,7 +541,7 @@ static bool r600_resource_commit(struct pipe_context *pctx,
 				 bool commit)
 {
 	struct r600_common_context *ctx = (struct r600_common_context *)pctx;
-	struct r600_resource *res = r600_resource(resource);
+	struct r600_resource *res = r600_as_resource(resource);
 
 	/*
 	 * Since buffer commitment changes cannot be pipelined, we need to
@@ -791,34 +789,6 @@ static const char* r600_get_name(struct pipe_screen* pscreen)
 	return rscreen->renderer_string;
 }
 
-static int r600_get_video_param(struct pipe_screen *screen,
-				enum pipe_video_profile profile,
-				enum pipe_video_entrypoint entrypoint,
-				enum pipe_video_cap param)
-{
-	switch (param) {
-	case PIPE_VIDEO_CAP_SUPPORTED:
-		return vl_profile_supported(screen, profile, entrypoint);
-	case PIPE_VIDEO_CAP_NPOT_TEXTURES:
-		return 1;
-	case PIPE_VIDEO_CAP_MAX_WIDTH:
-	case PIPE_VIDEO_CAP_MAX_HEIGHT:
-		return vl_video_buffer_max_size(screen);
-	case PIPE_VIDEO_CAP_PREFERRED_FORMAT:
-		return PIPE_FORMAT_NV12;
-	case PIPE_VIDEO_CAP_PREFERS_INTERLACED:
-		return false;
-	case PIPE_VIDEO_CAP_SUPPORTS_INTERLACED:
-		return false;
-	case PIPE_VIDEO_CAP_SUPPORTS_PROGRESSIVE:
-		return true;
-	case PIPE_VIDEO_CAP_MAX_LEVEL:
-		return vl_level_supported(screen, profile);
-	default:
-		return 0;
-	}
-}
-
 static uint64_t r600_get_timestamp(struct pipe_screen *screen)
 {
 	struct r600_common_screen *rscreen = (struct r600_common_screen*)screen;
@@ -937,21 +907,13 @@ struct pipe_resource *r600_resource_create_common(struct pipe_screen *screen,
 	}
 }
 
-static const struct nir_shader_compiler_options *
-r600_get_compiler_options(struct pipe_screen *screen,
-			  enum pipe_shader_type shader)
-{
-       struct r600_common_screen *rscreen = (struct r600_common_screen *)screen;
-       return &rscreen->nir_options;
-}
-
 extern bool r600_lower_to_scalar_instr_filter(const nir_instr *instr, const void *);
 
 static void r600_resource_destroy(struct pipe_screen *screen,
 				  struct pipe_resource *res)
 {
 	if (res->target == PIPE_BUFFER) {
-		if (r600_resource(res)->compute_global_bo)
+		if (r600_as_resource(res)->compute_global_bo)
 			r600_compute_global_buffer_destroy(screen, res);
 		else
 			r600_buffer_destroy(screen, res);
@@ -1043,7 +1005,6 @@ bool r600_common_screen_init(struct r600_common_screen *rscreen,
 	rscreen->b.get_disk_shader_cache = r600_get_disk_shader_cache;
 	rscreen->b.get_screen_fd = r600_get_screen_fd;
 	rscreen->b.get_timestamp = r600_get_timestamp;
-	rscreen->b.get_compiler_options = r600_get_compiler_options;
 	rscreen->b.fence_finish = r600_fence_finish;
 	rscreen->b.fence_reference = r600_fence_reference;
 	rscreen->b.resource_destroy = r600_resource_destroy;
@@ -1055,11 +1016,11 @@ bool r600_common_screen_init(struct r600_common_screen *rscreen,
 	if (rscreen->info.ip[AMD_IP_UVD].num_queues) {
 		rscreen->b.get_video_param = rvid_get_video_param;
 		rscreen->b.is_video_format_supported = rvid_is_format_supported;
-	} else {
-		rscreen->b.get_video_param = r600_get_video_param;
-		rscreen->b.is_video_format_supported = vl_video_buffer_is_format_supported;
 	}
 
+	for (unsigned i = 0; i <= MESA_SHADER_COMPUTE; i++)
+		rscreen->b.nir_options[i] = &rscreen->nir_options;
+
 	r600_init_screen_texture_functions(rscreen);
 	r600_init_screen_query_functions(rscreen);
 
@@ -1139,6 +1100,7 @@ bool r600_common_screen_init(struct r600_common_screen *rscreen,
 		.lower_flrp64 = true,
 		.lower_fdiv = true,
 		.lower_isign = true,
+		.lower_ineg = true,
 		.lower_fsign = true,
 		.lower_fmod = true,
 		.lower_uadd_carry = true,
@@ -1180,6 +1142,7 @@ bool r600_common_screen_init(struct r600_common_screen *rscreen,
 		.vectorize_tess_levels = 1,
 		.io_options = nir_io_mediump_is_32bit,
 		.vertex_id_zero_based = rscreen->info.gfx_level >= EVERGREEN,
+		.avoid_ternary_with_fabs = 1,
 	};
 
 	rscreen->nir_options = nir_options;
@@ -1220,8 +1183,8 @@ bool r600_common_screen_init(struct r600_common_screen *rscreen,
 	}
 
 	uint8_t indirect_supported_mask =
-		(uint8_t)BITFIELD_MASK(PIPE_SHADER_TYPES) &
-		~BITFIELD_BIT(PIPE_SHADER_FRAGMENT);
+		(uint8_t)BITFIELD_MASK(MESA_SHADER_STAGES) &
+		~BITFIELD_BIT(MESA_SHADER_FRAGMENT);
 	rscreen->nir_options.support_indirect_inputs = indirect_supported_mask;
 	rscreen->nir_options.support_indirect_outputs = indirect_supported_mask;
 
