@@ -350,7 +350,7 @@ static void set_random_image_attrs(struct pipe_resource *templ, bool allow_msaa,
       templ->nr_samples = 2 << (rand() % 3);
       break;
    default:
-      unreachable("invalid path");
+      UNREACHABLE("invalid path");
    }
 
    templ->usage = PIPE_USAGE_DEFAULT;
@@ -414,7 +414,19 @@ static void print_image_attrs(struct si_screen *sscreen, struct si_texture *tex)
 {
    const char *mode;
 
-   if (sscreen->info.gfx_level >= GFX9) {
+   if (sscreen->info.gfx_level >= GFX12) {
+      static const char *modes[32] = {
+         [ADDR3_LINEAR] = "LINEAR",
+         [ADDR3_256B_2D] = "256B_2D",
+         [ADDR3_4KB_2D] = "4KB_2D",
+         [ADDR3_64KB_2D] = "64KB_2D",
+         [ADDR3_256KB_2D] = "256KB_2D",
+         [ADDR3_4KB_3D] = "4KB_3D",
+         [ADDR3_64KB_3D] = "64KB_3D",
+         [ADDR3_256KB_3D] = "256KB_3D",
+      };
+      mode = modes[tex->surface.u.gfx9.swizzle_mode];
+   } else if (sscreen->info.gfx_level >= GFX9) {
       static const char *modes[32] = {
          [ADDR_SW_LINEAR] = "LINEAR",
          [ADDR_SW_4KB_S_X] = "4KB_S_X",
@@ -537,8 +549,10 @@ void si_test_image_copy_region(struct si_screen *sscreen)
 
       /* clear dst pixels */
       uint32_t zero = 0;
-      si_clear_buffer(sctx, dst, 0, sdst->surface.surf_size, &zero, 4, SI_OP_SYNC_BEFORE_AFTER,
-                      SI_COHERENCY_SHADER, SI_AUTO_SELECT_CLEAR_METHOD);
+      si_barrier_before_simple_buffer_op(sctx, 0, dst, NULL);
+      si_clear_buffer(sctx, dst, 0, sdst->surface.surf_size, &zero, 4,
+                      SI_AUTO_SELECT_CLEAR_METHOD, false);
+      si_barrier_after_simple_buffer_op(sctx, 0, dst, NULL);
 
       for (j = 0; j < num_partial_copies; j++) {
          int width, height, depth;
@@ -715,10 +729,16 @@ void si_test_blit(struct si_screen *sscreen, unsigned test_flags)
 
       /* clear dst pixels */
       uint32_t zero = 0;
+
+      /* Using 2 consecutive barriers calls results in a single merged barrier for both resources. */
+      si_barrier_before_simple_buffer_op(sctx, 0, gfx_dst, NULL);
+      si_barrier_before_simple_buffer_op(sctx, 0, comp_dst, NULL);
       si_clear_buffer(sctx, gfx_dst, 0, ((struct si_texture *)gfx_dst)->surface.surf_size, &zero,
-                      4, SI_OP_SYNC_BEFORE_AFTER, SI_COHERENCY_SHADER, SI_AUTO_SELECT_CLEAR_METHOD);
+                      4, SI_AUTO_SELECT_CLEAR_METHOD, false);
       si_clear_buffer(sctx, comp_dst, 0, ((struct si_texture *)comp_dst)->surface.surf_size, &zero,
-                      4, SI_OP_SYNC_BEFORE_AFTER, SI_COHERENCY_SHADER, SI_AUTO_SELECT_CLEAR_METHOD);
+                      4, SI_AUTO_SELECT_CLEAR_METHOD, false);
+      si_barrier_after_simple_buffer_op(sctx, 0, gfx_dst, NULL);
+      si_barrier_after_simple_buffer_op(sctx, 0, comp_dst, NULL);
 
       /* TODO: These two fix quite a lot of BCn cases. */
       /*si_clear_buffer(sctx, gfx_src, 0, ((struct si_texture *)gfx_src)->surface.surf_size, &zero,
@@ -927,9 +947,9 @@ void si_test_blit(struct si_screen *sscreen, unsigned test_flags)
 
       bool success;
       if (only_cb_resolve)
-         success = si_msaa_resolve_blit_via_CB(ctx, &info);
+         success = si_msaa_resolve_blit_via_CB(ctx, &info, false);
       else
-         success = si_compute_blit(sctx, &info, true);
+         success = si_compute_blit(sctx, &info, NULL, 0, 0, false);
 
       if (success) {
          printf(" %-7s", only_cb_resolve ? "resolve" : "comp");
