@@ -37,25 +37,11 @@ struct pipe_video_buffer *si_video_buffer_create_with_modifiers(struct pipe_cont
    for (i = 0; i < modifiers_count; i++) {
       uint64_t mod = modifiers[i];
 
-      if (ac_modifier_has_dcc(mod)) {
-         /* DCC not supported */
-         if (sscreen->info.gfx_level < GFX12)
-            continue;
-
-         /* Filter out non displayable modifiers */
-         if (sscreen->info.drm_minor < 63 &&
-             AMD_FMT_MOD_GET(DCC_MAX_COMPRESSED_BLOCK, mod) == AMD_FMT_MOD_DCC_BLOCK_256B)
-            continue;
-      }
+      if (!ac_modifier_supports_video(&sscreen->info, mod))
+         continue;
 
       if (mod != DRM_FORMAT_MOD_LINEAR) {
-         /* Linear only for UVD/VCE and VCN 1.0 */
-         if (sscreen->info.vcn_ip_version < VCN_2_0_0)
-            continue;
-
-         /* Only "S" swizzle modes supported */
-         if (sscreen->info.vcn_ip_version < VCN_2_2_0 &&
-             AMD_FMT_MOD_GET(TILE, mod) != AMD_FMT_MOD_TILE_GFX9_64K_S)
+         if (sscreen->multimedia_debug_flags & DBG(NO_VIDEO_TILING))
             continue;
 
          if (!sscreen->info.has_image_opcodes)
@@ -125,8 +111,6 @@ static struct pb_buffer_lean *si_uvd_set_dtb(struct ruvd_msg *msg, struct vl_vid
    enum ruvd_surface_type type =
       (sscreen->info.gfx_level >= GFX9) ? RUVD_SURFACE_TYPE_GFX9 : RUVD_SURFACE_TYPE_LEGACY;
 
-   msg->body.decode.dt_field_mode = buf->base.interlaced;
-
    si_uvd_set_dt_surfaces(msg, &luma->surface, (chroma) ? &chroma->surface : NULL, type);
 
    return luma->buffer.buf;
