@@ -30,7 +30,7 @@ svga_set_vertex_buffers(struct pipe_context *pipe,
 
    util_set_vertex_buffers_count(svga->curr.vb,
                                  &svga->curr.num_vertex_buffers,
-                                 buffers, count, true);
+                                 buffers, count);
 
    svga->dirty |= SVGA_NEW_VBUFFER;
 }
