@@ -119,7 +119,9 @@ pipeline_statistic_convert(enum pipe_statistics_query_index idx)
       [PIPE_STAT_QUERY_PS_INVOCATIONS] = VK_QUERY_PIPELINE_STATISTIC_FRAGMENT_SHADER_INVOCATIONS_BIT,
       [PIPE_STAT_QUERY_HS_INVOCATIONS] = VK_QUERY_PIPELINE_STATISTIC_TESSELLATION_CONTROL_SHADER_PATCHES_BIT,
       [PIPE_STAT_QUERY_DS_INVOCATIONS] = VK_QUERY_PIPELINE_STATISTIC_TESSELLATION_EVALUATION_SHADER_INVOCATIONS_BIT,
-      [PIPE_STAT_QUERY_CS_INVOCATIONS] = VK_QUERY_PIPELINE_STATISTIC_COMPUTE_SHADER_INVOCATIONS_BIT
+      [PIPE_STAT_QUERY_CS_INVOCATIONS] = VK_QUERY_PIPELINE_STATISTIC_COMPUTE_SHADER_INVOCATIONS_BIT,
+      [PIPE_STAT_QUERY_MS_INVOCATIONS] = VK_QUERY_PIPELINE_STATISTIC_MESH_SHADER_INVOCATIONS_BIT_EXT,
+      [PIPE_STAT_QUERY_TS_INVOCATIONS] = VK_QUERY_PIPELINE_STATISTIC_TASK_SHADER_INVOCATIONS_BIT_EXT,
    };
    assert(idx < ARRAY_SIZE(map));
    return map[idx];
@@ -172,7 +174,7 @@ zink_context_destroy_query_pools(struct zink_context *ctx)
 }
 
 static struct zink_query_pool *
-find_or_allocate_qp(struct zink_context *ctx, struct zink_query *q, unsigned idx)
+find_or_allocate_qp(struct zink_context *ctx, struct zink_query *q, unsigned idx, bool *vkq_needs_reset)
 {
    VkQueryPipelineStatisticFlags pipeline_stats = 0;
    if (q->type == PIPE_QUERY_PRIMITIVES_GENERATED && q->vkqtype != VK_QUERY_TYPE_PRIMITIVES_GENERATED_EXT)
@@ -208,6 +210,10 @@ find_or_allocate_qp(struct zink_context *ctx, struct zink_query *q, unsigned idx
 
    VkQueryPoolCreateInfo pool_create = {0};
    pool_create.sType = VK_STRUCTURE_TYPE_QUERY_POOL_CREATE_INFO;
+   if (screen->info.have_KHR_maintenance9) {
+      pool_create.flags = VK_QUERY_POOL_CREATE_RESET_BIT_KHR;
+      *vkq_needs_reset = false;
+   }
    pool_create.queryType = vk_query_type;
    pool_create.queryCount = NUM_QUERIES;
    pool_create.pipelineStatistics = pipeline_stats;
@@ -276,7 +282,7 @@ get_num_results(struct zink_query *q)
    default:
       debug_printf("unknown query: %s\n",
                    util_str_query_type(q->type, true));
-      unreachable("zink: unknown query type");
+      UNREACHABLE("zink: unknown query type");
    }
 }
 
@@ -324,7 +330,7 @@ convert_query_type(struct zink_screen *screen, enum pipe_query_type query_type,
    default:
       debug_printf("unknown query: %s\n",
                    util_str_query_type(query_type, true));
-      unreachable("zink: unknown query type");
+      UNREACHABLE("zink: unknown query type");
    }
 }
 
@@ -465,16 +471,17 @@ query_pool_get_range(struct zink_context *ctx, struct zink_query *q)
       /* try and find the active query for this */
       struct zink_vk_query *vkq;
       int xfb_idx = num_queries == 4 ? i : q->index;
+      bool vkq_needs_reset = true;
       if ((q->vkqtype == VK_QUERY_TYPE_TRANSFORM_FEEDBACK_STREAM_EXT ||
            (pool_idx == 1)) && ctx->curr_xfb_queries[xfb_idx]) {
          vkq = ctx->curr_xfb_queries[xfb_idx];
          vkq->refcount++;
          vkq->pool->refcount++;
       } else {
-         struct zink_query_pool *pool = find_or_allocate_qp(ctx, q, pool_idx);
+         struct zink_query_pool *pool = find_or_allocate_qp(ctx, q, pool_idx, &vkq_needs_reset);
          if (pool->last_range == NUM_QUERIES) {
             list_del(&pool->list);
-            pool = find_or_allocate_qp(ctx, q, pool_idx);
+            pool = find_or_allocate_qp(ctx, q, pool_idx, &vkq_needs_reset);
          }
          vkq = CALLOC_STRUCT(zink_vk_query);
          if (!vkq) {
@@ -484,7 +491,7 @@ query_pool_get_range(struct zink_context *ctx, struct zink_query *q)
 
          pool->refcount++;
          vkq->refcount = 1;
-         vkq->needs_reset = true;
+         vkq->needs_reset = vkq_needs_reset;
          vkq->pool = pool;
          vkq->started = false;
          vkq->query_id = pool->last_range++;
@@ -646,7 +653,7 @@ check_query_results(struct zink_query *query, union pipe_query_result *result,
       default:
          debug_printf("unhandled query type: %s\n",
                       util_str_query_type(query->type, true));
-         unreachable("unexpected query type");
+         UNREACHABLE("unexpected query type");
       }
    }
 }
