@@ -34,7 +34,7 @@
 #include <string.h>
 #include "util/compiler.h"
 #include "GL/glx.h"
-#include "glapi/glapi.h"
+#include "glapi/glapi/glapi.h"
 
 
 struct name_address_pair {
