@@ -262,6 +262,34 @@ StringFromCodecAPI( const GUID *Api )
    {
       return "CODECAPI_AVEncVideoOutputBitsUsedMapBlockSize";
    }
+   else if( *Api == CODECAPI_AVEncVideoSatdMapBlockSize )
+   {
+      return "CODECAPI_AVEncVideoSatdMapBlockSize";
+   }
+   else if( *Api == CODECAPI_AVEncVideoRateControlFramePreAnalysis )
+   {
+      return "CODECAPI_AVEncVideoRateControlFramePreAnalysis";
+   }
+   else if( *Api == CODECAPI_AVEncVideoRateControlFramePreAnalysisExternalReconDownscale )
+   {
+      return "CODECAPI_AVEncVideoRateControlFramePreAnalysisExternalReconDownscale";
+   }
+   else if( *Api == CODECAPI_AVEncVideoInputDeltaQPBlockSettings )
+   {
+      return "CODECAPI_AVEncVideoInputDeltaQPBlockSettings";
+   }
+   else if( *Api == CODECAPI_AVEncVideoInputAbsoluteQPBlockSettings )
+   {
+      return "CODECAPI_AVEncVideoInputAbsoluteQPBlockSettings";
+   }
+   else if( *Api == CODECAPI_AVEncWorkGlobalPriority )
+   {
+      return "CODECAPI_AVEncWorkGlobalPriority";
+   }
+   else if( *Api == CODECAPI_AVEncWorkProcessPriority )
+   {
+      return "CODECAPI_AVEncWorkProcessPriority";
+   }
    return "Unknown CodecAPI";
 }
 
@@ -284,6 +312,10 @@ StringFromVariant( VARIANT *Value )
    {
       return std::to_string( Value->boolVal );
    }
+   else if( ( Value->vt & ( VT_ARRAY | VT_UI1 ) ) == ( VT_ARRAY | VT_UI1 ) && Value->parray )
+   {
+      return "Array Blob mapping";
+   }
    return "Unsupported Variant";
 }
 
@@ -310,14 +342,23 @@ CDX12EncHMFT::IsSupported( const GUID *Api )
        *Api == CODECAPI_AVEncVideoSelectLayer || *Api == CODECAPI_AVEncVideoEncodeFrameTypeQP ||
        *Api == CODECAPI_AVEncSliceControlMode || *Api == CODECAPI_AVEncSliceControlSize ||
        *Api == CODECAPI_AVEncVideoMaxNumRefFrame || *Api == CODECAPI_AVEncVideoMeanAbsoluteDifference ||
-       *Api == CODECAPI_AVEncVideoMaxQP || *Api == CODECAPI_AVEncVideoGradualIntraRefresh || *Api == CODECAPI_AVScenarioInfo ||
-       *Api == CODECAPI_AVEncVideoROIEnabled || *Api == CODECAPI_AVEncVideoLTRBufferControl ||
-       *Api == CODECAPI_AVEncVideoMarkLTRFrame || *Api == CODECAPI_AVEncVideoUseLTRFrame )
+       *Api == CODECAPI_AVEncVideoMaxQP || *Api == CODECAPI_AVScenarioInfo || *Api == CODECAPI_AVEncVideoROIEnabled ||
+       *Api == CODECAPI_AVEncVideoLTRBufferControl || *Api == CODECAPI_AVEncVideoMarkLTRFrame ||
+       *Api == CODECAPI_AVEncVideoUseLTRFrame )
    {
       hr = S_OK;
       return hr;
    }
 
+   if( m_EncoderCapabilities.m_HWSupportsIntraRefreshModes != PIPE_VIDEO_ENC_INTRA_REFRESH_NONE )
+   {
+      if( *Api == CODECAPI_AVEncVideoGradualIntraRefresh )
+      {
+         hr = S_OK;
+         return hr;
+      }
+   }
+
    if( m_EncoderCapabilities.m_HWSupportDirtyRects.bits.supports_info_type_dirty )
    {
       if( *Api == CODECAPI_AVEncVideoDirtyRectEnabled )
@@ -363,6 +404,60 @@ CDX12EncHMFT::IsSupported( const GUID *Api )
       }
    }
 
+   if( m_EncoderCapabilities.m_HWSupportStatsSATDMapOutput.bits.supported )
+   {
+      if( *Api == CODECAPI_AVEncVideoSatdMapBlockSize )
+      {
+         hr = S_OK;
+         return hr;
+      }
+   }
+
+   if( m_EncoderCapabilities.m_TwoPassSupport.bits.supports_two_pass )
+   {
+      if( *Api == CODECAPI_AVEncVideoRateControlFramePreAnalysis )
+      {
+         hr = S_OK;
+         return hr;
+      }
+   }
+
+   if( ( *Api == CODECAPI_AVEncWorkGlobalPriority ) || ( *Api == CODECAPI_AVEncWorkProcessPriority ) )
+   {
+      if( m_EncoderCapabilities.m_bHWSupportsQueuePriorityManagement )
+      {
+         hr = S_OK;
+         return hr;
+      }
+   }
+
+   if( m_EncoderCapabilities.m_TwoPassSupport.bits.supports_1pass_recon_writing_skip )
+   {
+      if( *Api == CODECAPI_AVEncVideoRateControlFramePreAnalysisExternalReconDownscale )
+      {
+         hr = S_OK;
+         return hr;
+      }
+   }
+
+   if( m_EncoderCapabilities.m_HWSupportsVideoEncodeROI.bits.roi_rc_qp_delta_support )
+   {
+      if( *Api == CODECAPI_AVEncVideoInputDeltaQPBlockSettings )
+      {
+         hr = S_OK;
+         return hr;
+      }
+   }
+
+   if( m_EncoderCapabilities.m_HWSupportSpatialAdaptiveQuantization.bits.max_spatial_adaptive_quantization_strength > 0 )
+   {
+      if( *Api == CODECAPI_AVEncVideoEnableSpatialAdaptiveQuantization )
+      {
+         hr = S_OK;
+         return hr;
+      }
+   }
+
 done:
    return hr;
 }
@@ -571,17 +666,31 @@ CDX12EncHMFT::GetParameterValues( const GUID *Api, VARIANT **Values, ULONG *Valu
       }
       else if( *Api == CODECAPI_AVEncVideoGradualIntraRefresh )
       {
-         // Our HMFT doesn't support HMFT_INTRA_REFRESH_MODE_PERIODIC
-         *ValuesCount = 2;
-         CHECKNULL_GOTO( *Values = (VARIANT *) CoTaskMemAlloc( ( *ValuesCount ) * sizeof( VARIANT ) ), E_OUTOFMEMORY, done );
-         ( *Values )[0].vt = VT_UI4;
-         ( *Values )[0].ulVal = HMFT_INTRA_REFRESH_MODE_NONE;
-         ( *Values )[1].vt = VT_UI4;
-         ( *Values )[1].ulVal = HMFT_INTRA_REFRESH_MODE_CONTINUAL;
+
+         *ValuesCount = 0;   // Assume no support unless reported by driver's capabilities
+
+         if( m_EncoderCapabilities.m_HWSupportsIntraRefreshModes != PIPE_VIDEO_ENC_INTRA_REFRESH_NONE )
+         {
+            // Our HMFT doesn't support HMFT_INTRA_REFRESH_MODE_PERIODIC
+            *ValuesCount = 2;
+            CHECKNULL_GOTO( *Values = (VARIANT *) CoTaskMemAlloc( ( *ValuesCount ) * sizeof( VARIANT ) ), E_OUTOFMEMORY, done );
+            ( *Values )[0].vt = VT_UI4;
+            ( *Values )[0].ulVal = HMFT_INTRA_REFRESH_MODE_NONE;
+            ( *Values )[1].vt = VT_UI4;
+            ( *Values )[1].ulVal = HMFT_INTRA_REFRESH_MODE_CONTINUAL;
+         }
       }
       else if( *Api == CODECAPI_AVEncVideoLTRBufferControl )
       {
-         *ValuesCount = m_EncoderCapabilities.m_uiMaxHWSupportedLongTermReferences + 1;
+         // reserve one dpb spot for short term reference frame.
+         // when m_EncoderCapabilities.m_uiMaxHWSupportedLongTermReferences = m_EncoderCapabilities.m_uiMaxHWSupportedDPBCapacity,
+         // we subtract one.
+         ULONG numSupportedLTR = m_EncoderCapabilities.m_uiMaxHWSupportedLongTermReferences;
+         if( numSupportedLTR > 0 && numSupportedLTR == m_EncoderCapabilities.m_uiMaxHWSupportedDPBCapacity )
+         {
+            numSupportedLTR -= 1;
+         }
+         *ValuesCount = numSupportedLTR + 1;
          CHECKNULL_GOTO( *Values = (VARIANT *) CoTaskMemAlloc( ( *ValuesCount ) * sizeof( VARIANT ) ), E_OUTOFMEMORY, done );
          for( ULONG i = 0; i < *ValuesCount; i++ )
          {
@@ -813,6 +922,67 @@ CDX12EncHMFT::GetValue( const GUID *Api, VARIANT *Value )
             (ULONG) ( 1 << m_EncoderCapabilities.m_HWSupportStatsRCBitAllocationMapOutput.bits.log2_values_block_size ) :
             0;
    }
+   else if( *Api == CODECAPI_AVEncVideoSatdMapBlockSize )
+   {
+      Value->vt = VT_UI4;
+      Value->ulVal = m_EncoderCapabilities.m_HWSupportStatsSATDMapOutput.bits.supported ?
+                        (ULONG) ( 1 << m_EncoderCapabilities.m_HWSupportStatsSATDMapOutput.bits.log2_values_block_size ) :
+                        0;
+   }
+   else if( *Api == CODECAPI_AVEncVideoRateControlFramePreAnalysis )
+   {
+      Value->vt = VT_BOOL;
+      Value->boolVal = m_bRateControlFramePreAnalysis ? VARIANT_TRUE : VARIANT_FALSE;
+   }
+   else if( *Api == CODECAPI_AVEncVideoRateControlFramePreAnalysisExternalReconDownscale )
+   {
+      Value->vt = VT_BOOL;
+      Value->boolVal = m_bRateControlFramePreAnalysisExternalReconDownscale ? VARIANT_TRUE : VARIANT_FALSE;
+   }
+   else if( *Api == CODECAPI_AVEncWorkGlobalPriority )
+   {
+      Value->vt = VT_UI4;
+      Value->ulVal = (UINT32) m_WorkGlobalPriority;
+   }
+   else if( *Api == CODECAPI_AVEncWorkProcessPriority )
+   {
+      Value->vt = VT_UI4;
+      Value->ulVal = (UINT32) m_WorkProcessPriority;
+   }
+   else if( *Api == CODECAPI_AVEncVideoInputDeltaQPBlockSettings )
+   {
+      InputQPSettings hevcDeltaQPSettings;
+      hevcDeltaQPSettings.minBlockSize = 1 << m_EncoderCapabilities.m_HWSupportsVideoEncodeROI.bits.log2_roi_min_block_pixel_size;
+      hevcDeltaQPSettings.maxBlockSize = hevcDeltaQPSettings.minBlockSize;
+      hevcDeltaQPSettings.stepsBlockSize = 1;
+      hevcDeltaQPSettings.dataType = CODEC_API_QP_MAP_INT8;
+      hevcDeltaQPSettings.minValue = static_cast<INT16>( m_uiMinQP );
+      hevcDeltaQPSettings.maxValue = static_cast<INT16>( m_uiMaxQP );
+      hevcDeltaQPSettings.step = 1;
+
+      SAFEARRAYBOUND bound = { static_cast<ULONG>( sizeof( hevcDeltaQPSettings ) ),
+                               static_cast<LONG>( 0 ) };       // cElements , lower bound
+      SAFEARRAY *psa = SafeArrayCreate( VT_UI1, 1, &bound );   // dims == 1
+      if( !psa )
+      {
+         hr = E_OUTOFMEMORY;
+         CHECKHR_GOTO( hr, done );
+      }
+      void *pData = nullptr;
+      hr = SafeArrayAccessData( psa, &pData );
+      if( SUCCEEDED( hr ) && pData )
+      {
+         memcpy( pData, &hevcDeltaQPSettings, sizeof( hevcDeltaQPSettings ) );
+         SafeArrayUnaccessData( psa );
+      }
+      else
+      {
+         SafeArrayDestroy( psa );
+         CHECKHR_GOTO( hr, done );
+      }
+      Value->vt = VT_ARRAY | VT_UI1;
+      Value->parray = psa;
+   }
    else
    {
       hr = E_NOTIMPL;
@@ -1384,11 +1554,46 @@ CDX12EncHMFT::SetValue( const GUID *Api, VARIANT *Value )
    }
    else if( *Api == CODECAPI_AVEncVideoGradualIntraRefresh )
    {
-      debug_printf( "[dx12 hmft 0x%p] SET CODECAPI_AVEncVideoGradualIntraRefresh - %u\n", this, Value->ulVal );
-      if( Value->vt != VT_UI4 || ( ( Value->ulVal & 0xFFFF ) >= HMFT_INTRA_REFRESH_MODE_MAX ) )
+      UINT uiIntraRefreshMode = Value->ulVal & 0xFFFF;
+      UINT uiIntraRefreshSize = Value->ulVal >> 16 & 0xFFFF;
+
+      if( ( uiIntraRefreshMode != 0 ) &&
+          ( m_EncoderCapabilities.m_HWSupportsIntraRefreshModes == PIPE_VIDEO_ENC_INTRA_REFRESH_NONE ) )
       {
+         debug_printf( "[dx12 hmft 0x%p] User tried to set CODECAPI_AVEncVideoGradualIntraRefresh with mode %u, but this "
+                       "encoder does NOT support intra refresh.",
+                       this,
+                       uiIntraRefreshMode );
          CHECKHR_GOTO( E_INVALIDARG, done );
       }
+
+      if( uiIntraRefreshSize > m_EncoderCapabilities.m_uiMaxHWSupportedIntraRefreshSize )
+      {
+         debug_printf( "[dx12 hmft 0x%p] User tried to set CODECAPI_AVEncVideoGradualIntraRefresh with size %u, but this "
+                       "exceeds the maximum supported by hardware %u.",
+                       this,
+                       uiIntraRefreshSize,
+                       m_EncoderCapabilities.m_uiMaxHWSupportedIntraRefreshSize );
+         CHECKHR_GOTO( E_INVALIDARG, done );
+      }
+
+      debug_printf( "[dx12 hmft 0x%p] SET CODECAPI_AVEncVideoGradualIntraRefresh - %u\n", this, Value->ulVal );
+      if( Value->vt != VT_UI4 )
+      {
+         debug_printf( "[dx12 hmft 0x%p] User tried to set CODECAPI_AVEncVideoGradualIntraRefresh with invalid vt %u\n",
+                       this,
+                       Value->vt );
+         CHECKHR_GOTO( E_INVALIDARG, done );
+      }
+
+      if( uiIntraRefreshMode >= HMFT_INTRA_REFRESH_MODE_MAX )
+      {
+         debug_printf( "[dx12 hmft 0x%p] User tried to set CODECAPI_AVEncVideoGradualIntraRefresh with invalid mode %u\n",
+                       this,
+                       uiIntraRefreshMode );
+         CHECKHR_GOTO( E_INVALIDARG, done );
+      }
+
       m_uiIntraRefreshMode = Value->ulVal & 0xFFFF;
       m_uiIntraRefreshSize = Value->ulVal >> 16 & 0xFFFF;
    }
@@ -1456,6 +1661,36 @@ CDX12EncHMFT::SetValue( const GUID *Api, VARIANT *Value )
       }
       m_bVideoEnableSpatialAdaptiveQuantization = Value->ulVal ? TRUE : FALSE;
    }
+   else if( *Api == CODECAPI_AVEncWorkProcessPriority )
+   {
+      debug_printf( "[dx12 hmft 0x%p] SET CODECAPI_AVEncWorkProcessPriority - %u\n", this, Value->ulVal );
+      if( Value->vt != VT_UI4 )
+      {
+         CHECKHR_GOTO( E_INVALIDARG, done );
+      }
+
+      if( !m_EncoderCapabilities.m_bHWSupportsQueuePriorityManagement )
+      {
+         CHECKHR_GOTO( E_INVALIDARG, done );
+      }
+      m_WorkProcessPriority = (D3D12_COMMAND_QUEUE_PROCESS_PRIORITY) ( Value->ulVal );
+      m_bWorkProcessPrioritySet = TRUE;
+   }
+   else if( *Api == CODECAPI_AVEncWorkGlobalPriority )
+   {
+      debug_printf( "[dx12 hmft 0x%p] SET CODECAPI_AVEncWorkGlobalPriority - %u\n", this, Value->ulVal );
+      if( Value->vt != VT_UI4 )
+      {
+         CHECKHR_GOTO( E_INVALIDARG, done );
+      }
+
+      if( !m_EncoderCapabilities.m_bHWSupportsQueuePriorityManagement )
+      {
+         CHECKHR_GOTO( E_INVALIDARG, done );
+      }
+      m_WorkGlobalPriority = (D3D12_COMMAND_QUEUE_GLOBAL_PRIORITY) Value->ulVal;
+      m_bWorkGlobalPrioritySet = TRUE;
+   }
    else if( *Api == CODECAPI_AVEncVideoOutputQPMapBlockSize )
    {
       debug_printf( "[dx12 hmft 0x%p] SET CODECAPI_AVEncVideoOutputQPMapBlockSize - %u\n", this, Value->ulVal );
@@ -1509,6 +1744,71 @@ CDX12EncHMFT::SetValue( const GUID *Api, VARIANT *Value )
       }
       m_uiVideoOutputBitsUsedMapBlockSize = Value->ulVal;
    }
+   else if( *Api == CODECAPI_AVEncVideoSatdMapBlockSize )
+   {
+      debug_printf( "[dx12 hmft 0x%p] SET CODECAPI_AVEncVideoSatdMapBlockSize - %u\n", this, Value->ulVal );
+      if( Value->vt != VT_UI4 )
+      {
+         CHECKHR_GOTO( E_INVALIDARG, done );
+      }
+      if( !m_EncoderCapabilities.m_HWSupportStatsSATDMapOutput.bits.supported && Value->ulVal )
+      {
+         MFE_ERROR( "[dx12 hmft 0x%p] User tried to set CODECAPI_AVEncVideoSatdMapBlockSize as nonzero: %d, but this encoder "
+                    "does NOT support this feature.",
+                    this,
+                    Value->ulVal );
+         CHECKHR_GOTO( E_INVALIDARG, done );
+      }
+      if( m_EncoderCapabilities.m_HWSupportStatsSATDMapOutput.bits.supported && Value->ulVal &&
+          ( Value->ulVal != (ULONG) ( 1 << m_EncoderCapabilities.m_HWSupportStatsSATDMapOutput.bits.log2_values_block_size ) ) )
+      {
+         MFE_ERROR( "[dx12 hmft 0x%p] User MUST set CODECAPI_AVEncVideoSatdMapBlockSize as %d to enable this feature, or 0 to "
+                    "disable this feature.",
+                    this,
+                    ( 1 << m_EncoderCapabilities.m_HWSupportStatsSATDMapOutput.bits.log2_values_block_size ) );
+         CHECKHR_GOTO( E_INVALIDARG, done );
+      }
+      m_uiVideoSatdMapBlockSize = Value->ulVal;
+   }
+   else if( *Api == CODECAPI_AVEncVideoRateControlFramePreAnalysis )
+   {
+      debug_printf( "[dx12 hmft 0x%p] SET CODECAPI_AVEncVideoRateControlFramePreAnalysis - %s\n",
+                    this,
+                    (bool) Value->boolVal ? "true" : "false" );
+
+      if( Value->vt != VT_UI4 && Value->vt != VT_BOOL )
+      {
+         CHECKHR_GOTO( E_INVALIDARG, done );
+      }
+      if( !m_EncoderCapabilities.m_TwoPassSupport.bits.supports_two_pass && Value->boolVal )
+      {
+         MFE_ERROR( "[dx12 hmft 0x%p] User tried to set CODECAPI_AVEncVideoRateControlFramePreAnalysis, but this encoder does NOT "
+                    "support this feature.",
+                    this );
+         CHECKHR_GOTO( E_INVALIDARG, done );
+      }
+      m_bRateControlFramePreAnalysis = Value->boolVal == VARIANT_TRUE ? TRUE : FALSE;
+   }
+   else if( *Api == CODECAPI_AVEncVideoRateControlFramePreAnalysisExternalReconDownscale )
+   {
+      debug_printf( "[dx12 hmft 0x%p] SET CODECAPI_AVEncVideoRateControlFramePreAnalysisExternalReconDownscale - %s\n",
+                    this,
+                    (bool) Value->boolVal ? "true" : "false" );
+
+      if( Value->vt != VT_UI4 && Value->vt != VT_BOOL )
+      {
+         CHECKHR_GOTO( E_INVALIDARG, done );
+      }
+      if( !m_EncoderCapabilities.m_TwoPassSupport.bits.supports_1pass_recon_writing_skip && Value->boolVal )
+      {
+         MFE_ERROR( "[dx12 hmft 0x%p] User tried to set CODECAPI_AVEncVideoRateControlFramePreAnalysisExternalReconDownscale, but "
+                    "this encoder does NOT "
+                    "support this feature.",
+                    this );
+         CHECKHR_GOTO( E_INVALIDARG, done );
+      }
+      m_bRateControlFramePreAnalysisExternalReconDownscale = Value->boolVal == VARIANT_TRUE ? TRUE : FALSE;
+   }
    else
    {
       CHECKHR_GOTO( E_NOTIMPL, done );
