@@ -439,7 +439,12 @@ CDX12EncHMFT::OnInputTypeChanged()
       CHECKHR_GOTO( hr, done );
    }
 
-   CHECKHR_GOTO( ConfigureSampleAllocator(), done );
+   // when input type changes, clear m_spVideoSampleAllocator so we can recreate it later.
+   if( m_spVideoSampleAllocator )
+   {
+      m_spVideoSampleAllocator->UninitializeSampleAllocator();
+      m_spVideoSampleAllocator = nullptr;
+   }
 
 done:
    if( hr != S_OK )
@@ -542,7 +547,6 @@ CDX12EncHMFT::OnOutputTypeChanged()
    {
       CleanupEncoder();
    }
-   CHECKHR_GOTO( InitializeEncoder( m_outputPipeProfile, m_uiOutputWidth, m_uiOutputHeight ), done );
 
    if( m_gpuFeatureFlags.m_bDisableAsync )
    {
@@ -557,6 +561,21 @@ CDX12EncHMFT::OnOutputTypeChanged()
       }
    }
 
+   CHECKHR_GOTO( InitializeEncoder( m_outputPipeProfile, m_uiOutputWidth, m_uiOutputHeight ), done );
+
+   if( bResolutionChange )
+   {
+      ConfigureMapSampleAllocatorHelper( m_spSATDMapAllocator,
+                                         m_EncoderCapabilities.m_HWSupportStatsSATDMapOutput,
+                                         m_uiVideoSatdMapBlockSize,
+                                         m_bUseSATDMapAllocator );
+
+      ConfigureMapSampleAllocatorHelper( m_spBitsusedMapAllocator,
+                                         m_EncoderCapabilities.m_HWSupportStatsRCBitAllocationMapOutput,
+                                         m_uiVideoOutputBitsUsedMapBlockSize,
+                                         m_bUseBitsusedMapAllocator );
+   }
+
    // Indicate that we'll be adding MF_NALU_LENGTH_INFORMATION on each output sample that comes
    // MFSampleExtension_NALULengthInfo is equivalent to MF_NALU_LENGTH_INFORMATION
    (void) m_spOutputType->SetUINT32( MF_NALU_LENGTH_SET, 1 );
@@ -796,6 +815,49 @@ done:
    return hr;
 }
 
+// Helper function to calculate the max output bitstream size based on width, height, and format
+// This function uses an approach based on common video formats and their typical compression ratios
+UINT
+CalculateMaxOutputBitstreamSize( UINT uiWidth, UINT uiHeight, enum pipe_format format )
+{
+   assert( ( uiHeight > 16 ) && ( uiWidth > 16 ) && ( format != PIPE_FORMAT_NONE ) );
+
+   const UINT MIN_BUFFER_SIZE = 128 * 128 * 2;       // Minimum buffer size for very small frames: 128x128 pixels at 2 bytes/pixel
+   const UINT MAX_BUFFER_SIZE = 20 * 1024 * 1024;    // Maximum buffer size of 20MB
+   const float EXPECTED_COMPRESSION_FACTOR = 2.0f;   // Assume 50% of calculated size after compression of raw pixel sizes
+
+   UINT alignedWidth = ( uiWidth + 15 ) & ~15;
+   UINT alignedHeight = ( uiHeight + 15 ) & ~15;
+   UINT bufferSize = 0;
+   switch( format )
+   {
+      case PIPE_FORMAT_NV12:
+         // NV12: Y plane (1 byte/pixel) + UV plane (1/2 byte/pixel) = 1.5 bytes/pixel
+         bufferSize = alignedWidth * alignedHeight * 3 / 2;
+         break;
+      case PIPE_FORMAT_P010:
+         // P010: Y plane (2 bytes/pixel) + UV plane (1 byte/pixel) = 3 bytes/pixel
+         bufferSize = alignedWidth * alignedHeight * 3;
+         break;
+      case PIPE_FORMAT_AYUV:
+         // AYUV: 4 bytes/pixel
+         bufferSize = alignedWidth * alignedHeight * 4;
+         break;
+      default:
+         // Fallback formula for other formats: assume 15 bits/pixel (1.875 bytes/pixel)
+         bufferSize = ( ( ( alignedHeight ) * ( alignedWidth ) * 15 ) >> 3 );
+         break;
+   }
+
+   // Apply EXPECTED_COMPRESSION_FACTOR constant (% of calculated size)
+   bufferSize = static_cast<UINT>( std::ceil( bufferSize / EXPECTED_COMPRESSION_FACTOR ) );
+
+   // Clamp buffer size between minimum and maximum limits
+   bufferSize = std::max( MIN_BUFFER_SIZE, std::min( bufferSize, MAX_BUFFER_SIZE ) );
+
+   return bufferSize;
+}
+
 // internal function to initialize the encoder
 HRESULT
 CDX12EncHMFT::InitializeEncoder( pipe_video_profile videoProfile, UINT32 Width, UINT32 Height )
@@ -859,37 +921,57 @@ CDX12EncHMFT::InitializeEncoder( pipe_video_profile videoProfile, UINT32 Width,
          CHECKHR_GOTO( E_INVALIDARG, done );
       }
 
-#if ENCODE_WITH_TWO_PASS
-      encoderSettings.two_pass.enable = 1;
+      if( m_bRateControlFramePreAnalysis )
+      {
+         encoderSettings.two_pass.enable = 1;
 #if ENCODE_WITH_TWO_PASS_LOWEST_RES
-      encoderSettings.two_pass.pow2_downscale_factor = m_EncoderCapabilities.m_TwoPassSupport.bits.max_pow2_downscale_factor;
+         encoderSettings.two_pass.pow2_downscale_factor = m_EncoderCapabilities.m_TwoPassSupport.bits.max_pow2_downscale_factor;
 #else
-      encoderSettings.two_pass.pow2_downscale_factor = m_EncoderCapabilities.m_TwoPassSupport.bits.min_pow2_downscale_factor;
+         encoderSettings.two_pass.pow2_downscale_factor = m_EncoderCapabilities.m_TwoPassSupport.bits.min_pow2_downscale_factor;
 #endif   // ENCODE_WITH_TWO_PASS_LOWEST_RES
 
-#if ENCODE_WITH_TWO_PASS_EXTERNAL_DPB_RECON_SCALE
-      encoderSettings.two_pass.skip_1st_dpb_texture = m_EncoderCapabilities.m_TwoPassSupport.bits.supports_1pass_recon_writing_skip;
-#else
-      encoderSettings.two_pass.skip_1st_dpb_texture = 0u;
-#endif   // ENCODE_WITH_TWO_PASS_EXTERNAL_DPB_RECON_SCALE
+         encoderSettings.two_pass.skip_1st_dpb_texture = m_bRateControlFramePreAnalysisExternalReconDownscale ? true : false;
 
-      if( encoderSettings.two_pass.enable && ( encoderSettings.two_pass.pow2_downscale_factor > 0 ) )
-      {
-         struct pipe_video_codec blitterSettings = {};
-         blitterSettings.entrypoint = PIPE_VIDEO_ENTRYPOINT_PROCESSING;
-         blitterSettings.width = Width;
-         blitterSettings.height = Height;
-         CHECKNULL_GOTO( m_pPipeVideoBlitter = m_pPipeContext->create_video_codec( m_pPipeContext, &blitterSettings ),
-                         MF_E_UNEXPECTED,
-                         done );
+         if( encoderSettings.two_pass.enable && ( encoderSettings.two_pass.pow2_downscale_factor > 0 ) )
+         {
+            struct pipe_video_codec blitterSettings = {};
+            blitterSettings.entrypoint = PIPE_VIDEO_ENTRYPOINT_PROCESSING;
+            blitterSettings.width = Width;
+            blitterSettings.height = Height;
+            CHECKNULL_GOTO( m_pPipeVideoBlitter = m_pPipeContext->create_video_codec( m_pPipeContext, &blitterSettings ),
+                            MF_E_UNEXPECTED,
+                            done );
+         }
       }
 
-#endif   // ENCODE_WITH_TWO_PASS
+      struct d3d12_interop_device_info1 screen_interop_info = {};
+      if( ( m_pPipeContext->screen->interop_query_device_info( m_pPipeContext->screen,
+                                                               sizeof( d3d12_interop_device_info1 ),
+                                                               &screen_interop_info ) != 0 ) &&
+          ( screen_interop_info.set_video_encoder_max_async_queue_depth != nullptr ) )
+      {
+         screen_interop_info.set_video_encoder_max_async_queue_depth( m_pPipeContext,
+                                                                      ( m_bLowLatency ? 1 : MFT_INPUT_QUEUE_DEPTH ) );
+      }
 
       CHECKNULL_GOTO( m_pPipeVideoCodec = m_pPipeContext->create_video_codec( m_pPipeContext, &encoderSettings ),
                       MF_E_UNEXPECTED,
                       done );
 
+      // Calculate and cache the expected output buffer max bitstream size for a single frame
+      m_uiMaxOutputBitstreamSize = CalculateMaxOutputBitstreamSize( encoderSettings.width,
+                                                                    encoderSettings.height,
+                                                                    ConvertProfileToFormat( encoderSettings.profile ) );
+
+      debug_printf( "[dx12 hmft 0x%p] Calculated max output bitstream size: %u bytes (%u Kb, %u Mb) for %ux%u pipe_format %u\n",
+                    this,
+                    m_uiMaxOutputBitstreamSize,
+                    m_uiMaxOutputBitstreamSize / 1024,
+                    m_uiMaxOutputBitstreamSize / ( 1024 * 1024 ),
+                    encoderSettings.width,
+                    encoderSettings.height,
+                    ConvertProfileToFormat( encoderSettings.profile ) );
+
       // Create DX12 fence and share it as handle for using it with DX11/create_fence_win32
       CHECKHR_GOTO( m_spDevice->CreateFence( 0, D3D12_FENCE_FLAG_SHARED, IID_PPV_ARGS( &m_spStagingFence12 ) ), done );
       CHECKHR_GOTO( m_spDevice->CreateSharedHandle( m_spStagingFence12.Get(), NULL, GENERIC_ALL, NULL, &m_hSharedFenceHandle ),
@@ -991,6 +1073,65 @@ done:
    return hr;
 }
 
+// Utility function to configure the sample allocator to allocate map samples
+HRESULT
+CDX12EncHMFT::ConfigureMapSampleAllocator(
+   IMFVideoSampleAllocatorEx *spAllocator, UINT32 width, UINT32 height, GUID subtype, UINT32 poolSize )
+{
+   if( !spAllocator )
+      return E_POINTER;
+
+   spAllocator->UninitializeSampleAllocator();
+   HRESULT hr = spAllocator->SetDirectXManager( m_spDeviceManager.Get() );
+   if( FAILED( hr ) )
+      return hr;
+
+   ComPtr<IMFAttributes> spAttrs;
+   ComPtr<IMFMediaType> spMapType;
+
+   // Attributes for allocator
+   CHECKHR_GOTO( MFCreateAttributes( &spAttrs, 2 ), done );
+   CHECKHR_GOTO( spAttrs->SetUINT32( MF_SA_BUFFERS_PER_SAMPLE, 1 ), done );
+   CHECKHR_GOTO( spAttrs->SetUINT32( MF_MT_D3D_RESOURCE_VERSION, MF_D3D12_RESOURCE ), done );
+
+   // Media type for the map
+   CHECKHR_GOTO( MFCreateMediaType( &spMapType ), done );
+   CHECKHR_GOTO( spMapType->SetGUID( MF_MT_MAJOR_TYPE, MFMediaType_Video ), done );
+   CHECKHR_GOTO( spMapType->SetGUID( MF_MT_SUBTYPE, subtype ), done );
+   MFSetAttributeSize( spMapType.Get(), MF_MT_FRAME_SIZE, width, height );
+   CHECKHR_GOTO( spMapType->SetUINT32( MF_MT_D3D_RESOURCE_VERSION, MF_D3D12_RESOURCE ), done );
+
+   // Initialize the allocator
+   CHECKHR_GOTO( spAllocator->InitializeSampleAllocatorEx( 1, poolSize, spAttrs.Get(), spMapType.Get() ), done );
+
+done:
+   return hr;
+}
+
+// Helper function to configure map sample allocator.
+void
+CDX12EncHMFT::ConfigureMapSampleAllocatorHelper( ComPtr<IMFVideoSampleAllocatorEx> &allocator,
+                                                 const union pipe_enc_cap_gpu_stats_map &outputStatsMap,
+                                                 uint32_t blockSize,
+                                                 BOOL &useAllocatorFlag )
+{
+   if( allocator != nullptr && outputStatsMap.bits.supported && blockSize > 0 )
+   {
+      uint32_t actualBlockSize = ( 1u << outputStatsMap.bits.log2_values_block_size );
+      uint32_t width = static_cast<uint32_t>( std::ceil( m_uiOutputWidth / static_cast<float>( actualBlockSize ) ) );
+      uint32_t height = static_cast<uint32_t>( std::ceil( m_uiOutputHeight / static_cast<float>( actualBlockSize ) ) );
+
+      if( SUCCEEDED( ConfigureMapSampleAllocator( allocator.Get(), width, height, MFVideoFormat_L32, 10 ) ) )
+      {
+         useAllocatorFlag = TRUE;
+      }
+      else
+      {
+         useAllocatorFlag = FALSE;
+      }
+   }
+}
+
 // internal thread function to handle encoding and output
 void WINAPI
 CDX12EncHMFT::xThreadProc( void *pCtx )
@@ -1030,7 +1171,7 @@ CDX12EncHMFT::xThreadProc( void *pCtx )
       std::lock_guard<std::mutex> lock( pThis->m_lock );
       while( !bHasEncodingError && pThis->m_EncodingQueue.try_pop( pDX12EncodeContext ) )
       {
-         pipe_enc_feedback_metadata metadata = { };
+         pipe_enc_feedback_metadata metadata = {};
          unsigned int encoded_bitstream_bytes = 0u;
          ComPtr<IMFSample> spOutputSample;
          MFCreateSample( &spOutputSample );
@@ -1041,7 +1182,6 @@ CDX12EncHMFT::xThreadProc( void *pCtx )
 
             metadata.encode_result = PIPE_VIDEO_FEEDBACK_METADATA_ENCODE_FLAG_FAILED;   // default to failure
 
-#if ( USE_D3D12_PREVIEW_HEADERS && ( D3D12_PREVIEW_SDK_VERSION >= 717 ) )
             // If sliced fences supported, we asynchronously copy here every slice as it is ready
             // Otherwise, let's copy all the sliced together here after full frame completion (see below)
             if( pDX12EncodeContext->sliceNotificationMode == D3D12_VIDEO_ENCODER_COMPRESSED_BITSTREAM_NOTIFICATION_MODE_SUBREGIONS )
@@ -1050,29 +1190,30 @@ CDX12EncHMFT::xThreadProc( void *pCtx )
                // Wait for each slice fence and resolve offset/size as each slice is ready
                //
                ComPtr<IMFMediaBuffer> spMemoryBuffer;
-               // TODO: Estimate size of entire frame (e.g all slices) instead of assuming 8MBs here...
+               // Create a single large buffer to hold all slices using the max output bitstream size for the whole frame
+               // it can be possible that slices have padding between them so more size may be needed.
                // TODO: or even better allow multiple buffers (one per slice) in the MFSample or multiple MFSamples (one per slice)
                // with tight allocations
-               MFCreateMemoryBuffer( ( 1024 /*1K*/ * 1024 /*1MB*/ ) * 8 /*8 MB*/, &spMemoryBuffer );
+               MFCreateMemoryBuffer( pThis->m_uiMaxOutputBitstreamSize, &spMemoryBuffer );
                uint64_t output_buffer_offset = 0u;
                LPBYTE lpBuffer;
                spMemoryBuffer->Lock( &lpBuffer, NULL, NULL );
 
                uint32_t num_slice_buffers = static_cast<uint32_t>( pDX12EncodeContext->pSliceFences.size() );
+               std::vector<struct codec_unit_location_t> codec_unit_metadata;
+               const size_t max_default_init_alloc_count_nals = 64u;
+               codec_unit_metadata.reserve( max_default_init_alloc_count_nals );
                for( uint32_t slice_idx = 0; slice_idx < num_slice_buffers; slice_idx++ )
                {
                   assert( pDX12EncodeContext->pSliceFences[slice_idx] );
 
-                  bool fenceWaitResult =
-                     pThis->m_pPipeVideoCodec->fence_wait( pThis->m_pPipeVideoCodec,
-                                                           pDX12EncodeContext->pSliceFences[slice_idx],
-                                                           OS_TIMEOUT_INFINITE ) != 0;
+                  bool fenceWaitResult = pThis->m_pPipeVideoCodec->fence_wait( pThis->m_pPipeVideoCodec,
+                                                                               pDX12EncodeContext->pSliceFences[slice_idx],
+                                                                               OS_TIMEOUT_INFINITE ) != 0;
                   assert( fenceWaitResult );
-                  pThis->m_pPipeVideoCodec->destroy_fence( pThis->m_pPipeVideoCodec,
-                                                           pDX12EncodeContext->pSliceFences[slice_idx]);
+                  pThis->m_pPipeVideoCodec->destroy_fence( pThis->m_pPipeVideoCodec, pDX12EncodeContext->pSliceFences[slice_idx] );
                   if( fenceWaitResult )
                   {
-                     std::vector<struct codec_unit_location_t> codec_unit_metadata;
                      unsigned codec_unit_metadata_count = 0u;
                      pThis->m_pPipeVideoCodec->get_slice_bitstream_data( pThis->m_pPipeVideoCodec,
                                                                          pDX12EncodeContext->pAsyncCookie,
@@ -1080,6 +1221,7 @@ CDX12EncHMFT::xThreadProc( void *pCtx )
                                                                          NULL /*get size*/,
                                                                          &codec_unit_metadata_count );
                      assert( codec_unit_metadata_count > 0 );
+                     codec_unit_metadata.clear();
                      codec_unit_metadata.resize( codec_unit_metadata_count, {} );
                      pThis->m_pPipeVideoCodec->get_slice_bitstream_data( pThis->m_pPipeVideoCodec,
                                                                          pDX12EncodeContext->pAsyncCookie,
@@ -1119,22 +1261,25 @@ CDX12EncHMFT::xThreadProc( void *pCtx )
                   }
                }
 
-               memset(pDX12EncodeContext->pSliceFences.data(), 0, pDX12EncodeContext->pSliceFences.size() * sizeof(pipe_fence_handle *));
+               memset( pDX12EncodeContext->pSliceFences.data(),
+                       0,
+                       pDX12EncodeContext->pSliceFences.size() * sizeof( pipe_fence_handle * ) );
 
                spMemoryBuffer->Unlock();
                spMemoryBuffer->SetCurrentLength( static_cast<DWORD>( output_buffer_offset ) );
                spOutputSample->AddBuffer( spMemoryBuffer.Get() );
             }
-#endif   // (USE_D3D12_PREVIEW_HEADERS && (D3D12_PREVIEW_SDK_VERSION >= 717))
 
             // Still wait for pAsyncFence (full frame fence) before calling get_feedback for full frame stats
             // First wait on the D3D12 encoder_fence
             assert( pDX12EncodeContext->pAsyncFence );   // NULL returned pDX12EncodeContext->pAsyncFence indicates encode error
             if( pDX12EncodeContext->pAsyncFence )
             {
-               int wait_res = pThis->m_pPipeVideoCodec->fence_wait(pThis->m_pPipeVideoCodec, pDX12EncodeContext->pAsyncFence, OS_TIMEOUT_INFINITE);
-               HRESULT hr = wait_res > 0 ? S_OK : E_FAIL; // Based on p_video_codec interface
-               pThis->m_pPipeVideoCodec->destroy_fence(pThis->m_pPipeVideoCodec, pDX12EncodeContext->pAsyncFence);
+               int wait_res = pThis->m_pPipeVideoCodec->fence_wait( pThis->m_pPipeVideoCodec,
+                                                                    pDX12EncodeContext->pAsyncFence,
+                                                                    OS_TIMEOUT_INFINITE );
+               HRESULT hr = wait_res > 0 ? S_OK : E_FAIL;   // Based on p_video_codec interface
+               pThis->m_pPipeVideoCodec->destroy_fence( pThis->m_pPipeVideoCodec, pDX12EncodeContext->pAsyncFence );
                pDX12EncodeContext->pAsyncFence = nullptr;
 
                assert( SUCCEEDED( hr ) );
@@ -1186,7 +1331,7 @@ CDX12EncHMFT::xThreadProc( void *pCtx )
                      vpblit_params.dst_region.y1 = cur_pic_dpb_entry.downscaled_buffer->height;
 
                      pThis->m_pPipeVideoBlitter->begin_frame( pThis->m_pPipeVideoBlitter,
-                                                              pDX12EncodeContext->pDownscaledTwoPassPipeVideoBuffer,
+                                                              cur_pic_dpb_entry.downscaled_buffer,
                                                               &vpblit_params.base );
 
                      if( pThis->m_pPipeVideoBlitter->process_frame( pThis->m_pPipeVideoBlitter,
@@ -1201,7 +1346,7 @@ CDX12EncHMFT::xThreadProc( void *pCtx )
                      }
 
                      if( pThis->m_pPipeVideoBlitter->end_frame( pThis->m_pPipeVideoBlitter,
-                                                                pDX12EncodeContext->pDownscaledTwoPassPipeVideoBuffer,
+                                                                cur_pic_dpb_entry.downscaled_buffer,
                                                                 &vpblit_params.base ) != 0 )
                      {
                         assert( false );
@@ -1222,11 +1367,10 @@ CDX12EncHMFT::xThreadProc( void *pCtx )
                      // queue the fence wait into the next frame's encode GPU fence wait
 
                      ASSERTED bool finished =
-                        pThis->m_pPipeVideoCodec->fence_wait(pThis->m_pPipeVideoCodec,
-                                                             dst_surface_fence,
-                                                             OS_TIMEOUT_INFINITE ) != 0;
+                        pThis->m_pPipeVideoCodec->fence_wait( pThis->m_pPipeVideoCodec, dst_surface_fence, OS_TIMEOUT_INFINITE ) !=
+                        0;
                      assert( finished );
-                     pThis->m_pPipeVideoCodec->destroy_fence(pThis->m_pPipeVideoCodec, dst_surface_fence);
+                     pThis->m_pPipeVideoCodec->destroy_fence( pThis->m_pPipeVideoCodec, dst_surface_fence );
                   }
 #endif   // (MFT_CODEC_H264ENC || MFT_CODEC_H265ENC)
 
@@ -1344,12 +1488,25 @@ CDX12EncHMFT::xThreadProc( void *pCtx )
                   }
                }
 
+               // Conditionally attach SATD map
+               if( pThis->m_uiVideoSatdMapBlockSize && pDX12EncodeContext->pPipeResourceSATDMapStats != nullptr )
+               {
+                  HRESULT hr = MFAttachPipeResourceAsSampleExtension( pThis->m_pPipeContext,
+                                                                      pDX12EncodeContext->pPipeResourceSATDMapStats,
+                                                                      pDX12EncodeContext->pSyncObjectQueue,
+                                                                      MFSampleExtension_VideoEncodeSatdMap,
+                                                                      spOutputSample.Get() );
+
+                  if( FAILED( hr ) )
+                  {
+                     MFE_INFO( "[dx12 hmft 0x%p] SATDMap: MFAttachPipeResourceAsSampleExtension failed - hr=0x%08x", pThis, hr );
+                  }
+               }
+
                // If sliced fences supported, we asynchronously copied every slice as it was ready (see above)
                // into spMemoryBuffer. Otherwise, let's copy all the sliced together here after full frame completion
-#if ( USE_D3D12_PREVIEW_HEADERS && ( D3D12_PREVIEW_SDK_VERSION >= 717 ) )
                if( pDX12EncodeContext->sliceNotificationMode ==
                    D3D12_VIDEO_ENCODER_COMPRESSED_BITSTREAM_NOTIFICATION_MODE_FULL_FRAME )
-#endif   // (USE_D3D12_PREVIEW_HEADERS && (D3D12_PREVIEW_SDK_VERSION >= 717))
                {
                   // Readback full encoded frame bitstream from GPU memory onto CPU buffer
                   struct pipe_box box = { 0 };
@@ -1913,11 +2070,20 @@ CDX12EncHMFT::ProcessMessage( MFT_MESSAGE_TYPE eMessage, ULONG_PTR ulParam )
          std::lock_guard<std::mutex> lock( m_lock );
          CleanupEncoder();
          CHECKHR_GOTO( xOnSetD3DManager( ulParam ), done );
-         CHECKHR_GOTO( ConfigureSampleAllocator(), done );
          if( m_pPipeContext )
          {
             m_EncoderCapabilities.initialize( m_pPipeContext->screen, m_outputPipeProfile );
          }
+
+         ConfigureMapSampleAllocatorHelper( m_spSATDMapAllocator,
+                                            m_EncoderCapabilities.m_HWSupportStatsSATDMapOutput,
+                                            m_uiVideoSatdMapBlockSize,
+                                            m_bUseSATDMapAllocator );
+
+         ConfigureMapSampleAllocatorHelper( m_spBitsusedMapAllocator,
+                                            m_EncoderCapabilities.m_HWSupportStatsRCBitAllocationMapOutput,
+                                            m_uiVideoOutputBitsUsedMapBlockSize,
+                                            m_bUseBitsusedMapAllocator );
          break;
       }
    }
@@ -1930,6 +2096,38 @@ done:
    return hr;
 }
 
+static HRESULT
+GetQPMapBufferFromSampleLockHeld( IMFSample *pSample, BYTE **ppData, DWORD *pSize, ComPtr<IMFMediaBuffer> &outBuffer )
+{
+   if( !pSample || !ppData || !pSize )
+      return E_POINTER;
+
+   ComPtr<IMFAttributes> attrs;
+   HRESULT hr = pSample->QueryInterface( IID_PPV_ARGS( &attrs ) );
+   if( FAILED( hr ) || !attrs )
+      return hr;
+
+   ComPtr<IUnknown> unk;
+   hr = attrs->GetUnknown( MFSampleExtension_VideoEncodeInputDeltaQPMap, IID_PPV_ARGS( &unk ) );
+   if( FAILED( hr ) || !unk )
+      return hr;
+
+   hr = unk.As( &outBuffer );
+   if( FAILED( hr ) || !outBuffer )
+      return hr;
+
+   BYTE *pData = nullptr;
+   DWORD maxLen = 0, curLen = 0;
+   hr = outBuffer->Lock( &pData, &maxLen, &curLen );
+   if( FAILED( hr ) )
+      return hr;
+
+   *ppData = pData;
+   *pSize = curLen;
+
+   return S_OK;
+}
+
 // IMFTransform::ProcessInput
 // https://learn.microsoft.com/en-us/windows/win32/api/mftransform/nf-mftransform-imftransform-processinput
 HRESULT
@@ -1939,6 +2137,9 @@ CDX12EncHMFT::ProcessInput( DWORD dwInputStreamIndex, IMFSample *pSample, DWORD
    HRESULT hr = S_OK;
    UINT32 unChromaOnly = 0;
    LPDX12EncodeContext pDX12EncodeContext = nullptr;
+   BYTE *qpData = nullptr;
+   DWORD qpSize = 0;
+   ComPtr<IMFMediaBuffer> qpMapBuffer;
    std::lock_guard<std::mutex> lock( m_lock );
 
    CHECKHR_GOTO( IsUnlocked(), done );
@@ -1973,6 +2174,17 @@ CDX12EncHMFT::ProcessInput( DWORD dwInputStreamIndex, IMFSample *pSample, DWORD
 
    // setup the source buffer
    CHECKHR_HRGOTO( PrepareForEncode( pSample, &pDX12EncodeContext ), MF_E_INVALIDMEDIATYPE, done );
+   if( SUCCEEDED( GetQPMapBufferFromSampleLockHeld( pSample, &qpData, &qpSize, qpMapBuffer ) ) && qpMapBuffer )
+   {
+      pDX12EncodeContext->SetPipeQPMapBufferInfo( qpData, qpSize );
+   }
+   else
+   {
+      // make sure it's null/zero if we failed to get it
+      qpData = nullptr;
+      qpSize = 0;
+      qpMapBuffer = nullptr;
+   }
 
    // Submit work
    {
@@ -1984,7 +2196,6 @@ CDX12EncHMFT::ProcessInput( DWORD dwInputStreamIndex, IMFSample *pSample, DWORD
                                       pDX12EncodeContext->pPipeVideoBuffer,
                                       &pDX12EncodeContext->encoderPicInfo.base );
 
-#if ( USE_D3D12_PREVIEW_HEADERS && ( D3D12_PREVIEW_SDK_VERSION >= 717 ) )
       if( pDX12EncodeContext->sliceNotificationMode == D3D12_VIDEO_ENCODER_COMPRESSED_BITSTREAM_NOTIFICATION_MODE_SUBREGIONS )
       {
          m_pPipeVideoCodec->encode_bitstream_sliced( m_pPipeVideoCodec,
@@ -1995,7 +2206,6 @@ CDX12EncHMFT::ProcessInput( DWORD dwInputStreamIndex, IMFSample *pSample, DWORD
                                                      &pDX12EncodeContext->pAsyncCookie );
       }
       else
-#endif   // (USE_D3D12_PREVIEW_HEADERS && (D3D12_PREVIEW_SDK_VERSION >= 717))
       {
          m_pPipeVideoCodec->encode_bitstream( m_pPipeVideoCodec,
                                               pDX12EncodeContext->pPipeVideoBuffer,
@@ -2024,6 +2234,11 @@ CDX12EncHMFT::ProcessInput( DWORD dwInputStreamIndex, IMFSample *pSample, DWORD
       m_pPipeVideoCodec->flush( m_pPipeVideoCodec );
       HMFT_ETW_EVENT_STOP( "PipeFlush", this );
    }
+   // Release the QP map buffer after encode_bitstream call returns.
+   if( qpMapBuffer && qpSize != 0 && qpData != nullptr )
+   {
+      qpMapBuffer->Unlock();
+   }
    m_EncodingQueue.push( pDX12EncodeContext );
    // Moves the GOP tracker state to the next frame for having next
    // frame data in get_frame_descriptor() for next iteration
