@@ -1,3 +1,6 @@
+// Copyright 2020 Red Hat.
+// SPDX-License-Identifier: MIT
+
 use crate::api::icd::*;
 use crate::api::types::DeleteContextCB;
 use crate::api::util::bit_check;
@@ -13,6 +16,7 @@ use mesa_rust::pipe::context::RWFlags;
 use mesa_rust::pipe::fence::FenceFd;
 use mesa_rust::pipe::resource::*;
 use mesa_rust::pipe::screen::ResourceType;
+use mesa_rust::util;
 use mesa_rust_gen::*;
 use mesa_rust_util::conversion::*;
 use mesa_rust_util::properties::Properties;
@@ -113,6 +117,7 @@ pub struct Context {
     >,
     svm: Mutex<SVMContext>,
     pub gl_ctx_manager: Option<GLCtxManager>,
+    pub worker_queue: util::queue::Queue,
 }
 
 impl_cl_type_trait!(cl_context, Context, CL_INVALID_CONTEXT);
@@ -123,6 +128,9 @@ impl Context {
         properties: Properties<cl_context_properties>,
         gl_ctx_manager: Option<GLCtxManager>,
     ) -> Arc<Context> {
+        let worker_count = u32::max(util::cpu_count() / 2, 1);
+        let max_job_count = worker_count * 8;
+
         Arc::new(Self {
             base: CLObjectBase::new(RusticlTypes::Context),
             devs: devs,
@@ -133,6 +141,7 @@ impl Context {
                 svm_ptrs: TrackedPointers::new(),
             }),
             gl_ctx_manager: gl_ctx_manager,
+            worker_queue: util::queue::Queue::new(c"clctxworker", max_job_count, worker_count),
         })
     }
 
@@ -143,7 +152,7 @@ impl Context {
         copy: bool,
         bda: bool,
         res_type: ResourceType,
-    ) -> CLResult<HashMap<&'static Device, PipeResource>> {
+    ) -> CLResult<HashMap<&'static Device, PipeResourceOwned>> {
         let adj_size: u32 = size.try_into_with_err(CL_OUT_OF_HOST_MEMORY)?;
         let mut res = HashMap::new();
         let mut pipe_flags = 0;
@@ -184,7 +193,9 @@ impl Context {
                     d.helper_ctx()
                         .exec(|ctx| ctx.buffer_subdata(r, 0, user_ptr, size.try_into().unwrap()))
                 })
-                .for_each(|f| f.wait());
+                .for_each(|f| {
+                    f.wait();
+                });
         }
 
         Ok(res)
@@ -197,7 +208,7 @@ impl Context {
         user_ptr: *mut c_void,
         copy: bool,
         res_type: ResourceType,
-    ) -> CLResult<HashMap<&'static Device, PipeResource>> {
+    ) -> CLResult<HashMap<&'static Device, PipeResourceOwned>> {
         let pipe_format = format.to_pipe_format().unwrap();
 
         let width = desc.image_width.try_into_with_err(CL_OUT_OF_HOST_MEMORY)?;
@@ -256,7 +267,9 @@ impl Context {
                     d.helper_ctx()
                         .exec(|ctx| ctx.texture_subdata(r, &bx, user_ptr, stride, layer_stride))
                 })
-                .for_each(|f| f.wait());
+                .for_each(|f| {
+                    f.wait();
+                });
         }
 
         Ok(res)
@@ -407,7 +420,7 @@ impl Context {
         &self,
         ctx: &QueueContext,
         ptr: usize,
-    ) -> CLResult<Option<PipeResource>> {
+    ) -> CLResult<Option<PipeResourceOwned>> {
         let svm = self.svm.lock().unwrap();
 
         let Some(alloc) = svm.svm_ptrs.find_alloc_precise(ptr) else {
@@ -509,7 +522,7 @@ impl Context {
             ctx.clear_buffer(res, &pattern, offset as u32, size as u32);
         } else {
             let slice = unsafe {
-                slice::from_raw_parts_mut(svm_ptr as *mut _, size / mem::size_of_val(&pattern))
+                slice::from_raw_parts_mut(svm_ptr as *mut _, size / size_of_val(&pattern))
             };
 
             slice.fill(pattern);
@@ -618,7 +631,7 @@ impl Context {
         gl_target: cl_GLenum,
         format: cl_image_format,
         gl_props: GLMemProps,
-    ) -> CLResult<HashMap<&'static Device, PipeResource>> {
+    ) -> CLResult<HashMap<&'static Device, PipeResourceOwned>> {
         let mut res = HashMap::new();
         let target = cl_mem_type_to_texture_target_gl(image_type, gl_target);
         let pipe_format = if image_type == CL_MEM_OBJECT_BUFFER {
