@@ -29,16 +29,20 @@
 #define STW_CONTEXT_H
 
 #include <windows.h>
+#include <GL/gl.h>
+#include <stw_gdishim.h>
+#include <gldrv.h>
 
 struct hud_context;
 struct stw_framebuffer;
-struct st_context_iface;
+struct st_context;
+struct pipe_frontend_screen;
 
 struct stw_context
 {
-   struct st_context_iface *st;
+   struct st_context *st;
    DHGLRC dhglrc;
-   int iPixelFormat;
+   const struct stw_pixelformat_info *pfi;
    HDC hDrawDC;
    HDC hReadDC;
    BOOL shared;
@@ -51,9 +55,11 @@ struct stw_context
 
 struct stw_context *stw_create_context_attribs(HDC hdc, INT iLayerPlane,
                                                struct stw_context *shareCtx,
+                                               struct pipe_frontend_screen *fscreen,
                                                int majorVersion, int minorVersion,
                                                int contextFlags, int profileMask,
-                                               int iPixelFormat);
+                                               const struct stw_pixelformat_info *pfi,
+                                               int resetStrategy);
 
 DHGLRC stw_create_context_handle(struct stw_context *context, DHGLRC handle);
 
