@@ -29,7 +29,7 @@
 
 #include <windows.h>
 
-#include "glapi/glapi.h"
+#include "glapi/glapi/glapi.h"
 #include "util/u_debug.h"
 #include "util/u_math.h"
 #include "util/u_memory.h"
