@@ -36,14 +36,15 @@
 
 #include <windows.h>
 
-#include "pipe/p_format.h"
+#include "util/format/u_formats.h"
 #include "pipe/p_context.h"
 #include "util/u_inlines.h"
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 #include "util/u_math.h"
 #include "util/u_memory.h"
-#include "state_tracker/sw_winsys.h"
+#include "frontend/sw_winsys.h"
 #include "gdi_sw_winsys.h"
+#include "wgl/stw_gdishim.h"
 
 
 struct gdi_sw_displaytarget
@@ -57,9 +58,15 @@ struct gdi_sw_displaytarget
 
    void *data;
 
-   BITMAPINFO bmi;
+   BITMAPV5HEADER bmi;
 };
 
+struct gdi_sw_winsys {
+   struct sw_winsys base;
+
+   HDC (*acquire_hdc)(void *winsys_drawable_handle);
+   void (*release_hdc)(void *winsys_drawable_handle, HDC hdc);
+};
 
 /** Cast wrapper */
 static inline struct gdi_sw_displaytarget *
@@ -69,6 +76,14 @@ gdi_sw_displaytarget( struct sw_displaytarget *buf )
 }
 
 
+/** Cast wrapper */
+static inline struct gdi_sw_winsys *
+gdi_sw_winsys( struct sw_winsys *buf )
+{
+   return (struct gdi_sw_winsys *)buf;
+}
+
+
 static bool
 gdi_sw_is_displaytarget_format_supported( struct sw_winsys *ws,
                                                 unsigned tex_usage,
@@ -77,6 +92,12 @@ gdi_sw_is_displaytarget_format_supported( struct sw_winsys *ws,
    switch(format) {
    case PIPE_FORMAT_B8G8R8X8_UNORM:
    case PIPE_FORMAT_B8G8R8A8_UNORM:
+   case PIPE_FORMAT_B5G6R5_UNORM:
+   case PIPE_FORMAT_B5G5R5A1_UNORM:
+   case PIPE_FORMAT_B4G4R4A4_UNORM:
+   case PIPE_FORMAT_R10G10B10A2_UNORM:
+   case PIPE_FORMAT_R8G8B8X8_UNORM:
+   case PIPE_FORMAT_R8G8B8A8_UNORM:
       return true;
 
    /* TODO: Support other formats possible with BMPs, as described in 
@@ -149,17 +170,39 @@ gdi_sw_displaytarget_create(struct sw_winsys *winsys,
    if(!gdt->data)
       goto no_data;
 
-   gdt->bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
-   gdt->bmi.bmiHeader.biWidth = gdt->stride / cpp;
-   gdt->bmi.bmiHeader.biHeight= -(long)height;
-   gdt->bmi.bmiHeader.biPlanes = 1;
-   gdt->bmi.bmiHeader.biBitCount = bpp;
-   gdt->bmi.bmiHeader.biCompression = BI_RGB;
-   gdt->bmi.bmiHeader.biSizeImage = 0;
-   gdt->bmi.bmiHeader.biXPelsPerMeter = 0;
-   gdt->bmi.bmiHeader.biYPelsPerMeter = 0;
-   gdt->bmi.bmiHeader.biClrUsed = 0;
-   gdt->bmi.bmiHeader.biClrImportant = 0;
+   gdt->bmi.bV5Size = sizeof(BITMAPV5HEADER);
+   gdt->bmi.bV5Width = gdt->stride / cpp;
+   gdt->bmi.bV5Height = -(long)height;
+   gdt->bmi.bV5Planes = 1;
+   gdt->bmi.bV5BitCount = bpp;
+   gdt->bmi.bV5Compression = BI_RGB;
+   gdt->bmi.bV5SizeImage = 0;
+   gdt->bmi.bV5XPelsPerMeter = 0;
+   gdt->bmi.bV5YPelsPerMeter = 0;
+   gdt->bmi.bV5ClrUsed = 0;
+   gdt->bmi.bV5ClrImportant = 0;
+
+   if (format == PIPE_FORMAT_B5G6R5_UNORM) {
+      gdt->bmi.bV5Compression = BI_BITFIELDS;
+      gdt->bmi.bV5RedMask = 0xF800;
+      gdt->bmi.bV5GreenMask = 0x07E0;
+      gdt->bmi.bV5BlueMask = 0x001F;
+   } else if (format == PIPE_FORMAT_B4G4R4A4_UNORM) {
+      gdt->bmi.bV5Compression = BI_BITFIELDS;
+      gdt->bmi.bV5RedMask = 0x0F00;
+      gdt->bmi.bV5GreenMask = 0x00F0;
+      gdt->bmi.bV5BlueMask = 0x000F;
+   } else if (format == PIPE_FORMAT_R10G10B10A2_UNORM) {
+      gdt->bmi.bV5Compression = BI_BITFIELDS;
+      gdt->bmi.bV5RedMask = 0x000003FF;
+      gdt->bmi.bV5GreenMask = 0x000FFC00;
+      gdt->bmi.bV5BlueMask = 0x3FF00000;
+   } else if (format == PIPE_FORMAT_R8G8B8X8_UNORM || format == PIPE_FORMAT_R8G8B8A8_UNORM) {
+      gdt->bmi.bV5Compression = BI_BITFIELDS;
+      gdt->bmi.bV5RedMask = 0x0000ff;
+      gdt->bmi.bV5GreenMask = 0x00ff00;
+      gdt->bmi.bV5BlueMask = 0xff0000;
+   }
 
    *stride = gdt->stride;
    return (struct sw_displaytarget *)gdt;
@@ -202,20 +245,21 @@ gdi_sw_display( struct sw_winsys *winsys,
     StretchDIBits(hDC,
                   0, 0, gdt->width, gdt->height,
                   0, 0, gdt->width, gdt->height,
-                  gdt->data, &gdt->bmi, 0, SRCCOPY);
+                  gdt->data, (BITMAPINFO *)&gdt->bmi, 0, SRCCOPY);
 }
 
 static void
-gdi_sw_displaytarget_display(struct sw_winsys *winsys, 
+gdi_sw_displaytarget_display(struct sw_winsys *_winsys, 
                              struct sw_displaytarget *dt,
                              void *context_private,
+                             unsigned nboxes,
                              struct pipe_box *box)
 {
-    /* nasty:
-     */
-    HDC hDC = (HDC)context_private;
+    struct gdi_sw_winsys *winsys = gdi_sw_winsys(_winsys);
+    HDC hDC = winsys->acquire_hdc(context_private);
 
-    gdi_sw_display(winsys, dt, hDC);
+    gdi_sw_display(_winsys, dt, hDC);
+    winsys->release_hdc(context_private, hDC);
 }
 
 
@@ -226,24 +270,37 @@ gdi_sw_destroy(struct sw_winsys *winsys)
 }
 
 struct sw_winsys *
-gdi_create_sw_winsys(void)
+gdi_create_sw_winsys(
+    HDC (*acquire_hdc)(void *winsys_drawable_handle),
+    void (*release_hdc)(void *winsys_drawable_handle, HDC hdc))
 {
-   static struct sw_winsys *winsys;
+   static struct gdi_sw_winsys *winsys;
 
-   winsys = CALLOC_STRUCT(sw_winsys);
+   winsys = CALLOC_STRUCT(gdi_sw_winsys);
    if(!winsys)
       return NULL;
 
-   winsys->destroy = gdi_sw_destroy;
-   winsys->is_displaytarget_format_supported = gdi_sw_is_displaytarget_format_supported;
-   winsys->displaytarget_create = gdi_sw_displaytarget_create;
-   winsys->displaytarget_from_handle = gdi_sw_displaytarget_from_handle;
-   winsys->displaytarget_get_handle = gdi_sw_displaytarget_get_handle;
-   winsys->displaytarget_map = gdi_sw_displaytarget_map;
-   winsys->displaytarget_unmap = gdi_sw_displaytarget_unmap;
-   winsys->displaytarget_display = gdi_sw_displaytarget_display;
-   winsys->displaytarget_destroy = gdi_sw_displaytarget_destroy;
+   winsys->acquire_hdc = acquire_hdc;
+   winsys->release_hdc = release_hdc;
 
-   return winsys;
+   winsys->base.destroy = gdi_sw_destroy;
+   winsys->base.is_displaytarget_format_supported = gdi_sw_is_displaytarget_format_supported;
+   winsys->base.displaytarget_create = gdi_sw_displaytarget_create;
+   winsys->base.displaytarget_from_handle = gdi_sw_displaytarget_from_handle;
+   winsys->base.displaytarget_get_handle = gdi_sw_displaytarget_get_handle;
+   winsys->base.displaytarget_map = gdi_sw_displaytarget_map;
+   winsys->base.displaytarget_unmap = gdi_sw_displaytarget_unmap;
+   winsys->base.displaytarget_display = gdi_sw_displaytarget_display;
+   winsys->base.displaytarget_destroy = gdi_sw_displaytarget_destroy;
+
+   return &winsys->base;
 }
 
+
+HDC gdi_sw_acquire_hdc_by_value(void *context_private) {
+   return (HDC)context_private;
+};
+
+void gdi_sw_release_hdc_by_value(void *context_private, HDC hdc) {
+   // Nothing to do
+};
