@@ -335,6 +335,8 @@ custom_decodes = {
     "vkCmdBeginRenderPass" : emit_global_state_wrapped_decoding,
     "vkCmdBeginRenderPass2" : emit_global_state_wrapped_decoding,
     "vkCmdBeginRenderPass2KHR" : emit_global_state_wrapped_decoding,
+    "vkCmdSetEvent" : emit_global_state_wrapped_decoding,
+    "vkCmdResetEvent" : emit_global_state_wrapped_decoding,
 }
 
 
