@@ -162,14 +162,8 @@ class ResourceTracker {
 
     void on_vkGetPhysicalDeviceFeatures2(void* context, VkPhysicalDevice physicalDevice,
                                          VkPhysicalDeviceFeatures2* pFeatures);
-    void on_vkGetPhysicalDeviceFeatures2KHR(void* context, VkPhysicalDevice physicalDevice,
-                                            VkPhysicalDeviceFeatures2* pFeatures);
-    void on_vkGetPhysicalDeviceProperties(void* context, VkPhysicalDevice physicalDevice,
-                                          VkPhysicalDeviceProperties* pProperties);
     void on_vkGetPhysicalDeviceProperties2(void* context, VkPhysicalDevice physicalDevice,
                                            VkPhysicalDeviceProperties2* pProperties);
-    void on_vkGetPhysicalDeviceProperties2KHR(void* context, VkPhysicalDevice physicalDevice,
-                                              VkPhysicalDeviceProperties2* pProperties);
 
     void on_vkGetPhysicalDeviceMemoryProperties(
         void* context, VkPhysicalDevice physicalDevice,
@@ -177,13 +171,6 @@ class ResourceTracker {
     void on_vkGetPhysicalDeviceMemoryProperties2(
         void* context, VkPhysicalDevice physicalDevice,
         VkPhysicalDeviceMemoryProperties2* pMemoryProperties);
-    void on_vkGetPhysicalDeviceMemoryProperties2KHR(
-        void* context, VkPhysicalDevice physicalDevice,
-        VkPhysicalDeviceMemoryProperties2* pMemoryProperties);
-    void on_vkGetDeviceQueue(void* context, VkDevice device, uint32_t queueFamilyIndex,
-                             uint32_t queueIndex, VkQueue* pQueue);
-    void on_vkGetDeviceQueue2(void* context, VkDevice device, const VkDeviceQueueInfo2* pQueueInfo,
-                              VkQueue* pQueue);
 
     VkResult on_vkCreateInstance(void* context, VkResult input_result,
                                  const VkInstanceCreateInfo* createInfo,
@@ -218,9 +205,6 @@ class ResourceTracker {
     void on_vkGetImageMemoryRequirements2(void* context, VkDevice device,
                                           const VkImageMemoryRequirementsInfo2* pInfo,
                                           VkMemoryRequirements2* pMemoryRequirements);
-    void on_vkGetImageMemoryRequirements2KHR(void* context, VkDevice device,
-                                             const VkImageMemoryRequirementsInfo2* pInfo,
-                                             VkMemoryRequirements2* pMemoryRequirements);
     VkResult on_vkGetImageDrmFormatModifierPropertiesEXT(
         void* context, VkResult input_result, VkDevice device, VkImage image,
         VkImageDrmFormatModifierPropertiesEXT* pProperties);
@@ -229,9 +213,6 @@ class ResourceTracker {
                                   VkImage image, VkDeviceMemory memory, VkDeviceSize memoryOffset);
     VkResult on_vkBindImageMemory2(void* context, VkResult input_result, VkDevice device,
                                    uint32_t bindingCount, const VkBindImageMemoryInfo* pBindInfos);
-    VkResult on_vkBindImageMemory2KHR(void* context, VkResult input_result, VkDevice device,
-                                      uint32_t bindingCount,
-                                      const VkBindImageMemoryInfo* pBindInfos);
 
     VkResult on_vkCreateBuffer(void* context, VkResult input_result, VkDevice device,
                                const VkBufferCreateInfo* pCreateInfo,
@@ -244,19 +225,6 @@ class ResourceTracker {
     void on_vkGetBufferMemoryRequirements2(void* context, VkDevice device,
                                            const VkBufferMemoryRequirementsInfo2* pInfo,
                                            VkMemoryRequirements2* pMemoryRequirements);
-    void on_vkGetBufferMemoryRequirements2KHR(void* context, VkDevice device,
-                                              const VkBufferMemoryRequirementsInfo2* pInfo,
-                                              VkMemoryRequirements2* pMemoryRequirements);
-
-    VkResult on_vkBindBufferMemory(void* context, VkResult input_result, VkDevice device,
-                                   VkBuffer buffer, VkDeviceMemory memory,
-                                   VkDeviceSize memoryOffset);
-    VkResult on_vkBindBufferMemory2(void* context, VkResult input_result, VkDevice device,
-                                    uint32_t bindInfoCount,
-                                    const VkBindBufferMemoryInfo* pBindInfos);
-    VkResult on_vkBindBufferMemory2KHR(void* context, VkResult input_result, VkDevice device,
-                                       uint32_t bindInfoCount,
-                                       const VkBindBufferMemoryInfo* pBindInfos);
 
     VkResult on_vkCreateSemaphore(void* context, VkResult, VkDevice device,
                                   const VkSemaphoreCreateInfo* pCreateInfo,
@@ -351,13 +319,6 @@ class ResourceTracker {
     void on_vkDestroySamplerYcbcrConversion(void* context, VkDevice device,
                                             VkSamplerYcbcrConversion ycbcrConversion,
                                             const VkAllocationCallbacks* pAllocator);
-    VkResult on_vkCreateSamplerYcbcrConversionKHR(
-        void* context, VkResult input_result, VkDevice device,
-        const VkSamplerYcbcrConversionCreateInfo* pCreateInfo,
-        const VkAllocationCallbacks* pAllocator, VkSamplerYcbcrConversion* pYcbcrConversion);
-    void on_vkDestroySamplerYcbcrConversionKHR(void* context, VkDevice device,
-                                               VkSamplerYcbcrConversion ycbcrConversion,
-                                               const VkAllocationCallbacks* pAllocator);
 
     VkResult on_vkCreateSampler(void* context, VkResult input_result, VkDevice device,
                                 const VkSamplerCreateInfo* pCreateInfo,
@@ -368,11 +329,6 @@ class ResourceTracker {
         const VkPhysicalDeviceExternalFenceInfo* pExternalFenceInfo,
         VkExternalFenceProperties* pExternalFenceProperties);
 
-    void on_vkGetPhysicalDeviceExternalFencePropertiesKHR(
-        void* context, VkPhysicalDevice physicalDevice,
-        const VkPhysicalDeviceExternalFenceInfo* pExternalFenceInfo,
-        VkExternalFenceProperties* pExternalFenceProperties);
-
     VkResult on_vkCreateFence(void* context, VkResult input_result, VkDevice device,
                               const VkFenceCreateInfo* pCreateInfo,
                               const VkAllocationCallbacks* pAllocator, VkFence* pFence);
@@ -468,60 +424,30 @@ class ResourceTracker {
         const VkAllocationCallbacks* pAllocator,
         VkDescriptorUpdateTemplate* pDescriptorUpdateTemplate);
 
-    VkResult on_vkCreateDescriptorUpdateTemplateKHR(
-        void* context, VkResult input_result, VkDevice device,
-        const VkDescriptorUpdateTemplateCreateInfo* pCreateInfo,
-        const VkAllocationCallbacks* pAllocator,
-        VkDescriptorUpdateTemplate* pDescriptorUpdateTemplate);
-
     void on_vkUpdateDescriptorSetWithTemplate(void* context, VkDevice device,
                                               VkDescriptorSet descriptorSet,
                                               VkDescriptorUpdateTemplate descriptorUpdateTemplate,
                                               const void* pData);
 
-    void on_vkUpdateDescriptorSetWithTemplateKHR(void* context, VkDevice device,
-                                              VkDescriptorSet descriptorSet,
-                                              VkDescriptorUpdateTemplate descriptorUpdateTemplate,
-                                              const void* pData);
-
     void on_vkGetPhysicalDeviceFormatProperties2(void* context, VkPhysicalDevice physicalDevice,
                                                  VkFormat format,
                                                  VkFormatProperties2* pFormatProperties);
 
-    void on_vkGetPhysicalDeviceFormatProperties2KHR(void* context, VkPhysicalDevice physicalDevice,
-                                                    VkFormat format,
-                                                    VkFormatProperties2* pFormatProperties);
-
     VkResult on_vkGetPhysicalDeviceImageFormatProperties2(
         void* context, VkResult input_result, VkPhysicalDevice physicalDevice,
         const VkPhysicalDeviceImageFormatInfo2* pImageFormatInfo,
         VkImageFormatProperties2* pImageFormatProperties);
 
-    VkResult on_vkGetPhysicalDeviceImageFormatProperties2KHR(
-        void* context, VkResult input_result, VkPhysicalDevice physicalDevice,
-        const VkPhysicalDeviceImageFormatInfo2* pImageFormatInfo,
-        VkImageFormatProperties2* pImageFormatProperties);
-
     void on_vkGetPhysicalDeviceExternalBufferProperties(
         void* context, VkPhysicalDevice physicalDevice,
         const VkPhysicalDeviceExternalBufferInfo* pExternalBufferInfo,
         VkExternalBufferProperties* pExternalBufferProperties);
 
-    void on_vkGetPhysicalDeviceExternalBufferPropertiesKHR(
-        void* context, VkPhysicalDevice physicalDevice,
-        const VkPhysicalDeviceExternalBufferInfoKHR* pExternalBufferInfo,
-        VkExternalBufferPropertiesKHR* pExternalBufferProperties);
-
     void on_vkGetPhysicalDeviceExternalSemaphoreProperties(
         void* context, VkPhysicalDevice physicalDevice,
         const VkPhysicalDeviceExternalSemaphoreInfo* pExternalSemaphoreInfo,
         VkExternalSemaphoreProperties* pExternalSemaphoreProperties);
 
-    void on_vkGetPhysicalDeviceExternalSemaphorePropertiesKHR(
-        void* context, VkPhysicalDevice physicalDevice,
-        const VkPhysicalDeviceExternalSemaphoreInfo* pExternalSemaphoreInfo,
-        VkExternalSemaphoreProperties* pExternalSemaphoreProperties);
-
     void registerEncoderCleanupCallback(const VkEncoder* encoder, void* handle,
                                         CleanupCallback callback);
     void unregisterEncoderCleanupCallback(const VkEncoder* encoder, void* handle);
@@ -660,11 +586,6 @@ class ResourceTracker {
     const VkPhysicalDeviceMemoryProperties& getPhysicalDeviceMemoryProperties(
         void* context, VkDevice device, VkPhysicalDevice physicalDevice);
 
-    VkResult on_vkGetPhysicalDeviceImageFormatProperties2_common(
-        bool isKhr, void* context, VkResult input_result, VkPhysicalDevice physicalDevice,
-        const VkPhysicalDeviceImageFormatInfo2* pImageFormatInfo,
-        VkImageFormatProperties2* pImageFormatProperties);
-
     void on_vkGetPhysicalDeviceExternalBufferProperties_common(
         bool isKhr, void* context, VkPhysicalDevice physicalDevice,
         const VkPhysicalDeviceExternalBufferInfo* pExternalBufferInfo,
@@ -827,7 +748,7 @@ class ResourceTracker {
     };
 
     struct VkQueue_Info {
-        VkDevice device;
+        uint32_t placeholder;
     };
 
     // custom guest-side structs for images/buffers because of AHardwareBuffer :((
@@ -964,6 +885,8 @@ class ResourceTracker {
     void transformImageMemoryRequirementsForGuestLocked(VkImage image, VkMemoryRequirements* reqs);
     CoherentMemoryPtr freeCoherentMemoryLocked(VkDeviceMemory memory, VkDeviceMemory_Info& info);
 
+    void EmitGuestAndHostTraceMarker(VkEncoder* encoder);
+
     std::recursive_mutex mLock;
 
     std::optional<const VkPhysicalDeviceMemoryProperties> mCachedPhysicalDeviceMemoryProps;
