@@ -369,8 +369,8 @@ struct pvr_winsys_ops {
 
    VkResult (*buffer_get_fd)(struct pvr_winsys_bo *bo, int *const fd_out);
 
-   VkResult (*buffer_map)(struct pvr_winsys_bo *bo);
-   void (*buffer_unmap)(struct pvr_winsys_bo *bo);
+   VkResult (*buffer_map)(struct pvr_winsys_bo *bo, void *addr);
+   VkResult (*buffer_unmap)(struct pvr_winsys_bo *bo, bool reserve);
 
    VkResult (*heap_alloc)(struct pvr_winsys_heap *heap,
                           uint64_t size,
