@@ -57,7 +57,7 @@ isl_tiling_to_i915_tiling(enum isl_tiling tiling)
       return I915_TILING_NONE;
    }
 
-   unreachable("Invalid ISL tiling");
+   UNREACHABLE("Invalid ISL tiling");
 }
 
 enum isl_tiling
@@ -74,7 +74,7 @@ isl_tiling_from_i915_tiling(uint32_t tiling)
       return ISL_TILING_Y0;
    }
 
-   unreachable("Invalid i915 tiling");
+   UNREACHABLE("Invalid i915 tiling");
 }
 
 /** Sentinel is DRM_FORMAT_MOD_INVALID. */
@@ -208,7 +208,7 @@ isl_drm_modifier_get_score(const struct intel_device_info *devinfo,
     * and don't want to provide a default value either, so we don't
     * use debug_get_num_option() here.
     */
-   const char *mod_str = getenv("INTEL_MODIFIER_OVERRIDE");
+   const char *mod_str = os_get_option("INTEL_MODIFIER_OVERRIDE");
    if (mod_str != NULL) {
       return modifier == strtoul(mod_str, NULL, 0);
    }
@@ -292,7 +292,7 @@ isl_drm_modifier_get_score(const struct intel_device_info *devinfo,
 
       return 5;
    case I915_FORMAT_MOD_4_TILED_LNL_CCS:
-      if (devinfo->platform != INTEL_PLATFORM_LNL)
+      if (devinfo->ver < 20 || devinfo->has_local_mem)
          return 0;
 
       if (INTEL_DEBUG(DEBUG_NO_CCS))
@@ -300,10 +300,13 @@ isl_drm_modifier_get_score(const struct intel_device_info *devinfo,
 
       return 4;
    case I915_FORMAT_MOD_4_TILED_BMG_CCS:
-      /* FIXME: Disable BMG modifier for
-       * https://gitlab.freedesktop.org/mesa/mesa/-/issues/13868
-       */
-      return 0;
+      if (devinfo->ver < 20 || !devinfo->has_local_mem)
+         return 0;
+
+      if (INTEL_DEBUG(DEBUG_NO_CCS))
+         return 0;
+
+      return 4;
    }
 }
 
