@@ -104,7 +104,9 @@
 
 /* Allocations are always at least 64 byte aligned, so 1 is an invalid value.
  * We use it to indicate the free list is empty. */
-#define EMPTY UINT32_MAX
+#define ANV_FREE_LIST_EMPTY_VAL UINT32_MAX
+
+#define ANV_FREE_LIST_EMPTY ((union anv_free_list) { { ANV_FREE_LIST_EMPTY_VAL, 0 } })
 
 /* On FreeBSD PAGE_SIZE is already defined in
  * /usr/include/machine/param.h that is indirectly
@@ -329,7 +331,7 @@ anv_free_list_pop(union anv_free_list *list,
    union anv_free_list current, new, old;
 
    current.u64 = list->u64;
-   while (current.offset != EMPTY) {
+   while (current.offset != ANV_FREE_LIST_EMPTY_VAL) {
       __sync_synchronize();
       new.offset = table->map[current.offset].next;
       new.count = current.count + 1;
@@ -598,13 +600,13 @@ anv_block_pool_grow(struct anv_block_pool *pool, struct anv_block_state *state,
    return pool->size;
 }
 
-static VkResult
-anv_block_pool_alloc_new(struct anv_block_pool *pool,
-                         struct anv_block_state *pool_state,
-                         uint32_t block_size,
-                         int64_t *offset,
-                         uint32_t *padding)
+VkResult
+anv_block_pool_alloc(struct anv_block_pool *pool,
+                     uint32_t block_size,
+                     int64_t *offset,
+                     uint32_t *padding)
 {
+   struct anv_block_state *pool_state = &pool->state;
    struct anv_block_state state, old, new;
 
    /* Most allocations won't generate any padding */
@@ -661,14 +663,6 @@ anv_block_pool_alloc_new(struct anv_block_pool *pool,
    }
 }
 
-VkResult
-anv_block_pool_alloc(struct anv_block_pool *pool,
-                     uint32_t block_size,
-                     int64_t *offset, uint32_t *padding)
-{
-   return anv_block_pool_alloc_new(pool, &pool->state, block_size, offset, padding);
-}
-
 VkResult
 anv_state_pool_init(struct anv_state_pool *pool,
                     struct anv_device *device,
@@ -1383,7 +1377,7 @@ anv_scratch_pool_finish(struct anv_device *device, struct anv_scratch_pool *pool
 
 struct anv_bo *
 anv_scratch_pool_alloc(struct anv_device *device, struct anv_scratch_pool *pool,
-                       gl_shader_stage stage, unsigned per_thread_scratch)
+                       mesa_shader_stage stage, unsigned per_thread_scratch)
 {
    if (per_thread_scratch == 0)
       return NULL;
@@ -1827,8 +1821,9 @@ anv_device_map_bo(struct anv_device *device,
    if (real != bo) {
       offset += (bo->offset - real->offset);
 
+      const uint64_t page_size = device->physical->page_size;
       /* KMD rounds munmap() to whole pages, so here doing some adjustments */
-      const uint64_t munmap_offset = ROUND_DOWN_TO(offset, 4096);
+      const uint64_t munmap_offset = ROUND_DOWN_TO(offset, page_size);
       if (munmap_offset != offset) {
          offset_adjustment = offset - munmap_offset;
          size += offset_adjustment;
@@ -1838,7 +1833,7 @@ anv_device_map_bo(struct anv_device *device,
             placed_addr -= offset_adjustment;
       }
 
-      assert((offset & (4096 - 1)) == 0);
+      assert((offset & (page_size - 1)) == 0);
    }
 
    void *map = device->kmd_backend->gem_mmap(device, bo, offset, size, placed_addr);
@@ -1866,14 +1861,15 @@ anv_device_unmap_bo(struct anv_device *device,
 
    struct anv_bo *real = anv_bo_get_real(bo);
    if (real != bo) {
+      const uint64_t page_size = device->physical->page_size;
       uint64_t slab_offset = bo->offset - real->offset;
 
-      if (ROUND_DOWN_TO(slab_offset, 4096) != slab_offset) {
-         slab_offset -= ROUND_DOWN_TO(slab_offset, 4096);
+      if (ROUND_DOWN_TO(slab_offset, page_size) != slab_offset) {
+         slab_offset -= ROUND_DOWN_TO(slab_offset, page_size);
          map -= slab_offset;
          map_size += slab_offset;
       }
-      assert(((uintptr_t)map & (4096 - 1)) == 0);
+      assert(((uintptr_t)map & (page_size - 1)) == 0);
    }
 
    if (replace) {
