@@ -23,19 +23,18 @@
 
 #include "anv_nir.h"
 #include "nir_builder.h"
-#include "compiler/brw_nir.h"
+#include "compiler/brw/brw_nir.h"
 #include "util/mesa-sha1.h"
 
 bool
 anv_nir_compute_push_layout(nir_shader *nir,
                             const struct anv_physical_device *pdevice,
                             enum brw_robustness_flags robust_flags,
-                            bool fragment_dynamic,
-                            bool mesh_dynamic,
+                            const struct anv_nir_push_layout_info *push_info,
+                            struct brw_base_prog_key *prog_key,
                             struct brw_stage_prog_data *prog_data,
                             struct anv_pipeline_bind_map *map,
                             const struct anv_pipeline_push_map *push_map,
-                            enum anv_descriptor_set_layout_type desc_type,
                             void *mem_ctx)
 {
    const struct brw_compiler *compiler = pdevice->compiler;
@@ -93,7 +92,7 @@ anv_nir_compute_push_layout(nir_shader *nir,
    const bool needs_wa_18019110168 =
       nir->info.stage == MESA_SHADER_FRAGMENT &&
       brw_nir_fragment_shader_needs_wa_18019110168(
-         devinfo, mesh_dynamic ? INTEL_SOMETIMES : INTEL_NEVER, nir);
+         devinfo, push_info->mesh_dynamic ? INTEL_SOMETIMES : INTEL_NEVER, nir);
 
    if (push_ubo_ranges && (robust_flags & BRW_ROBUSTNESS_UBO)) {
       /* We can't on-the-fly adjust our push ranges because doing so would
@@ -112,7 +111,7 @@ anv_nir_compute_push_layout(nir_shader *nir,
    }
 
    if (nir->info.stage == MESA_SHADER_FRAGMENT) {
-      if (fragment_dynamic) {
+      if (push_info->fragment_dynamic) {
          const uint32_t fs_msaa_flags_start =
             anv_drv_const_offset(gfx.fs_msaa_flags);
          const uint32_t fs_msaa_flags_end =
@@ -133,6 +132,20 @@ anv_nir_compute_push_layout(nir_shader *nir,
       }
    }
 
+   const bool needs_dyn_tess_config =
+      (nir->info.stage == MESA_SHADER_TESS_CTRL &&
+       (container_of(prog_key, struct brw_tcs_prog_key, base)->input_vertices == 0 ||
+        push_info->separate_tessellation)) ||
+      (nir->info.stage == MESA_SHADER_TESS_EVAL &&
+       push_info->separate_tessellation);
+   if (needs_dyn_tess_config) {
+      const uint32_t tess_config_start = anv_drv_const_offset(gfx.tess_config);
+      const uint32_t tess_config_end = tess_config_start +
+                                       anv_drv_const_size(gfx.tess_config);
+      push_start = MIN2(push_start, tess_config_start);
+      push_end = MAX2(push_end, tess_config_end);
+   }
+
    if (nir->info.stage == MESA_SHADER_COMPUTE && devinfo->verx10 < 125) {
       /* For compute shaders, we always have to have the subgroup ID.  The
        * back-end compiler will "helpfully" add it for us in the last push
@@ -245,7 +258,7 @@ anv_nir_compute_push_layout(nir_shader *nir,
     */
    const bool needs_padding_per_primitive =
       needs_wa_18019110168 ||
-      (mesh_dynamic &&
+      (push_info->mesh_dynamic &&
        (nir->info.inputs_read & VARYING_BIT_PRIMITIVE_ID));
 
    unsigned n_push_ranges = 0;
@@ -274,9 +287,8 @@ anv_nir_compute_push_layout(nir_shader *nir,
       }
 
       const unsigned max_push_buffers = needs_padding_per_primitive ? 3 : 4;
-      unsigned range_start_reg = push_constant_range.length;
 
-      for (int i = 0; i < 4; i++) {
+      for (unsigned i = 0; i < 4; i++) {
          struct brw_ubo_range *ubo_range = &prog_data->ubo_ranges[i];
          if (ubo_range->length == 0)
             continue;
@@ -301,11 +313,8 @@ anv_nir_compute_push_layout(nir_shader *nir,
          /* We only bother to shader-zero pushed client UBOs */
          if (binding->set < MAX_SETS &&
              (robust_flags & BRW_ROBUSTNESS_UBO)) {
-            prog_data->zero_push_reg |= BITFIELD64_RANGE(range_start_reg,
-                                                         ubo_range->length);
+            prog_data->robust_ubo_ranges |= (uint8_t) (1 << i);
          }
-
-         range_start_reg += ubo_range->length;
       }
    } else if (push_constant_range.length > 0) {
       /* For Ivy Bridge, the push constants packets have a different
@@ -349,11 +358,25 @@ anv_nir_compute_push_layout(nir_shader *nir,
 
    assert(n_push_ranges <= 4);
 
+   if (nir->info.stage == MESA_SHADER_TESS_CTRL && needs_dyn_tess_config) {
+      struct brw_tcs_prog_data *tcs_prog_data = brw_tcs_prog_data(prog_data);
+
+      const uint32_t tess_config_offset = anv_drv_const_offset(gfx.tess_config);
+      assert(tess_config_offset >= push_start);
+      tcs_prog_data->tess_config_param = (tess_config_offset - push_start) / 4;
+   }
+   if (nir->info.stage == MESA_SHADER_TESS_EVAL && push_info->separate_tessellation) {
+      struct brw_tes_prog_data *tes_prog_data = brw_tes_prog_data(prog_data);
+
+      const uint32_t tess_config_offset = anv_drv_const_offset(gfx.tess_config);
+      assert(tess_config_offset >= push_start);
+      tes_prog_data->tess_config_param = (tess_config_offset - push_start) / 4;
+   }
    if (nir->info.stage == MESA_SHADER_FRAGMENT) {
       struct brw_wm_prog_data *wm_prog_data =
          container_of(prog_data, struct brw_wm_prog_data, base);
 
-      if (fragment_dynamic) {
+      if (push_info->fragment_dynamic) {
          const uint32_t fs_msaa_flags_offset =
             anv_drv_const_offset(gfx.fs_msaa_flags);
          assert(fs_msaa_flags_offset >= push_start);
@@ -371,7 +394,7 @@ anv_nir_compute_push_layout(nir_shader *nir,
    }
 
 #if 0
-   fprintf(stderr, "stage=%s push ranges:\n", gl_shader_stage_name(nir->info.stage));
+   fprintf(stderr, "stage=%s push ranges:\n", mesa_shader_stage_name(nir->info.stage));
    for (unsigned i = 0; i < ARRAY_SIZE(map->push_ranges); i++)
       fprintf(stderr, "   range%i: %03u-%03u set=%u index=%u\n", i,
               map->push_ranges[i].start,
