@@ -82,6 +82,8 @@ anv_shader_deserialize(struct vk_device *vk_device,
    blob_copy_bytes(blob, data.bind_map.input_attachments,
                    sizeof(data.bind_map.input_attachments));
    blob_copy_bytes(blob, data.bind_map.push_ranges, sizeof(data.bind_map.push_ranges));
+   blob_copy_bytes(blob, data.bind_map.dynamic_descriptors,
+                   sizeof(data.bind_map.dynamic_descriptors));
 
    if (blob->overrun)
       return vk_error(device, VK_ERROR_UNKNOWN);
@@ -159,6 +161,8 @@ anv_shader_serialize(struct vk_device *device,
                     sizeof(shader->bind_map.input_attachments));
    blob_write_bytes(blob, shader->bind_map.push_ranges,
                     sizeof(shader->bind_map.push_ranges));
+   blob_write_bytes(blob, shader->bind_map.dynamic_descriptors,
+                    sizeof(shader->bind_map.dynamic_descriptors));
 
    return !blob->out_of_memory;
 }
