@@ -31,13 +31,21 @@ void state_pool_padding_test(void)
    struct anv_physical_device physical_device = {};
    struct anv_device device = {};
    struct anv_state_pool state_pool;
+   const uint32_t _1Gb = 1024 * 1024 * 1024;
 
    test_device_info_init(&physical_device.info);
    anv_device_set_physical(&device, &physical_device);
    device.kmd_backend = anv_kmd_backend_get(INTEL_KMD_TYPE_STUB);
    pthread_mutex_init(&device.mutex, NULL);
    anv_bo_cache_init(&device.bo_cache, &device);
-   anv_state_pool_init(&state_pool, &device, "test", 4096, 0, 4096);
+   anv_state_pool_init(&state_pool, &device,
+                       &(struct anv_state_pool_params) {
+                          .name         = "test",
+                          .base_address = 4096,
+                          .start_offset = 0,
+                          .block_size   = 4096,
+                          .max_size     = _1Gb,
+                       });
 
    /* Get the size of the underlying block_pool */
    struct anv_block_pool *bp = &state_pool.block_pool;
