@@ -22,6 +22,7 @@
 
 #include "xe/anv_device.h"
 #include "anv_private.h"
+#include "vk_debug_utils.h"
 
 #include "drm-uapi/gpu_scheduler.h"
 #include "drm-uapi/xe_drm.h"
@@ -56,6 +57,8 @@ VkResult anv_xe_device_setup_vm(struct anv_device *device)
                        "intel_bind_timeline_init failed");
    }
 
+   device->protected_session_id = DRM_XE_PXP_HWDRM_DEFAULT_SESSION;
+
    return VK_SUCCESS;
 }
 
@@ -70,7 +73,7 @@ drm_sched_priority_to_vk_priority(enum drm_sched_priority drm_sched_priority)
    case DRM_SCHED_PRIORITY_HIGH:
       return VK_QUEUE_GLOBAL_PRIORITY_HIGH_KHR;
    default:
-      unreachable("Invalid drm_sched_priority");
+      UNREACHABLE("Invalid drm_sched_priority");
       return VK_QUEUE_GLOBAL_PRIORITY_LOW_KHR;
    }
 }
@@ -85,7 +88,6 @@ anv_xe_physical_device_get_parameters(struct anv_physical_device *device)
       return vk_errorf(device, VK_ERROR_INITIALIZATION_FAILED,
                        "unable to query device config");
 
-   device->has_exec_timeline = true;
    device->has_vm_control = true;
    device->max_context_priority =
          drm_sched_priority_to_vk_priority(config->info[DRM_XE_QUERY_CONFIG_MAX_EXEC_QUEUE_PRIORITY]);
@@ -98,7 +100,7 @@ VkResult
 anv_xe_physical_device_init_memory_types(struct anv_physical_device *device)
 {
    if (anv_physical_device_has_vram(device)) {
-      if (device->info.ver >= 20 && !INTEL_DEBUG(DEBUG_NO_CCS)) {
+      if (device->info.ver >= 20) {
          device->memory.types[device->memory.type_count++] = (struct anv_memory_type) {
             .propertyFlags = VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT,
             .heapIndex = 0,
@@ -148,13 +150,11 @@ anv_xe_physical_device_init_memory_types(struct anv_physical_device *device)
          .heapIndex = 0,
       };
    } else {
-      if (device->info.ver >= 20 && !INTEL_DEBUG(DEBUG_NO_CCS)) {
-         device->memory.types[device->memory.type_count++] = (struct anv_memory_type) {
-            .propertyFlags = VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT,
-            .heapIndex = 0,
-            .compressed = true,
-         };
-      }
+      device->memory.types[device->memory.type_count++] = (struct anv_memory_type) {
+         .propertyFlags = VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT,
+         .heapIndex = 0,
+         .compressed = device->info.ver >= 20,
+      };
       device->memory.types[device->memory.type_count++] = (struct anv_memory_type) {
          .propertyFlags = VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT |
                           VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT |
@@ -208,5 +208,11 @@ anv_xe_device_check_status(struct vk_device *vk_device)
       }
    }
 
+   if (result != VK_SUCCESS)
+      return result;
+
+   if (INTEL_DEBUG(DEBUG_SHADER_PRINT))
+      result = vk_check_printf_status(vk_device, &device->printf);
+
    return result;
 }
