@@ -215,7 +215,7 @@ anv_nir_lower_multiview(nir_shader *shader, uint32_t view_mask)
     */
    nir_def *view_index = build_view_index(&state);
 
-   assert(view_index->parent_instr->block == nir_start_block(entrypoint));
+   assert(nir_def_block(view_index) == nir_start_block(entrypoint));
    b->cursor = nir_after_instr(view_index->parent_instr);
 
    /* Unless there is only one possible view index (that would be set
