@@ -32,6 +32,7 @@
 #ifndef MENUMS_H
 #define MENUMS_H
 
+#include <stdbool.h>
 #include "util/macros.h"
 
 /**
@@ -49,6 +50,19 @@ typedef enum
    API_OPENGL_LAST = API_OPENGL_CORE
 } gl_api;
 
+/**
+ * Checks if the api is for GLES 2.0 or later
+ */
+static inline bool
+_mesa_is_api_gles2(gl_api api)
+{
+#if HAVE_OPENGL_ES_2
+   return api == API_OPENGLES2;
+#else
+   return false;
+#endif
+}
+
 /**
  * An index for each type of texture object.  These correspond to the GL
  * texture target enums, such as GL_TEXTURE_2D, GL_TEXTURE_CUBE_MAP, etc.
@@ -90,7 +104,7 @@ typedef enum
  *
  *    result_bit = logic_op & (1 << (2 * src_bit + dst_bit))
  */
-enum PACKED gl_logicop_mode {
+enum ENUM_PACKED gl_logicop_mode {
    COLOR_LOGICOP_CLEAR = 0,
    COLOR_LOGICOP_NOR = 1,
    COLOR_LOGICOP_AND_INVERTED = 2,
