@@ -200,14 +200,16 @@ st_update_fp( struct st_context *st )
 void
 st_update_vp( struct st_context *st )
 {
-   struct gl_program *vp;
-
    /* find active shader and params -- Should be covered by
     * ST_NEW_VERTEX_PROGRAM
     */
-   assert(st->ctx->VertexProgram._Current);
-   vp = st->ctx->VertexProgram._Current;
-   assert(vp->info.stage == MESA_SHADER_VERTEX);
+   struct gl_program *vp = st->ctx->VertexProgram._Current;
+
+   if (!vp) {
+      _mesa_reference_program(st->ctx, &st->vp, NULL);
+      cso_set_vertex_shader_handle(st->cso_context, NULL);
+      return;
+   }
 
    if (st->shader_has_one_variant[MESA_SHADER_VERTEX] &&
        !st->ctx->Array._PerVertexEdgeFlagsEnabled) {
@@ -280,8 +282,8 @@ st_update_common_program(struct st_context *st, struct gl_program *prog,
 
    key.st = st->has_shareable_shaders ? NULL : st;
 
-   if (pipe_shader == PIPE_SHADER_GEOMETRY ||
-       pipe_shader == PIPE_SHADER_TESS_EVAL) {
+   if (pipe_shader == MESA_SHADER_GEOMETRY ||
+       pipe_shader == MESA_SHADER_TESS_EVAL) {
       key.clamp_color = st->clamp_vert_color_in_shader &&
                         st->ctx->Light._ClampVertexColor &&
                         (prog->info.outputs_written &
@@ -291,7 +293,7 @@ st_update_common_program(struct st_context *st, struct gl_program *prog,
                           VARYING_SLOT_BFC1));
 
       if (st->lower_ucp && st_user_clip_planes_enabled(st->ctx) &&
-          (pipe_shader == PIPE_SHADER_GEOMETRY ||
+          (pipe_shader == MESA_SHADER_GEOMETRY ||
              !st->ctx->GeometryProgram._Current))
          key.lower_ucp = st->ctx->Transform.ClipPlanesEnabled;
 
@@ -314,7 +316,7 @@ st_update_gp(struct st_context *st)
 {
    void *shader = st_update_common_program(st,
                                            st->ctx->GeometryProgram._Current,
-                                           PIPE_SHADER_GEOMETRY, &st->gp);
+                                           MESA_SHADER_GEOMETRY, &st->gp);
    cso_set_geometry_shader_handle(st->cso_context, shader);
 }
 
@@ -324,7 +326,7 @@ st_update_tcp(struct st_context *st)
 {
    void *shader = st_update_common_program(st,
                                            st->ctx->TessCtrlProgram._Current,
-                                           PIPE_SHADER_TESS_CTRL, &st->tcp);
+                                           MESA_SHADER_TESS_CTRL, &st->tcp);
    cso_set_tessctrl_shader_handle(st->cso_context, shader);
 }
 
@@ -334,7 +336,7 @@ st_update_tep(struct st_context *st)
 {
    void *shader = st_update_common_program(st,
                                            st->ctx->TessEvalProgram._Current,
-                                           PIPE_SHADER_TESS_EVAL, &st->tep);
+                                           MESA_SHADER_TESS_EVAL, &st->tep);
    cso_set_tesseval_shader_handle(st->cso_context, shader);
 }
 
@@ -344,6 +346,24 @@ st_update_cp(struct st_context *st)
 {
    void *shader = st_update_common_program(st,
                                            st->ctx->ComputeProgram._Current,
-                                           PIPE_SHADER_COMPUTE, &st->cp);
+                                           MESA_SHADER_COMPUTE, &st->cp);
    cso_set_compute_shader_handle(st->cso_context, shader);
 }
+
+void
+st_update_tp(struct st_context *st)
+{
+   void *shader = st_update_common_program(st,
+                                           st->ctx->TaskProgram._Current,
+                                           MESA_SHADER_TASK, &st->tp);
+   cso_set_task_shader_handle(st->cso_context, shader);
+}
+
+void
+st_update_mp(struct st_context *st)
+{
+   void *shader = st_update_common_program(st,
+                                           st->ctx->MeshProgram._Current,
+                                           MESA_SHADER_MESH, &st->mp);
+   cso_set_mesh_shader_handle(st->cso_context, shader);
+}
