@@ -140,7 +140,7 @@ st_make_bitmap_texture(struct gl_context *ctx, GLsizei width, GLsizei height,
     * Create texture to hold bitmap pattern.
     */
    pt = st_texture_create(st, st->internal_target, st->bitmap.tex_format,
-                          0, width, height, 1, 1, 0,
+                          0, width, height, 1, 1, 0, PIPE_RESOURCE_FLAG_MAP_UNSYNCHRONIZED,
                           PIPE_BIND_SAMPLER_VIEW, false,
                           PIPE_COMPRESSION_FIXED_RATE_NONE);
    if (!pt) {
@@ -149,7 +149,7 @@ st_make_bitmap_texture(struct gl_context *ctx, GLsizei width, GLsizei height,
    }
 
    dest = pipe_texture_map(st->pipe, pt, 0, 0,
-                            PIPE_MAP_WRITE,
+                            PIPE_MAP_WRITE | PIPE_MAP_UNSYNCHRONIZED,
                             0, 0, width, height, &transfer);
 
    /* Put image into texture transfer */
@@ -226,6 +226,8 @@ setup_render_state(struct gl_context *ctx,
    cso_set_tessctrl_shader_handle(cso, NULL);
    cso_set_tesseval_shader_handle(cso, NULL);
    cso_set_geometry_shader_handle(cso, NULL);
+   cso_set_task_shader_handle(cso, NULL);
+   cso_set_mesh_shader_handle(cso, NULL);
 
    /* user samplers, plus our bitmap sampler */
    {
@@ -237,7 +239,7 @@ setup_render_state(struct gl_context *ctx,
          samplers[i] = &st->state.frag_samplers[i];
       }
       samplers[fpv->bitmap_sampler] = &st->bitmap.sampler;
-      cso_set_samplers(cso, PIPE_SHADER_FRAGMENT, num,
+      cso_set_samplers(cso, MESA_SHADER_FRAGMENT, num,
                        (const struct pipe_sampler_state **) samplers);
    }
 
@@ -246,13 +248,13 @@ setup_render_state(struct gl_context *ctx,
       unsigned extra_sampler_views = 0;
       struct pipe_sampler_view *sampler_views[PIPE_MAX_SAMPLERS];
       unsigned num_views =
-         st_get_sampler_views(st, PIPE_SHADER_FRAGMENT, fp, sampler_views, &extra_sampler_views);
+         st_get_sampler_views(st, MESA_SHADER_FRAGMENT, fp, sampler_views, &extra_sampler_views);
 
       num_views = MAX2(fpv->bitmap_sampler + 1, num_views);
       sampler_views[fpv->bitmap_sampler] = sv;
-      pipe->set_sampler_views(pipe, PIPE_SHADER_FRAGMENT, 0, num_views, 0,
+      pipe->set_sampler_views(pipe, MESA_SHADER_FRAGMENT, 0, num_views, 0,
                               sampler_views);
-      st->state.num_sampler_views[PIPE_SHADER_FRAGMENT] = num_views;
+      st->state.num_sampler_views[MESA_SHADER_FRAGMENT] = num_views;
 
       for (unsigned i = 0; i < num_views; i++)
          pipe->sampler_view_release(pipe, sampler_views[i]);
@@ -283,11 +285,10 @@ restore_render_state(struct gl_context *ctx)
     * use them.
     */
    cso_restore_state(cso, CSO_UNBIND_FS_SAMPLERVIEWS);
-   st->state.num_sampler_views[PIPE_SHADER_FRAGMENT] = 0;
+   st->state.num_sampler_views[MESA_SHADER_FRAGMENT] = 0;
 
    ctx->Array.NewVertexElements = true;
-   ctx->NewDriverState |= ST_NEW_VERTEX_ARRAYS |
-                          ST_NEW_FS_SAMPLER_VIEWS;
+   ST_SET_STATE2(ctx->NewDriverState, ST_NEW_VERTEX_ARRAYS, ST_NEW_FS_SAMPLER_VIEWS);
 }
 
 
@@ -347,7 +348,7 @@ draw_bitmap_quad(struct gl_context *ctx, GLint x, GLint y, GLfloat z,
    restore_render_state(ctx);
 
    /* We uploaded modified constants, need to invalidate them. */
-   ctx->NewDriverState |= ST_NEW_FS_CONSTANTS;
+   ST_SET_STATE(ctx->NewDriverState, ST_NEW_FS_CONSTANTS);
 }
 
 
@@ -372,7 +373,7 @@ reset_cache(struct st_context *st)
    cache->texture = st_texture_create(st, st->internal_target,
                                       st->bitmap.tex_format, 0,
                                       BITMAP_CACHE_WIDTH, BITMAP_CACHE_HEIGHT,
-                                      1, 1, 0,
+                                      1, 1, 0, PIPE_RESOURCE_FLAG_MAP_UNSYNCHRONIZED,
                                       PIPE_BIND_SAMPLER_VIEW,
                                       false,
                                       PIPE_COMPRESSION_FIXED_RATE_NONE);
@@ -415,7 +416,7 @@ create_cache_trans(struct st_context *st)
     * Subsequent glBitmap calls will write into the texture image.
     */
    cache->buffer = pipe_texture_map(pipe, cache->texture, 0, 0,
-                                     PIPE_MAP_WRITE, 0, 0,
+                                     PIPE_MAP_WRITE | PIPE_MAP_UNSYNCHRONIZED, 0, 0,
                                      BITMAP_CACHE_WIDTH,
                                      BITMAP_CACHE_HEIGHT, &cache->trans);
 
@@ -639,7 +640,11 @@ st_Bitmap(struct gl_context *ctx, GLint x, GLint y,
     * for bitmap drawing uses no constants and the FS constants are
     * explicitly uploaded in the draw_bitmap_quad() function.
     */
-   st_validate_state(st, ST_PIPELINE_META_STATE_MASK & ~ST_NEW_CONSTANTS);
+   ST_PIPELINE_META_STATE_MASK(mask);
+   st_state_bitset constants = {0};
+   ST_SET_SHADER_STATES(constants, CONSTANTS);
+   BITSET_ANDNOT(mask, mask, constants);
+   st_validate_state(st, mask);
 
    struct pipe_sampler_view *view = NULL;
 
