@@ -313,10 +313,11 @@ st_RenderMode(struct gl_context *ctx, GLenum newMode )
       ctx->Driver.DrawGalliumMultiMode = st_feedback_draw_vbo_multi_mode;
       /* need to generate/use a vertex program that emits pos/color/tex */
       if (vp)
-         ctx->NewDriverState |= ST_NEW_VERTEX_PROGRAM(ctx, vp);
+         ST_SET_VERTEX_PROGRAM_STATES(ctx->NewDriverState, ctx, vp);
    }
 
    /* Restore geometry shader states when leaving GL_SELECT mode. */
    if (ctx->RenderMode == GL_SELECT && ctx->Const.HardwareAcceleratedSelect)
-      ctx->NewDriverState |= ST_NEW_GS_SSBOS | ST_NEW_GS_CONSTANTS | ST_NEW_GS_STATE;
+      ST_SET_STATE3(ctx->NewDriverState, ST_NEW_GS_SSBOS, ST_NEW_GS_CONSTANTS,
+                    ST_NEW_GS_STATE);
 }
