@@ -31,16 +31,16 @@
 #define ST_FORMAT_H
 
 #include "main/formats.h"
-#include "main/glheader.h"
+#include "util/glheader.h"
 
-#include "pipe/p_defines.h"
-#include "pipe/p_format.h"
+#include "util/format/u_formats.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 struct gl_context;
+struct st_context;
 struct pipe_screen;
 
 
@@ -55,11 +55,12 @@ extern enum pipe_format
 st_choose_format(struct st_context *st, GLenum internalFormat,
                  GLenum format, GLenum type,
                  enum pipe_texture_target target, unsigned sample_count,
-                 unsigned bindings, boolean allow_dxt);
+                 unsigned storage_sample_count,
+                 unsigned bindings, bool swap_bytes, bool allow_dxt);
 
 extern enum pipe_format
-st_choose_renderbuffer_format(struct st_context *st,
-                              GLenum internalFormat, unsigned sample_count);
+st_choose_matching_format_noverify(struct st_context *st,
+                                   GLenum format, GLenum type, GLboolean swapBytes);
 
 extern enum pipe_format
 st_choose_matching_format(struct st_context *st, unsigned bind,
@@ -69,14 +70,14 @@ extern mesa_format
 st_ChooseTextureFormat(struct gl_context * ctx, GLenum target,
                        GLint internalFormat,
                        GLenum format, GLenum type);
-
+bool
+st_QueryTextureFormatSupport(struct gl_context *ctx, GLenum target, GLenum internalFormat);
 void
 st_QueryInternalFormat(struct gl_context *ctx, GLenum target,
                        GLenum internalFormat, GLenum pname, GLint *params);
 
 extern void
-st_translate_color(const union gl_color_union *colorIn,
-                   union pipe_color_union *colorOut,
+st_translate_color(union pipe_color_union *color,
                    GLenum baseFormat, GLboolean is_integer);
 
 #ifdef __cplusplus
