@@ -188,7 +188,6 @@ st_pbo_draw(struct st_context *st, const struct st_pbo_addresses *addr,
             unsigned surface_width, unsigned surface_height)
 {
    struct cso_context *cso = st->cso_context;
-   struct pipe_context *pipe = st->pipe;
 
    /* Setup vertex and geometry shaders */
    if (!st->pbo.vs) {
@@ -211,7 +210,11 @@ st_pbo_draw(struct st_context *st, const struct st_pbo_addresses *addr,
 
    cso_set_tesseval_shader_handle(cso, NULL);
 
+   cso_set_task_shader_handle(cso, NULL);
+   cso_set_mesh_shader_handle(cso, NULL);
+
    /* Upload vertices */
+   struct pipe_resource *releasebuf = NULL;
    {
       struct pipe_vertex_buffer vbo = {0};
       struct cso_velems_state velem;
@@ -224,7 +227,7 @@ st_pbo_draw(struct st_context *st, const struct st_pbo_addresses *addr,
       float *verts = NULL;
 
       u_upload_alloc(st->pipe->stream_uploader, 0, 8 * sizeof(float), 4,
-                     &vbo.buffer_offset, &vbo.buffer.resource, (void **) &verts);
+                     &vbo.buffer_offset, &vbo.buffer.resource, &releasebuf, (void **) &verts);
       if (!verts)
          return false;
 
@@ -248,7 +251,7 @@ st_pbo_draw(struct st_context *st, const struct st_pbo_addresses *addr,
       velem.velems[0].dual_slot = false;
 
       cso_set_vertex_elements(cso, &velem);
-      cso_set_vertex_buffers(cso, 1, true, &vbo);
+      cso_set_vertex_buffers(cso, 1, &vbo);
    }
 
    /* Upload constants */
@@ -260,9 +263,7 @@ st_pbo_draw(struct st_context *st, const struct st_pbo_addresses *addr,
       cb.buffer_offset = 0;
       cb.buffer_size = sizeof(addr->constants);
 
-      pipe->set_constant_buffer(pipe, PIPE_SHADER_FRAGMENT, 0, false, &cb);
-
-      pipe_resource_reference(&cb.buffer, NULL);
+      pipe_upload_constant_buffer0(st->pipe, MESA_SHADER_FRAGMENT, &cb);
    }
 
    /* Rasterizer state */
@@ -277,6 +278,7 @@ st_pbo_draw(struct st_context *st, const struct st_pbo_addresses *addr,
       cso_draw_arrays_instanced(cso, MESA_PRIM_TRIANGLE_STRIP,
                                 0, 4, 0, addr->depth);
    }
+   pipe_resource_release(st->pipe, releasebuf);
 
    return true;
 }
@@ -285,7 +287,7 @@ void *
 st_pbo_create_vs(struct st_context *st)
 {
    const nir_shader_compiler_options *options =
-      st_get_nir_compiler_options(st, MESA_SHADER_VERTEX);
+      st->screen->nir_options[MESA_SHADER_VERTEX];
 
    nir_builder b = nir_builder_init_simple_shader(MESA_SHADER_VERTEX, options,
                                                   "st/pbo VS");
@@ -325,7 +327,7 @@ void *
 st_pbo_create_gs(struct st_context *st)
 {
    const nir_shader_compiler_options *options =
-      st_get_nir_compiler_options(st, MESA_SHADER_GEOMETRY);
+      st->screen->nir_options[MESA_SHADER_GEOMETRY];
 
    nir_builder b = nir_builder_init_simple_shader(MESA_SHADER_GEOMETRY, options,
                                                   "st/pbo GS");
@@ -396,7 +398,7 @@ create_fs(struct st_context *st, bool download,
           bool need_layer)
 {
    const nir_shader_compiler_options *options =
-      st_get_nir_compiler_options(st, MESA_SHADER_FRAGMENT);
+      st->screen->nir_options[MESA_SHADER_FRAGMENT];
 
    nir_builder b = nir_builder_init_simple_shader(MESA_SHADER_FRAGMENT, options,
                                                   download ?
@@ -513,6 +515,7 @@ create_fs(struct st_context *st, bool download,
    tex->coord_components =
       glsl_get_sampler_coordinate_components(tex_var->type);
    tex->is_array = target >= PIPE_TEXTURE_1D_ARRAY;
+   tex->can_speculate = true;
 
    tex->dest_type = nir_get_nir_type_for_glsl_base_type(glsl_get_sampler_result_type(tex_var->type));
    tex->src[0].src_type = nir_tex_src_texture_deref;
@@ -640,7 +643,7 @@ st_init_pbo_helpers(struct st_context *st)
    st->pbo.upload_enabled =
       screen->caps.texture_buffer_objects &&
       screen->caps.texture_buffer_offset_alignment >= 1 &&
-      screen->shader_caps[PIPE_SHADER_FRAGMENT].integers;
+      screen->shader_caps[MESA_SHADER_FRAGMENT].integers;
    if (!st->pbo.upload_enabled)
       return;
 
@@ -648,7 +651,7 @@ st_init_pbo_helpers(struct st_context *st)
       st->pbo.upload_enabled &&
       screen->caps.sampler_view_target &&
       screen->caps.framebuffer_no_attachment &&
-      screen->shader_caps[PIPE_SHADER_FRAGMENT].max_shader_images >= 1;
+      screen->shader_caps[MESA_SHADER_FRAGMENT].max_shader_images >= 1;
 
    st->pbo.rgba_only =
       screen->caps.buffer_sampler_view_rgba_only;
