@@ -120,7 +120,7 @@ struct nak_xfb_info {
 #pragma GCC diagnostic push
 #pragma GCC diagnostic error "-Wpadded"
 struct nak_shader_info {
-   gl_shader_stage stage;
+   mesa_shader_stage stage;
 
    /** Shader model */
    uint8_t sm;
@@ -146,7 +146,7 @@ struct nak_shader_info {
    uint32_t num_instrs;
 
    /** Number of cycles used by fixed-latency instructions */
-   uint32_t num_static_cycles;
+   uint64_t num_static_cycles;
 
    /** Number of spills from GPRs to Memory */
    uint32_t num_spills_to_mem;
@@ -211,6 +211,8 @@ struct nak_shader_info {
       struct nak_xfb_info xfb;
    } vtg;
 
+   uint8_t _pad1[4];
+
    /** Shader header for 3D stages */
    uint32_t hdr[32];
 };
@@ -242,8 +244,7 @@ struct nak_qmd_cbuf {
 struct nak_qmd_info {
    uint64_t addr;
 
-   uint16_t smem_size;
-   uint16_t smem_max;
+   uint32_t smem_size;
 
    uint32_t global_size[3];
 
