@@ -28,7 +28,7 @@ pub fn src_is_upred_reg(src: &Src) -> bool {
 pub fn src_is_reg(src: &Src, reg_file: RegFile) -> bool {
     match &src.src_ref {
         SrcRef::Zero | SrcRef::True | SrcRef::False => true,
-        SrcRef::SSA(ssa) => ssa.file() == Some(reg_file),
+        SrcRef::SSA(ssa) => ssa.file() == reg_file,
         SrcRef::Imm32(_) | SrcRef::CBuf(_) => false,
         SrcRef::Reg(_) => panic!("Not in SSA form"),
     }
@@ -177,7 +177,7 @@ pub trait LegalizeBuildHelpers: SSABuilder {
         };
 
         if DEBUG.annotate() {
-            self.push_instr(Instr::new_boxed(OpAnnotate {
+            self.push_instr(Instr::new(OpAnnotate {
                 annotation: "copy generated by legalizer".into(),
             }));
         }
@@ -446,33 +446,30 @@ fn legalize_instr(
                         && vec.comps() > 1
                         && !pinned.contains(vec)
                     {
-                        b.copy_ssa_ref(vec, vec.file().unwrap().to_warp());
+                        b.copy_ssa_ref(vec, vec.file().to_warp());
                     }
                 }
                 SrcRef::CBuf(CBufRef {
                     buf: CBuf::BindlessSSA(handle),
                     ..
-                }) => assert!(pinned.contains(handle)),
+                }) => assert!(pinned.contains(&SSARef::new(handle))),
                 _ => (),
             }
         }
     }
 
     // OpBreak and OpBSsy impose additional RA constraints
-    match &mut instr.op {
-        Op::Break(OpBreak {
-            bar_in, bar_out, ..
-        })
-        | Op::BSSy(OpBSSy {
-            bar_in, bar_out, ..
-        }) => {
-            let bar_in_ssa = bar_in.src_ref.as_ssa().unwrap();
-            if !bar_out.is_none() && bl.is_live_after_ip(&bar_in_ssa[0], ip) {
-                let gpr = b.bmov_to_gpr(bar_in.clone());
-                let tmp = b.bmov_to_bar(gpr.into());
-                *bar_in = tmp.into();
-            }
+    let mut legalize_break_bssy = |bar_in: &mut Src, bar_out: &mut Dst| {
+        let bar_in_ssa = bar_in.src_ref.as_ssa().unwrap();
+        if !bar_out.is_none() && bl.is_live_after_ip(&bar_in_ssa[0], ip) {
+            let gpr = b.bmov_to_gpr(bar_in.clone());
+            let tmp = b.bmov_to_bar(gpr.into());
+            *bar_in = tmp.into();
         }
+    };
+    match &mut instr.op {
+        Op::Break(op) => legalize_break_bssy(&mut op.bar_in, &mut op.bar_out),
+        Op::BSSy(op) => legalize_break_bssy(&mut op.bar_in, &mut op.bar_out),
         _ => (),
     }
 
