@@ -6,7 +6,6 @@ mod assign_regs;
 mod builder;
 mod calc_instr_deps;
 mod const_tracker;
-mod dataflow;
 mod from_nir;
 mod ir;
 mod legalize;
@@ -27,6 +26,7 @@ mod opt_uniform_instrs;
 mod qmd;
 mod reg_tracker;
 mod repair_ssa;
+mod sm120_instr_latencies;
 mod sm20;
 mod sm30_instr_latencies;
 mod sm32;
