@@ -105,7 +105,7 @@ impl RegLatencySM80 {
             Op::IAdd3(_) | Op::IAdd3X(_) => CoupledAlu,
 
             Op::BMsk(_) => CoupledAlu,
-            // Sgxt => CoupledAlu,
+            Op::Sgxt(_) => CoupledAlu,
             Op::Lop3(_) => CoupledAlu,
             Op::Flo(_) => Decoupled,
             Op::ISetP(_) => CoupledAlu,
@@ -249,7 +249,7 @@ impl RegLatencySM80 {
             Op::Isberd(_) => DecoupledAgu,
             Op::LdTram(_) => DecoupledAgu,
             Op::Shfl(_) => DecoupledAgu,
-            //Op::LdSm(_) => DecoupledAgu
+            Op::Ldsm(_) => DecoupledAgu,
             x => {
                 panic!("Illegal instuction in reg category {}", x);
             }
@@ -846,6 +846,7 @@ impl PredLatencySM80 {
     fn op_category(op: &Op) -> PredLatencySM80 {
         match op {
             Op::Atom(_) => PredLatencySM80::Decoupled,
+            Op::Bra(_) => PredLatencySM80::Decoupled,
             Op::DSetP(_) => PredLatencySM80::RedirectedFP64,
             Op::FMnMx(_) | Op::FSetP(_) => PredLatencySM80::Coupled,
             Op::HFma2(_) => PredLatencySM80::FP16,
@@ -1094,7 +1095,7 @@ impl URegLatencySM80 {
             Op::PSetP(_) => vcoupled,
             // UR2UP
             Op::Sel(_) => vcoupled,
-            // SGXT
+            Op::Sgxt(_) => vcoupled,
             Op::Shf(_) => vcoupled,
             Op::Shfl(_) => vdecoupled,
 
@@ -1291,6 +1292,7 @@ impl UPredLatencySM80 {
             | Op::LeaX(_)
             | Op::Lop3(_)
             | Op::Mov(_) => Udp,
+            Op::Bra(_) => Bra_Jmp,
             Op::Ldc(_) => Uldc_Mma,
             Op::PLop3(_) => {
                 if uniform_op {
@@ -1458,7 +1460,7 @@ impl SM80Latency {
     ) -> u32 {
         let dst_file = match &write.dsts_as_slice()[dst_idx] {
             Dst::None => return 0,
-            Dst::SSA(vec) => vec.file().unwrap(),
+            Dst::SSA(vec) => vec.file(),
             Dst::Reg(reg) => reg.file(),
         };
 
@@ -1515,7 +1517,7 @@ impl SM80Latency {
     pub fn war(read: &Op, src_idx: usize, write: &Op, dst_idx: usize) -> u32 {
         let dst_file = match &write.dsts_as_slice()[dst_idx] {
             Dst::None => return 0,
-            Dst::SSA(vec) => vec.file().unwrap(),
+            Dst::SSA(vec) => vec.file(),
             Dst::Reg(reg) => reg.file(),
         };
 
@@ -1567,7 +1569,7 @@ impl SM80Latency {
     ) -> u32 {
         let dst_file = match &a.dsts_as_slice()[a_dst_idx] {
             Dst::None => return 0,
-            Dst::SSA(vec) => vec.file().unwrap(),
+            Dst::SSA(vec) => vec.file(),
             Dst::Reg(reg) => reg.file(),
         };
 
