@@ -432,12 +432,14 @@ cs_dst64(struct cs_builder *b, struct cs_index dst)
    return cs_dst_tuple(b, dst, 2, BITFIELD_MASK(2));
 }
 
+#define CS_MAX_REG_TUPLE_SIZE 16
+
 static inline struct cs_index
 cs_reg_tuple(ASSERTED struct cs_builder *b, uint8_t reg, uint8_t size)
 {
    assert(reg + size <= b->conf.nr_registers - b->conf.nr_kernel_registers &&
           "overflowed register file");
-   assert(size <= 16 && "unsupported");
+   assert(size <= CS_MAX_REG_TUPLE_SIZE && "unsupported");
 
    return (struct cs_index){
       .type = CS_INDEX_REGISTER,
@@ -492,6 +494,25 @@ cs_extract32(struct cs_builder *b, struct cs_index idx, unsigned word)
    return cs_reg32(b, idx.reg + word);
 }
 
+static inline struct cs_index
+cs_extract64(struct cs_builder *b, struct cs_index idx, unsigned word)
+{
+   assert(idx.type == CS_INDEX_REGISTER && "unsupported");
+   assert(word + 1 < idx.size && "overrun");
+
+   return cs_reg64(b, idx.reg + word);
+}
+
+static inline struct cs_index
+cs_extract_tuple(struct cs_builder *b, struct cs_index idx, unsigned word,
+                 unsigned size)
+{
+   assert(idx.type == CS_INDEX_REGISTER && "unsupported");
+   assert(word + size < idx.size && "overrun");
+
+   return cs_reg_tuple(b, idx.reg + word, size);
+}
+
 static inline struct cs_block *
 cs_cur_block(struct cs_builder *b)
 {
@@ -513,6 +534,17 @@ cs_reserve_instrs(struct cs_builder *b, uint32_t num_instrs)
    if (unlikely(!cs_is_valid(b)))
       return false;
 
+   /* Make sure we have sufficient capacity if we wont allocate more */
+   if (b->conf.alloc_buffer == NULL) {
+      if (unlikely(b->cur_chunk.size + num_instrs > b->cur_chunk.buffer.capacity)) {
+         assert(!"Out of CS space");
+         b->invalid = true;
+         return false;
+      }
+
+      return true;
+   }
+
    /* Lazy root chunk allocation. */
    if (unlikely(!b->root_chunk.buffer.cpu)) {
       b->root_chunk.buffer = b->conf.alloc_buffer(b->conf.cookie);
@@ -530,8 +562,10 @@ cs_reserve_instrs(struct cs_builder *b, uint32_t num_instrs)
     * We actually do this a few instructions before running out, because the
     * sequence to jump to a new queue takes multiple instructions.
     */
-   if (unlikely((b->cur_chunk.size + num_instrs + JUMP_SEQ_INSTR_COUNT) >
-                b->cur_chunk.buffer.capacity)) {
+   bool jump_to_next_chunk =
+      (b->cur_chunk.size + num_instrs + JUMP_SEQ_INSTR_COUNT) >
+      b->cur_chunk.buffer.capacity;
+   if (unlikely(jump_to_next_chunk)) {
       /* Now, allocate a new chunk */
       struct cs_buffer newbuf = b->conf.alloc_buffer(b->conf.cookie);
 
@@ -1068,9 +1102,9 @@ cs_invert_cond(enum mali_cs_condition cond)
    case MALI_CS_CONDITION_GEQUAL:
       return MALI_CS_CONDITION_LESS;
    case MALI_CS_CONDITION_ALWAYS:
-      unreachable("cannot invert ALWAYS");
+      UNREACHABLE("cannot invert ALWAYS");
    default:
-      unreachable("invalid cond");
+      UNREACHABLE("invalid cond");
    }
 }
 
@@ -1114,7 +1148,7 @@ cs_branch_label_cond64(struct cs_builder *b, struct cs_label *label,
       cs_branch_label_cond32(b, label, MALI_CS_CONDITION_EQUAL, val_hi);
       break;
    default:
-      unreachable("unsupported 64bit condition");
+      UNREACHABLE("unsupported 64bit condition");
    }
 
    cs_set_label(b, &false_label);
