@@ -75,7 +75,7 @@ pan_kmod_dev_free(const struct pan_kmod_dev *dev, void *data)
 
 static inline void
 pan_kmod_bo_init(struct pan_kmod_bo *bo, struct pan_kmod_dev *dev,
-                 struct pan_kmod_vm *exclusive_vm, size_t size, uint32_t flags,
+                 struct pan_kmod_vm *exclusive_vm, uint64_t size, uint32_t flags,
                  uint32_t handle)
 {
    bo->dev = dev;
@@ -88,11 +88,12 @@ pan_kmod_bo_init(struct pan_kmod_bo *bo, struct pan_kmod_dev *dev,
 
 static inline void
 pan_kmod_vm_init(struct pan_kmod_vm *vm, struct pan_kmod_dev *dev,
-                 uint32_t handle, uint32_t flags)
+                 uint32_t handle, uint32_t flags, uint64_t pgsize_bitmap)
 {
    vm->dev = dev;
    vm->handle = handle;
    vm->flags = flags;
+   vm->pgsize_bitmap = pgsize_bitmap;
 }
 
 static inline int
