@@ -61,7 +61,8 @@ pan_shader_compile(nir_shader *s, struct pan_compile_inputs *inputs,
 
    memset(info, 0, sizeof(*info));
 
-   NIR_PASS(_, s, nir_lower_printf_buffer, 0, LIBPAN_PRINTF_BUFFER_SIZE - 8);
+   NIR_PASS(_, s, nir_inline_sysval, nir_intrinsic_load_printf_buffer_size,
+            LIBPAN_PRINTF_BUFFER_SIZE - 8);
 
    if (arch >= 6)
       bifrost_compile_shader_nir(s, inputs, binary, info);
