@@ -29,9 +29,7 @@ static void
 finish_render_desc_ringbuf(struct panvk_gpu_queue *queue)
 {
    struct panvk_device *dev = to_panvk_device(queue->vk.base.device);
-   struct panvk_instance *instance =
-      to_panvk_instance(dev->vk.physical->instance);
-   bool tracing_enabled = instance->debug_flags & PANVK_DEBUG_TRACE;
+   const bool tracing_enabled = PANVK_DEBUG(TRACE);
    struct panvk_desc_ringbuf *ringbuf = &queue->render_desc_ringbuf;
 
    panvk_pool_free_mem(&ringbuf->syncobj);
@@ -74,9 +72,7 @@ static VkResult
 init_render_desc_ringbuf(struct panvk_gpu_queue *queue)
 {
    struct panvk_device *dev = to_panvk_device(queue->vk.base.device);
-   struct panvk_instance *instance =
-      to_panvk_instance(dev->vk.physical->instance);
-   bool tracing_enabled = instance->debug_flags & PANVK_DEBUG_TRACE;
+   const bool tracing_enabled = PANVK_DEBUG(TRACE);
    uint32_t flags = panvk_device_adjust_bo_flags(dev, PAN_KMOD_BO_FLAG_NO_MMAP);
    struct panvk_desc_ringbuf *ringbuf = &queue->render_desc_ringbuf;
    uint64_t dev_addr = 0;
@@ -190,7 +186,7 @@ finish_subqueue_tracing(struct panvk_gpu_queue *queue,
    struct panvk_subqueue *subq = &queue->subqueues[subqueue];
 
    if (subq->tracebuf.addr.dev) {
-      size_t pgsize = getpagesize();
+      uint64_t pgsize = panvk_get_gpu_page_size(dev);
 
       pandecode_inject_free(dev->debug.decode_ctx, subq->tracebuf.addr.dev,
                             subq->tracebuf.size);
@@ -228,12 +224,9 @@ init_subqueue_tracing(struct panvk_gpu_queue *queue,
 {
    struct panvk_device *dev = to_panvk_device(queue->vk.base.device);
    struct panvk_subqueue *subq = &queue->subqueues[subqueue];
-   struct panvk_instance *instance =
-      to_panvk_instance(dev->vk.physical->instance);
-   unsigned debug = instance->debug_flags;
    uint64_t dev_addr;
 
-   if (!(debug & PANVK_DEBUG_TRACE))
+   if (!PANVK_DEBUG(TRACE))
       return VK_SUCCESS;
 
    subq->reg_file =
@@ -265,7 +258,7 @@ init_subqueue_tracing(struct panvk_gpu_queue *queue,
    }
 
    /* Add a guard page. */
-   size_t pgsize = getpagesize();
+   uint64_t pgsize = panvk_get_gpu_page_size(dev);
    dev_addr = panvk_as_alloc(dev, subq->tracebuf.size + pgsize, pgsize);
 
    if (!dev_addr)
@@ -342,9 +335,6 @@ init_subqueue(struct panvk_gpu_queue *queue, enum panvk_subqueue_id subqueue)
    struct panvk_subqueue *subq = &queue->subqueues[subqueue];
    const struct panvk_physical_device *phys_dev =
       to_panvk_physical_device(queue->vk.base.device->physical);
-   struct panvk_instance *instance =
-      to_panvk_instance(dev->vk.physical->instance);
-   unsigned debug = instance->debug_flags;
    struct panvk_cs_sync64 *syncobjs = panvk_priv_mem_host_addr(queue->syncobjs);
 
    VkResult result = init_subqueue_tracing(queue, subqueue);
@@ -369,7 +359,7 @@ init_subqueue(struct panvk_gpu_queue *queue, enum panvk_subqueue_id subqueue)
    /* When tracing is enabled, we want to use a non-cached pool, so can get
     * up-to-date context even if the CS crashed in the middle. */
    struct panvk_pool *mempool =
-      (debug & PANVK_DEBUG_TRACE) ? &dev->mempools.rw_nc : &dev->mempools.rw;
+      PANVK_DEBUG(TRACE) ? &dev->mempools.rw_nc : &dev->mempools.rw;
 
    subq->context = panvk_pool_alloc_mem(mempool, alloc_info);
    if (!panvk_priv_mem_host_addr(subq->context))
@@ -464,7 +454,7 @@ init_subqueue(struct panvk_gpu_queue *queue, enum panvk_subqueue_id subqueue)
       cs_req_res(&b, CS_COMPUTE_RES);
       break;
    default:
-      unreachable("Unknown subqueue");
+      UNREACHABLE("Unknown subqueue");
       break;
    }
 
@@ -504,7 +494,7 @@ init_subqueue(struct panvk_gpu_queue *queue, enum panvk_subqueue_id subqueue)
 
    drmSyncobjReset(dev->drm_fd, &queue->syncobj_handle, 1);
 
-   if (debug & PANVK_DEBUG_TRACE) {
+   if (PANVK_DEBUG(TRACE)) {
       pandecode_user_msg(dev->debug.decode_ctx, "Init subqueue %d binary\n\n",
                          subqueue);
       pandecode_cs_binary(dev->debug.decode_ctx, qsubmit.stream_addr,
@@ -535,9 +525,6 @@ static VkResult
 init_queue(struct panvk_gpu_queue *queue)
 {
    struct panvk_device *dev = to_panvk_device(queue->vk.base.device);
-   struct panvk_instance *instance =
-      to_panvk_instance(dev->vk.physical->instance);
-   unsigned debug = instance->debug_flags;
    VkResult result;
 
    struct panvk_pool_alloc_info alloc_info = {
@@ -565,7 +552,7 @@ init_queue(struct panvk_gpu_queue *queue)
          goto err_cleanup_queue;
    }
 
-   if (debug & PANVK_DEBUG_TRACE)
+   if (PANVK_DEBUG(TRACE))
       pandecode_next_frame(dev->debug.decode_ctx);
 
    return VK_SUCCESS;
@@ -711,7 +698,6 @@ cleanup_tiler(struct panvk_gpu_queue *queue)
 }
 
 struct panvk_queue_submit {
-   const struct panvk_instance *instance;
    const struct panvk_physical_device *phys_dev;
    struct panvk_device *dev;
    struct panvk_gpu_queue *queue;
@@ -750,10 +736,10 @@ static void
 panvk_queue_submit_init(struct panvk_queue_submit *submit,
                         struct vk_queue *vk_queue)
 {
+   MESA_TRACE_FUNC();
    struct vk_device *vk_dev = vk_queue->base.device;
 
    *submit = (struct panvk_queue_submit){
-      .instance = to_panvk_instance(vk_dev->physical->instance),
       .phys_dev = to_panvk_physical_device(vk_dev->physical),
       .dev = to_panvk_device(vk_dev),
       .queue = container_of(vk_queue, struct panvk_gpu_queue, vk),
@@ -763,8 +749,7 @@ panvk_queue_submit_init(struct panvk_queue_submit *submit,
       u_trace_should_process(&submit->dev->utrace.utctx) &&
       submit->phys_dev->kmod.props.timestamp_frequency;
 
-   submit->force_sync =
-      submit->instance->debug_flags & (PANVK_DEBUG_TRACE | PANVK_DEBUG_SYNC);
+   submit->force_sync = PANVK_DEBUG(TRACE) || PANVK_DEBUG(SYNC);
 }
 
 static void
@@ -772,6 +757,7 @@ panvk_queue_submit_init_storage(
    struct panvk_queue_submit *submit, const struct vk_queue_submit *vk_submit,
    struct panvk_queue_submit_stack_storage *stack_storage)
 {
+   MESA_TRACE_FUNC();
    submit->utrace.first_subqueue = PANVK_SUBQUEUE_COUNT;
    for (uint32_t i = 0; i < vk_submit->command_buffer_count; i++) {
       struct panvk_cmd_buffer *cmdbuf = container_of(
@@ -862,7 +848,7 @@ static void
 panvk_queue_submit_init_utrace(struct panvk_queue_submit *submit,
                                const struct vk_queue_submit *vk_submit)
 {
-   struct panvk_device *dev = submit->dev;
+   MESA_TRACE_FUNC();
 
    if (!submit->utrace.queue_mask)
       return;
@@ -873,6 +859,7 @@ panvk_queue_submit_init_utrace(struct panvk_queue_submit *submit,
     */
    struct panvk_utrace_flush_data *next = submit->utrace.data_storage;
    submit->utrace.data[submit->utrace.last_subqueue] = next++;
+   submit->utrace.data[submit->utrace.last_subqueue]->free_self = true;
 
    u_foreach_bit(i, submit->utrace.queue_mask) {
       if (i != submit->utrace.last_subqueue)
@@ -883,19 +870,16 @@ panvk_queue_submit_init_utrace(struct panvk_queue_submit *submit,
          .subqueue = i,
          .sync = wait ? submit->queue->utrace.sync : NULL,
          .wait_value = wait ? submit->queue->utrace.next_value : 0,
+         .free_self = false,
       };
    }
-
-   if (submit->utrace.needs_clone) {
-      struct panvk_pool *clone_pool = &submit->utrace.data_storage->clone_pool;
-      panvk_per_arch(utrace_clone_init_pool)(clone_pool, dev);
-   }
 }
 
 static void
 panvk_queue_submit_init_waits(struct panvk_queue_submit *submit,
                               const struct vk_queue_submit *vk_submit)
 {
+   MESA_TRACE_FUNC();
    if (!submit->needs_waits)
       return;
 
@@ -928,6 +912,7 @@ static void
 panvk_queue_submit_init_cmdbufs(struct panvk_queue_submit *submit,
                                 const struct vk_queue_submit *vk_submit)
 {
+   MESA_TRACE_FUNC();
    struct panvk_device *dev = submit->dev;
 
    for (uint32_t i = 0; i < vk_submit->command_buffer_count; i++) {
@@ -959,17 +944,30 @@ panvk_queue_submit_init_cmdbufs(struct panvk_queue_submit *submit,
          if (!u_trace_has_points(ut))
             continue;
 
-         const bool free_data = ut == submit->utrace.last_ut;
+         /* The last subqueue frees the flush data itself. */
+         bool free_data = ut == submit->utrace.last_ut;
 
          struct u_trace clone_ut;
          if (!(cmdbuf->flags & VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT)) {
             u_trace_init(&clone_ut, &dev->utrace.utctx);
 
-            struct panvk_pool *clone_pool =
-               &submit->utrace.data_storage->clone_pool;
+            const uint64_t root_buf_size = sizeof(uint64_t) * 1024;
+            struct panvk_utrace_buf *cs_root_buf =
+               panvk_utrace_create_buffer(&dev->utrace.utctx, root_buf_size);
+            assert(cs_root_buf);
+            /* For every sq, the cs buffer needs to be freed. */
+            free_data = true;
+
+            const struct cs_buffer cs_root = (struct cs_buffer){
+               .cpu = cs_root_buf->host,
+               .gpu = cs_root_buf->dev,
+               .capacity = root_buf_size / sizeof(uint64_t),
+            };
+
+            submit->utrace.data[j]->clone_cs_root = cs_root_buf;
             struct cs_builder clone_builder;
-            panvk_per_arch(utrace_clone_init_builder)(&clone_builder,
-                                                      clone_pool);
+            panvk_per_arch(utrace_clone_init_builder)(&clone_builder, dev,
+                                                      &cs_root);
 
             u_trace_clone_append(
                u_trace_begin_iterator(ut), u_trace_end_iterator(ut), &clone_ut,
@@ -1001,6 +999,7 @@ static void
 panvk_queue_submit_init_signals(struct panvk_queue_submit *submit,
                                 const struct vk_queue_submit *vk_submit)
 {
+   MESA_TRACE_FUNC();
    struct panvk_gpu_queue *queue = submit->queue;
 
    if (!submit->needs_signals)
@@ -1027,11 +1026,10 @@ static VkResult
 panvk_queue_submit_ioctl(struct panvk_queue_submit *submit)
 {
    const struct panvk_device *dev = submit->dev;
-   const struct panvk_instance *instance = submit->instance;
    struct panvk_gpu_queue *queue = submit->queue;
    int ret;
 
-   if (instance->debug_flags & PANVK_DEBUG_TRACE) {
+   if (PANVK_DEBUG(TRACE)) {
       /* If we're tracing, we need to reset the desc ringbufs and the CS
        * tracebuf. */
       for (uint32_t i = 0; i < ARRAY_SIZE(queue->subqueues); i++) {
@@ -1108,11 +1106,10 @@ panvk_queue_submit_process_signals(struct panvk_queue_submit *submit,
 static void
 panvk_queue_submit_process_debug(const struct panvk_queue_submit *submit)
 {
-   const struct panvk_instance *instance = submit->instance;
    struct panvk_gpu_queue *queue = submit->queue;
    struct pandecode_context *decode_ctx = submit->dev->debug.decode_ctx;
 
-   if (instance->debug_flags & PANVK_DEBUG_TRACE) {
+   if (PANVK_DEBUG(TRACE)) {
       const struct pan_kmod_dev_props *props = &submit->phys_dev->kmod.props;
 
       for (uint32_t i = 0; i < submit->qsubmit_count; i++) {
@@ -1153,10 +1150,10 @@ panvk_queue_submit_process_debug(const struct panvk_queue_submit *submit)
       }
    }
 
-   if (instance->debug_flags & PANVK_DEBUG_DUMP)
+   if (PANVK_DEBUG(DUMP))
       pandecode_dump_mappings(decode_ctx);
 
-   if (instance->debug_flags & PANVK_DEBUG_TRACE)
+   if (PANVK_DEBUG(TRACE))
       pandecode_next_frame(decode_ctx);
 
    /* validate last after the command streams are dumped */
@@ -1167,6 +1164,7 @@ panvk_queue_submit_process_debug(const struct panvk_queue_submit *submit)
 VkResult
 panvk_per_arch(gpu_queue_submit)(struct vk_queue *vk_queue, struct vk_queue_submit *vk_submit)
 {
+   MESA_TRACE_FUNC();
    struct panvk_queue_submit_stack_storage stack_storage;
    struct panvk_queue_submit submit;
    VkResult result = VK_SUCCESS;
@@ -1213,7 +1211,7 @@ get_panthor_group_priority(const VkDeviceQueueCreateInfo *create_info)
    case VK_QUEUE_GLOBAL_PRIORITY_REALTIME_KHR:
       return PANTHOR_GROUP_PRIORITY_REALTIME;
    default:
-      unreachable("Invalid global priority");
+      UNREACHABLE("Invalid global priority");
    }
 }
 
