@@ -44,7 +44,7 @@ att_set_clear_preload(const VkRenderingAttachmentInfo *att, bool *clear, bool *p
       *preload |= att->storeOp == VK_ATTACHMENT_STORE_OP_NONE;
       break;
    default:
-      unreachable("Unsupported loadOp");
+      UNREACHABLE("Unsupported loadOp");
    }
 }
 
@@ -129,11 +129,9 @@ render_state_set_z_attachment(struct panvk_cmd_buffer *cmdbuf,
    state->render.zs_pview = iview->pview;
    fbinfo->zs.view.zs = &state->render.zs_pview;
 
-   /* D32_S8 is a multiplanar format, so we need to adjust the format of the
-    * depth-only view to match the one of the depth plane.
-    */
-   if (iview->pview.format == PIPE_FORMAT_Z32_FLOAT_S8X24_UINT)
-      state->render.zs_pview.format = PIPE_FORMAT_Z32_FLOAT;
+   /* Fixup view format when the image is multiplanar. */
+   if (panvk_image_is_planar_depth_stencil(img))
+      state->render.zs_pview.format = panvk_image_depth_only_pfmt(img);
 
    state->render.zs_pview.planes[0] = (struct pan_image_plane_ref){
       .image = &img->planes[0].image,
@@ -149,13 +147,12 @@ render_state_set_z_attachment(struct panvk_cmd_buffer *cmdbuf,
     * If we touch the depth component, we need to make sure the stencil
     * component is preserved, hence the preload, and the view format adjusment.
     */
-   if (img->vk.format == VK_FORMAT_D24_UNORM_S8_UINT) {
+   if (panvk_image_is_interleaved_depth_stencil(img)) {
       fbinfo->zs.preload.s = true;
       cmdbuf->state.gfx.render.zs_pview.format =
-         PIPE_FORMAT_Z24_UNORM_S8_UINT;
+         img->planes[0].image.props.format;
    } else {
-      state->render.zs_pview.format =
-         vk_format_to_pipe_format(vk_format_depth_only(img->vk.format));
+      state->render.zs_pview.format = panvk_image_depth_only_pfmt(img);
    }
 
    if (att->loadOp == VK_ATTACHMENT_LOAD_OP_CLEAR)
@@ -195,19 +192,15 @@ render_state_set_s_attachment(struct panvk_cmd_buffer *cmdbuf,
    state->render.s_pview = iview->pview;
    fbinfo->zs.view.s = &state->render.s_pview;
 
-   /* D32_S8 is a multiplanar format, so we need to adjust the format of the
-    * stencil-only view to match the one of the stencil plane.
-    */
-   state->render.s_pview.format = img->vk.format == VK_FORMAT_D24_UNORM_S8_UINT
-                                     ? PIPE_FORMAT_Z24_UNORM_S8_UINT
-                                     : PIPE_FORMAT_S8_UINT;
-   if (img->vk.format == VK_FORMAT_D32_SFLOAT_S8_UINT) {
+   if (panvk_image_is_planar_depth_stencil(img)) {
+      state->render.s_pview.format = panvk_image_stencil_only_pfmt(img);
       state->render.s_pview.planes[0] = (struct pan_image_plane_ref){0};
       state->render.s_pview.planes[1] = (struct pan_image_plane_ref){
          .image = &img->planes[1].image,
          .plane_idx = 0,
       };
    } else {
+      state->render.s_pview.format = panvk_image_stencil_only_pfmt(img);
       state->render.s_pview.planes[0] = (struct pan_image_plane_ref){
          .image = &img->planes[0].image,
          .plane_idx = 0,
@@ -224,7 +217,7 @@ render_state_set_s_attachment(struct panvk_cmd_buffer *cmdbuf,
     * and the format is D24S8, we can combine them in a single view
     * addressing both components.
     */
-   if (img->vk.format == VK_FORMAT_D24_UNORM_S8_UINT &&
+   if (state->render.s_pview.format == PIPE_FORMAT_X24S8_UINT &&
        state->render.z_attachment.iview &&
        state->render.z_attachment.iview->vk.image == iview->vk.image) {
       state->render.zs_pview.format = PIPE_FORMAT_Z24_UNORM_S8_UINT;
@@ -236,6 +229,7 @@ render_state_set_s_attachment(struct panvk_cmd_buffer *cmdbuf,
     * is not supported on the stencil-only slot on Bifrost.
     */
    } else if (img->vk.format == VK_FORMAT_D24_UNORM_S8_UINT &&
+              state->render.s_pview.format == PIPE_FORMAT_X24S8_UINT &&
               fbinfo->zs.view.zs == NULL) {
       fbinfo->zs.view.zs = &state->render.s_pview;
       state->render.s_pview.format = PIPE_FORMAT_Z24_UNORM_S8_UINT;
@@ -294,8 +288,8 @@ panvk_per_arch(cmd_init_render_state)(struct panvk_cmd_buffer *cmdbuf,
       pRenderingInfo->layerCount;
    cmdbuf->state.gfx.render.view_mask = pRenderingInfo->viewMask;
    *fbinfo = (struct pan_fb_info){
-      .tile_buf_budget = pan_query_optimal_tib_size(phys_dev->model),
-      .z_tile_buf_budget = pan_query_optimal_z_tib_size(phys_dev->model),
+      .tile_buf_budget = pan_query_optimal_tib_size(PAN_ARCH, phys_dev->model),
+      .z_tile_buf_budget = pan_query_optimal_z_tib_size(PAN_ARCH, phys_dev->model),
       .nr_samples = 0,
       .rt_count = pRenderingInfo->colorAttachmentCount,
    };
@@ -716,7 +710,8 @@ panvk_per_arch(cmd_prepare_draw_sysvals)(struct panvk_cmd_buffer *cmdbuf,
 {
    const struct panvk_device *dev = to_panvk_device(cmdbuf->vk.base.device);
    struct vk_color_blend_state *cb = &cmdbuf->vk.dynamic_graphics_state.cb;
-   const struct panvk_shader *fs = get_fs(cmdbuf);
+   const struct panvk_shader_variant *fs =
+      panvk_shader_only_variant(get_fs(cmdbuf));
    uint32_t noperspective_varyings = fs ? fs->info.varyings.noperspective : 0;
    BITSET_DECLARE(dirty_sysvals, MAX_SYSVAL_FAUS) = {0};
 
@@ -736,7 +731,7 @@ panvk_per_arch(cmd_prepare_draw_sysvals)(struct panvk_cmd_buffer *cmdbuf,
    if (dyn_gfx_state_dirty(cmdbuf, CB_BLEND_CONSTANTS)) {
       for (unsigned i = 0; i < ARRAY_SIZE(cb->blend_constants); i++) {
          set_gfx_sysval(cmdbuf, dirty_sysvals, blend.constants[i],
-                        CLAMP(cb->blend_constants[i], 0.0f, 1.0f));
+                        cb->blend_constants[i]);
       }
    }
 
@@ -815,7 +810,8 @@ panvk_per_arch(cmd_prepare_draw_sysvals)(struct panvk_cmd_buffer *cmdbuf,
    if (dyn_gfx_state_dirty(cmdbuf, INPUT_ATTACHMENT_MAP))
       prepare_iam_sysvals(cmdbuf, dirty_sysvals);
 
-   const struct panvk_shader *vs = cmdbuf->state.gfx.vs.shader;
+   const struct panvk_shader_variant *vs =
+      panvk_shader_hw_variant(cmdbuf->state.gfx.vs.shader);
 
 #if PAN_ARCH < 9
    struct panvk_descriptor_state *desc_state = &cmdbuf->state.gfx.desc_state;
@@ -915,10 +911,6 @@ panvk_per_arch(CmdBindIndexBuffer2)(VkCommandBuffer commandBuffer,
       cmdbuf->state.gfx.ib.size = panvk_buffer_range(buf, offset, size);
       assert(cmdbuf->state.gfx.ib.size <= UINT32_MAX);
       cmdbuf->state.gfx.ib.dev_addr = panvk_buffer_gpu_ptr(buf, offset);
-#if PAN_ARCH < 9
-      cmdbuf->state.gfx.ib.host_addr =
-         buf && buf->host_ptr ? buf->host_ptr + offset : NULL;
-#endif
    } else {
       cmdbuf->state.gfx.ib.size = 0;
       /* In case of NullDescriptors, we need to set a non-NULL address and rely
@@ -926,9 +918,6 @@ panvk_per_arch(CmdBindIndexBuffer2)(VkCommandBuffer commandBuffer,
        * that this only works for v10+, as v9 does not have a way to specify the
        * index buffer size. */
       cmdbuf->state.gfx.ib.dev_addr = PAN_ARCH >= 10 ? 0x1000 : 0;
-#if PAN_ARCH < 9
-      cmdbuf->state.gfx.ib.host_addr = 0;
-#endif
    }
    cmdbuf->state.gfx.ib.index_size = vk_index_type_to_bytes(indexType);
 
