@@ -74,7 +74,7 @@ struct lower_desc_ctx {
    bool null_descriptor_support;
    nir_address_format ubo_addr_format;
    nir_address_format ssbo_addr_format;
-   struct panvk_shader *shader;
+   struct panvk_shader_variant *shader;
 };
 
 static nir_address_format
@@ -91,7 +91,7 @@ addr_format_for_desc_type(VkDescriptorType desc_type,
       return ctx->ssbo_addr_format;
 
    default:
-      unreachable("Unsupported descriptor type");
+      UNREACHABLE("Unsupported descriptor type");
    }
 }
 
@@ -218,10 +218,14 @@ shader_desc_idx(uint32_t set, uint32_t binding,
 static nir_address_format
 addr_format_for_type(VkDescriptorType type, const struct lower_desc_ctx *ctx)
 {
+   /* Mutable must imply that both formats are the same. */
+   assert(type != VK_DESCRIPTOR_TYPE_MUTABLE_EXT || ctx->ubo_addr_format == ctx->ssbo_addr_format);
+
    switch (type) {
    case VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER:
    case VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC:
    case VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK:
+   case VK_DESCRIPTOR_TYPE_MUTABLE_EXT:
       return ctx->ubo_addr_format;
 
    case VK_DESCRIPTOR_TYPE_STORAGE_BUFFER:
@@ -229,7 +233,7 @@ addr_format_for_type(VkDescriptorType type, const struct lower_desc_ctx *ctx)
       return ctx->ssbo_addr_format;
 
    default:
-      unreachable("Unsupported descriptor type");
+      UNREACHABLE("Unsupported descriptor type");
       return ~0;
    }
 }
@@ -323,7 +327,7 @@ build_res_index(nir_builder *b, uint32_t set, uint32_t binding,
 #endif
 
    default:
-      unreachable("Unsupported descriptor type");
+      UNREACHABLE("Unsupported descriptor type");
    }
 }
 
@@ -358,7 +362,7 @@ build_res_reindex(nir_builder *b, nir_def *orig, nir_def *delta,
 #endif
 
    default:
-      unreachable("Unhandled address format");
+      UNREACHABLE("Unhandled address format");
    }
 }
 
@@ -432,7 +436,7 @@ build_buffer_addr_for_res_index(nir_builder *b, nir_def *res_index,
 #endif
 
    default:
-      unreachable("Unhandled address format");
+      UNREACHABLE("Unhandled address format");
    }
 }
 
@@ -464,7 +468,7 @@ lower_res_intrinsic(nir_builder *b, nir_intrinsic_instr *intrin,
       break;
 
    default:
-      unreachable("Unhandled resource intrinsic");
+      UNREACHABLE("Unhandled resource intrinsic");
    }
 
    assert(intrin->def.bit_size == res->bit_size);
@@ -768,7 +772,7 @@ lower_tex(nir_builder *b, nir_tex_instr *tex, const struct lower_desc_ctx *ctx)
          res = load_tex_samples(b, deref, dim, ctx);
          break;
       default:
-         unreachable("Unsupported texture query op");
+         UNREACHABLE("Unsupported texture query op");
       }
 
       nir_def_replace(&tex->def, res);
@@ -797,14 +801,14 @@ lower_tex(nir_builder *b, nir_tex_instr *tex, const struct lower_desc_ctx *ctx)
       struct panvk_subdesc_info subdesc =
          get_sampler_subdesc_info(bind_layout->type, plane);
       uint32_t desc_stride = get_desc_array_stride(bind_layout, subdesc.type);
-
-      tex->sampler_index =
-         shader_desc_idx(set, binding, subdesc, ctx) +
-         index_imm * desc_stride;
+      uint32_t sampler_index = shader_desc_idx(set, binding, subdesc, ctx) + index_imm * desc_stride;
 
       if (index_ssa != NULL) {
          nir_def *offset = nir_imul_imm(b, index_ssa, desc_stride);
+         offset = nir_iadd_imm(b, offset, sampler_index);
          nir_tex_instr_add_src(tex, nir_tex_src_sampler_offset, offset);
+      } else {
+         tex->sampler_index = sampler_index;
       }
       progress = true;
    } else {
@@ -830,14 +834,14 @@ lower_tex(nir_builder *b, nir_tex_instr *tex, const struct lower_desc_ctx *ctx)
       struct panvk_subdesc_info subdesc =
          get_tex_subdesc_info(bind_layout->type,  plane);
       uint32_t desc_stride = get_desc_array_stride(bind_layout, subdesc.type);
-
-      tex->texture_index =
-         shader_desc_idx(set, binding, subdesc, ctx) +
-         index_imm * desc_stride;
+      uint32_t texture_index = shader_desc_idx(set, binding, subdesc, ctx) + index_imm * desc_stride;
 
       if (index_ssa != NULL) {
          nir_def *offset = nir_imul_imm(b, index_ssa, desc_stride);
+         offset = nir_iadd_imm(b, offset, texture_index);
          nir_tex_instr_add_src(tex, nir_tex_src_texture_offset, offset);
+      } else {
+         tex->texture_index = texture_index;
       }
       progress = true;
    }
@@ -858,7 +862,8 @@ get_img_index(nir_builder *b, nir_deref_instr *deref,
       get_binding_layout(set, binding, ctx);
    assert(bind_layout->type == VK_DESCRIPTOR_TYPE_STORAGE_IMAGE ||
           bind_layout->type == VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER ||
-          bind_layout->type == VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER);
+          bind_layout->type == VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER ||
+          bind_layout->type == VK_DESCRIPTOR_TYPE_MUTABLE_EXT);
 
    unsigned img_offset = shader_desc_idx(set, binding, NO_SUBDESC, ctx);
 
@@ -872,7 +877,7 @@ get_img_index(nir_builder *b, nir_deref_instr *deref,
 
 struct panvk_lower_input_attachment_load_ctx {
    uint32_t ro_color_mask;
-   struct panvk_shader *shader;
+   struct panvk_shader_variant *shader;
 };
 
 static bool
@@ -890,7 +895,7 @@ lower_input_attachment_load(nir_builder *b, nir_intrinsic_instr *intr,
       return false;
 
    const struct panvk_lower_input_attachment_load_ctx *ctx = data;
-   struct panvk_shader *shader = ctx->shader;
+   struct panvk_shader_variant *shader = ctx->shader;
    nir_variable *var = nir_deref_instr_get_variable(deref);
    assert(var);
 
@@ -918,6 +923,8 @@ lower_input_attachment_load(nir_builder *b, nir_intrinsic_instr *intr,
       nir_def *is_color_att = nir_ilt_imm(b, target, 8);
       nir_def *load_color, *load_zs;
       nir_io_semantics iosem = {0};
+      iosem.fb_fetch_output = true;
+      iosem.fb_fetch_output_coherent = !!(nir_intrinsic_access(intr) & ACCESS_COHERENT);
 
       nir_push_if(b, is_color_att);
       {
@@ -1066,7 +1073,7 @@ readonly_color_mask(nir_shader *nir,
 static bool
 lower_input_attachment_loads(nir_shader *nir,
                              const struct vk_graphics_pipeline_state *state,
-                             struct panvk_shader *shader)
+                             struct panvk_shader_variant *shader)
 {
    bool progress = false;
    struct panvk_lower_input_attachment_load_ctx ia_load_ctx = {
@@ -1075,7 +1082,7 @@ lower_input_attachment_loads(nir_shader *nir,
    };
 
    NIR_PASS(progress, nir, nir_shader_intrinsics_pass,
-            lower_input_attachment_load, nir_metadata_control_flow,
+            lower_input_attachment_load, nir_metadata_none,
             &ia_load_ctx);
 
    /* Lower the remaining input attachment loads. */
@@ -1111,7 +1118,7 @@ lower_img_intrinsic(nir_builder *b, nir_intrinsic_instr *intr,
          res = load_img_samples(b, deref, dim, ctx);
          break;
       default:
-         unreachable("Unsupported image query op");
+         UNREACHABLE("Unsupported image query op");
       }
 
       nir_def_replace(&intr->def, res);
@@ -1295,7 +1302,7 @@ create_copy_table(nir_shader *nir, struct lower_desc_ctx *ctx)
       dummy_sampler_idx = 0;
       break;
    default:
-      unreachable("unexpected stage");
+      UNREACHABLE("unexpected stage");
    }
    desc_info->dummy_sampler_handle = pan_res_handle(0, dummy_sampler_idx);
 
@@ -1453,7 +1460,7 @@ collect_instr_desc_access(nir_builder *b, nir_instr *instr, void *data)
 }
 
 static void
-upload_shader_desc_info(struct panvk_device *dev, struct panvk_shader *shader,
+upload_shader_desc_info(struct panvk_device *dev, struct panvk_shader_variant *shader,
                         const struct panvk_shader_desc_info *desc_info)
 {
 #if PAN_ARCH < 9
@@ -1496,7 +1503,7 @@ panvk_per_arch(nir_lower_descriptors)(
    nir_shader *nir, struct panvk_device *dev,
    const struct vk_pipeline_robustness_state *rs, uint32_t set_layout_count,
    struct vk_descriptor_set_layout *const *set_layouts,
-   const struct vk_graphics_pipeline_state *state, struct panvk_shader *shader)
+   const struct vk_graphics_pipeline_state *state, struct panvk_shader_variant *shader)
 {
    struct lower_desc_ctx ctx = {
       .shader = shader,
