@@ -7,7 +7,16 @@
 
 #pragma once
 
+#ifndef ANDROID_LIBPERFETTO
 #include <perfetto.h>
+#else
+#include <perfetto/tracing.h>
+#include <perfetto/common/gpu_counter_descriptor.gen.h>
+#include <perfetto/common/gpu_counter_descriptor.pbzero.h>
+#include <perfetto/config/gpu/gpu_counter_config.pbzero.h>
+#include <perfetto/trace/clock_snapshot.pbzero.h>
+#include <perfetto/trace/gpu/gpu_counter_event.pbzero.h>
+#endif
 
 #define PPS_LOG PERFETTO_LOG
 #define PPS_LOG_IMPORTANT PERFETTO_ILOG
@@ -27,6 +36,8 @@ enum class State {
 /// @return True if ok, false otherwise
 bool check(int res, const char *msg);
 
+void make_thread_rt();
+
 /// @param num Numerator
 /// @param den Denominator
 /// @return A ratio between two floating point numbers, or 0 if the denominator is 0
