@@ -112,7 +112,7 @@ vn_cmd_get_cached_storage(struct vn_command_buffer *cmd,
       barriers_size = barrier_count * sizeof(VkImageMemoryBarrier2);
       break;
    default:
-      unreachable("invalid barrier_type");
+      UNREACHABLE("invalid barrier_type");
    }
 
    size_t total_size =
@@ -218,8 +218,6 @@ vn_cmd_fix_image_memory_barrier_common(const struct vn_image *img,
        *new_layout != VK_IMAGE_LAYOUT_PRESENT_SRC_KHR)
       return result;
 
-   assert(img->wsi.is_wsi);
-
    /* prime blit src or no layout transition */
    if (img->wsi.is_prime_blit_src || *old_layout == *new_layout) {
       if (*old_layout == VK_IMAGE_LAYOUT_PRESENT_SRC_KHR)
@@ -2389,7 +2387,7 @@ vn_CmdPushDescriptorSetWithTemplate2(VkCommandBuffer commandBuffer,
       stage_flags = VK_SHADER_STAGE_RAYGEN_BIT_KHR;
       break;
    default:
-      unreachable("bad pipeline bind point in the template");
+      UNREACHABLE("bad pipeline bind point in the template");
       break;
    }
    const VkPushDescriptorSetInfo info = {
