@@ -30,6 +30,11 @@ vn_wsi_create_image(struct vn_device *dev,
                     const VkAllocationCallbacks *alloc,
                     struct vn_image **out_img);
 
+bool
+vn_wsi_validate_image_format_info(
+   struct vn_physical_device *physical_dev,
+   const VkPhysicalDeviceImageFormatInfo2 *info);
+
 #else
 
 static inline VkResult
@@ -53,6 +58,13 @@ vn_wsi_create_image(struct vn_device *dev,
    return VK_ERROR_OUT_OF_HOST_MEMORY;
 }
 
+static inline bool
+vn_wsi_validate_image_format_info(struct vn_physical_device *physical_dev,
+                                  const VkPhysicalDeviceImageFormatInfo2 *info)
+{
+   return true;
+}
+
 #endif /* VN_USE_WSI_PLATFORM */
 
 #endif /* VN_WSI_H */
