@@ -24,6 +24,7 @@
 #include "vk_fence.h"
 
 #include "util/os_time.h"
+#include "util/perf/cpu_trace.h"
 
 #ifndef _WIN32
 #include <unistd.h>
@@ -151,7 +152,7 @@ vk_common_CreateFence(VkDevice _device,
                       VkFence *pFence)
 {
    VK_FROM_HANDLE(vk_device, device, _device);
-   struct vk_fence *fence;
+   struct vk_fence *fence = NULL;
 
    VkResult result = vk_fence_create(device, pCreateInfo, pAllocator, &fence);
    if (result != VK_SUCCESS)
@@ -252,6 +253,8 @@ vk_common_WaitForFences(VkDevice _device,
                         VkBool32 waitAll,
                         uint64_t timeout)
 {
+   MESA_TRACE_FUNC();
+
    VK_FROM_HANDLE(vk_device, device, _device);
 
    if (vk_device_is_lost(device))
@@ -268,7 +271,7 @@ vk_common_WaitForFences(VkDevice _device,
       VK_FROM_HANDLE(vk_fence, fence, pFences[i]);
       waits[i] = (struct vk_sync_wait) {
          .sync = vk_fence_get_active_sync(fence),
-         .stage_mask = ~(VkPipelineStageFlags2KHR)0,
+         .stage_mask = ~(VkPipelineStageFlags2)0,
       };
    }
 
@@ -470,7 +473,7 @@ vk_common_GetFenceFdKHR(VkDevice _device,
       break;
 
    default:
-      unreachable("Invalid fence export handle type");
+      UNREACHABLE("Invalid fence export handle type");
    }
 
    /* From the Vulkan 1.2.194 spec:
