@@ -41,7 +41,7 @@ static void
 vk_shader_init(struct vk_shader *shader,
                struct vk_device *device,
                const struct vk_shader_ops *ops,
-               gl_shader_stage stage)
+               mesa_shader_stage stage)
 {
    vk_object_base_init(device, &shader->base, VK_OBJECT_TYPE_SHADER_EXT);
    shader->ops = ops;
@@ -51,7 +51,7 @@ vk_shader_init(struct vk_shader *shader,
 void *
 vk_shader_zalloc(struct vk_device *device,
                  const struct vk_shader_ops *ops,
-                 gl_shader_stage stage,
+                 mesa_shader_stage stage,
                  const VkAllocationCallbacks *alloc,
                  size_t size)
 {
@@ -79,7 +79,7 @@ void *
 vk_shader_multizalloc(struct vk_device *device,
                       struct vk_multialloc *ma,
                       const struct vk_shader_ops *ops,
-                      gl_shader_stage stage,
+                      mesa_shader_stage stage,
                       const VkAllocationCallbacks *alloc)
 {
    struct vk_shader *shader =
@@ -103,7 +103,7 @@ vk_shader_free(struct vk_device *device,
 }
 
 int
-vk_shader_cmp_graphics_stages(gl_shader_stage a, gl_shader_stage b)
+vk_shader_cmp_graphics_stages(mesa_shader_stage a, mesa_shader_stage b)
 {
    static const int stage_order[MESA_SHADER_MESH + 1] = {
       [MESA_SHADER_VERTEX] = 1,
@@ -121,8 +121,26 @@ vk_shader_cmp_graphics_stages(gl_shader_stage a, gl_shader_stage b)
    return stage_order[a] - stage_order[b];
 }
 
+int
+vk_shader_cmp_rt_stages(mesa_shader_stage a, mesa_shader_stage b)
+{
+   static const int stage_order[MESA_SHADER_CALLABLE + 1] = {
+      [MESA_SHADER_RAYGEN] = 1,
+      [MESA_SHADER_ANY_HIT] = 2,
+      [MESA_SHADER_CLOSEST_HIT] = 3,
+      [MESA_SHADER_MISS] = 4,
+      [MESA_SHADER_INTERSECTION] = 5,
+      [MESA_SHADER_CALLABLE] = 6,
+   };
+
+   assert(a < ARRAY_SIZE(stage_order) && stage_order[a] > 0);
+   assert(b < ARRAY_SIZE(stage_order) && stage_order[b] > 0);
+
+   return stage_order[a] - stage_order[b];
+}
+
 struct stage_idx {
-   gl_shader_stage stage;
+   mesa_shader_stage stage;
    uint32_t idx;
 };
 
@@ -140,28 +158,28 @@ vk_shader_to_nir(struct vk_device *device,
 {
    const struct vk_device_shader_ops *ops = device->shader_ops;
 
-   const gl_shader_stage stage = vk_to_mesa_shader_stage(info->stage);
+   const mesa_shader_stage stage = vk_to_mesa_shader_stage(info->stage);
    const nir_shader_compiler_options *nir_options =
       ops->get_nir_options(device->physical, stage, rs);
    struct spirv_to_nir_options spirv_options =
       ops->get_spirv_options(device->physical, stage, rs);
 
-   enum gl_subgroup_size subgroup_size = vk_get_subgroup_size(
-      vk_spirv_version(info->pCode, info->codeSize),
-      stage, info->pNext,
-      info->flags & VK_SHADER_CREATE_ALLOW_VARYING_SUBGROUP_SIZE_BIT_EXT,
-      info->flags &VK_SHADER_CREATE_REQUIRE_FULL_SUBGROUPS_BIT_EXT);
-
    nir_shader *nir = vk_spirv_to_nir(device,
                                      info->pCode, info->codeSize,
                                      stage, info->pName,
-                                     subgroup_size,
                                      info->pSpecializationInfo,
                                      &spirv_options, nir_options,
                                      false /* internal */, NULL);
    if (nir == NULL)
       return NULL;
 
+   vk_set_subgroup_size(
+      device, nir,
+      vk_spirv_version(info->pCode, info->codeSize),
+      info->pNext,
+      info->flags & VK_SHADER_CREATE_ALLOW_VARYING_SUBGROUP_SIZE_BIT_EXT,
+      info->flags & VK_SHADER_CREATE_REQUIRE_FULL_SUBGROUPS_BIT_EXT);
+
    if (ops->preprocess_nir != NULL)
       ops->preprocess_nir(device->physical, nir, rs);
 
@@ -518,7 +536,7 @@ vk_common_CreateShadersEXT(VkDevice _device,
       }
 
       default:
-         unreachable("Unknown shader code type");
+         UNREACHABLE("Unknown shader code type");
       }
 
       if (first_fail_or_success == VK_SUCCESS)
@@ -562,8 +580,7 @@ vk_common_CreateShadersEXT(VkDevice _device,
          }
       } else {
          for (uint32_t l = 0; l < linked_count; l++) {
-            if (infos[l].nir != NULL)
-               ralloc_free(infos[l].nir);
+            ralloc_free(infos[l].nir);
          }
       }
 
@@ -598,7 +615,7 @@ vk_common_CmdBindShadersEXT(VkCommandBuffer commandBuffer,
    struct vk_device *device = cmd_buffer->base.device;
    const struct vk_device_shader_ops *ops = device->shader_ops;
 
-   STACK_ARRAY(gl_shader_stage, stages, stageCount);
+   STACK_ARRAY(mesa_shader_stage, stages, stageCount);
    STACK_ARRAY(struct vk_shader *, shaders, stageCount);
 
    VkShaderStageFlags vk_stages = 0;
